@@ -3,7 +3,11 @@
 
 # We import the names "as" themselves here (and here only) to tell linting tools
 # that they are explicitly being exported here (and not just unused).
+from arraykit import mloc as mloc
+from arraykit import isna_element as isna_element
 
+from static_frame.core.archive_npy import NPY as NPY
+from static_frame.core.archive_npy import NPZ as NPZ
 from static_frame.core.batch import Batch as Batch
 from static_frame.core.bus import Bus as Bus
 from static_frame.core.display import Display as Display
@@ -26,8 +30,8 @@ from static_frame.core.exception import LocInvalid
 from static_frame.core.exception import StoreFileMutation
 from static_frame.core.frame import Frame as Frame
 from static_frame.core.frame import FrameAssign as FrameAssign
-from static_frame.core.frame import FrameAssignILoc as FrameAssignILoc
 from static_frame.core.frame import FrameAssignBLoc as FrameAssignBLoc
+from static_frame.core.frame import FrameAssignILoc as FrameAssignILoc
 from static_frame.core.frame import FrameGO as FrameGO
 from static_frame.core.frame import FrameHE as FrameHE
 from static_frame.core.hloc import HLoc as HLoc
@@ -37,6 +41,7 @@ from static_frame.core.index import IndexGO as IndexGO
 from static_frame.core.index_auto import IndexAutoFactory as IndexAutoFactory
 from static_frame.core.index_auto import IndexAutoFactoryType
 from static_frame.core.index_auto import IndexAutoInitializer as IndexAutoInitializer
+from static_frame.core.index_auto import IndexDefaultFactory as IndexDefaultFactory
 from static_frame.core.index_datetime import IndexDate as IndexDate
 from static_frame.core.index_datetime import IndexDateGO as IndexDateGO
 from static_frame.core.index_datetime import IndexHour as IndexHour
@@ -61,6 +66,7 @@ from static_frame.core.index_level import IndexLevel as IndexLevel
 from static_frame.core.index_level import IndexLevelGO as IndexLevelGO
 from static_frame.core.interface_meta import InterfaceMeta as InterfaceMeta
 from static_frame.core.node_dt import InterfaceDatetime
+from static_frame.core.node_fill_value import InterfaceFillValue
 from static_frame.core.node_iter import IterNodeApplyType as IterNodeApplyType
 from static_frame.core.node_iter import IterNodeAxis
 from static_frame.core.node_iter import IterNodeDelegate as IterNodeDelegate
@@ -78,10 +84,12 @@ from static_frame.core.node_selector import InterfaceGetItem as InterfaceGetItem
 from static_frame.core.node_selector import InterfaceSelectDuo
 from static_frame.core.node_selector import InterfaceSelectQuartet
 from static_frame.core.node_selector import InterfaceSelectTrio
-from static_frame.core.node_transpose import InterfaceTranspose
+from static_frame.core.node_re import InterfaceRe
 from static_frame.core.node_str import InterfaceString
+from static_frame.core.node_transpose import InterfaceTranspose
 from static_frame.core.platform import Platform as Platform
 from static_frame.core.quilt import Quilt as Quilt
+from static_frame.core.yarn import Yarn as Yarn
 from static_frame.core.series import Series as Series
 from static_frame.core.series import SeriesAssign as SeriesAssign
 from static_frame.core.series import SeriesHE as SeriesHE
@@ -97,10 +105,9 @@ from static_frame.core.util import GetItemKeyTypeCompound as GetItemKeyTypeCompo
 from static_frame.core.util import IndexInitializer as IndexInitializer
 from static_frame.core.util import IndexSpecifier as IndexSpecifier
 from static_frame.core.util import KeyOrKeys as KeyOrKeys
-from static_frame.core.util import mloc as mloc
 from static_frame.core.util import PathSpecifierOrFileLike as PathSpecifierOrFileLike
 from static_frame.core.util import SeriesInitializer as SeriesInitializer
 
-__version__ = '0.8.5' # use -dev for new version in development
+__version__ = '0.8.35' # use -dev for new version in development
 
 
