@@ -3,7 +3,7 @@ import typing as tp
 
 import numpy as np
 
-from static_frame.core.util import immutable_filter
+from arraykit import immutable_filter
 from static_frame.core.util import DTYPE_OBJECT
 from static_frame.core.util import array_deepcopy
 
@@ -32,13 +32,13 @@ class ArrayGO:
         Args:
             own_iterable: flag iterable as ownable by this instance.
         '''
-        if isinstance(iterable, np.ndarray):
+        if iterable.__class__ is np.ndarray:
             if own_iterable:
                 self._array = iterable
-                self._array.flags.writeable = False
+                self._array.flags.writeable = False #type: ignore
             else:
                 self._array = immutable_filter(iterable)
-            if self._array.dtype != self._DTYPE:
+            if self._array.dtype != self._DTYPE: #type: ignore
                 raise NotImplementedError('only object arrays are supported')
             self._recache = False
             self._array_mutable = None
