@@ -1,6 +1,8 @@
 import typing as tp
 from concurrent.futures import ProcessPoolExecutor
 from concurrent.futures import ThreadPoolExecutor
+# import multiprocessing as mp
+# mp_context = mp.get_context('spawn')
 
 import numpy as np
 
@@ -27,6 +29,7 @@ from static_frame.core.store_zip import StoreZipCSV
 from static_frame.core.store_zip import StoreZipParquet
 from static_frame.core.store_zip import StoreZipPickle
 from static_frame.core.store_zip import StoreZipTSV
+from static_frame.core.store_zip import StoreZipNPZ
 from static_frame.core.util import AnyCallable
 from static_frame.core.util import Bloc2DKeyType
 from static_frame.core.util import DEFAULT_SORT_KIND
@@ -39,6 +42,8 @@ from static_frame.core.util import KeyOrKeys
 from static_frame.core.util import NameType
 from static_frame.core.util import PathSpecifier
 from static_frame.core.util import UFunc
+from static_frame.core.style_config import StyleConfig
+from static_frame.core.exception import BatchIterableInvalid
 
 
 FrameOrSeries = tp.Union[Frame, Series]
@@ -52,7 +57,7 @@ GeneratorFrameItems = tp.Callable[..., IteratorFrameItems]
 def normalize_container(post: tp.Any
         ) -> FrameOrSeries:
     # post might be an element, promote to a Series to permit concatenation
-    if isinstance(post, np.ndarray):
+    if post.__class__ is np.ndarray:
         if post.ndim == 1:
             return Series(post)
         elif post.ndim == 2:
@@ -138,8 +143,7 @@ class Batch(ContainerOperand, StoreClientMixin):
                 max_workers=max_workers,
                 chunksize=chunksize,
                 use_threads=use_threads,
-                                )
-
+                )
 
     @classmethod
     @doc_inject(selector='batch_constructor')
@@ -162,7 +166,7 @@ class Batch(ContainerOperand, StoreClientMixin):
                 max_workers=max_workers,
                 chunksize=chunksize,
                 use_threads=use_threads,
-                                )
+                )
 
     @classmethod
     @doc_inject(selector='batch_constructor')
@@ -210,6 +214,28 @@ class Batch(ContainerOperand, StoreClientMixin):
                 use_threads=use_threads,
                                 )
 
+    @classmethod
+    @doc_inject(selector='batch_constructor')
+    def from_zip_npz(cls,
+            fp: PathSpecifier,
+            *,
+            config: StoreConfigMapInitializer = None,
+            max_workers: tp.Optional[int] = None,
+            chunksize: int = 1,
+            use_threads: bool = False,
+            ) -> 'Batch':
+        '''
+        Given a file path to zipped NPZ :obj:`Batch` store, return a :obj:`Batch` instance.
+
+        {args}
+        '''
+        store = StoreZipNPZ(fp)
+        return cls._from_store(store,
+                config=config,
+                max_workers=max_workers,
+                chunksize=chunksize,
+                use_threads=use_threads,
+                )
 
     @classmethod
     @doc_inject(selector='batch_constructor')
@@ -232,7 +258,7 @@ class Batch(ContainerOperand, StoreClientMixin):
                 max_workers=max_workers,
                 chunksize=chunksize,
                 use_threads=use_threads,
-                                )
+                )
 
 
     @classmethod
@@ -257,7 +283,7 @@ class Batch(ContainerOperand, StoreClientMixin):
                 max_workers=max_workers,
                 chunksize=chunksize,
                 use_threads=use_threads,
-                                )
+                )
 
 
     @classmethod
@@ -281,7 +307,7 @@ class Batch(ContainerOperand, StoreClientMixin):
                 max_workers=max_workers,
                 chunksize=chunksize,
                 use_threads=use_threads,
-                                )
+                )
 
 
     @classmethod
@@ -305,10 +331,9 @@ class Batch(ContainerOperand, StoreClientMixin):
                 max_workers=max_workers,
                 chunksize=chunksize,
                 use_threads=use_threads,
-                                )
+                )
 
     #---------------------------------------------------------------------------
-
     @doc_inject(selector='batch_init')
     def __init__(self,
             items: IteratorFrameItems,
@@ -378,7 +403,9 @@ class Batch(ContainerOperand, StoreClientMixin):
 
 
     def display(self,
-            config: tp.Optional[DisplayConfig] = None
+            config: tp.Optional[DisplayConfig] = None,
+            *,
+            style_config: tp.Optional[StyleConfig] = None,
             ) -> Display:
         '''Provide a :obj:`Series`-style display of the :obj:`Batch`. Note that if the held iterator is a generator, this display will exhaust the generator.
         '''
@@ -390,7 +417,10 @@ class Batch(ContainerOperand, StoreClientMixin):
         display_cls = Display.from_values((),
                 header=DisplayHeader(self.__class__, self._name),
                 config=config)
-        return series._display(config, display_cls)
+        return series._display(config,
+                display_cls=display_cls,
+                style_config=style_config,
+                )
 
     def __repr__(self) -> str:
         '''Provide a display of the :obj:`Batch` that does not exhaust the generator.
@@ -404,6 +434,16 @@ class Batch(ContainerOperand, StoreClientMixin):
     #---------------------------------------------------------------------------
     # core function application routines
 
+    def _iter_items(self) -> IteratorFrameItems:
+        '''Iter pairs in items, providing helpful exception of a pair is not found. Thies is necessary as we cannot validate the items until we actually do an iteration, and the iterable might be an iterator.
+        '''
+        for pair in self._items:
+            try:
+                label, frame = pair
+            except ValueError:
+                raise BatchIterableInvalid() from None
+            yield label, frame
+
     def _apply_pool(self,
             labels: tp.List[tp.Hashable],
             arg_iter: tp.Iterator[tp.Tuple[tp.Any, ...]],
@@ -456,13 +496,13 @@ class Batch(ContainerOperand, StoreClientMixin):
         '''
         if self._max_workers is None:
             def gen() -> IteratorFrameItems:
-                for label, frame in self._items:
+                for label, frame in self._iter_items():
                     yield label, call_attr((frame, attr, args, kwargs))
             return self._derive(gen)
 
         labels = []
         def arg_gen() -> tp.Iterator[tp.Tuple[FrameOrSeries, str, tp.Any, tp.Any]]:
-            for label, frame in self._items:
+            for label, frame in self._iter_items():
                 labels.append(label)
                 yield frame, attr, args, kwargs
 
@@ -474,13 +514,13 @@ class Batch(ContainerOperand, StoreClientMixin):
         '''
         if self._max_workers is None:
             def gen() -> IteratorFrameItems:
-                for label, frame in self._items:
+                for label, frame in self._iter_items():
                     yield label, call_func((frame, func))
             return self._derive(gen)
 
         labels = []
         def arg_gen() -> tp.Iterator[tp.Tuple[FrameOrSeries, AnyCallable]]:
-            for label, frame in self._items:
+            for label, frame in self._iter_items():
                 labels.append(label)
                 yield frame, func
 
@@ -495,7 +535,7 @@ class Batch(ContainerOperand, StoreClientMixin):
         '''
         if self._max_workers is None:
             def gen() -> IteratorFrameItems:
-                for label, frame in self._items:
+                for label, frame in self._iter_items():
                     try:
                         yield label, call_func((frame, func))
                     except exception:
@@ -504,7 +544,7 @@ class Batch(ContainerOperand, StoreClientMixin):
 
         labels = []
         def arg_gen() -> tp.Iterator[tp.Tuple[FrameOrSeries, AnyCallable]]:
-            for label, frame in self._items:
+            for label, frame in self._iter_items():
                 labels.append(label)
                 yield frame, func
 
@@ -520,13 +560,13 @@ class Batch(ContainerOperand, StoreClientMixin):
         '''
         if self._max_workers is None:
             def gen() -> IteratorFrameItems:
-                for label, frame in self._items:
+                for label, frame in self._iter_items():
                     yield label, call_func_items((frame, func, label))
             return self._derive(gen)
 
         labels = []
         def arg_gen() -> tp.Iterator[tp.Tuple[FrameOrSeries, AnyCallable, tp.Hashable]]:
-            for label, frame in self._items:
+            for label, frame in self._iter_items():
                 labels.append(label)
                 yield frame, func, label
 
@@ -541,7 +581,7 @@ class Batch(ContainerOperand, StoreClientMixin):
         '''
         if self._max_workers is None:
             def gen() -> IteratorFrameItems:
-                for label, frame in self._items:
+                for label, frame in self._iter_items():
                     try:
                         yield label, call_func_items((frame, func, label))
                     except exception:
@@ -550,7 +590,7 @@ class Batch(ContainerOperand, StoreClientMixin):
 
         labels = []
         def arg_gen() -> tp.Iterator[tp.Tuple[FrameOrSeries, AnyCallable, tp.Hashable]]:
-            for label, frame in self._items:
+            for label, frame in self._iter_items():
                 labels.append(label)
                 yield frame, func, label
 
@@ -638,7 +678,7 @@ class Batch(ContainerOperand, StoreClientMixin):
         '''
         Iterator of :obj:`Frame` labels.
         '''
-        for k, _ in self._items:
+        for k, _ in self._iter_items():
             yield k
 
     def __iter__(self) -> tp.Iterator[tp.Hashable]:
@@ -652,13 +692,13 @@ class Batch(ContainerOperand, StoreClientMixin):
         '''
         Return an iterator of values (:obj:`Frame` or :obj:`Series`) stored in this :obj:`Batch`.
         '''
-        return (v for _, v in self._items)
+        return (v for _, v in self._iter_items())
 
     def items(self) -> IteratorFrameItems:
         '''
         Iterator of labels, :obj:`Frame`.
         '''
-        return self._items.__iter__()
+        return self._iter_items()
 
     _items_store = items
 
@@ -676,6 +716,7 @@ class Batch(ContainerOperand, StoreClientMixin):
     def _ufunc_binary_operator(self, *,
             operator: UFunc,
             other: tp.Any,
+            fill_value: object = np.nan,
             ) -> 'Batch':
         return self._apply_attr(
                 attr='_ufunc_binary_operator',
@@ -1042,6 +1083,24 @@ class Batch(ContainerOperand, StoreClientMixin):
                 axis=axis,
                 )
 
+    def cov(self, *,
+            axis: int = 1,
+            ddof: int = 1,
+            ) -> 'Batch':
+        '''
+        Compute a covariance matrix.
+
+        Args:
+            axis: if 0, each row represents a variable, with observations as columns; if 1, each column represents a variable, with observations as rows. Defaults to 1.
+            ddof: Delta degrees of freedom, defaults to 1.
+        '''
+        return self._apply_attr(
+                attr='cov',
+                axis=axis,
+                ddof=ddof,
+                )
+
+
     #---------------------------------------------------------------------------
     # utility function to numpy array
 
