@@ -1,7 +1,6 @@
 import typing as tp
-import operator as operator_mod
 from functools import partial
-
+from collections import deque
 import numpy as np
 
 from static_frame.core.display import Display
@@ -19,6 +18,8 @@ from static_frame.core.util import ufunc_all
 from static_frame.core.util import ufunc_any
 from static_frame.core.util import ufunc_nanall
 from static_frame.core.util import ufunc_nanany
+from static_frame.core.util import OPERATORS
+from static_frame.core.style_config import StyleConfig
 
 if tp.TYPE_CHECKING:
     from static_frame.core.frame import Frame #pylint: disable=W0611 #pragma: no cover
@@ -28,7 +29,7 @@ T = tp.TypeVar('T')
 #-------------------------------------------------------------------------------
 class ContainerBase(metaclass=InterfaceMeta):
     '''
-    Root of all containers. Most containers, like Series, Frame, and Index, inherit from ContainerOperand; only Bus inherits from ContainerBase.
+    Root of all containers. The core containers, like Series, Frame, and Index, inherit from ContainerOperand. The higher-order containers, like Bus, Quilt, Batch, and Yarn, inherit from ContainerBase.
     '''
     __slots__ = EMPTY_TUPLE
 
@@ -48,7 +49,9 @@ class ContainerBase(metaclass=InterfaceMeta):
         return InterfaceSummary.to_frame(self.__class__)
 
     def display(self,
-            config: tp.Optional[DisplayConfig] = None
+            config: tp.Optional[DisplayConfig] = None,
+            *,
+            style_config: tp.Optional[StyleConfig] = None,
             ) -> Display:
         raise NotImplementedError()
 
@@ -110,6 +113,12 @@ class ContainerBase(metaclass=InterfaceMeta):
         '''
         raise ValueError('The truth value of a container is ambiguous. For a truthy indicator of non-empty status, use the `size` attribute.')
 
+    #---------------------------------------------------------------------------
+    def to_visidata(self) -> None:
+        '''Open an interactive VisiData session.
+        '''
+        from static_frame.core.display_visidata import view_sf #pragma: no cover
+        view_sf(self) #type: ignore [no-untyped-call] #pragma: no cover
 
 
 class ContainerOperand(ContainerBase):
@@ -126,112 +135,101 @@ class ContainerOperand(ContainerBase):
     def _ufunc_binary_operator(self: T, *,
             operator: UFunc,
             other: tp.Any,
+            fill_value: object = np.nan,
             ) -> T:
         raise NotImplementedError() #pragma: no cover
 
     #---------------------------------------------------------------------------
     def __pos__(self) -> 'ContainerOperand':
-        return self._ufunc_unary_operator(operator_mod.__pos__)
+        return self._ufunc_unary_operator(OPERATORS['__pos__'])
 
     def __neg__(self) -> 'ContainerOperand':
-        return self._ufunc_unary_operator(operator_mod.__neg__)
+        return self._ufunc_unary_operator(OPERATORS['__neg__'])
 
     def __abs__(self) -> 'ContainerOperand':
-        return self._ufunc_unary_operator(operator_mod.__abs__)
+        return self._ufunc_unary_operator(OPERATORS['__abs__'])
 
     def __invert__(self) -> 'ContainerOperand':
-        return self._ufunc_unary_operator(operator_mod.__invert__)
+        return self._ufunc_unary_operator(OPERATORS['__invert__'])
 
     #---------------------------------------------------------------------------
     def __add__(self, other: tp.Any) -> tp.Any:
-        return self._ufunc_binary_operator(operator=operator_mod.__add__, other=other)
+        return self._ufunc_binary_operator(operator=OPERATORS['__add__'], other=other)
 
     def __sub__(self, other: tp.Any) -> tp.Any:
-        return self._ufunc_binary_operator(operator=operator_mod.__sub__, other=other)
+        return self._ufunc_binary_operator(operator=OPERATORS['__sub__'], other=other)
 
     def __mul__(self, other: tp.Any) -> tp.Any:
-        return self._ufunc_binary_operator(operator=operator_mod.__mul__, other=other)
+        return self._ufunc_binary_operator(operator=OPERATORS['__mul__'], other=other)
 
     def __matmul__(self, other: tp.Any) -> tp.Any:
-        return self._ufunc_binary_operator(operator=operator_mod.__matmul__, other=other)
+        return self._ufunc_binary_operator(operator=OPERATORS['__matmul__'], other=other)
 
     def __truediv__(self, other: tp.Any) -> tp.Any:
-        return self._ufunc_binary_operator(operator=operator_mod.__truediv__, other=other)
+        return self._ufunc_binary_operator(operator=OPERATORS['__truediv__'], other=other)
 
     def __floordiv__(self, other: tp.Any) -> tp.Any:
-        return self._ufunc_binary_operator(operator=operator_mod.__floordiv__, other=other)
+        return self._ufunc_binary_operator(operator=OPERATORS['__floordiv__'], other=other)
 
     def __mod__(self, other: tp.Any) -> tp.Any:
-        return self._ufunc_binary_operator(operator=operator_mod.__mod__, other=other)
+        return self._ufunc_binary_operator(operator=OPERATORS['__mod__'], other=other)
 
     # def __divmod__:
 
     def __pow__(self, other: tp.Any) -> tp.Any:
-        return self._ufunc_binary_operator(operator=operator_mod.__pow__, other=other)
+        return self._ufunc_binary_operator(operator=OPERATORS['__pow__'], other=other)
 
     def __lshift__(self, other: tp.Any) -> tp.Any:
-        return self._ufunc_binary_operator(operator=operator_mod.__lshift__, other=other)
+        return self._ufunc_binary_operator(operator=OPERATORS['__lshift__'], other=other)
 
     def __rshift__(self, other: tp.Any) -> tp.Any:
-        return self._ufunc_binary_operator(operator=operator_mod.__rshift__, other=other)
+        return self._ufunc_binary_operator(operator=OPERATORS['__rshift__'], other=other)
 
     def __and__(self, other: tp.Any) -> tp.Any:
-        return self._ufunc_binary_operator(operator=operator_mod.__and__, other=other)
+        return self._ufunc_binary_operator(operator=OPERATORS['__and__'], other=other)
 
     def __xor__(self, other: tp.Any) -> tp.Any:
-        return self._ufunc_binary_operator(operator=operator_mod.__xor__, other=other)
+        return self._ufunc_binary_operator(operator=OPERATORS['__xor__'], other=other)
 
     def __or__(self, other: tp.Any) -> tp.Any:
-        return self._ufunc_binary_operator(operator=operator_mod.__or__, other=other)
+        return self._ufunc_binary_operator(operator=OPERATORS['__or__'], other=other)
 
     def __lt__(self, other: tp.Any) -> tp.Any:
-        return self._ufunc_binary_operator(operator=operator_mod.__lt__, other=other)
+        return self._ufunc_binary_operator(operator=OPERATORS['__lt__'], other=other)
 
     def __le__(self, other: tp.Any) -> tp.Any:
-        return self._ufunc_binary_operator(operator=operator_mod.__le__, other=other)
+        return self._ufunc_binary_operator(operator=OPERATORS['__le__'], other=other)
 
     def __eq__(self, other: tp.Any) -> tp.Any:
-        return self._ufunc_binary_operator(operator=operator_mod.__eq__, other=other)
+        return self._ufunc_binary_operator(operator=OPERATORS['__eq__'], other=other)
 
     def __ne__(self, other: tp.Any) -> tp.Any:
-        return self._ufunc_binary_operator(operator=operator_mod.__ne__, other=other)
+        return self._ufunc_binary_operator(operator=OPERATORS['__ne__'], other=other)
 
     def __gt__(self, other: tp.Any) -> tp.Any:
-        return self._ufunc_binary_operator(operator=operator_mod.__gt__, other=other)
+        return self._ufunc_binary_operator(operator=OPERATORS['__gt__'], other=other)
 
     def __ge__(self, other: tp.Any) -> tp.Any:
-        return self._ufunc_binary_operator(operator=operator_mod.__ge__, other=other)
+        return self._ufunc_binary_operator(operator=OPERATORS['__ge__'], other=other)
 
     #---------------------------------------------------------------------------
     def __radd__(self, other: tp.Any) -> tp.Any:
-        operator = lambda rhs, lhs: operator_mod.__add__(lhs, rhs)
-        operator.__name__ = 'r' + operator_mod.__add__.__name__
-        return self._ufunc_binary_operator(operator=operator, other=other)
+        return self._ufunc_binary_operator(operator=OPERATORS['__radd__'], other=other)
 
     def __rsub__(self, other: tp.Any) -> tp.Any:
-        operator = lambda rhs, lhs: operator_mod.__sub__(lhs, rhs)
-        operator.__name__ = 'r' + operator_mod.__sub__.__name__
-        return self._ufunc_binary_operator(operator=operator, other=other)
+        return self._ufunc_binary_operator(operator=OPERATORS['__rsub__'], other=other)
 
     def __rmul__(self, other: tp.Any) -> tp.Any:
-        operator = lambda rhs, lhs: operator_mod.__mul__(lhs, rhs)
-        operator.__name__ = 'r' + operator_mod.__mul__.__name__
-        return self._ufunc_binary_operator(operator=operator, other=other)
+        return self._ufunc_binary_operator(operator=OPERATORS['__rmul__'], other=other)
 
     def __rmatmul__(self, other: tp.Any) -> tp.Any:
-        operator = lambda rhs, lhs: operator_mod.__matmul__(lhs, rhs)
-        operator.__name__ = 'r' + operator_mod.__matmul__.__name__
-        return self._ufunc_binary_operator(operator=operator, other=other)
+        return self._ufunc_binary_operator(operator=OPERATORS['__rmatmul__'], other=other)
 
     def __rtruediv__(self, other: tp.Any) -> tp.Any:
-        operator = lambda rhs, lhs: operator_mod.__truediv__(lhs, rhs)
-        operator.__name__ = 'r' + operator_mod.__truediv__.__name__
-        return self._ufunc_binary_operator(operator=operator, other=other)
+        return self._ufunc_binary_operator(operator=OPERATORS['__rtruediv__'], other=other)
 
     def __rfloordiv__(self, other: tp.Any) -> tp.Any:
-        operator = lambda rhs, lhs: operator_mod.__floordiv__(lhs, rhs)
-        operator.__name__ = 'r' + operator_mod.__floordiv__.__name__
-        return self._ufunc_binary_operator(operator=operator, other=other)
+        return self._ufunc_binary_operator(operator=OPERATORS['__rfloordiv__'], other=other)
 
     # --------------------------------------------------------------------------
     # ufunc axis skipna methods: applied along an axis, reducing dimensionality.
@@ -312,7 +310,7 @@ class ContainerOperand(ContainerBase):
                 ufunc=np.sum,
                 ufunc_skipna=np.nansum,
                 composable=False,
-                dtypes=EMPTY_TUPLE, # float or int, row type will match
+                dtypes=EMPTY_TUPLE, # float or int, row type will match except Boolean
                 size_one_unity=True
                 )
 
@@ -525,3 +523,19 @@ class ContainerOperand(ContainerBase):
         # modify the active display to be for HTML
         return repr(self.display(config))
 
+
+#-------------------------------------------------------------------------------
+# TODO: replace usage with ContainerMap; use this in a test to validate
+
+def container_opperand_map() -> tp.Dict[str, tp.Type[ContainerOperand]]:
+    '''Return a mapping of ContainerOperand types, from name to cls. Note that other modules must be loaded before this returns usable results.
+    '''
+    def collector() -> tp.Iterator[tp.Type[ContainerOperand]]:
+        targets = deque((ContainerOperand,))
+        while targets:
+            target = targets.popleft()
+            for part in target.__subclasses__():
+                targets.append(part)
+                yield part
+
+    return {cls.__name__: cls for cls in collector()}
