@@ -8,14 +8,15 @@ from functools import partial
 import datetime
 from fractions import Fraction
 import typing as tp
+from enum import Enum
 
 import numpy as np
 from numpy import char as npc
+from arraykit import column_2d_filter
 
 from static_frame.core.index_base import IndexBase
 from static_frame.core.util import AnyCallable
 from static_frame.core.util import Bloc2DKeyType
-from static_frame.core.util import column_2d_filter
 from static_frame.core.util import concat_resolved
 from static_frame.core.util import DEFAULT_SORT_KIND
 from static_frame.core.util import DepthLevelSpecifier
@@ -39,26 +40,81 @@ from static_frame.core.util import INT_TYPES
 from static_frame.core.util import NameType
 from static_frame.core.util import is_dtype_specifier
 from static_frame.core.util import is_mapping
+from static_frame.core.util import BoolOrBools
+from static_frame.core.util import BOOL_TYPES
+from static_frame.core.util import EMPTY_TUPLE
+
+from static_frame.core.rank import rank_1d
+from static_frame.core.rank import RankMethod
 
 from static_frame.core.exception import AxisInvalid
 
+
 if tp.TYPE_CHECKING:
     import pandas as pd #pylint: disable=W0611 #pragma: no cover
     from static_frame.core.type_blocks import TypeBlocks #pylint: disable=W0611,C0412 #pragma: no cover
     from static_frame.core.series import Series #pylint: disable=W0611,C0412 #pragma: no cover
     from static_frame.core.frame import Frame #pylint: disable=W0611,C0412 #pragma: no cover
     from static_frame.core.index_hierarchy import IndexHierarchy #pylint: disable=W0611,C0412 #pragma: no cover
+    from static_frame.core.index_auto import IndexAutoFactory #pylint: disable=W0611,C0412 #pragma: no cover
+    from static_frame.core.index_auto import IndexDefaultFactory #pylint: disable=W0611,C0412 #pragma: no cover
     from static_frame.core.index_auto import IndexAutoFactoryType #pylint: disable=W0611,C0412 #pragma: no cover
     from static_frame.core.quilt import Quilt #pylint: disable=W0611,C0412 #pragma: no cover
+    from static_frame.core.container import ContainerOperand #pylint: disable=W0611,C0412 #pragma: no cover
 
 
 
+class ContainerMap:
+
+    _map: tp.Optional[tp.Dict[str, tp.Type['ContainerOperand']]] = None
+
+    @classmethod
+    def _update_map(cls) -> None:
+        from static_frame.core.frame import Frame
+        from static_frame.core.frame import FrameGO
+        from static_frame.core.frame import FrameHE
+        from static_frame.core.series import Series
+        from static_frame.core.series import SeriesHE
+        from static_frame.core.index import Index
+        from static_frame.core.index import IndexGO
+        from static_frame.core.index_hierarchy import IndexHierarchy
+        from static_frame.core.index_hierarchy import IndexHierarchyGO
+        from static_frame.core.index_datetime import IndexDate
+        from static_frame.core.index_datetime import IndexDateGO
+        from static_frame.core.index_datetime import IndexYear
+        from static_frame.core.index_datetime import IndexYearGO
+        from static_frame.core.index_datetime import IndexYearMonth
+        from static_frame.core.index_datetime import IndexYearMonthGO
+        from static_frame.core.index_datetime import IndexMinute
+        from static_frame.core.index_datetime import IndexMinuteGO
+        from static_frame.core.index_datetime import IndexSecond
+        from static_frame.core.index_datetime import IndexSecondGO
+        from static_frame.core.index_datetime import IndexMillisecond
+        from static_frame.core.index_datetime import IndexMillisecondGO
+        from static_frame.core.index_datetime import IndexMicrosecond
+        from static_frame.core.index_datetime import IndexMicrosecondGO
+        from static_frame.core.index_datetime import IndexNanosecond
+        from static_frame.core.index_datetime import IndexNanosecondGO
+
+        cls._map = {k: v for k, v in locals().items() if v is not cls}
+
+    # @staticmethod
+    # def cls_to_str(cls: tp.Type['ContainerOperand']) -> str:
+    #     return
+
+    @classmethod
+    def str_to_cls(cls, name: str) -> tp.Type['ContainerOperand']:
+        if cls._map is None:
+            cls._update_map()
+        return cls._map[name] #type: ignore #pylint: disable=unsubscriptable-object
+
+
 def get_col_dtype_factory(
         dtypes: DtypesSpecifier,
         columns: tp.Optional[tp.Sequence[tp.Hashable]],
         ) -> tp.Callable[[int], np.dtype]:
     '''
-    Return a function, or None, to get values from a DtypeSpecifier.
+    Return a function, or None, to get values from a DtypeSpecifier by integer column positions.
 
     Args:
         columns: In common usage in Frame constructors, ``columns`` is a reference to a mutable list that is assigned column labels when processing data (and before this function is called). Columns can also be an ``Index``.
@@ -86,6 +142,7 @@ def get_col_dtype_factory(
         if is_element:
             return dtypes
         if is_map:
+            # mappings can be incomplete
             return dtypes.get(columns[col_idx], None) #type: ignore
         # NOTE: dtypes might be a generator deferred until this function is called; if so, realize here; INVALID_ITERABLE_FOR_ARRAY (dict_values, etc) do not have __getitem__,
         if not hasattr(dtypes, '__len__') or not hasattr(dtypes, '__getitem__'):
@@ -189,24 +246,67 @@ def pandas_to_numpy(
     array.flags.writeable = False
     return array
 
+def df_slice_to_arrays(*,
+        part: 'pd.DataFrame',
+        column_ilocs: range,
+        get_col_dtype: tp.Optional[tp.Callable[[int], np.dtype]],
+        pdvu1: bool,
+        own_data: bool,
+        ) -> tp.Iterator[np.ndarray]:
+    '''
+    Given a slice of a DataFrame, extract an array and optionally convert dtypes. If dtypes are provided, they are read with iloc positions given by `columns_ilocs`.
+    '''
+    if pdvu1:
+        array = part.values
+        if own_data:
+            array.flags.writeable = False
+    else:
+        array = pandas_to_numpy(part, own_data=own_data)
 
+    if get_col_dtype:
+        assert len(column_ilocs) == array.shape[1]
+        for col, iloc in enumerate(column_ilocs):
+            # use iloc to get dtype
+            dtype = get_col_dtype(iloc)
+            if dtype is None or dtype == array.dtype:
+                yield array[NULL_SLICE, col]
+            else:
+                yield array[NULL_SLICE, col].astype(dtype)
+    else:
+        yield array
+
+
+
+#---------------------------------------------------------------------------
 def index_from_optional_constructor(
-        value: IndexInitializer,
+        value: tp.Union[IndexInitializer, 'IndexAutoFactory'],
         *,
         default_constructor: IndexConstructor,
-        explicit_constructor: tp.Optional[IndexConstructor] = None,
+        explicit_constructor: tp.Union[IndexConstructor, 'IndexDefaultFactory', None] = None,
         ) -> IndexBase:
     '''
     Given a value that is an IndexInitializer (which means it might be an Index), determine if that value is really an Index, and if so, determine if a copy has to be made; otherwise, use the default_constructor. If an explicit_constructor is given, that is always used.
     '''
     # NOTE: this might return an own_index flag to show callers when a new index has been created
+    # NOTE: do not pass `name` here; instead, partial contstuctors if necessary
+    from static_frame.core.index_auto import IndexAutoFactory
+    from static_frame.core.index_auto import IndexDefaultFactory
+
+    if isinstance(value, IndexAutoFactory):
+        return value.to_index(
+                default_constructor=default_constructor, #type: ignore
+                explicit_constructor=explicit_constructor,
+                )
 
     if explicit_constructor:
+        if isinstance(explicit_constructor, IndexDefaultFactory):
+            # partial the default constructor with a name argument
+            return explicit_constructor(default_constructor)(value)
         return explicit_constructor(value)
 
     # default constructor could be a function with a STATIC attribute
     if isinstance(value, IndexBase):
-        # if default is STATIC, and value is not STATIC, get an immutabel
+        # if default is STATIC, and value is not STATIC, get an immutable
         if is_static(default_constructor):
             if not value.STATIC:
                 # v: ~S, dc: S, use immutable alternative
@@ -223,6 +323,71 @@ def index_from_optional_constructor(
     # cannot always determine static status from constructors; fallback on using default constructor
     return default_constructor(value)
 
+def index_from_optional_constructors(
+        value: tp.Union[np.ndarray, tp.Iterable[tp.Hashable]],
+        *,
+        depth: int,
+        default_constructor: IndexConstructor,
+        explicit_constructors: IndexConstructors = None,
+        ) -> tp.Tuple[tp.Optional[IndexBase], bool]:
+    '''For scenarios here `index_depth` is the primary way of specifying index creation from a data source and the returned index might be an `IndexHierarchy`. Note that we do not take `name` or `continuation_token` here, but expect constructors to be appropriately partialed.
+    '''
+    if depth == 0:
+        index = None
+        own_index = False
+    elif depth == 1:
+        if not explicit_constructors:
+            explicit_constructor = None
+        elif callable(explicit_constructors):
+            explicit_constructor = explicit_constructors
+        else:
+            if len(explicit_constructors) != 1:
+                raise RuntimeError('Cannot specify multiple index constructors for depth 1 indicies.')
+            explicit_constructor = explicit_constructors[0]
+
+        index = index_from_optional_constructor(
+                value,
+                default_constructor=default_constructor,
+                explicit_constructor=explicit_constructor,
+                )
+        own_index = True
+    else:
+        # if depth is > 1, the default constructor is expected to be an IndexHierarchy, and explicit constructors are optionally provided `index_constructors`
+        if callable(explicit_constructors):
+            explicit_constructors = [explicit_constructors] * depth
+        # default_constructor is an IH type
+        index = default_constructor(
+                value,
+                index_constructors=explicit_constructors
+                )
+        own_index = True
+    return index, own_index
+
+
+def index_from_optional_constructors_deferred(
+        *,
+        depth: int,
+        default_constructor: IndexConstructor,
+        explicit_constructors: IndexConstructors = None,
+        ) -> tp.Callable[
+                [tp.Union[np.ndarray, tp.Iterable[tp.Hashable]]],
+                tp.Optional[IndexBase]]:
+    '''
+    Partial `index_from_optional_constructors` for all args except `value`; only return the Index, ignoring the own_index Boolean.
+    '''
+    def func(
+            value: tp.Union[np.ndarray, tp.Iterable[tp.Hashable]],
+            ) -> tp.Optional[IndexBase]:
+        # drop the own_index Boolean
+        index, _ = index_from_optional_constructors(value,
+                depth=depth,
+                default_constructor=default_constructor,
+                explicit_constructors=explicit_constructors,
+                )
+        return index
+    return func
+
+
 def index_constructor_empty(
         index: tp.Union[IndexInitializer, 'IndexAutoFactoryType']
         ) -> bool:
@@ -563,7 +728,7 @@ def bloc_key_normalize(
                 fill_value=False
                 )
         bloc_key = bloc_frame.values # shape must match post reindex
-    elif isinstance(key, np.ndarray):
+    elif key.__class__ is np.ndarray:
         bloc_key = key
         if bloc_key.shape != container.shape:
             raise RuntimeError(f'bloc {bloc_key.shape} must match shape {container.shape}')
@@ -586,14 +751,16 @@ def key_to_ascending_key(key: GetItemKeyType, size: int) -> GetItemKeyType:
     from static_frame.core.frame import Frame
     from static_frame.core.series import Series
 
-    if isinstance(key, slice):
-        return slice_to_ascending_slice(key, size=size)
+    if key.__class__ is slice:
+        return slice_to_ascending_slice(key, size=size) #type: ignore
 
     if isinstance(key, str) or not hasattr(key, '__len__'):
         return key
 
-    if isinstance(key, np.ndarray):
+    if key.__class__ is np.ndarray:
         # array first as not truthy
+        if key.dtype == bool: #type: ignore
+            return key
         return np.sort(key, kind=DEFAULT_SORT_KIND)
 
     if not len(key): #type: ignore
@@ -744,7 +911,7 @@ def apply_binary_operator(*,
             result = npc.add(other, values)
         elif operator_name == 'mul' or operator_name == 'rmul':
             result = npc.multiply(values, other)
-        else:
+        else: # WARNING_RAISED
             result = operator(values, other)
     else:
         result = operator(values, other)
@@ -842,6 +1009,9 @@ def key_from_container_key(
     '''
     Unpack selection values from another Index, Series, or ILoc selection.
     '''
+    # PERF: do not do comparisons if key is not a Container or SF object
+    if not hasattr(key, 'STATIC'):
+        return key
 
     from static_frame.core.index import Index
     from static_frame.core.index import ILoc
@@ -865,10 +1035,10 @@ def key_from_container_key(
         else:
             # For all other Series types, we simply assume that the values are to be used as keys in the IH. This ignores the index, but it does not seem useful to require the Series, used like this, to have a matching index value, as the index and values would need to be identical to have the desired selection.
             key = key.values
-    elif expand_iloc and isinstance(key, ILoc):
+    elif expand_iloc and key.__class__ is ILoc:
         # realize as Boolean array
         array = np.full(len(index), False)
-        array[key.key] = True
+        array[key.key] = True #type: ignore
         key = array
 
     # detect and fail on Frame?
@@ -876,11 +1046,17 @@ def key_from_container_key(
 
 
 #---------------------------------------------------------------------------
+class ManyToOneType(Enum):
+    CONCAT = 0
+    UNION = 1
+    INTERSECT = 2
+
 
 def _index_many_to_one(
         indices: tp.Iterable[IndexBase],
         cls_default: tp.Type[IndexBase],
-        array_processor: tp.Callable[[tp.Iterable[np.ndarray]], np.ndarray]
+        many_to_one_type: ManyToOneType,
+        explicit_constructor: tp.Optional[IndexInitializer] = None,
         ) -> IndexBase:
     '''
     Given multiple Index objects, combine them. Preserve name and index type if aligned, and handle going to GO if the default class is GO.
@@ -888,12 +1064,30 @@ def _index_many_to_one(
     Args:
         indices: can be a generator
         cls_default: Default Index class to be used if no alignment of classes; also used to determine if result Index should be static or mutable.
+        explicit_constructor: Alternative constructor that will override normal evaluation.
     '''
+    from static_frame.core.index_auto import IndexAutoFactory
+
+    array_processor: tp.Callable[[tp.Iterable[np.ndarray]], np.ndarray]
+
+    if many_to_one_type is ManyToOneType.UNION:
+        array_processor = partial(ufunc_set_iter,
+                union=True,
+                assume_unique=True)
+    elif many_to_one_type is ManyToOneType.INTERSECT:
+        array_processor = partial(ufunc_set_iter,
+                union=False,
+                assume_unique=True)
+    elif many_to_one_type is ManyToOneType.CONCAT:
+        array_processor = concat_resolved
+
     indices_iter = iter(indices)
     try:
         index = next(indices_iter)
     except StopIteration:
-        return cls_default.from_labels(())
+        if explicit_constructor is not None:
+            return explicit_constructor(EMPTY_TUPLE) #type: ignore
+        return cls_default.from_labels(EMPTY_TUPLE)
 
     arrays = [index.values]
 
@@ -903,12 +1097,18 @@ def _index_many_to_one(
     cls_first = index.__class__
     cls_aligned = True
 
+    # if we are unioning we can give back an index_auto_aligned
+    index_auto_aligned = (many_to_one_type is not ManyToOneType.CONCAT
+            and index.ndim == 1
+            and index._map is None #type: ignore
+            )
+
     # if IndexHierarchy, collect index_types generators
     if index.ndim == 2:
         depth_first = index.depth
         index_types_gen = [index._levels.index_types()] #type: ignore
         index_types_aligned = True
-    else:
+    else: # for 1D we ignore this
         index_types_aligned = False
 
     for index in indices_iter:
@@ -917,13 +1117,26 @@ def _index_many_to_one(
             name_aligned = False
         if cls_aligned and index.__class__ != cls_first:
             cls_aligned = False
+        if index_auto_aligned and (index.ndim != 1 or index._map is not None): #type: ignore
+            index_auto_aligned = False
 
         if index_types_aligned and index.ndim == 2 and index.depth == depth_first:
             index_types_gen.append(index._levels.index_types()) #type: ignore
         else:
             index_types_aligned = False
 
-    if index_types_aligned:
+    name = name_first if name_aligned else None
+    if index_auto_aligned:
+        if many_to_one_type is ManyToOneType.UNION:
+            size = max(a.size for a in arrays)
+        elif many_to_one_type is ManyToOneType.INTERSECT:
+            size = min(a.size for a in arrays)
+        return IndexAutoFactory(size, name=name).to_index(
+                default_constructor=cls_default,
+                explicit_constructor=explicit_constructor,
+                )
+
+    if index_types_aligned: # for IndexHierarchy
         # all depths are already aligned
         index_constructors = []
         for types in zip(*index_types_gen):
@@ -932,7 +1145,7 @@ def _index_many_to_one(
             else: # assume this is always a 1D index
                 index_constructors.append(cls_default)
 
-    if cls_aligned:
+    if cls_aligned and explicit_constructor is None:
         if cls_default.STATIC and not cls_first.STATIC:
             # default is static but aligned is mutable
             constructor = cls_first._IMMUTABLE_CONSTRUCTOR.from_labels #type: ignore
@@ -941,36 +1154,46 @@ def _index_many_to_one(
             constructor = cls_first._MUTABLE_CONSTRUCTOR.from_labels #type: ignore
         else:
             constructor = cls_first.from_labels
+    elif explicit_constructor is not None:
+        constructor = explicit_constructor
     else:
         constructor = cls_default.from_labels
 
-    name = name_first if name_aligned else None
-
     # returns an immutable array
     array = array_processor(arrays)
 
-    if index_types_aligned:
-        return constructor(array, name=name, index_constructors=index_constructors) #type: ignore
+    if index_types_aligned: # IndexHierarchy
+        return constructor(array, #type: ignore
+                name=name,
+                index_constructors=index_constructors,
+                )
     return constructor(array, name=name) #type: ignore
 
 def index_many_concat(
         indices: tp.Iterable[IndexBase],
         cls_default: tp.Type[IndexBase],
+        explicit_constructor: tp.Optional[IndexInitializer] = None,
         ) -> tp.Optional[IndexBase]:
-    return _index_many_to_one(indices, cls_default, concat_resolved)
+    return _index_many_to_one(indices,
+            cls_default,
+            ManyToOneType.CONCAT,
+            explicit_constructor,
+            )
 
 def index_many_set(
         indices: tp.Iterable[IndexBase],
         cls_default: tp.Type[IndexBase],
         union: bool,
+        explicit_constructor: tp.Optional[IndexInitializer] = None,
         ) -> tp.Optional[IndexBase]:
     '''
-    Given multiple Index objects, concatenate them in order. Preserve name and index type if aligned.
+    Given multiple Index objects, union them. Preserve name and index type if aligned.
     '''
-    array_processor = partial(ufunc_set_iter,
-            union=union,
-            assume_unique=True)
-    return _index_many_to_one(indices, cls_default, array_processor)
+    return _index_many_to_one(indices,
+            cls_default,
+            ManyToOneType.UNION if union else ManyToOneType.INTERSECT,
+            explicit_constructor,
+            )
 
 
 #-------------------------------------------------------------------------------
@@ -1031,10 +1254,43 @@ def container_to_exporter_attr(container_type: tp.Type['Frame']) -> str:
         return 'to_frame_he'
     raise NotImplementedError(f'no handling for {container_type}')
 
+def frame_to_frame(
+        frame: 'Frame',
+        container_type: tp.Type['Frame'],
+        ) -> 'Frame':
+    if frame.__class__ is container_type:
+        return frame
+    func = getattr(frame, container_to_exporter_attr(container_type))
+    return func() # type: ignore
+
+def prepare_values_for_lex(
+        *,
+        ascending: BoolOrBools = True,
+        values_for_lex: tp.Optional[tp.Iterable[np.ndarray]],
+        ) -> tp.Tuple[bool, tp.Optional[tp.Iterable[np.ndarray]]]:
+    '''Prepare values for lexical sorting; assumes values have already been collected in reverse order. If ascending is an element and values_for_lex is None, this function is pass through.
+    '''
+    asc_is_element = isinstance(ascending, BOOL_TYPES)
+    if not asc_is_element:
+        ascending = tuple(ascending) #type: ignore
+        if values_for_lex is None or len(ascending) != len(values_for_lex): #type: ignore
+            raise RuntimeError(f'Multiple ascending values must match number of arrays selected.')
+        # values for lex are in reversed order; thus take ascending reversed
+        values_for_lex_post = []
+        for asc, a in zip(reversed(ascending), values_for_lex):
+            # if not ascending, replace with an inverted dense rank
+            if not asc:
+                values_for_lex_post.append(
+                        rank_1d(a, method=RankMethod.DENSE, ascending=False))
+            else:
+                values_for_lex_post.append(a)
+        values_for_lex = values_for_lex_post
+
+    return asc_is_element, values_for_lex
 
 def sort_index_for_order(
         index: IndexBase,
-        ascending: bool,
+        ascending: BoolOrBools,
         kind: str,
         key: tp.Optional[tp.Callable[[IndexBase], tp.Union[np.ndarray, IndexBase]]],
         ) -> np.ndarray:
@@ -1043,7 +1299,7 @@ def sort_index_for_order(
     # cfs is container_for_sort
     if key:
         cfs = key(index)
-        cfs_is_array = isinstance(cfs, np.ndarray)
+        cfs_is_array = cfs.__class__ is np.ndarray
         if cfs_is_array:
             cfs_depth = 1 if cfs.ndim == 1 else cfs.shape[1]
         else:
@@ -1055,6 +1311,7 @@ def sort_index_for_order(
         cfs_is_array = False
         cfs_depth = cfs.depth
 
+    asc_is_element: bool
     # argsort lets us do the sort once and reuse the results
     if cfs_depth > 1:
         if cfs_is_array:
@@ -1062,13 +1319,23 @@ def sort_index_for_order(
         else: # cfs is an IndexHierarchy
             values_for_lex = [cfs.values_at_depth(i)
                     for i in range(cfs.depth-1, -1, -1)]
+
+        asc_is_element, values_for_lex = prepare_values_for_lex( #type: ignore
+                ascending=ascending,
+                values_for_lex=values_for_lex,
+                )
         order = np.lexsort(values_for_lex)
     else:
         # depth is 1
+        asc_is_element = isinstance(ascending, BOOL_TYPES)
+        if not asc_is_element:
+            raise RuntimeError(f'Multiple ascending values not permitted.')
+
         v = cfs if cfs_is_array else cfs.values
         order = np.argsort(v, kind=kind)
 
-    if not ascending:
+    if asc_is_element and not ascending:
+        # NOTE: if asc is not an element, then ascending Booleans have already been applied to values_for_lex
         order = order[::-1]
     return order
 
@@ -1131,19 +1398,4 @@ class MessagePackElement:
 
 
 
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
 
