@@ -4,6 +4,7 @@ import os
 import html
 import inspect
 import platform
+import re
 from functools import partial
 from collections import namedtuple
 
@@ -11,6 +12,7 @@ import numpy as np
 
 from static_frame.core.display_color import HexColor
 from static_frame.core.util import _gen_skip_middle
+from static_frame.core.util import CallableToIterType
 from static_frame.core.util import COMPLEX_TYPES
 from static_frame.core.util import DTYPE_INT_KINDS
 from static_frame.core.util import DTYPE_STR_KINDS
@@ -19,6 +21,11 @@ from static_frame.core.display_config import DisplayConfig
 from static_frame.core.display_config import _DISPLAY_FORMAT_HTML
 from static_frame.core.display_config import _DISPLAY_FORMAT_MAP
 from static_frame.core.display_config import _DISPLAY_FORMAT_TERMINAL
+from static_frame.core.style_config import StyleConfig
+
+if tp.TYPE_CHECKING:
+    from static_frame.core.index_base import IndexBase # pylint: disable=unused-import #pragma: no cover
+
 
 _module = sys.modules[__name__]
 
@@ -125,6 +132,26 @@ class DisplayTypeFrame(DisplayTypeCategory):
             return False
         return issubclass(t, Frame)
 
+class DisplayTypeBus(DisplayTypeCategory):
+    CONFIG_ATTR = 'type_color_bus'
+
+    @staticmethod
+    def in_category(t: tp.Union[type, np.dtype]) -> bool:
+        from static_frame import Bus
+        if not inspect.isclass(t):
+            return False
+        return issubclass(t, Bus)
+
+class DisplayTypeQuilt(DisplayTypeCategory):
+    CONFIG_ATTR = 'type_color_quilt'
+
+    @staticmethod
+    def in_category(t: tp.Union[type, np.dtype]) -> bool:
+        from static_frame import Quilt
+        if not inspect.isclass(t):
+            return False
+        return issubclass(t, Quilt)
+
 
 class DisplayTypeCategoryFactory:
 
@@ -139,7 +166,9 @@ class DisplayTypeCategoryFactory:
             DisplayTypeTimeDelta,
             DisplayTypeIndex,
             DisplayTypeSeries,
-            DisplayTypeFrame
+            DisplayTypeFrame,
+            DisplayTypeBus,
+            DisplayTypeQuilt,
             )
 
     _TYPE_TO_CATEGORY_CACHE: tp.Dict[tp.Union[type, np.dtype], tp.Type[DisplayTypeCategory]] = {}
@@ -231,7 +260,7 @@ class DisplayActive:
 #-------------------------------------------------------------------------------
 class DisplayHeader:
     '''
-    Wraper for passing in display header that have a name attribute.
+    Wrapper for passing in display header that has a name attribute, such as a :obj:`Series` or :obj:`Frame` that displays the ``name`` attribute.
     '''
     __slots__ = ('cls', 'name')
 
@@ -257,8 +286,9 @@ class DisplayHeader:
 
 HeaderInitializer = tp.Optional[tp.Union[str, DisplayHeader]]
 
-# store formating string, raw string
+# An NT that stores a formatted strings ``format_str`` as well as a ``raw`` string
 DisplayCell = namedtuple('DisplayCell', ('format_str', 'raw'))
+
 FORMAT_EMPTY = '{}'
 
 class Display:
@@ -271,6 +301,7 @@ class Display:
         '_outermost',
         '_index_depth',
         '_header_depth',
+        '_style_config',
         )
 
     CHAR_MARGIN = 1
@@ -278,6 +309,7 @@ class Display:
     ELLIPSIS = '...' # this string is appended to truncated entries
     CELL_ELLIPSIS = DisplayCell(FORMAT_EMPTY, ELLIPSIS)
     ELLIPSIS_CENTER_SENTINEL = object()
+    ANSI_ESCAPE = re.compile(r'\x1B(?:[@-Z\\-_]|\[[0-?]*[ -/]*[@-~])')
 
     #---------------------------------------------------------------------------
     # utility methods
@@ -288,18 +320,18 @@ class Display:
             config: DisplayConfig
             ) -> tp.Tuple[str, tp.Type[DisplayTypeCategory]]:
         '''
-        Apply delimters to type, for either numpy types or Python classes.
+        Return the `type_input` as a string, applying delimters to either numpy dtypes or Python classes.
         '''
         if isinstance(type_input, np.dtype):
             type_str = str(type_input)
             type_ref = type_input
-        elif inspect.isclass(type_input):
-            assert isinstance(type_input, type)
-            type_str = type_input.__name__
-            type_ref = type_input
         elif isinstance(type_input, DisplayHeader):
             type_str = repr(type_input)
             type_ref = type_input.cls
+        elif inspect.isclass(type_input):
+            # assert isinstance(type_input, type)
+            type_str = type_input.__name__
+            type_ref = type_input
         else:
             raise NotImplementedError('no handling for this input', type_input)
 
@@ -327,13 +359,10 @@ class Display:
         if config.display_format in _DISPLAY_FORMAT_HTML:
             return HexColor.format_html(color, FORMAT_EMPTY)
 
-        if config.display_format in _DISPLAY_FORMAT_TERMINAL:
-            if terminal_ansi():
-                return HexColor.format_terminal(color, FORMAT_EMPTY)
+        if config.display_format in _DISPLAY_FORMAT_TERMINAL and terminal_ansi():
+            return HexColor.format_terminal(color, FORMAT_EMPTY) #pragma: no cover
             # if not a compatible terminal, return label unaltered
-            return FORMAT_EMPTY
 
-        # RST and other text displays
         return FORMAT_EMPTY
 
     @classmethod
@@ -357,10 +386,10 @@ class Display:
             return DisplayCell(format_str, type_str_raw)
 
         # ContainerOperand needs to import Display
-        from static_frame.core.container import ContainerOperand
+        from static_frame.core.container import ContainerBase
 
         # handling for all other values that are stringable
-        if isinstance(value, ContainerOperand):
+        if isinstance(value, ContainerBase):
             # NOTE: we do not use type delimieters as ths is an instance, not a class
             msg = value.__class__.__name__
         else:
@@ -377,6 +406,12 @@ class Display:
                 msg = config.value_format_complex_scientific.format(value)
             else:
                 msg = config.value_format_complex_positional.format(value)
+        elif (isinstance(value, str) and
+                config.display_format in _DISPLAY_FORMAT_TERMINAL):
+            # When in a TERMINAL mode, separate ASCII color markup found in strings and provide the original string as the formatted string; this will permit correct spacing.
+            msg, count = cls.ANSI_ESCAPE.subn('', msg)
+            if count:
+                return DisplayCell(value, msg)
 
         return DisplayCell(FORMAT_EMPTY, msg)
 
@@ -386,11 +421,13 @@ class Display:
     @classmethod
     def from_values(cls,
             values: np.ndarray,
+            *,
             header: object,
             config: tp.Optional[DisplayConfig] = None,
             outermost: bool = False,
             index_depth: int = 0,
-            header_depth: int = 0
+            header_depth: int = 0,
+            style_config: tp.Optional[StyleConfig] = None,
             ) -> 'Display':
         '''
         Given a 1 or 2D ndarray, return a Display instance. Generally 2D arrays are passed here only from TypeBlocks.
@@ -408,7 +445,8 @@ class Display:
             else:
                 rows.append([cls.CELL_EMPTY])
 
-        if isinstance(values, np.ndarray) and values.ndim == 2:
+        if values.__class__ is np.ndarray and values.ndim == 2:
+            # NOTE: this is generally only used by TypeBlocks
             # get rows from numpy string formatting
             np_rows = np.array_str(values).split('\n')
             last_idx = len(np_rows) - 1
@@ -439,22 +477,129 @@ class Display:
                     rows.append([cls.to_cell(v, config=config)])
 
         # add the types to the last row
-        if isinstance(values, np.ndarray) and config.type_show:
+        if values.__class__ is np.ndarray and config.type_show:
             rows.append([cls.to_cell(values.dtype, config=config, is_dtype=True)])
 
         return cls(rows,
                 config=config,
                 outermost=outermost,
                 index_depth=index_depth,
-                header_depth=header_depth)
+                header_depth=header_depth,
+                style_config=style_config,
+                )
 
+    @classmethod
+    def from_params(cls,
+            *,
+            index: "IndexBase",
+            columns: "IndexBase",
+            header: DisplayHeader,
+            column_forward_iter: CallableToIterType,
+            column_reverse_iter: CallableToIterType,
+            column_default_iter: CallableToIterType,
+            config: tp.Optional[DisplayConfig] = None,
+            style_config: tp.Optional[StyleConfig] = None,
+            ) -> 'Display':
+        '''
+        Args:
+            {config}
+        '''
+        config = config or DisplayActive.get()
+        index_depth = index.depth if config.include_index else 0
+
+        # always get the index Display (even if we are not going to use it) to determine how many rows we need (which may include types, as well as truncation with ellipsis).
+        display_index = index.display(config=config)
+
+        # header depth used for HTML and other foramtting; needs to be adjusted if removing types and/or columns and types, When showing types on a Frame, we need 2: one for the Frame type, the other for the index type.
+        header_depth = (columns.depth * config.include_columns) + (2 * config.type_show)
+
+        # create an empty display based on index display
+        d = cls([list() for _ in range(len(display_index))],
+                config=config,
+                outermost=True,
+                index_depth=index_depth,
+                header_depth=header_depth,
+                style_config=style_config,
+                )
+
+        if config.include_index:
+            # this will add more rows to accomodate the index if it is bigger due to types
+            d.extend_display(display_index)
+            header_column = '' if config.type_show else None
+        else:
+            header_column = None
+
+        if len(columns) > config.display_columns:
+            # columns as they will look after application of truncation and insertion of ellipsis
+            data_half_count = cls.truncate_half_count(config.display_columns)
+
+            column_gen = partial(_gen_skip_middle,
+                    forward_iter=column_forward_iter,
+                    forward_count=data_half_count,
+                    reverse_iter=column_reverse_iter,
+                    reverse_count=data_half_count,
+                    center_sentinel=cls.ELLIPSIS_CENTER_SENTINEL
+                    )
+        else:
+            column_gen = column_default_iter # type: ignore
+
+        for column in column_gen():
+            if column is cls.ELLIPSIS_CENTER_SENTINEL:
+                d.extend_ellipsis()
+            else:
+                d.extend_iterable(column, header=header_column)
+
+        #-----------------------------------------------------------------------
+        config_transpose = config.to_transpose()
+
+        #-----------------------------------------------------------------------
+        # prepare header display of container class
+        header_displays = []
+        if config.type_show:
+            display_cls = cls.from_values((), header=header, config=config_transpose)
+            header_displays.append(display_cls.flatten())
+
+        #-----------------------------------------------------------------------
+        # prepare columns display
+        if config.include_columns:
+            # need to apply the config_transpose such that it truncates it based on the the max columns, not the max rows
+            display_columns = columns.display(config=config_transpose)
+
+            if config.type_show:
+                index_depth_extend = index.depth - 1
+                spacer_insert_index = 1 # after the first, the name
+            elif not config.type_show and config.include_index:
+                index_depth_extend = index.depth
+                spacer_insert_index = 0
+            elif not config.include_index: # type_show must be False
+                index_depth_extend = 0
+                spacer_insert_index = 0
+
+            # add spacers to from of columns when we have a hierarchical index
+            for _ in range(index_depth_extend):
+                # will need a width equal to the column depth
+                row = [cls.to_cell('', config=config) for _ in range(columns.depth)]
+                spacer = cls([row])
+                display_columns.insert_displays(spacer, insert_index=spacer_insert_index)
+
+            if columns.depth > 1:
+                display_columns_horizontal = display_columns.transform()
+            else: # can just flatten a single column into one row
+                display_columns_horizontal = display_columns.flatten()
+
+            header_displays.append(display_columns_horizontal)
+
+        if header_displays:
+            d.insert_displays(*header_displays)
+
+        return d
 
     #---------------------------------------------------------------------------
     # core cell-to-rwo expansion routines
 
     @staticmethod
     def truncate_half_count(count_target: int) -> int:
-        '''Given a target number of rows or columns, return the count of half as found in presentation where one column is used for the elipsis. The number returned will always be odd. For example, given a target of 5 we allocate 2 per half (plus 1 reserved for middle).
+        '''Given a target number of rows or columns, return the count of half as found in presentation where one column is used for the ellipsis. The number returned will always be odd. For example, given a target of 5 we allocate 2 per half (plus 1 reserved for middle).
         '''
         if count_target <= 4:
             return 1 # practical floor for all values of 4 or less
@@ -466,7 +611,7 @@ class Display:
             rows: tp.Sequence[tp.Sequence[DisplayCell]],
             col_idx_src: int,
             col_last_src: int,
-            row_indices: tp.Iterable[tp.Optional[int]],
+            row_indices: tp.Iterable[int],
             config: tp.Optional[DisplayConfig] = None,
             ) -> tp.Tuple[int, int]:
         '''
@@ -491,14 +636,11 @@ class Display:
         max_width = 0
         for row_idx_src in row_indices:
             # get existing max width, up to the max
-            if row_idx_src is not None:
-                row = rows[row_idx_src]
-                if col_idx_src >= len(row): # this row does not have this column
-                    continue
-                cell = row[col_idx_src]
-                max_width = max(max_width, len(cell.raw))
-            else:
-                max_width = max(max_width, len(cls.ELLIPSIS))
+            row = rows[row_idx_src]
+            if col_idx_src >= len(row): # this row does not have this column
+                continue
+            cell = row[col_idx_src]
+            max_width = max(max_width, len(cell.raw))
             # if already exceeded max width, stop iterating
             if max_width >= width_limit:
                 break
@@ -570,7 +712,7 @@ class Display:
                         # must truncate if cell width is greater than max width
                         width_truncate = max_width - len(cls.CELL_ELLIPSIS.raw)
 
-                        # TODO: this is truncating scientific notation
+                        # NOTE: this might truncate scientific notation
                         cell_raw = cell_raw[:width_truncate] + cls.ELLIPSIS
                         if is_html:
                             cell_raw = html.escape(cell_raw)
@@ -606,11 +748,13 @@ class Display:
             outermost: bool = False,
             index_depth: int = 0,
             header_depth: int = 0,
+            *,
+            style_config: tp.Optional[StyleConfig] = None,
             ) -> None:
         '''Define rows as a list of lists, for each row; the contained DisplayCell instances may be of different size, but they are expected to be aligned vertically in final presentation.
 
         Args:
-            header_depth: columns depth plus any addtional lines used for headers
+            header_depth: columns depth plus any additional lines used for headers
         '''
         config = config or DisplayActive.get()
 
@@ -619,26 +763,25 @@ class Display:
         self._outermost = outermost
         self._index_depth = index_depth
         self._header_depth = header_depth
+        self._style_config = style_config
 
     def __repr__(self) -> str:
-        rows = self._to_rows_cells(self,
-                self._config,
-                )
+        rows = self._to_rows_cells(self, self._config)
+        dfc = _DISPLAY_FORMAT_MAP[self._config.display_format]
 
         if self._outermost:
-            dfc = _DISPLAY_FORMAT_MAP[self._config.display_format]
             header = []
             body = []
             for idx, row in enumerate(rows):
+                iloc_row = idx - self._header_depth
+                row = ''.join(dfc.markup_row(row,
+                        index_depth=self._index_depth,
+                        iloc_row=iloc_row,
+                        style_config=self._style_config,
+                        )).rstrip()
                 if idx < self._header_depth:
-                    row = ''.join(dfc.markup_row(row,
-                            header_depth=np.inf
-                            )).rstrip()
                     header.append(row)
                 else:
-                    row = ''.join(dfc.markup_row(row,
-                            header_depth=self._index_depth
-                            )).rstrip()
                     body.append(row)
 
             outermost = []
@@ -648,8 +791,11 @@ class Display:
 
             body_str = dfc.markup_body(dfc.LINE_SEP.join(body))
             outermost.append(body_str)
-            return dfc.markup_outermost(dfc.LINE_SEP.join(outermost))
-
+            return dfc.markup_outermost(
+                    dfc.LINE_SEP.join(outermost),
+                    style_config=self._style_config,
+                    )
+        # use mostly for debugging; strings in rows already have space / padding
         return dfc.LINE_SEP.join(''.join(r) for r in rows)
 
     def to_rows(self) -> tp.Iterable[str]:
@@ -659,9 +805,10 @@ class Display:
         post = []
         for idx, row in enumerate(self._to_rows_cells(self, self._config)):
             line = ''.join(row).rstrip()
-            if idx < self._header_depth:
-                if line == '': # type removal led to an empty line
-                    continue
+            # NOTE: apparently do not need special handling for empty lines
+            # if idx < self._header_depth:
+            #     if line == '': # type removal led to an empty line
+            #         continue
             post.append(line)
         return post
 
@@ -752,11 +899,6 @@ class Display:
         rows.extend(self._rows[insert_index:])
         self._rows = rows
 
-    # def drop_row(self, index: int = 0) -> None:
-    #     '''Remove a row in place.
-    #     '''
-    #     self._rows = self._rows[:index] + self._rows[index+1:]
-
 
     #---------------------------------------------------------------------------
     # return a new display
@@ -784,5 +926,3 @@ class Display:
             for idx, cell in enumerate(r):
                 rows[idx].append(cell)
         return self.__class__(rows, config=self._config)
-
-
