@@ -342,7 +342,14 @@ class DOC_TEMPLATE:
             {DEEPCOPY_FROM_BUS}
             '''
             )
-
+    rank = dict(
+            skipna='''skipna: If ``True``, exclude NA values (NaN or None) from ranking, replacing those values with ``fill_value``. ''',
+            ascending='''ascending: If ``True``, the lowest ranks correspond to the lowest values. The default is ``True``.''',
+            ascendings='''ascending: Boolean, or iterable of Booleans; if ``True``, the lowest ranks correspond to the lowest values; if an iterable, apply per column or row. The default is ``True``.''',
+            start='''start: The reference value for the lowest rank. Some ranking methodologies (mean, max) may not return this value given some inputs. The default is 0; for ranks that start from 1, provide a value of 1.''',
+            fill_value='''fill_value: A value to be used to fill NA values ignored in ranking when ``skipna`` is ``True``. The default is ``np.nan`` but can be set to any value to force NA values to the "bottom" or "top" of a rank as needed.''',
+            axis='''axis: Integer specifying axis of ranking, where 0 ranks vertically (within each column) and 1 ranks horizontally (within each row)'''
+            )
     reindex = dict(
             doc='''Return a new :obj:`{class_name}` with labels defined by the provided index. The size and ordering of the data is determined by the newly provided index, where data will continue to be aligned under labels found in both the new and the old index. Labels found only in the new index will be filled with ``fill_value``.
             ''',
@@ -394,9 +401,10 @@ class DOC_TEMPLATE:
             key_iloc=ILOC_SELECTOR,
             )
     sort = dict(
-            ascending='If True, sort in ascending order; if False, sort in descending order.',
-            kind='Name of the sort algorithm as passed to NumPy.',
-            key='A function that is used to pre-process the selected columns or rows and derive new values to sort by.'
+            ascending='ascending: If True, sort in ascending order; if False, sort in descending order.',
+            ascendings='''ascending: Boolean, or iterable of Booleans; if ``True``, the lowest ranks correspond to the lowest values; if an iterable, apply per column or row. The default is ``True``.''',
+            kind='kind: Name of the sort algorithm as passed to NumPy.',
+            key='key: A function that is used to pre-process the selected columns or rows and derive new values to sort by.'
             )
 
     store_client_exporter = dict(
