@@ -36,6 +36,10 @@ class ErrorInitQuilt(ErrorInit):
     '''Error in Quilt initialization.
     '''
 
+class ErrorInitYarn(ErrorInit):
+    '''Error in Yarn initialization.
+    '''
+
 class ErrorInitStore(ErrorInit):
     '''Error in Store initialization.
     '''
@@ -55,6 +59,19 @@ class LocInvalid(RuntimeError):
 class AxisInvalid(RuntimeError):
     pass
 
+class RelabelInvalid(RuntimeError):
+    def __init__(self) -> None:
+        super().__init__('Relabelling with an unordered iterable is not permitted.')
+
+class BatchIterableInvalid(RuntimeError):
+    def __init__(self) -> None:
+        super().__init__('Batch iterable does not yield expected pair of label, Frame.')
+
+class InvalidDatetime64Comparison(RuntimeError):
+    def __init__(self) -> None:
+        super().__init__('Cannot perform set operations on datetime64 of different units; use astype to align units before comparison.')
+
+
 #-------------------------------------------------------------------------------
 
 class StoreFileMutation(RuntimeError):
@@ -69,9 +86,21 @@ class StoreParameterConflict(RuntimeError):
 
 class NotImplementedAxis(NotImplementedError):
     def __init__(self) -> None:
-        super().__init__('iteration along this axis is too inefficient; create a consolidated Frame with Quilt.to_frame()')
+        super().__init__('Iteration along this axis is too inefficient; create a consolidated Frame with Quilt.to_frame()')
 
 
+#-------------------------------------------------------------------------------
+# NOTE: these are dervied from ValueError to match NumPy convention
+
+class ErrorNPYEncode(ValueError):
+    '''
+    Error encoding an NPY file.
+    '''
+
+class ErrorNPYDecode(ValueError):
+    '''
+    Error decoding an NPY file.
+    '''
 
 #-------------------------------------------------------------------------------
 
