@@ -1,14 +1,16 @@
 import typing as tp
-from collections.abc import KeysView
-import operator as operator_mod
 from itertools import zip_longest
-from functools import reduce
 from copy import deepcopy
+from collections import Counter
 
 import numpy as np
 
 from automap import AutoMap
 from automap import FrozenAutoMap
+from arraykit import immutable_filter
+from arraykit import mloc
+from arraykit import name_filter
+from arraykit import resolve_dtype
 
 
 from static_frame.core.container import ContainerOperand
@@ -24,7 +26,6 @@ from static_frame.core.display import DisplayHeader
 from static_frame.core.doc_str import doc_inject
 from static_frame.core.exception import ErrorInitIndex
 from static_frame.core.exception import ErrorInitIndexNonUnique
-from static_frame.core.exception import LocEmpty
 from static_frame.core.exception import LocInvalid
 from static_frame.core.index_base import IndexBase
 from static_frame.core.node_dt import InterfaceDatetime
@@ -35,24 +36,24 @@ from static_frame.core.node_selector import InterfaceGetItem
 from static_frame.core.node_selector import InterfaceSelectDuo
 from static_frame.core.node_selector import TContainer
 from static_frame.core.node_str import InterfaceString
+from static_frame.core.node_re import InterfaceRe
 
 from static_frame.core.util import array_shift
 from static_frame.core.util import array_sample
+from static_frame.core.util import arrays_equal
 from static_frame.core.util import array2d_to_tuples
-from static_frame.core.util import BOOL_TYPES
-from static_frame.core.util import CallableOrMapping
+
 from static_frame.core.util import dtype_from_element
 from static_frame.core.util import DEFAULT_SORT_KIND
 from static_frame.core.util import DepthLevelSpecifier
 from static_frame.core.util import DTYPE_DATETIME_KIND
+from static_frame.core.util import DTYPE_OBJECTABLE_KINDS
 from static_frame.core.util import DTYPE_INT_DEFAULT
 from static_frame.core.util import DTYPE_BOOL
 from static_frame.core.util import DtypeSpecifier
 from static_frame.core.util import EMPTY_ARRAY
-from static_frame.core.util import EMPTY_SLICE
 from static_frame.core.util import EMPTY_TUPLE
 from static_frame.core.util import GetItemKeyType
-from static_frame.core.util import immutable_filter
 from static_frame.core.util import IndexInitializer
 from static_frame.core.util import INT_TYPES
 from static_frame.core.util import intersect1d
@@ -62,24 +63,22 @@ from static_frame.core.util import iterable_to_array_1d
 from static_frame.core.util import KEY_ITERABLE_TYPES
 from static_frame.core.util import KeyIterableTypes
 from static_frame.core.util import KeyTransformType
-from static_frame.core.util import mloc
 from static_frame.core.util import NAME_DEFAULT
-from static_frame.core.util import name_filter
 from static_frame.core.util import NameType
 from static_frame.core.util import NULL_SLICE
-from static_frame.core.util import resolve_dtype
 from static_frame.core.util import setdiff1d
-from static_frame.core.util import SLICE_ATTRS
-from static_frame.core.util import SLICE_START_ATTR
-from static_frame.core.util import SLICE_STEP_ATTR
-from static_frame.core.util import SLICE_STOP_ATTR
 from static_frame.core.util import slice_to_inclusive_slice
 from static_frame.core.util import to_datetime64
 from static_frame.core.util import UFunc
-from static_frame.core.util import ufunc_axis_skipna
+from static_frame.core.util import array_ufunc_axis_skipna
 from static_frame.core.util import union1d
 from static_frame.core.util import PositionsAllocator
 from static_frame.core.util import array_deepcopy
+from static_frame.core.util import DTYPE_OBJECT
+from static_frame.core.util import IndexConstructor
+from static_frame.core.style_config import StyleConfig
+from static_frame.core.loc_map import LocMap
+
 
 if tp.TYPE_CHECKING:
     import pandas #pylint: disable=W0611 #pragma: no cover
@@ -101,6 +100,7 @@ class ILoc(metaclass=ILocMeta):
     '''A wrapper for embedding ``iloc`` specifications within a single axis argument of a ``loc`` selection.
     '''
 
+    STATIC = True
     __slots__ = (
             'key',
             )
@@ -108,161 +108,6 @@ class ILoc(metaclass=ILocMeta):
     def __init__(self, key: GetItemKeyType):
         self.key = key
 
-class LocMap:
-
-    @staticmethod
-    def map_slice_args(
-            label_to_pos: tp.Callable[[tp.Iterable[tp.Hashable]], int],
-            key: slice,
-            labels: tp.Optional[np.ndarray] = None,
-            offset: tp.Optional[int] = 0
-            ) -> tp.Iterator[tp.Union[int, None]]:
-        '''Given a slice ``key`` and a label-to-position mapping, yield each integer argument necessary to create a new iloc slice. If the ``key`` defines a region with no constituents, raise ``LocEmpty``
-
-        Args:
-            label_to_pos: callable into mapping (can be a get() method from a dictionary)
-        '''
-        offset_apply = not offset is None
-
-        for field in SLICE_ATTRS:
-            attr = getattr(key, field)
-            if attr is None:
-                yield None
-
-            elif isinstance(attr, np.datetime64):
-                assert labels is not None
-                # if a datetime, we assume that the labels are ordered;
-                if attr.dtype == labels.dtype:
-                    if field != SLICE_STEP_ATTR:
-                        pos: tp.Optional[int] = label_to_pos(attr)
-                        if pos is None:
-                            # if same type, and that atter is not in labels, we fail, just as we do in then non-datetime64 case. Only when datetimes are given in a different unit are we "loose" about matching.
-                            raise LocInvalid('Invalid loc given in a slice', attr, field)
-                    else: # step
-                        pos = attr # should be an integer
-
-                    if field == SLICE_STOP_ATTR:
-                        pos += 1 #type: ignore  # stop is inclusive
-
-                elif field == SLICE_START_ATTR:
-                    # convert to the type of the atrs; this should get the relevant start
-                    pos: tp.Optional[int] = label_to_pos(attr.astype(labels.dtype)) #type: ignore
-                    if pos is None: # we did not find a start position
-                        matches = np.flatnonzero(labels.astype(attr.dtype) == attr)
-                        if len(matches):
-                            pos = matches[0]
-                        else:
-                            raise LocEmpty()
-
-                elif field == SLICE_STOP_ATTR:
-                    # convert labels to the slice attr value, compare, then get last
-                    # add one, as this is an inclusive stop
-                    # pos = np.flatnonzero(labels.astype(attr.dtype) == attr)[-1] + 1
-                    matches = np.flatnonzero(labels.astype(attr.dtype) == attr)
-                    if len(matches):
-                        pos = matches[-1] + 1
-                    else:
-                        raise LocEmpty()
-
-                elif field == SLICE_STEP_ATTR:
-                    pos = attr
-
-                if offset_apply and field != SLICE_STEP_ATTR:
-                    pos += offset #type: ignore
-
-                yield pos
-
-            else:
-                if field != SLICE_STEP_ATTR:
-                    pos = label_to_pos(attr)
-                    if pos is None:
-                        # NOTE: could raise LocEmpty() to silently handle this
-                        raise LocInvalid('Invalid loc given in a slice', attr, field)
-                    if offset_apply:
-                        pos += offset #type: ignore
-                else: # step
-                    pos = attr # should be an integer
-
-                if field == SLICE_STOP_ATTR:
-                    # loc selections are inclusive, so iloc gets one more
-                    pos += 1 #type: ignore
-
-                yield pos
-
-    @classmethod
-    def loc_to_iloc(cls, *,
-            label_to_pos: tp.Dict[tp.Hashable, int],
-            labels: np.ndarray,
-            positions: np.ndarray,
-            key: GetItemKeyType,
-            offset: tp.Optional[int] = None,
-            partial_selection: bool = False,
-            ) -> GetItemKeyType:
-        '''
-        Note: all SF objects (Series, Index) need to be converted to basic types before being passed as `key` to this function.
-
-        Args:
-            offset: in the context of an IndexHierarchical, the iloc positions returned from this funcition need to be shifted.
-            partial_selection: if True and key is an iterable of labels that includes labels not in the mapping, available matches will be returned rather than raising.
-        Returns:
-            An integer mapped slice, or GetItemKey type that is based on integers, compatible with TypeBlocks
-        '''
-        offset_apply = not offset is None
-
-        # ILoc is handled prior to this call, in the Index._loc_to_iloc method
-
-        if isinstance(key, slice):
-            if offset_apply and key == NULL_SLICE:
-                # when offset is defined (even if it is zero), null slice is not sufficiently specific; need to convert to an explicit slice relative to the offset
-                return slice(offset, len(positions) + offset) #type: ignore
-            try:
-                return slice(*cls.map_slice_args(
-                        label_to_pos.get, #type: ignore
-                        key,
-                        labels,
-                        offset)
-                        )
-            except LocEmpty:
-                return EMPTY_SLICE
-
-        if isinstance(key, np.datetime64):
-            # convert this to the target representation, do a Boolean selection
-            if labels.dtype != key.dtype:
-                key = labels.astype(key.dtype) == key
-            # if not different type, keep it the same so as to do a direct, single element selection
-
-        is_array = isinstance(key, np.ndarray)
-        is_list = isinstance(key, list)
-
-        # can be an iterable of labels (keys) or an iterable of Booleans
-        if is_array or is_list:
-            if is_array and key.dtype.kind == DTYPE_DATETIME_KIND:
-                if labels.dtype != key.dtype:
-                    labels_ref = labels.astype(key.dtype)
-                    # let Boolean key advance to next branch
-                    key = reduce(operator_mod.or_, (labels_ref == k for k in key))
-
-            if is_array and key.dtype == DTYPE_BOOL:
-                if offset_apply:
-                    return positions[key] + offset
-                return positions[key]
-
-            # map labels to integer positions, return a list of integer positions
-            # NOTE: we may miss the opportunity to identify contiguous keys and extract a slice
-            # NOTE: we do more branching here to optimize performance
-            if partial_selection:
-                if offset_apply:
-                    return [label_to_pos[k] + offset for k in key if k in label_to_pos] #type: ignore
-                return [label_to_pos[k] for k in key if k in label_to_pos]
-            if offset_apply:
-                return [label_to_pos[k] + offset for k in key] #type: ignore
-            return [label_to_pos[k] for k in key]
-
-        # if a single element (an integer, string, or date, we just get the integer out of the map
-        if offset_apply:
-            return label_to_pos[key] + offset #type: ignore
-        return label_to_pos[key]
-
 
 def immutable_index_filter(index: I) -> IndexBase:
     '''Return an immutable index. All index objects handle converting from mutable to immutable via the __init__ constructor; but need to use appropriate class between Index and IndexHierarchy.'''
@@ -316,7 +161,7 @@ class Index(IndexBase):
     _name: NameType
 
     #---------------------------------------------------------------------------
-    # methods used in __init__ that are customized in dervied classes; there, we need to mutate instance state, this these are instance methods
+    # methods used in __init__ that are customized in derived classes; there, we need to mutate instance state, this these are instance methods
     @staticmethod
     def _extract_labels(
             mapping: tp.Optional[tp.Dict[tp.Hashable, int]],
@@ -334,8 +179,8 @@ class Index(IndexBase):
             labels: might be an expired Generator, but if it is an immutable ndarray, we can use it without a copy.
         '''
         # pre-fetching labels for faster get_item construction
-        if isinstance(labels, np.ndarray):
-            if dtype is not None and dtype != labels.dtype:
+        if labels.__class__ is np.ndarray:
+            if dtype is not None and dtype != labels.dtype: #type: ignore
                 raise ErrorInitIndex('invalid label dtype for this Index')
             return immutable_filter(labels)
 
@@ -359,7 +204,7 @@ class Index(IndexBase):
             positions: tp.Optional[tp.Sequence[int]]
             ) -> np.ndarray:
         # positions is either None or an ndarray
-        if isinstance(positions, np.ndarray):
+        if positions.__class__ is np.ndarray:
             return immutable_filter(positions)
         return PositionsAllocator.get(size)
 
@@ -377,6 +222,19 @@ class Index(IndexBase):
         '''
         return cls(labels, name=name)
 
+
+    @staticmethod
+    def _error_init_index_non_unique(labels: tp.Iterable[tp.Hashable]) -> ErrorInitIndexNonUnique:
+        labels_counter = Counter(labels)
+        if len(labels_counter) == 0: # generator consumed
+            msg = 'Labels have non-unique values. Details from iterators not available.'
+        else:
+            labels_all = sum(labels_counter.values())
+            labels_duplicated = [repr(p[0]) for p in labels_counter.most_common(10) if p[1] > 1]
+            msg = f'Labels have {labels_all - len(labels_counter)} non-unique values, including {", ".join(labels_duplicated)}.'
+
+        return ErrorInitIndexNonUnique(msg)
+
     #---------------------------------------------------------------------------
     @doc_inject(selector='index_init')
     def __init__(self,
@@ -408,7 +266,9 @@ class Index(IndexBase):
 
         #-----------------------------------------------------------------------
         # handle all Index subclasses
-        if isinstance(labels, IndexBase):
+        if labels.__class__ is np.ndarray:
+            pass
+        elif isinstance(labels, IndexBase):
             if labels._recache:
                 labels._update_array_cache()
             if name is NAME_DEFAULT:
@@ -427,42 +287,50 @@ class Index(IndexBase):
                 labels = labels.__iter__()
         elif isinstance(labels, ContainerOperand):
             # it is a Series or similar
-            array = labels.values # NOTE: should we take values or keys here?
+            array = labels.values
             if array.ndim == 1:
                 labels = array
             else:
                 labels = array2d_to_tuples(array)
-        # else: assume an iterable suitable for labels usage
+        # else: assume an iterable suitable for labels usage, we will identify strings later
 
         #-----------------------------------------------------------------------
         if is_typed:
             # do not need to check arrays, as will and checked to match dtype_extract in _extract_labels
-            if not isinstance(labels, np.ndarray):
+            if not labels.__class__ is np.ndarray:
                 # for now, assume that if _DTYPE is defined, we have a date
                 labels = (to_datetime64(v, dtype_extract) for v in labels)
             # coerce to target type
-            elif labels.dtype != dtype_extract:
-                labels = labels.astype(dtype_extract)
+            elif labels.dtype != dtype_extract: #type: ignore
+                labels = labels.astype(dtype_extract) #type: ignore
                 labels.flags.writeable = False #type: ignore
+            labels_for_automap = labels
 
         self._name = None if name is NAME_DEFAULT else name_filter(name)
 
         if self._map is None: # if _map not shared from another Index
             if not loc_is_iloc:
+                # PERF: calling tolist before initializing AutoMap is shown to be about 2x faster, but can only be done with NumPy dtypes that are equivalent after conversion to Python objects
+                if not is_typed:
+                    if labels.__class__ is np.ndarray and labels.dtype.kind in DTYPE_OBJECTABLE_KINDS: #type: ignore
+                        labels_for_automap = labels.tolist() #type: ignore
+                    elif isinstance(labels, str):
+                        # NOTE: this is necessary as otherwise a malformed Index will be created, whereby the _map will treat the string as an iterable of chars, while the labels will not and have a single string value. This is consisten as other elements (ints, Booleans) are rejected on instantiation of the AutoMap
+                        raise ErrorInitIndex('Cannot create an Index from a single string; provide an iterable of strings.')
+                    else:
+                        labels_for_automap = labels
+                else:
+                    labels_for_automap = labels
                 try:
-                    self._map = FrozenAutoMap(labels) if self.STATIC else AutoMap(labels)
+                    self._map = FrozenAutoMap(labels_for_automap) if self.STATIC else AutoMap(labels_for_automap)
                 except ValueError: # Automap will raise ValueError of non-unique values are encountered
-                    pass
-                if self._map is None:
-                    raise ErrorInitIndexNonUnique(
-                            f'labels ({len(tuple(labels))}) have non-unique values ({len(set(labels))})'
-                            )
+                    raise self._error_init_index_non_unique(labels_for_automap) from None
                 size = len(self._map)
-            else: # must assume labels are unique
-                # labels must not be a generator, but we assume that internal clients that provided loc_is_iloc will not give a generator
+            else:
+                # if loc_is_iloc, labels must be positions and we assume that internal clients that provided loc_is_iloc will not give a generator
                 size = len(labels) #type: ignore
                 if positions is None:
-                    positions = PositionsAllocator.get(size)
+                    positions = labels
         else: # map shared from another Index
             size = len(self._map)
 
@@ -471,9 +339,17 @@ class Index(IndexBase):
         self._positions = self._extract_positions(size, positions)
 
         if self._DTYPE and self._labels.dtype != self._DTYPE:
-            raise ErrorInitIndex('invalid label dtype for this Index', #pragma: no cover
+            raise ErrorInitIndex('Invalid label dtype for this Index', #pragma: no cover
                     self._labels.dtype, self._DTYPE)
 
+        # NOTE: to implement GH # 374; do this after final self._labels creation as user may pass a dtype argument
+        if not is_typed and self._labels.dtype.kind == DTYPE_DATETIME_KIND:
+            from static_frame.core.exception import deprecated #pragma: no cover
+            deprecated('Creating an Index with a datetime64 array is deprecated and will be removed in v0.9; use an Index subclass (e.g. IndexDate) or supply an `index_constructors` argument') #pragma: no cover
+            # raise ErrorInitIndex('Cannot create an Index with a datetime64 array; use an Index subclass (e.g. IndexDate) or supply an `index_constructors` argument')
+
+
+
     #---------------------------------------------------------------------------
     def __setstate__(self, state: tp.Tuple[None, tp.Dict[str, tp.Any]]) -> None:
         '''
@@ -484,9 +360,7 @@ class Index(IndexBase):
         self._labels.flags.writeable = False
 
     def __deepcopy__(self: I, memo: tp.Dict[int, tp.Any]) -> I:
-        if self._recache:
-            self._update_array_cache()
-
+        assert not self._recache # __deepcopy__ is implemented on derived GO class
         obj = self.__new__(self.__class__)
         obj._map = deepcopy(self._map, memo) #type: ignore
         obj._labels = array_deepcopy(self._labels, memo) #type: ignore
@@ -592,7 +466,7 @@ class Index(IndexBase):
         '''
         if self._recache:
             self._update_array_cache()
-        return tp.cast(tp.Tuple[int, ...], self._labels.shape)
+        return self._labels.shape #type: ignore
 
     @property
     def ndim(self) -> int:
@@ -604,7 +478,7 @@ class Index(IndexBase):
         '''
         if self._recache:
             self._update_array_cache()
-        return tp.cast(int, self._labels.ndim)
+        return self._labels.ndim #type: ignore
 
     @property
     def size(self) -> int:
@@ -616,7 +490,7 @@ class Index(IndexBase):
         '''
         if self._recache:
             self._update_array_cache()
-        return tp.cast(int, self._labels.size)
+        return self._labels.size #type: ignore
 
     @property
     def nbytes(self) -> int:
@@ -628,15 +502,7 @@ class Index(IndexBase):
         '''
         if self._recache:
             self._update_array_cache()
-        return tp.cast(int, self._labels.nbytes)
-
-    # def __bool__(self) -> bool:
-    #     '''
-    #     True if this container has size.
-    #     '''
-    #     if self._recache:
-    #         self._update_array_cache()
-    #     return bool(self._labels.size)
+        return self._labels.nbytes #type: ignore
 
     #---------------------------------------------------------------------------
     # set operations
@@ -664,7 +530,7 @@ class Index(IndexBase):
                 # if self._DTYPE is defined, the default constructor does not take a dtype argument
                 return self.__class__(())
 
-        if isinstance(other, np.ndarray):
+        if other.__class__ is np.ndarray:
             operand = other
             assume_unique = False
         elif isinstance(other, IndexBase):
@@ -695,7 +561,7 @@ class Index(IndexBase):
             if self.STATIC: # immutable, no selection, can return self
                 return self
             labels = self._labels # already immutable
-        elif isinstance(key, np.ndarray) and key.dtype == bool:
+        elif key.__class__ is np.ndarray and key.dtype == bool: #type: ignore
             # can use labels, as we already recached
             # use Boolean area to select indices from positions, as np.delete does not work with arrays
             labels = np.delete(self._labels, self._positions[key], axis=0)
@@ -724,14 +590,14 @@ class Index(IndexBase):
     @doc_inject(select='astype')
     def astype(self, dtype: DtypeSpecifier) -> 'Index':
         '''
-        Return an Index with type determined by `dtype` argument. Note that for Index, this is a simple function, whereas for ``IndexHierarchy``, this is an interface exposing both a callable and a getitem interface.
+        Return an Index with type determined by `dtype` argument. If a `datetime64` dtype is provided, the appropriate ``Index`` subclass will be returned. Note that for Index, this is a simple function, whereas for ``IndexHierarchy``, this is an interface exposing both a callable and a getitem interface.
 
         Args:
             {dtype}
         '''
-        from static_frame.core.index_datetime import _dtype_to_index_cls
+        from static_frame.core.index_datetime import dtype_to_index_cls
         array = self.values.astype(dtype)
-        cls = _dtype_to_index_cls(self.STATIC, array.dtype)
+        cls = dtype_to_index_cls(self.STATIC, array.dtype)
         return cls(
                 array,
                 name=self._name
@@ -747,14 +613,11 @@ class Index(IndexBase):
         if self._recache:
             self._update_array_cache()
 
-        blocks = (self._labels,)
-        cls = Index if self.STATIC else IndexGO
-
         def blocks_to_container(blocks: tp.Iterator[np.ndarray]) -> np.ndarray:
             return next(blocks)
 
         return InterfaceString(
-                blocks=blocks,
+                blocks=(self._labels,),
                 blocks_to_container=blocks_to_container,
                 )
 
@@ -766,17 +629,34 @@ class Index(IndexBase):
         if self._recache:
             self._update_array_cache()
 
-        blocks = (self.values,)
-        cls = Index if self.STATIC else IndexGO
-
         def blocks_to_container(blocks: tp.Iterator[np.ndarray]) -> np.ndarray:
             return next(blocks)
 
         return InterfaceDatetime(
-                blocks=blocks,
+                blocks=(self.values,),
                 blocks_to_container=blocks_to_container,
                 )
 
+    def via_re(self,
+            pattern: str,
+            flags: int = 0,
+            ) -> InterfaceRe[np.ndarray]:
+        '''
+        Interface for applying regular expressions to elements in this container.
+        '''
+        if self._recache:
+            self._update_array_cache()
+
+        def blocks_to_container(blocks: tp.Iterator[np.ndarray]) -> np.ndarray:
+            return next(blocks)
+
+        return InterfaceRe(
+                blocks=(self._labels,),
+                blocks_to_container=blocks_to_container,
+                pattern=pattern,
+                flags=flags,
+                )
+
     #---------------------------------------------------------------------------
 
     def _update_array_cache(self) -> None:
@@ -793,6 +673,8 @@ class Index(IndexBase):
     @doc_inject()
     def display(self,
             config: tp.Optional[DisplayConfig] = None,
+            *,
+            style_config: tp.Optional[StyleConfig] = None,
             ) -> Display:
         '''{doc}
 
@@ -819,6 +701,7 @@ class Index(IndexBase):
                 outermost=True,
                 index_depth=0,
                 header_depth=header_depth,
+                style_config=style_config,
                 )
 
     #---------------------------------------------------------------------------
@@ -874,6 +757,17 @@ class Index(IndexBase):
         self._depth_level_validate(depth_level)
         yield from zip_longest(self.values, EMPTY_TUPLE, fillvalue=1)
 
+    @property
+    def index_types(self) -> 'Series':
+        '''
+        Return a Series of Index classes for each index depth.
+
+        Returns:
+            :obj:`Series`
+        '''
+        from static_frame.core.series import Series
+        return Series((self.__class__,), index=(self._name,), dtype=DTYPE_OBJECT)
+
 
     #---------------------------------------------------------------------------
 
@@ -893,7 +787,7 @@ class Index(IndexBase):
                     )
 
         return self.__class__(
-                (mapper(x) for x in self._labels), #type: ignore
+                (mapper(x) for x in self._labels),
                 name=self._name
                 )
 
@@ -915,60 +809,52 @@ class Index(IndexBase):
         Returns:
             Return GetItemKey type that is based on integers, compatible with TypeBlocks
         '''
-        from static_frame.core.series import Series
-
-        if self._recache:
-            self._update_array_cache()
-
-        if isinstance(key, ILoc):
-            return key.key
+        if key.__class__ is ILoc:
+            return key.key #type: ignore
 
         key = key_from_container_key(self, key)
 
         if self._map is None and offset is None: # loc_is_iloc
-            if isinstance(key, np.ndarray):
-                if key.dtype == bool:
+            if key.__class__ is np.ndarray:
+                if key.dtype == bool: #type: ignore
                     return key
-                if key.dtype != DTYPE_INT_DEFAULT:
+                if key.dtype != DTYPE_INT_DEFAULT: #type: ignore
                     # if key is an np.array, it must be an int or bool type
                     # could use tolist(), but we expect all keys to be integers
-                    return key.astype(DTYPE_INT_DEFAULT)
-            elif isinstance(key, slice):
-                key = slice_to_inclusive_slice(key)
+                    return key.astype(DTYPE_INT_DEFAULT) #type: ignore
+            elif key.__class__ is slice:
+                key = slice_to_inclusive_slice(key) #type: ignore
             return key
 
         if self._map is None and offset is not None: # loc_is_iloc
-            if isinstance(key, slice):
+            if key.__class__ is slice:
                 if key == NULL_SLICE:
                     return slice(offset, self.__len__() + offset)
+                return slice_to_inclusive_slice(key, offset) #type: ignore
 
-                key = slice_to_inclusive_slice(key)
+            if key.__class__ is np.ndarray:
+                # PERF: isolate for usage of _positions
+                if self._recache:
+                    self._update_array_cache()
 
-                # NOTE: this slice transformation could be a utility function in ArrayKit
-                def slice_attrs() -> tp.Iterator[int]:
-                    for attr in SLICE_ATTRS:
-                        if attr != SLICE_STEP_ATTR:
-                            yield getattr(key, attr) + offset
-                        else:
-                            yield getattr(key, attr)
-
-                return slice(*slice_attrs())
-
-            if isinstance(key, np.ndarray):
-                if key.dtype == DTYPE_BOOL:
+                if key.dtype == DTYPE_BOOL: #type: ignore
                     return self._positions[key] + offset
-                if key.dtype != DTYPE_INT_DEFAULT:
-                    key = key.astype(DTYPE_INT_DEFAULT)
+                if key.dtype != DTYPE_INT_DEFAULT: #type: ignore
+                    key = key.astype(DTYPE_INT_DEFAULT) #type: ignore
                 return key + offset
 
             if isinstance(key, list):
-               return [k + offset for k in key]
+                return [k + offset for k in key]
             # a single element
             return key + offset # type: ignore
 
         if key_transform:
             key = key_transform(key)
 
+        # PERF: isolate for usage of _positions
+        if self._recache:
+            self._update_array_cache()
+
         return LocMap.loc_to_iloc(
                 label_to_pos=self._map, #type: ignore
                 labels=self._labels,
@@ -987,7 +873,7 @@ class Index(IndexBase):
             key: a label key.
         '''
         if self._map is None: # loc is iloc
-            is_bool_array = isinstance(key, np.ndarray) and key.dtype == DTYPE_BOOL
+            is_bool_array = key.__class__ is np.ndarray and key.dtype == DTYPE_BOOL #type: ignore
 
             try:
                 result = self._positions[key]
@@ -1004,7 +890,7 @@ class Index(IndexBase):
 
             if isinstance(key, slice):
                 if key == NULL_SLICE:
-                    return slice(0, self.__len__())
+                    return NULL_SLICE
                 if key.stop >= len(self):
                     # while a valid slice of positions, loc lookups do not permit over-stating boundaries
                     raise LocInvalid(f'Invalid loc: {key}')
@@ -1015,29 +901,38 @@ class Index(IndexBase):
         return self._loc_to_iloc(key)
 
     def _extract_iloc(self,
-            key: GetItemKeyType
+            key: GetItemKeyType,
             ) -> tp.Union['Index', tp.Hashable]:
-        '''Extract a new index given an iloc key
+        '''Extract a new index given an iloc key.
         '''
         if self._recache:
             self._update_array_cache()
 
         if key is None:
             labels = self._labels
-        elif isinstance(key, slice):
+            loc_is_iloc = self._map is None
+        elif key.__class__ is slice:
             if key == NULL_SLICE:
                 labels = self._labels
+                loc_is_iloc = self._map is None
             else:
                 # if labels is an np array, this will be a view; if a list, a copy
                 labels = self._labels[key]
+                labels.flags.writeable = False
+                loc_is_iloc = False
         elif isinstance(key, KEY_ITERABLE_TYPES):
             # we assume Booleans have been normalized to integers here
             # can select directly from _labels[key] if if key is a list
             labels = self._labels[key]
+            labels.flags.writeable = False
+            loc_is_iloc = False
         else: # select a single label value
             return self._labels[key] #type: ignore
 
-        return self.__class__(labels=labels, name=self._name)
+        return self.__class__(labels=labels,
+                loc_is_iloc=loc_is_iloc,
+                name=self._name,
+                )
 
     def _extract_loc(self: I,
             key: GetItemKeyType
@@ -1068,7 +963,8 @@ class Index(IndexBase):
 
     def _ufunc_binary_operator(self, *,
             operator: UFunc,
-            other: tp.Any
+            other: tp.Any,
+            fill_value: object = np.nan,
             ) -> np.ndarray:
         '''
         Binary operators applied to an index always return an NP array. This deviates from Pandas, where some operations (multiplying an int index by an int) result in a new Index, while other operations result in a np.array (using == on two Index).
@@ -1088,7 +984,7 @@ class Index(IndexBase):
         if issubclass(other.__class__, Index):
             other = other.values # operate on labels to labels
             other_is_array = True
-        elif isinstance(other, np.ndarray):
+        elif other.__class__ is np.ndarray:
             other_is_array = True
 
         if operator.__name__ == 'matmul':
@@ -1121,7 +1017,7 @@ class Index(IndexBase):
             self._update_array_cache()
 
         # do not need to pass on composabel here
-        return ufunc_axis_skipna(
+        return array_ufunc_axis_skipna(
                 array=self._labels,
                 skipna=skipna,
                 axis=0,
@@ -1142,7 +1038,7 @@ class Index(IndexBase):
         '''
         if self._recache:
             self._update_array_cache()
-        return tp.cast(tp.Iterator[tp.Hashable], self._labels.__iter__())
+        yield from self._labels.__iter__()
 
     def __reversed__(self) -> tp.Iterator[tp.Hashable]:
         '''
@@ -1218,21 +1114,8 @@ class Index(IndexBase):
             return False
         if compare_dtype and self.dtype != other.dtype:
             return False
+        return arrays_equal(self.values, other.values, skipna=skipna)
 
-        eq = self.values == other.values
-
-        # NOTE: will only be False, or an array
-        if eq is False:
-            return eq
-
-        if skipna:
-            isna_both = (isna_array(self.values, include_none=False)
-                    & isna_array(other.values, include_none=False))
-            eq[isna_both] = True
-
-        if not eq.all(): # avoid returning a NumPy Bool
-            return False
-        return True
 
     @doc_inject(selector='sort')
     def sort(self,
@@ -1243,9 +1126,9 @@ class Index(IndexBase):
         '''Return a new Index with the labels sorted.
 
         Args:
-            ascending: {ascending}
-            kind: {kind}
-            key: {key}
+            {ascending}
+            {kind}
+            {key}
         '''
         order = sort_index_for_order(self, kind=kind, ascending=ascending, key=key) #type: ignore [arg-type]
 
@@ -1321,14 +1204,37 @@ class Index(IndexBase):
         return Series(self.values, name=self._name)
 
 
-    def level_add(self, level: tp.Hashable) -> 'IndexHierarchy':
+    def level_add(self,
+            level: tp.Hashable,
+            *,
+            index_constructor: IndexConstructor = None,
+            ) -> 'IndexHierarchy':
         '''Return an IndexHierarchy with an added root level.
+
+        Args:
+            level: A hashable to used as the new root.
+            *,
+            index_constructor:
         '''
         from static_frame import IndexHierarchy
         from static_frame import IndexHierarchyGO
+        from static_frame import Index
+        from static_frame import IndexGO
 
         cls = IndexHierarchy if self.STATIC else IndexHierarchyGO
-        return cls.from_tree({level: self.values}, name=self._name)
+        cls_depth = Index if self.STATIC else IndexGO
+
+        if index_constructor is None:
+            # cannot assume new depth is the same index subclass
+            index_constructors = (cls_depth, self.__class__)
+        else:
+            index_constructors = (index_constructor, self.__class__) #type: ignore
+
+        return cls.from_tree(
+                {level: self.values},
+                name=self._name,
+                index_constructors=index_constructors,
+                )
 
 
     def to_pandas(self) -> 'pandas.Index':
@@ -1336,6 +1242,8 @@ class Index(IndexBase):
         '''
         import pandas
         # must copy to remove immutability, decouple reference
+        if self._map is None:
+            return pandas.RangeIndex(self.__len__(), name=self._name)
         return pandas.Index(self.values.copy(),
                 name=self._name)
 
@@ -1355,7 +1263,8 @@ _INDEX_GO_SLOTS = (
 class _IndexGOMixin:
 
     STATIC = False
-    __slots__ = () # define in derived class
+    # NOTE: must define in derived class or get TypeError: multiple bases have instance lay-out conflict
+    __slots__ = ()
 
     _map: tp.Optional[AutoMap]
     _labels: np.ndarray
@@ -1489,7 +1398,7 @@ def _index_initializer_needs_init(
         return False
     if isinstance(value, IndexBase):
         return False
-    if isinstance(value, np.ndarray):
-        return bool(len(value))
+    if value.__class__ is np.ndarray:
+        return bool(len(value)) #type: ignore
     return bool(value)
 
