@@ -1,6 +1,7 @@
 import typing as tp
 
 import numpy as np
+from arraykit import resolve_dtype
 
 from static_frame.core.container import ContainerOperand
 from static_frame.core.display import Display
@@ -16,9 +17,15 @@ from static_frame.core.util import PathSpecifierOrFileLike
 from static_frame.core.util import UFunc
 from static_frame.core.util import write_optional_file
 from static_frame.core.util import iterable_to_array_1d
-from static_frame.core.util import resolve_dtype
 from static_frame.core.util import dtype_from_element
+from static_frame.core.util import EMPTY_TUPLE
 
+from static_frame.core.style_config import StyleConfig
+from static_frame.core.style_config import style_config_css_factory
+from static_frame.core.style_config import STYLE_CONFIG_DEFAULT
+from static_frame.core.node_re import InterfaceRe
+from static_frame.core.node_dt import InterfaceDatetime
+from static_frame.core.node_str import InterfaceString
 
 if tp.TYPE_CHECKING:
     import pandas #pylint: disable=W0611 #pragma: no cover
@@ -30,10 +37,10 @@ I = tp.TypeVar('I', bound='IndexBase')
 
 class IndexBase(ContainerOperand):
     '''
-    All indices are dervied from ``IndexBase``, including ``Index`` and ``IndexHierarchy``.
+    All indices are derived from ``IndexBase``, including ``Index`` and ``IndexHierarchy``.
     '''
 
-    __slots__ = () # defined in dervied classes
+    __slots__ = () # defined in derived classes
 
     #---------------------------------------------------------------------------
 
@@ -147,7 +154,7 @@ class IndexBase(ContainerOperand):
             *,
             name: tp.Optional[tp.Hashable] = None
             ) -> I:
-        raise NotImplementedError()
+        raise NotImplementedError() #pragma: no cover
 
     def __init__(self, initializer: tp.Any = None,
             *,
@@ -180,6 +187,12 @@ class IndexBase(ContainerOperand):
             ) -> np.ndarray:
         raise NotImplementedError() #pragma: no cover
 
+    @property
+    def index_types(self) -> 'Series':
+        # NOTE: this implementation is here due to pydoc.render_doc call that led to calling this base class method
+        from static_frame.core.series import Series
+        return Series(EMPTY_TUPLE) # pragma: no cover
+
     def _extract_iloc(self: I, key: GetItemKeyType) -> tp.Union[I, tp.Hashable]:
         raise NotImplementedError() #pragma: no cover
 
@@ -192,6 +205,9 @@ class IndexBase(ContainerOperand):
     def relabel(self: I, mapper: 'RelabelInput') -> I:
         raise NotImplementedError() #pragma: no cover
 
+    def rename(self: I, name: NameType) -> I:
+        raise NotImplementedError() #pragma: no cover
+
     def _drop_iloc(self: I, key: GetItemKeyType) -> I:
         raise NotImplementedError() #pragma: no cover
 
@@ -214,7 +230,11 @@ class IndexBase(ContainerOperand):
     def level_add(self, level: tp.Hashable) -> 'IndexHierarchy':
         raise NotImplementedError() #pragma: no cover
 
-    def display(self, config: tp.Optional[DisplayConfig] = None) -> Display:
+    def display(self,
+            config: tp.Optional[DisplayConfig] = None,
+            *,
+            style_config: tp.Optional[StyleConfig] = None,
+            ) -> Display:
         raise NotImplementedError()
 
     #---------------------------------------------------------------------------
@@ -250,7 +270,7 @@ class IndexBase(ContainerOperand):
             {side_left}
         '''
         if not isinstance(values, str) and hasattr(values, '__len__'):
-            if not isinstance(values, np.ndarray):
+            if not values.__class__ is np.ndarray:
                 values, _ = iterable_to_array_1d(values)
         return np.searchsorted(self.values, #type: ignore [no-any-return]
                 values,
@@ -297,12 +317,12 @@ class IndexBase(ContainerOperand):
     def _loc_to_iloc(self,
             key: GetItemKeyType,
             ) -> GetItemKeyType:
-        raise NotImplementedError()
+        raise NotImplementedError() #pragma: no cover
 
     def loc_to_iloc(self,
             key: GetItemKeyType,
             ) -> GetItemKeyType:
-        raise NotImplementedError()
+        raise NotImplementedError() #pragma: no cover
 
     def __getitem__(self: I,
             key: GetItemKeyType
@@ -318,6 +338,9 @@ class IndexBase(ContainerOperand):
         '''{}'''
         return self._name
 
+    def _name_is_names(self) -> bool:
+        return isinstance(self._name, tuple) and len(self._name) == self.depth
+
     @property
     def names(self) -> tp.Tuple[str, ...]:
         '''
@@ -331,17 +354,15 @@ class IndexBase(ContainerOperand):
             if name and depth == 1:
                 yield str(name)
             # try to use name only if it is a tuple of the right size
-            elif name and isinstance(name, tuple) and len(name) == depth:
-                for n in name:
+            elif name and self._name_is_names():
+                for n in name: #type: ignore [attr-defined]
                     yield str(n)
             else:
                 for i in range(depth):
                     yield template.format(i)
 
-        names = tuple(gen())
-        # if len(names) != depth:
-        #     raise RuntimeError(f'unexpected names formation: {names}, does not meet depth {depth}')
-        return names
+        return tuple(gen())
+
 
     #---------------------------------------------------------------------------
     # transformations resulting in reduced dimensionality
@@ -437,12 +458,30 @@ class IndexBase(ContainerOperand):
                 size_one_unity=size_one_unity
                 )
 
+    #---------------------------------------------------------------------------
+    # via interfaces
+
+    @property
+    def via_str(self) -> InterfaceString[np.ndarray]:
+        raise NotImplementedError() #pragma: no cover
+
+    @property
+    def via_dt(self) -> InterfaceDatetime[np.ndarray]:
+        raise NotImplementedError() #pragma: no cover
+
+    def via_re(self,
+            pattern: str,
+            flags: int = 0,
+            ) -> InterfaceRe[np.ndarray]:
+        raise NotImplementedError() #pragma: no cover
+
     #---------------------------------------------------------------------------
     # exporters
 
     @doc_inject(class_name='Index')
     def to_html(self,
-            config: tp.Optional[DisplayConfig] = None
+            config: tp.Optional[DisplayConfig] = None,
+            style_config: tp.Optional[StyleConfig] = STYLE_CONFIG_DEFAULT,
             ) -> str:
         '''
         {}
@@ -451,7 +490,9 @@ class IndexBase(ContainerOperand):
         config = config.to_display_config(
                 display_format=DisplayFormats.HTML_TABLE,
                 )
-        return repr(self.display(config))
+
+        style_config = style_config_css_factory(style_config, self)
+        return repr(self.display(config, style_config=style_config))
 
     @doc_inject(class_name='Index')
     def to_html_datatables(self,
