@@ -57,13 +57,19 @@ class IndexDatetime(Index):
             labels: IndexInitializer,
             *,
             name: NameType = NAME_DEFAULT,
+            loc_is_iloc: bool = False,
             ):
         '''Initializer.
 
         {args}
         '''
+        assert not loc_is_iloc
         # __init__ here leaves out the dtype argument, reducing the signature to arguments relevant for these derived classes
-        Index.__init__(self, labels=labels, name=name)
+        Index.__init__(self,
+                labels=labels,
+                name=name,
+                loc_is_iloc=loc_is_iloc,
+                )
 
     #---------------------------------------------------------------------------
     # dict like interface
@@ -78,7 +84,9 @@ class IndexDatetime(Index):
 
     def _ufunc_binary_operator(self, *,
             operator: tp.Callable[..., tp.Any],
-            other: object) -> np.ndarray:
+            other: object,
+            fill_value: object = np.nan,
+            ) -> np.ndarray:
 
         if self._recache:
             self._update_array_cache()
@@ -511,7 +519,7 @@ _DTYPE_TO_CLASS = {cls._DTYPE: cls for cls in (
         IndexNanosecond
         )}
 
-def _dtype_to_index_cls(static: bool, dtype: np.dtype) -> tp.Type[Index]:
+def dtype_to_index_cls(static: bool, dtype: np.dtype) -> tp.Type[Index]:
     '''
     Given an the class of the Index from which this is valled, as well as the dtype of the resultant array, return the appropriate Index class.
     '''
