@@ -1,10 +1,12 @@
 import typing as tp
-from itertools import chain
 from ast import literal_eval
 from copy import deepcopy
+from itertools import chain
+from itertools import zip_longest
+from itertools import repeat
 
 import numpy as np
-
+from arraykit import name_filter
 
 from static_frame.core.array_go import ArrayGO
 from static_frame.core.container_util import index_from_optional_constructor
@@ -25,11 +27,13 @@ from static_frame.core.index import ILoc
 
 from static_frame.core.index import Index
 from static_frame.core.index import IndexGO
+from static_frame.core.util import iterable_to_array_1d
 from static_frame.core.util import PositionsAllocator
 from static_frame.core.index import mutable_immutable_index_filter
 from static_frame.core.index_base import IndexBase
 from static_frame.core.index_level import IndexLevel
 from static_frame.core.index_level import IndexLevelGO
+from static_frame.core.index_level import TreeNodeT
 from static_frame.core.index_auto import RelabelInput
 from static_frame.core.index_datetime import IndexDatetime
 
@@ -42,7 +46,7 @@ from static_frame.core.node_selector import InterfaceGetItem
 from static_frame.core.node_selector import TContainer
 from static_frame.core.node_str import InterfaceString
 from static_frame.core.node_transpose import InterfaceTranspose
-
+from static_frame.core.node_re import InterfaceRe
 from static_frame.core.type_blocks import TypeBlocks
 
 from static_frame.core.util import DEFAULT_SORT_KIND
@@ -50,7 +54,7 @@ from static_frame.core.util import DepthLevelSpecifier
 from static_frame.core.util import DtypeSpecifier
 from static_frame.core.util import EMPTY_TUPLE
 from static_frame.core.util import DTYPE_BOOL
-
+from static_frame.core.util import DTYPE_OBJECT
 from static_frame.core.util import GetItemKeyType
 from static_frame.core.util import IndexConstructor
 from static_frame.core.util import IndexConstructors
@@ -59,7 +63,6 @@ from static_frame.core.util import INT_TYPES
 from static_frame.core.util import intersect2d
 from static_frame.core.util import isin
 from static_frame.core.util import NAME_DEFAULT
-from static_frame.core.util import name_filter
 from static_frame.core.util import NameType
 from static_frame.core.util import NULL_SLICE
 from static_frame.core.util import setdiff2d
@@ -71,6 +74,11 @@ from static_frame.core.util import iterable_to_array_2d
 from static_frame.core.util import array_sample
 from static_frame.core.util import key_to_datetime_key
 from static_frame.core.util import concat_resolved
+from static_frame.core.util import CONTINUATION_TOKEN_INACTIVE
+from static_frame.core.util import BoolOrBools
+from static_frame.core.util import isna_array
+
+from static_frame.core.style_config import StyleConfig
 
 
 if tp.TYPE_CHECKING:
@@ -81,7 +89,6 @@ if tp.TYPE_CHECKING:
 
 IH = tp.TypeVar('IH', bound='IndexHierarchy')
 
-CONTINUATION_TOKEN_INACTIVE = object()
 
 #-------------------------------------------------------------------------------
 class IndexHierarchy(IndexBase):
@@ -111,29 +118,48 @@ class IndexHierarchy(IndexBase):
     _UFUNC_INTERSECTION = intersect2d
     _UFUNC_DIFFERENCE = setdiff2d
     _NDIM: int = 2
+
     #---------------------------------------------------------------------------
     # constructors
 
     @classmethod
     def from_product(cls: tp.Type[IH],
             *levels: IndexInitializer,
-            name: NameType = None
+            name: NameType = None,
+            index_constructors: tp.Optional[IndexConstructors] = None,
             ) -> IH:
         '''
         Given groups of iterables, return an ``IndexHierarchy`` made of the product of a values in those groups, where the first group is the top-most hierarchy.
 
+        Args:
+            *levels: index initializers (or Index instances) for each level
+            name:
+            index_consructors:
+
         Returns:
             :obj:`static_frame.IndexHierarchy`
 
         '''
         indices = [] # store in a list, where index is depth
-        for lvl in levels:
-            if not isinstance(lvl, Index): # Index, not IndexBase
-                lvl = cls._INDEX_CONSTRUCTOR(lvl)
-            indices.append(lvl)
+        if index_constructors is None:
+            for lvl in levels:
+                if not isinstance(lvl, Index): # Index, not IndexBase
+                    lvl = cls._INDEX_CONSTRUCTOR(lvl)
+                indices.append(lvl)
+        else:
+            if callable(index_constructors): # support a single constrctor
+                pair_iter = zip(levels, repeat(index_constructors))
+            else:
+                pair_iter = zip_longest(levels, index_constructors)
+
+            for lvl, constructor in pair_iter:
+                if constructor is None:
+                    raise ErrorInitIndex(f'Levels and index_constructors must be the same length.')
+                # we call the constructor on all lvl, even if it is already an Index
+                indices.append(constructor(lvl))
 
         if len(indices) == 1:
-            raise RuntimeError('only one level given')
+            raise ErrorInitIndex('Cannot create IndexHierarchy from only one level.')
 
         # build name from index names, assuming they are all specified
         if name is None:
@@ -169,9 +195,10 @@ class IndexHierarchy(IndexBase):
 
     @classmethod
     def from_tree(cls: tp.Type[IH],
-            tree: tp.Any,
+            tree: TreeNodeT,
             *,
-            name: NameType = None
+            name: NameType = None,
+            index_constructors: tp.Optional[IndexConstructors] = None,
             ) -> IH:
         '''
         Convert into a ``IndexHierarchy`` a dictionary defining keys to either iterables or nested dictionaries of the same.
@@ -179,7 +206,13 @@ class IndexHierarchy(IndexBase):
         Returns:
             :obj:`static_frame.IndexHierarchy`
         '''
-        return cls(cls._LEVEL_CONSTRUCTOR.from_tree(tree), name=name)
+        return cls(
+                cls._LEVEL_CONSTRUCTOR.from_tree(
+                        tree,
+                        index_constructors
+                        ),
+                name=name,
+                )
 
     @classmethod
     def from_labels(cls: tp.Type[IH],
@@ -196,7 +229,11 @@ class IndexHierarchy(IndexBase):
 
         Args:
             labels: an iterator or generator of tuples.
+            *,
+            name:
             reorder_for_hierarchy: reorder the labels to produce a hierarchable Index, assuming hierarchability is possible.
+            index_constructors:
+            depth_reference:
             continuation_token: a Hashable that will be used as a token to identify when a value in a label should use the previously encountered value at the same depth.
 
         Returns:
@@ -266,8 +303,8 @@ class IndexHierarchy(IndexBase):
             # each label is an iterable
             for d, v in enumerate(label):
                 if continuation_token is not CONTINUATION_TOKEN_INACTIVE:
-                    if v == continuation_token:
-                        # might check that observed_last[d] != token
+                    if v == continuation_token and observed_last[d] is not token:
+                        # only set v to observed_last if it is not the token; if we have not had a previous value that is not a token, the only thing to do is keep v unchanged
                         v = observed_last[d]
 
                 # shared implementation with from_labels -----------------------
@@ -305,6 +342,7 @@ class IndexHierarchy(IndexBase):
             items: tp.Iterable[tp.Tuple[tp.Hashable, Index]],
             *,
             index_constructor: tp.Optional[IndexConstructor] = None,
+            name: NameType = None,
             ) -> IH:
         '''
         Given an iterable of pairs of label, :obj:`Index`, produce an :obj:`IndexHierarchy` where the labels are depth 0, the indices are depth 1.
@@ -338,8 +376,7 @@ class IndexHierarchy(IndexBase):
                 own_index=True,
                 depth_reference=2, # depth always 2
                 )
-        # import ipdb; ipdb.set_trace()
-        return cls(levels)
+        return cls(levels, name=name)
 
     @classmethod
     def from_labels_delimited(cls: tp.Type[IH],
@@ -378,7 +415,7 @@ class IndexHierarchy(IndexBase):
             try:
                 return tuple(literal_eval(p) for p in parts)
             except ValueError as e:
-                raise ValueError('A label is malformed. This is most likely due to not quoting a string label') from e
+                raise ValueError('A label is malformed. This may be due to not quoting a string label') from e
 
         return cls.from_labels(
                 (to_label(label) for label in labels),
@@ -617,7 +654,8 @@ class IndexHierarchy(IndexBase):
     def _iter_label_items(self,
             depth_level: tp.Optional[DepthLevelSpecifier] = None,
             ) -> tp.Iterator[tp.Tuple[int, tp.Hashable]]:
-
+        '''This function is not directly called in iter_label or related routines, fulfills the expectations of the IterNodeDepthLevel interface.
+        '''
         yield from enumerate(self._iter_label(depth_level=depth_level))
 
 
@@ -677,7 +715,6 @@ class IndexHierarchy(IndexBase):
                 blocks_to_container=blocks_to_container,
                 )
 
-
     @property
     def via_T(self) -> InterfaceTranspose['IndexHierarchy']:
         '''
@@ -687,6 +724,26 @@ class IndexHierarchy(IndexBase):
                 container=self,
                 )
 
+    def via_re(self,
+            pattern: str,
+            flags: int = 0,
+            ) -> InterfaceRe[np.ndarray]:
+        '''
+        Interface for applying regular expressions to elements in this container.
+        '''
+        if self._recache:
+            self._update_array_cache()
+
+        def blocks_to_container(blocks: tp.Iterator[np.ndarray]) -> np.ndarray:
+            return TypeBlocks.from_blocks(blocks).values
+
+        return InterfaceRe(
+                blocks=self._blocks._blocks,
+                blocks_to_container=blocks_to_container,
+                pattern=pattern,
+                flags=flags,
+                )
+
     #---------------------------------------------------------------------------
 
     def _update_array_cache(self) -> None:
@@ -774,14 +831,6 @@ class IndexHierarchy(IndexBase):
             self._update_array_cache()
         return self._blocks.nbytes
 
-    # def __bool__(self) -> bool:
-    #     '''
-    #     True if this container has size.
-    #     '''
-    #     if self._recache:
-    #         return bool(self._levels.__len__()) and bool(self._levels.depth)
-    #     return bool(self._blocks.size)
-
     #---------------------------------------------------------------------------
 
     def __len__(self) -> int:
@@ -792,7 +841,9 @@ class IndexHierarchy(IndexBase):
 
     @doc_inject()
     def display(self,
-            config: tp.Optional[DisplayConfig] = None
+            config: tp.Optional[DisplayConfig] = None,
+            *,
+            style_config: tp.Optional[StyleConfig] = None,
             ) -> Display:
         '''{doc}
 
@@ -827,7 +878,9 @@ class IndexHierarchy(IndexBase):
                         config=config,
                         outermost=True,
                         index_depth=0,
-                        header_depth=header_depth)
+                        header_depth=header_depth,
+                        style_config=style_config,
+                        )
             else:
                 sub_display.extend_iterable(col, header=header_sub)
 
@@ -984,7 +1037,7 @@ class IndexHierarchy(IndexBase):
         Return a Series of Index classes for each index depth.
 
         Returns:
-            :obj:`static_frame.Series`
+            :obj:`Series`
         '''
         from static_frame.core.series import Series
 
@@ -996,7 +1049,7 @@ class IndexHierarchy(IndexBase):
             labels = None
 
         # NOTE: consider caching index_types
-        return Series(self._levels.index_types(), index=labels)
+        return Series(self._levels.index_types(), index=labels, dtype=DTYPE_OBJECT)
 
     #---------------------------------------------------------------------------
     def relabel(self, mapper: RelabelInput) -> 'IndexHierarchy':
@@ -1029,11 +1082,79 @@ class IndexHierarchy(IndexBase):
                 index_constructors=index_constructors,
                 )
 
+
+    def relabel_at_depth(self,
+            mapper: RelabelInput,
+            depth_level: DepthLevelSpecifier = 0
+            ) -> "IndexHierarchy":
+        '''
+        Return a new :obj:`IndexHierarchy` after applying `mapper` to a level or each individual level specified by `depth_level`.
+
+        `mapper` can be a callable, mapping, or iterable.
+            - If a callable, it must accept a single value, and return a single value.
+            - If a mapping, it must map a single value to a single value.
+            - If a iterable, it must be the same length as `self`.
+
+        This call:
+
+        >>> index.relabel_at_depth(mapper, depth_level=[0, 1, 2])
+
+        is equivalent to:
+
+        >>> for level in [0, 1, 2]:
+        >>>     index = index.relabel_at_depth(mapper, depth_level=level)
+
+        albeit more efficient.
+        '''
+        if self._recache:
+            self._update_array_cache()
+
+        if isinstance(depth_level, INT_TYPES):
+            depth_level = [depth_level]
+            target_depths: tp.Container[int] = depth_level
+        else:
+            depth_level = sorted(depth_level)
+            target_depths = set(depth_level)
+
+            if len(target_depths) != len(depth_level):
+                raise ValueError('depth_levels must be unique')
+
+            if not depth_level:
+                raise ValueError('depth_level must be non-empty')
+
+        if any(level < 0 or level >= self.depth for level in depth_level):
+            raise ValueError(f'Invalid depth level found. Valid levels: [0-{self.depth - 1}]')
+
+        if callable(mapper) or hasattr(mapper, 'get'):
+            return self.__class__(
+                    levels=self._levels._relabel_at_depth(mapper, depth_level),
+                    name=self._name,
+                    )
+
+        values, _ = iterable_to_array_1d(mapper, count=len(self))
+
+        if len(values) != len(self):
+            raise ValueError('Iterable must provide a value for each label')
+
+        def gen() -> tp.Iterator[np.ndarray]:
+            for depth_idx in range(self.depth):
+                if depth_idx in target_depths:
+                    yield values
+                else:
+                    yield self._blocks._extract_array_column(depth_idx)
+
+        return self.__class__._from_type_blocks(
+                TypeBlocks.from_blocks(gen()),
+                name=self._name,
+                index_constructors=tuple(self._levels.index_types()),
+                own_blocks=True
+                )
+
     def rehierarch(self: IH,
             depth_map: tp.Sequence[int]
             ) -> IH:
         '''
-        Return a new `IndexHierarchy` that conforms to the new depth assignments given be `depth_map`.
+        Return a new :obj:`IndexHierarchy` that conforms to the new depth assignments given be `depth_map`.
         '''
         if self._recache:
             self._update_array_cache()
@@ -1058,21 +1179,21 @@ class IndexHierarchy(IndexBase):
         '''
         from static_frame.core.series import Series
 
-        if isinstance(key, ILoc):
-            return key.key
+        if key.__class__ is ILoc:
+            return key.key # type: ignore
 
         if isinstance(key, IndexHierarchy):
             # default iteration of IH is as tuple
             return [self._levels.leaf_loc_to_iloc(k) for k in key]
 
-        if isinstance(key, np.ndarray) and key.dtype == DTYPE_BOOL:
+        if (key.__class__ is np.ndarray) and key.dtype == DTYPE_BOOL: # type: ignore
             return self.positions[key]
 
-        if isinstance(key, HLoc):
+        if key.__class__ is HLoc:
             # unpack any Series, Index, or ILoc into the context of this IndexHierarchy
             key = HLoc(tuple(
                     key_from_container_key(self, k, expand_iloc=True)
-                    for k in key))
+                    for k in key)) # type: ignore
         else:
             key = key_from_container_key(self, key)
 
@@ -1153,6 +1274,7 @@ class IndexHierarchy(IndexBase):
             operator: UFunc,
             other: tp.Any,
             axis: int = 0,
+            fill_value: object = np.nan,
             ) -> np.ndarray:
         '''
         Binary operators applied to an index always return an NP array. This deviates from Pandas, where some operations (multiplying an int index by an int) result in a new Index, while other operations result in a np.array (using == on two Index).
@@ -1316,16 +1438,16 @@ class IndexHierarchy(IndexBase):
     @doc_inject(selector='sort')
     def sort(self: IH,
             *,
-            ascending: bool = True,
+            ascending: BoolOrBools = True,
             kind: str = DEFAULT_SORT_KIND,
             key: tp.Optional[tp.Callable[['IndexHierarchy'], tp.Union[np.ndarray, 'IndexHierarchy']]] = None,
             ) -> IH:
         '''Return a new Index with the labels sorted.
 
         Args:
-            ascending: {ascending}
-            kind: {kind}
-            key: {key}
+            {ascendings}
+            {kind}
+            {key}
         '''
         if self._recache:
             self._update_array_cache()
@@ -1382,7 +1504,7 @@ class IndexHierarchy(IndexBase):
 
     @doc_inject(selector='fillna')
     def fillna(self, value: tp.Any) -> 'IndexHierarchy':
-        '''Return an :obj:`IndexHierarchy` after replacing null (NaN or None) with the supplied value.
+        '''Return an :obj:`IndexHierarchy` after replacing NA (NaN or None) with the supplied value.
 
         Args:
             {value}
@@ -1390,7 +1512,7 @@ class IndexHierarchy(IndexBase):
         if self._recache:
             self._update_array_cache()
 
-        blocks = self._blocks.fillna(value, None)
+        blocks = self._blocks.fill_missing_by_unit(value, None, func=isna_array)
         index_constructors = tuple(self._levels.index_types())
 
         return self.__class__._from_type_blocks(blocks,
@@ -1545,12 +1667,21 @@ class IndexHierarchy(IndexBase):
         mi.names = self.names
         return mi
 
+    def to_tree(self) -> TreeNodeT:
+        '''Returns the tree representation of an IndexHierarchy
+        '''
+        return self._levels.traverse() # type: ignore
+
     def flat(self) -> IndexBase:
         '''Return a flat, one-dimensional index of tuples for each level.
         '''
-        return self._INDEX_CONSTRUCTOR(self.__iter__())
+        return self._INDEX_CONSTRUCTOR(self.__iter__(), name=self._name)
 
-    def level_add(self: IH, level: tp.Hashable) -> IH:
+    def level_add(self: IH,
+            level: tp.Hashable,
+            *,
+            index_constructor: IndexConstructor = None,
+            ) -> IH:
         '''Return an IndexHierarchy with a new root (outer) level added.
         '''
         if self.STATIC: # can reuse levels
@@ -1558,8 +1689,10 @@ class IndexHierarchy(IndexBase):
         else:
             levels_src = self._levels.to_index_level()
 
+        index_cls = self._INDEX_CONSTRUCTOR if index_constructor is None else index_constructor
+
         levels = self._LEVEL_CONSTRUCTOR(
-                index=self._INDEX_CONSTRUCTOR((level,)),
+                index=index_cls((level,)),
                 targets=ArrayGO([levels_src], own_iterable=True),
                 offset=0,
                 own_index=True
@@ -1576,12 +1709,28 @@ class IndexHierarchy(IndexBase):
 
         return self.__class__(levels, name=self._name)
 
-    def level_drop(self, count: int = 1) -> tp.Union[Index, 'IndexHierarchy']:
+    def level_drop(self,
+            count: int = 1,
+            ) -> tp.Union[Index, 'IndexHierarchy']:
         '''Return an IndexHierarchy with one or more leaf levels removed. This might change the size of the resulting index if the resulting levels are not unique.
 
         Args:
-            count: A positive value is the number of depths to remove from the root (outer) side of the hierarchy; a negative values is the number of depths to remove from the leaf (inner) side of the hierarchy.
+            count: A positive value is the number of depths to remove from the root (outer) side of the hierarchy; a negative value is the number of depths to remove from the leaf (inner) side of the hierarchy.
         '''
+        if self._recache:
+            self._update_array_cache()
+
+        # NOTE: this was implement with a bipolar ``count`` to specify what to drop, but it could have been implemented with a depth level specifier, supporting arbitrary removals. The approach taken here is likely faster as we reuse levels.
+        if self._name_is_names():
+            if count < 0:
+                name = self._name[:count] #type: ignore
+            elif count > 0:
+                name = self._name[count:] #type: ignore
+            if len(name) == 1:
+                name = name[0]
+        else:
+            name = self._name
+
         if count < 0: # remove from inner
             levels = self._levels.to_index_level()
             for _ in range(abs(count)):
@@ -1595,18 +1744,19 @@ class IndexHierarchy(IndexBase):
                         levels_stack.extend(level.targets) #type: ignore
                 if levels.targets is None:  # if no targets, at the root
                     break
+
             if levels.targets is None: # fall back to 1D index
-                return levels.index
+                return levels.index.rename(name)
 
             # if we have TypeBlocks and levels is the same length
-            if not self._recache and levels.__len__() == self.__len__():
+            if levels.__len__() == self.__len__():
                 blocks = self._blocks.iloc[NULL_SLICE, :count]
                 return self.__class__(levels,
-                        name=self._name,
                         blocks=blocks,
-                        own_blocks=True
+                        own_blocks=True,
+                        name=name,
                         )
-            return self.__class__(levels, name=self._name)
+            return self.__class__(levels, name=name)
 
         elif count > 0: # remove from outer
             levels = self._levels.to_index_level()
@@ -1619,20 +1769,19 @@ class IndexHierarchy(IndexBase):
                         targets.extend(target.targets)
                 index = levels.index.__class__(labels)
                 if not targets:
-                    return index
+                    return index.rename(name)
                 levels = levels.__class__(
                         index=index,
                         targets=ArrayGO(targets, own_iterable=True))
 
-            # if we have TypeBlocks and levels is the same length
-            if not self._recache and levels.__len__() == self.__len__():
-                blocks = self._blocks.iloc[NULL_SLICE, count:]
-                return self.__class__(levels,
-                        name=self._name,
-                        blocks=blocks,
-                        own_blocks=True
-                        )
-            return self.__class__(levels, name=self._name)
+            # lengths must be equal if innner index components are unique; otherwise exception will have been raised
+            # assert levels.__len__() == self.__len__()
+            blocks = self._blocks.iloc[NULL_SLICE, count:]
+            return self.__class__(levels,
+                    name=name,
+                    blocks=blocks,
+                    own_blocks=True,
+                    )
 
         raise NotImplementedError('no handling for a 0 count drop level.')
 
@@ -1703,7 +1852,7 @@ class IndexHierarchyAsType:
 
     def __call__(self, dtype: DtypeSpecifier) -> IndexHierarchy:
 
-        from static_frame.core.index_datetime import _dtype_to_index_cls
+        from static_frame.core.index_datetime import dtype_to_index_cls
         container = self.container
 
         if container._recache:
@@ -1719,12 +1868,12 @@ class IndexHierarchyAsType:
 
         dtype_post = blocks.dtypes[self.key] # can select element or array
         if isinstance(dtype_post, np.dtype):
-            index_constructors[self.key] = _dtype_to_index_cls(
+            index_constructors[self.key] = dtype_to_index_cls(
                     container.STATIC,
                     dtype_post)
         else: # assign iterable
             index_constructors[self.key] = [
-                    _dtype_to_index_cls(container.STATIC, dt)
+                    dtype_to_index_cls(container.STATIC, dt)
                     for dt in dtype_post]
 
         return container.__class__._from_type_blocks(
