@@ -1,5 +1,3 @@
-
-
 import typing as tp
 from collections import deque
 from itertools import zip_longest
@@ -7,7 +5,7 @@ from itertools import repeat
 from copy import deepcopy
 
 import numpy as np
-
+from arraykit import resolve_dtype_iter
 
 from static_frame.core.array_go import ArrayGO
 from static_frame.core.container_util import index_from_optional_constructor
@@ -17,21 +15,22 @@ from static_frame.core.hloc import HLoc
 from static_frame.core.index import ILoc
 from static_frame.core.index import Index
 from static_frame.core.index import IndexGO
-from static_frame.core.index import LocMap
 from static_frame.core.index import mutable_immutable_index_filter
+from static_frame.core.index_auto import RelabelInput
 from static_frame.core.index_base import IndexBase
 from static_frame.core.type_blocks import TypeBlocks
+from static_frame.core.loc_map import LocMap
 from static_frame.core.util import GetItemKeyType
 from static_frame.core.util import GetItemKeyTypeCompound
 from static_frame.core.util import IndexConstructor
 from static_frame.core.util import IndexConstructors
 from static_frame.core.util import IndexInitializer
-from static_frame.core.util import INT_TYPES
+from static_frame.core.util import DTYPE_OBJECT
 from static_frame.core.util import DTYPE_BOOL
+from static_frame.core.util import EMPTY_TUPLE
+from static_frame.core.util import INT_TYPES
 from static_frame.core.util import KEY_ITERABLE_TYPES
 from static_frame.core.util import KEY_MULTIPLE_TYPES
-from static_frame.core.util import resolve_dtype_iter
-from static_frame.core.util import EMPTY_TUPLE
 # from static_frame.core.exception import LocInvalid
 
 
@@ -43,6 +42,9 @@ INDEX_LEVEL_SLOTS = (
             '_length',
             )
 
+TreeNodeT = tp.Dict[tp.Any, tp.Union[tp.Union[Index, tp.Tuple[tp.Any]], "TreeNodeT"]]
+
+
 class IndexLevel:
     '''
     A nestable representation of an Index, where labels in that index optionally point to other Index objects.
@@ -97,22 +99,22 @@ class IndexLevel:
 
     @classmethod
     def from_tree(cls,
-            tree: tp.Any, # recursively defined
+            tree: TreeNodeT,
             index_constructors: tp.Optional[IndexConstructors] = None,
             depth_reference: tp.Optional[int] = None,
             ) -> 'IndexLevel':
         '''
         Convert a tree structure to an IndexLevel instance. As a tree structure is a dictionary of keys to either a sequence of hashables or a dict of other keys, there is no way to represent a zero length, non-zero depth structure.
         '''
-        # tree: tp.Dict[tp.Hashable, tp.Union[Sequence[tp.Hashable], tp.Dict]]
-
         def get_index(labels: IndexInitializer, depth: int) -> Index:
             explicit_constructor: tp.Optional[IndexConstructor]
 
-            if index_constructors is not None:
-                explicit_constructor = index_constructors[depth]
-            else:
+            if index_constructors is None:
                 explicit_constructor = None
+            elif callable(index_constructors):
+                explicit_constructor = index_constructors # it is a single callable
+            else: # it is a sequence
+                explicit_constructor = index_constructors[depth]
 
             return index_from_optional_constructor(labels, #type: ignore
                     default_constructor=cls._INDEX_CONSTRUCTOR,
@@ -173,6 +175,82 @@ class IndexLevel:
             self._length = 0
 
     #---------------------------------------------------------------------------
+    def traverse(self) -> tp.Union[Index, TreeNodeT]:
+        '''Traverses the IndexLevel tree via preorder traversal
+        '''
+        if self.targets is None:
+            return self.index
+
+        tree: TreeNodeT = {}
+
+        sentinel: tp.Any = object()
+        for target, label in zip_longest(self.targets, self.index, fillvalue=sentinel):
+            assert target is not sentinel and label is not sentinel, f"Malformed IndexLevel {self}"
+            tree[label] = target.traverse()
+
+        return tree
+
+    @staticmethod
+    def _pad(s: str) -> str:
+        msg = f"{' ' * 4}"
+        return msg + f"\n{msg}".join(s.split("\n"))
+
+    @classmethod
+    def _format(cls, body: str, name: str = "") -> str:
+        if name:
+            name = ": " + name
+
+        if not body.strip():
+            return f"{cls.__name__}{name} {{ ... }},"
+
+        return f"{cls.__name__}{name} {{\n{body}\n}},"
+
+    def _repr(self, *, name: str, depth: int) -> str:
+        """
+        Since this is recursive, we can quickly (and easily) attempt to display
+        massive trees. To prevent this, we truncate the two following cases:
+
+        1. Levels that are 5+ levels deep
+        2. Only display the first & last 2 levels of any given level
+        """
+        if depth >= 4:
+            return ""
+
+        if self.targets is None:
+            # NOTE: str(self.index) does not permit passing a DisplayConfig
+            return self._format(self._pad(str(self.index)), name=name)
+
+        if len(self.index) <= 3:
+            def gen() -> tp.Iterator[tp.Tuple[tp.Hashable, "IndexLevel"]]:
+                yield from zip(self.index, self.targets)
+
+        else:
+
+            forward_index_iter = iter(self.index.values)
+            reverse_index_iter = reversed(self.index)
+            forward_targets_iter = iter(self.targets)
+            reverse_targets_iter = reversed(self.targets) # type: ignore
+
+            def gen() -> tp.Iterator[tp.Tuple[tp.Hashable, "IndexLevel"]]:
+                yield next(forward_index_iter), next(forward_targets_iter)
+                yield next(forward_index_iter), next(forward_targets_iter)
+
+                last = next(reverse_index_iter), next(reverse_targets_iter)
+                yield next(reverse_index_iter), next(reverse_targets_iter)
+                yield last
+
+        results: tp.List[str] = []
+        for label, target in gen():
+            results.append(target._repr(name=repr(label), depth=depth + 1))
+
+        if len(results) == 4 and any(results):
+            results = results[:2] + ["{ ... },"] + results[-2:]
+
+        return self._format(self._pad("\n".join(results)), name=name)
+
+    def __repr__(self) -> str:
+        return self._repr(name="", depth=0).rstrip(',')
+
     def __deepcopy__(self, memo: tp.Dict[int, tp.Any]) -> 'IndexLevel':
         obj = self.__new__(self.__class__)
         obj.index = deepcopy(self.index, memo)
@@ -184,7 +262,6 @@ class IndexLevel:
         memo[id(self)] = obj
         return obj #type: ignore
 
-
     #---------------------------------------------------------------------------
     def depths(self) -> tp.Iterator[int]:
         '''
@@ -208,11 +285,7 @@ class IndexLevel:
         '''
         Called once over the life of an instance to set self._depth; this is not re-evaluated over the life of the instance.
         '''
-        if not len(self.index):
-            raise AssertionError('zero-length indices should have depth set through depth_reference')
-
-        if self.targets is None: # this may not need to be here
-            return 1
+        assert len(self.index), 'zero-length indices should have depth set through depth_reference'
 
         # if we need to recurse to the max depth
         level, depth = self, 1
@@ -272,11 +345,9 @@ class IndexLevel:
                         zip_longest(index, targets[1:], fillvalue=None)
                         ):
                     if level_next is not None:
-                        # if the next offset is zero, we are moving to a component that is under a fresh hierarchy
-                        if level_next.offset > 0:
-                            delta = level_next.offset - transversed
-                        else:
-                            delta = len(targets[i])
+                        # offset will always be > 0, as we skip the first
+                        assert level_next.offset > 0
+                        delta = level_next.offset - transversed
                         yield label, delta
                         # get only the incremental addition for this label
                         transversed += delta
@@ -311,10 +382,9 @@ class IndexLevel:
                         zip_longest(index, targets[1:], fillvalue=None)
                         ):
                     if level_next is not None:
-                        if level_next.offset > 0:
-                            delta = level_next.offset - transversed
-                        else:
-                            delta = len(targets[i])
+                        # offset will always be > 0, as we skip the first
+                        assert level_next.offset > 0
+                        delta = level_next.offset - transversed
                         yield from repeat(label, delta)
                         transversed += delta
                     else:
@@ -330,6 +400,70 @@ class IndexLevel:
                 next_depth = depth + 1
                 levels.extend((lvl, next_depth) for lvl in level.targets)
 
+    def _relabel_at_single_depth(self,
+            mapper: RelabelInput, depth_level: int
+            ) -> 'IndexLevel':
+        '''
+        Returns a new IndexLevel instance with relabeled indices at the specified `depth_level` according to `mapper`.
+
+        It will recursive through the internal index levels to apply a `relabel` to all the appropriate indices.
+        '''
+        if depth_level == 0:
+            return self.__class__(
+                index=self.index.relabel(mapper),
+                targets=self.targets,
+                offset=self.offset,
+                own_index=True,
+                depth_reference=self._depth
+            )
+
+        # This will only happen if `relabel_at_depth` was not given a sorted
+        # `depth_level` argument.
+        assert self.targets is not None, f"Invalid depth_level={depth_level}"
+
+        new_targets = np.empty(len(self.targets), dtype=DTYPE_OBJECT)
+        for i, target in enumerate(self.targets):
+            new_targets[i] =  target._relabel_at_single_depth(mapper, depth_level - 1)
+
+        return self.__class__(
+            index=self.index,
+            targets=ArrayGO(new_targets, own_iterable=True),
+            offset=self.offset,
+            own_index=False,
+            depth_reference=self._depth
+        )
+
+    def _relabel_at_depth(self, mapper: RelabelInput, depth_level: tp.List[int]) -> 'IndexLevel':
+        '''
+        Returns a new IndexLevel instance with relabeled indices at the specified `depth_level`s according to `mapper`, where `mapper` is a callable or a mapping.
+
+        Note: There is a strong expectation that depth_level is sorted.
+        '''
+        assert depth_level, "Invalid depth_levels should have already been sanitized"
+
+        if len(depth_level) == 1:
+            return self._relabel_at_single_depth(mapper, depth_level[0])
+
+        if depth_level[0] == 0:
+            index = self.index.relabel(mapper)
+            target_depths = [level - 1 for level in depth_level[1:]]
+            own_index = True
+        else:
+            index = self.index
+            target_depths = [level - 1 for level in depth_level]
+            own_index = False
+
+        new_targets = np.empty(len(self.targets), dtype=DTYPE_OBJECT)
+        for i, target in enumerate(self.targets):
+            new_targets[i] = target._relabel_at_depth(mapper, target_depths)
+
+        return self.__class__(
+            index=index,
+            targets=ArrayGO(new_targets, own_iterable=True),
+            offset=self.offset,
+            own_index=own_index,
+            depth_reference=self._depth
+        )
 
     def index_array_at_depth(self,
             depth_level: int = 0
@@ -482,16 +616,16 @@ class IndexLevel:
         '''
         from static_frame.core.series import Series
 
-        if isinstance(key, slice):
+        if key.__class__ is slice:
             # given a top-level definition of a slice (and if that slice results in a single value), we can get a value range
             return slice(*LocMap.map_slice_args(self.leaf_loc_to_iloc, key))
 
         if isinstance(key, KEY_ITERABLE_TYPES): # iterables of leaf-locs
-            if isinstance(key, np.ndarray) and key.dtype == bool:
+            if key.__class__ is np.ndarray and key.dtype == bool: #type: ignore
                 return key # keep as Boolean
             return [self.leaf_loc_to_iloc(x) for x in key]
 
-        if not isinstance(key, HLoc): # assume a leaf loc tuple
+        if not key.__class__ is HLoc: # assume a leaf loc tuple
             if not isinstance(key, tuple):
                 raise KeyError(f'{key} cannot be used for loc selection from IndexHierarchy; try HLoc')
             return self.leaf_loc_to_iloc(key)
@@ -503,13 +637,13 @@ class IndexLevel:
         while levels:
             level, depth, offset = levels.popleft()
 
-            depth_key = key[depth]
+            depth_key = key[depth] # type: ignore
             # NOTE: depth_key should not be Series or Index at this point; IndexHierarchy is responsible for unpacking / reindexing prior to this call
             next_offset = offset + level.offset
 
-            if isinstance(depth_key, np.ndarray) and depth_key.dtype == DTYPE_BOOL:
+            if depth_key.__class__ is np.ndarray and depth_key.dtype == DTYPE_BOOL: #type: ignore
                 # NOTE: use length of level, not length of index, as need to observe all leafs covered at this node.
-                depth_key = depth_key[next_offset: next_offset + len(level)]
+                depth_key = depth_key[next_offset: next_offset + len(level)] #type: ignore
                 if len(depth_key) > len(level.index):
                     # given leaf-Boolean, determine what upper nodes to select
                     depth_key = level.values_at_depth(0)[depth_key]
@@ -547,15 +681,15 @@ class IndexLevel:
         if iloc_count == 0:
             raise KeyError('no matching keys across all levels')
 
-        if iloc_count == 1 and not key.has_key_multiple():
+        if iloc_count == 1 and not key.has_key_multiple(): # type: ignore
             return ilocs[0] # drop to a single iloc selection
 
         # NOTE: might be able to combine contiguous ilocs into a single slice
         iloc_flat: tp.List[GetItemKeyType] = [] # combine into one flat iloc
         length = self.__len__()
         for part in ilocs:
-            if isinstance(part, slice):
-                iloc_flat.extend(range(*part.indices(length)))
+            if part.__class__ is slice:
+                iloc_flat.extend(range(*part.indices(length))) # type: ignore
             elif isinstance(part, INT_TYPES):
                 iloc_flat.append(part)
             else: # assume it is an iterable
@@ -742,11 +876,9 @@ class IndexLevel:
             if level_self.targets is not None and level_other.targets is not None: # not terminus
                 levels_self.extend(level_self.targets)
                 levels_other.extend(level_other.targets)
-            if level_self.targets is None and level_other.targets is None: # terminus
+            elif level_self.targets is None and level_other.targets is None: # terminus
                 continue
-            if level_self.targets is None or level_other.targets is None:
-                # at least one is at a terminus
-                return False
+            # if targets is None for one and not the other, the depths are not equal
 
         if not levels_self and not levels_other:
             return True # both exhausted
@@ -783,15 +915,12 @@ class IndexLevel:
                 depth_reference=self.depth,
                 )
 
-
     def to_type_blocks(self) -> TypeBlocks:
         '''
         Provide a correctly typed TypeBlocks representation.
         '''
-        try:
-            depth_count = self.depth
-        except StopIteration:
-            # assume we have no depth or length
+        depth_count = self.depth
+        if depth_count == 0:
             return TypeBlocks.from_zero_size_shape()
 
         return TypeBlocks.from_blocks(
@@ -819,11 +948,9 @@ class IndexLevelGO(IndexLevel):
     def extend(self, level: IndexLevel) -> None:
         '''Extend this IndexLevel with another IndexLevel, assuming that it has compatible depth and Index types.
         '''
-        depth = self.depth
-
         if level.targets is None:
             raise RuntimeError('found IndexLevel with None as targets')
-        if depth != level.depth:
+        if self.depth != level.depth:
             raise RuntimeError('level for extension does not have necessary levels.')
 
         for type_self, type_other in zip(self.index_types(), level.index_types()):
