@@ -31,16 +31,19 @@ from static_frame.core.node_selector import InterfaceGetItem
 from static_frame.core.node_selector import InterfaceSelectDuo
 from static_frame.core.node_selector import InterfaceSelectTrio
 from static_frame.core.node_selector import TContainer
+from static_frame.core.node_re import InterfaceRe
 from static_frame.core.node_str import InterfaceString
 from static_frame.core.node_transpose import InterfaceTranspose
+from static_frame.core.node_fill_value import InterfaceFillValue
 from static_frame.core.store import StoreConfig
 from static_frame.core.store_filter import StoreFilter
 from static_frame.core.type_blocks import TypeBlocks
 from static_frame.core.util import AnyCallable
 from static_frame.core.util import DT64_S
 from static_frame.core.quilt import Quilt
-
-
+from static_frame.core.yarn import Yarn
+from static_frame.core.archive_npy import NPY
+from static_frame.core.archive_npy import NPZ
 
 #-------------------------------------------------------------------------------
 
@@ -90,7 +93,7 @@ UFUNC_AXIS_SKIPNA: tp.Dict[str, UfuncSkipnaAttrs] = {
         # 'all': UfuncSkipnaAttrs(ufunc_all, ufunc_nanall),
         # 'any': UfuncSkipnaAttrs(ufunc_any, ufunc_nanany),
         'sum': UfuncSkipnaAttrs(np.sum, np.nansum),
-        'min': UfuncSkipnaAttrs( np.min, np.nanmin),
+        'min': UfuncSkipnaAttrs(np.min, np.nanmin),
         'max': UfuncSkipnaAttrs(np.max, np.nanmax),
         'mean': UfuncSkipnaAttrs(np.mean, np.nanmean),
         'median': UfuncSkipnaAttrs(np.median, np.nanmedian),
@@ -172,8 +175,14 @@ def _get_signatures(
         delegate_func: tp.Optional[AnyCallable] = None,
         delegate_name: str = '',
         max_args: int = MAX_ARGS,
+        name_no_args: tp.Optional[str] = None,
         ) -> tp.Tuple[str, str]:
+    '''
+    Utility to get two versions of ``func`` and ``delegate_func`` signatures
 
+    Args:
+        name_no_args: If this signature has a ``delegate_func``, the root name might need to be provided in a version with no arguments (if the root itself is a function).
+    '''
     if delegate_func:
         delegate = _get_parameters(delegate_func, max_args=max_args)
         if delegate_name and delegate_name != '__call__':
@@ -188,10 +197,11 @@ def _get_signatures(
 
     signature = f'{name}{_get_parameters(func, is_getitem, max_args=max_args)}{delegate}'
 
+    name_no_args = name if not name_no_args else name_no_args
     if is_getitem:
-        signature_no_args = f'{name}[]{delegate_no_args}'
+        signature_no_args = f'{name_no_args}[]{delegate_no_args}'
     else:
-        signature_no_args = f'{name}(){delegate_no_args}'
+        signature_no_args = f'{name_no_args}(){delegate_no_args}'
 
     return signature, signature_no_args
 
@@ -211,6 +221,7 @@ class Features:
         '__class_getitem__',
         '__annotations__',
         '__doc__',
+        '__del__',
         '__delattr__',
         '__dir__',
         '__dict__',
@@ -286,9 +297,12 @@ class InterfaceGroup:
     AccessorDatetime = 'Accessor Datetime'
     AccessorString = 'Accessor String'
     AccessorTranspose = 'Accessor Transpose'
+    AccessorFillValue = 'Accessor Fill Value'
+    AccessorRe = 'Accessor Regular Expression'
 
 # NOTE: order from definition retained
-INTERFACE_GROUP_ORDER = tuple(v for k, v in vars(InterfaceGroup).items() if not k.startswith('_'))
+INTERFACE_GROUP_ORDER = tuple(v for k, v in vars(InterfaceGroup).items()
+        if not k.startswith('_'))
 
 class InterfaceRecord(tp.NamedTuple):
 
@@ -532,15 +546,32 @@ class InterfaceRecord(tp.NamedTuple):
             group = InterfaceGroup.AccessorDatetime
         elif cls_interface is InterfaceTranspose:
             group = InterfaceGroup.AccessorTranspose
+        elif cls_interface is InterfaceFillValue:
+            group = InterfaceGroup.AccessorFillValue
+        elif cls_interface is InterfaceRe:
+            group = InterfaceGroup.AccessorRe
         else:
-            raise NotImplementedError()
+            raise NotImplementedError() #pragma: no cover
+
+        terminus_name_no_args: tp.Optional[str]
 
         for field in cls_interface.INTERFACE: # apply, map, etc
             delegate_obj = getattr(cls_interface, field)
             delegate_reference = f'{cls_interface.__name__}.{field}'
             doc = Features.scrub_doc(delegate_obj.__doc__)
 
-            terminus_name = f'{name}.{field}'
+            if cls_interface in (InterfaceFillValue, InterfaceRe):
+                terminus_sig, terminus_sig_no_args = _get_signatures(
+                        name,
+                        obj,
+                        is_getitem=False,
+                        max_args=max_args,
+                        )
+                terminus_name = f'{terminus_sig}.{field}'
+                terminus_name_no_args = f'{terminus_sig_no_args}.{field}'
+            else:
+                terminus_name = f'{name}.{field}'
+                terminus_name_no_args = None
 
             if isinstance(delegate_obj, property):
                 # some date tools are properties
@@ -560,7 +591,10 @@ class InterfaceRecord(tp.NamedTuple):
                         terminus_name,
                         delegate_obj,
                         max_args=max_args,
+                        name_no_args=terminus_name_no_args,
                         )
+                # if group == InterfaceGroup.AccessorRe:
+                #     print(signature, signature_no_args)
                 yield cls(cls_name,
                         group,
                         signature,
@@ -784,12 +818,20 @@ class InterfaceSummary(Features):
             elif target is Bus:
                 f = Frame.from_elements((0,), name='frame')
                 instance = target.from_frames((f,)) #type: ignore
+            elif target is Yarn:
+                f = Frame.from_elements((0,), name='frame')
+                instance = Yarn.from_buses(
+                    (Bus.from_frames((f,), name='bus'),),
+                    retain_labels=False,
+                    )
             elif target is Quilt:
                 f = Frame.from_elements((0,), name='frame')
                 bus = Bus.from_frames((f,))
                 instance = target(bus, retain_labels=False) #type: ignore
             elif target is Batch:
                 instance = Batch(iter(()))
+            elif target is NPY or target is NPZ: #type: ignore
+                instance = target
             elif issubclass(target, IndexHierarchy):
                 instance = target.from_labels(((0,0),))
             elif issubclass(target, (IndexYearMonth, IndexYear, IndexDate)):
@@ -814,7 +856,7 @@ class InterfaceSummary(Features):
                 # getting interface off of the class will recurse
                 yield name_attr, None, ContainerBase.__class__.interface #type: ignore
 
-        # force tehse to be ordered at the bottom
+        # force these to be ordered at the bottom
         selectors = ('__getitem__', 'iloc', 'loc')
         selectors_found = set()
 
@@ -841,7 +883,6 @@ class InterfaceSummary(Features):
             *,
             max_args: int = MAX_ARGS
             ) -> tp.Iterator[InterfaceRecord]:
-
         for name_attr, obj, obj_cls in cls.name_obj_iter(target):
             doc = ''
 
@@ -881,29 +922,33 @@ class InterfaceSummary(Features):
                 yield from InterfaceRecord.gen_from_iterator(**kwargs)
             elif isinstance(obj, InterfaceGetItem) or name == cls.GETITEM:
                 yield from InterfaceRecord.from_getitem(**kwargs)
-            elif isinstance(obj, InterfaceString):
+
+            elif obj.__class__ in (InterfaceString, InterfaceDatetime, InterfaceTranspose):
                 yield from InterfaceRecord.gen_from_accessor(
-                            cls_interface=InterfaceString,
-                            **kwargs,
-                            )
-            elif isinstance(obj, InterfaceDatetime):
-                yield from InterfaceRecord.gen_from_accessor(
-                            cls_interface=InterfaceDatetime,
-                            **kwargs,
-                            )
-            elif isinstance(obj, InterfaceTranspose):
-                yield from InterfaceRecord.gen_from_accessor(
-                            cls_interface=InterfaceTranspose,
-                            **kwargs,
-                            )
+                        cls_interface=obj.__class__,
+                        **kwargs,
+                        )
             elif obj.__class__ in (InterfaceSelectDuo, InterfaceSelectTrio):
                 yield from InterfaceRecord.gen_from_selection(
                         cls_interface=obj.__class__,
-                        **kwargs)
+                        **kwargs,
+                        )
             elif obj.__class__ in (InterfaceAssignTrio, InterfaceAssignQuartet):
                 yield from InterfaceRecord.gen_from_assignment(
                         cls_interface=obj.__class__,
-                        **kwargs)
+                        **kwargs,
+                        )
+            # InterfaceFillValue, InterfaceRe are methods, must match name
+            elif name == 'via_fill_value':
+                yield from InterfaceRecord.gen_from_accessor(
+                        cls_interface=InterfaceFillValue,
+                        **kwargs,
+                        )
+            elif name == 'via_re':
+                yield from InterfaceRecord.gen_from_accessor(
+                        cls_interface=InterfaceRe,
+                        **kwargs,
+                        )
             elif callable(obj): # general methods
                 yield from InterfaceRecord.gen_from_method(**kwargs)
             else: # attributes
