@@ -7,8 +7,11 @@ from enum import Enum
 from functools import partial
 from concurrent.futures import ProcessPoolExecutor
 from concurrent.futures import ThreadPoolExecutor
+# import multiprocessing as mp
+# mp_context = mp.get_context('spawn')
 
 import numpy as np
+from arraykit import name_filter
 
 from static_frame.core.doc_str import doc_inject
 from static_frame.core.util import AnyCallable
@@ -19,32 +22,44 @@ from static_frame.core.util import Mapping
 from static_frame.core.util import NameType
 from static_frame.core.util import TupleConstructorType
 from static_frame.core.util import iterable_to_array_1d
+from static_frame.core.util import IndexConstructor
+# from static_frame.core.util import array_from_iterator
 
 
 if tp.TYPE_CHECKING:
     from static_frame.core.frame import Frame # pylint: disable=W0611 #pragma: no cover
     from static_frame.core.series import Series # pylint: disable=W0611 #pragma: no cover
-    from static_frame.core.index import Index # pylint: disable=W0611 #pragma: no cover
     from static_frame.core.quilt import Quilt # pylint: disable=W0611 #pragma: no cover
     from static_frame.core.bus import Bus # pylint: disable=W0611 #pragma: no cover
+    from static_frame.core.yarn import Yarn # pylint: disable=W0611 #pragma: no cover
 
 
-FrameOrSeries = tp.TypeVar('FrameOrSeries', 'Frame', 'Series', 'Bus', 'Quilt')
+FrameOrSeries = tp.TypeVar('FrameOrSeries', 'Frame', 'Series', 'Bus', 'Quilt', 'Yarn')
+PoolArgGen = tp.Callable[[], tp.Union[tp.Iterator[tp.Any], tp.Iterator[tp.Tuple[tp.Any, tp.Any]]]]
 # FrameSeriesIndex = tp.TypeVar('FrameSeriesIndex', 'Frame', 'Series', 'Index')
 
 
-class IterNodeApplyType(Enum):
-    SERIES_ITEMS = 1
-    SERIES_ITEMS_FLAT = 2 # do not use index class on container
-    FRAME_ELEMENTS = 3
-    INDEX_LABELS = 4
-
-
 class IterNodeType(Enum):
     VALUES = 1
     ITEMS = 2
 
 
+class IterNodeApplyType(Enum):
+    SERIES_VALUES = 0
+    SERIES_ITEMS = 1 # only used for iter_window_*
+    SERIES_ITEMS_GROUP_VALUES = 2
+    SERIES_ITEMS_GROUP_LABELS = 3
+    FRAME_ELEMENTS = 4
+    INDEX_LABELS = 5
+
+    @classmethod
+    def is_items(cls, apply_type: 'IterNodeApplyType') -> bool:
+        if apply_type is cls.SERIES_VALUES or apply_type is cls.INDEX_LABELS:
+            return False
+        return True
+
+
+
 class IterNodeDelegate(tp.Generic[FrameOrSeries]):
     '''
     Delegate returned from :obj:`static_frame.IterNode`, providing iteration as well as a family of apply methods.
@@ -54,7 +69,8 @@ class IterNodeDelegate(tp.Generic[FrameOrSeries]):
             '_func_values',
             '_func_items',
             '_yield_type',
-            '_apply_constructor'
+            '_apply_constructor',
+            '_apply_type',
             )
 
     INTERFACE = (
@@ -77,7 +93,8 @@ class IterNodeDelegate(tp.Generic[FrameOrSeries]):
             func_values: tp.Callable[..., tp.Iterable[tp.Any]],
             func_items: tp.Callable[..., tp.Iterable[tp.Tuple[tp.Any, tp.Any]]],
             yield_type: IterNodeType,
-            apply_constructor: tp.Callable[..., FrameOrSeries]
+            apply_constructor: tp.Callable[..., FrameOrSeries],
+            apply_type: IterNodeApplyType,
         ) -> None:
         '''
         Args:
@@ -87,6 +104,7 @@ class IterNodeDelegate(tp.Generic[FrameOrSeries]):
         self._func_items = func_items
         self._yield_type = yield_type
         self._apply_constructor: tp.Callable[..., FrameOrSeries] = apply_constructor
+        self._apply_type = apply_type
 
     #---------------------------------------------------------------------------
 
@@ -99,17 +117,14 @@ class IterNodeDelegate(tp.Generic[FrameOrSeries]):
 
         pool_executor = ThreadPoolExecutor if use_threads else ProcessPoolExecutor
 
-        yt_is_values = self._yield_type is IterNodeType.VALUES
-
-        if not callable(func):
+        if not callable(func): # support array, Series mapping
             func = getattr(func, '__getitem__')
 
         # use side effect list population to create keys when iterating over values
         func_keys = []
+        arg_gen: PoolArgGen
 
-        arg_gen: tp.Callable[[], tp.Union[tp.Iterator[tp.Any], tp.Iterator[tp.Tuple[tp.Any, tp.Any]]]]
-
-        if yt_is_values:
+        if self._yield_type is IterNodeType.VALUES:
             def arg_gen() -> tp.Iterator[tp.Any]: #pylint: disable=E0102
                 for k, v in self._func_items():
                     func_keys.append(k)
@@ -125,6 +140,24 @@ class IterNodeDelegate(tp.Generic[FrameOrSeries]):
                     executor.map(func, arg_gen(), chunksize=chunksize)
                     )
 
+    def _apply_iter_parallel(self,
+            func: AnyCallable,
+            max_workers: tp.Optional[int] = None,
+            chunksize: int = 1,
+            use_threads: bool = False,
+            ) -> tp.Iterator[tp.Any]:
+
+        pool_executor = ThreadPoolExecutor if use_threads else ProcessPoolExecutor
+        if not callable(func): # support array, Series mapping
+            func = getattr(func, '__getitem__')
+
+        # use side effect list population to create keys when iterating over values
+        arg_gen = (self._func_values if self._yield_type is IterNodeType.VALUES
+                else self._func_items)
+
+        with pool_executor(max_workers=max_workers) as executor:
+            yield from executor.map(func, arg_gen(), chunksize=chunksize)
+
     #---------------------------------------------------------------------------
     # public interface
 
@@ -154,14 +187,19 @@ class IterNodeDelegate(tp.Generic[FrameOrSeries]):
         Args:
             {mapping}
         '''
-        yield from (v for _, v in self.map_any_iter_items(mapping))
+        get = getattr(mapping, 'get')
+        if self._yield_type is IterNodeType.VALUES:
+            yield from (get(v, v) for v in self._func_values())
+        else:
+            yield from (get((k,  v), v) for k, v in self._func_items())
 
     @doc_inject(selector='map_any')
     def map_any(self,
             mapping: Mapping,
             *,
-            dtype: DtypeSpecifier = None, # can be DtypesSpecifier in some contexts
+            dtype: DtypeSpecifier = None,
             name: NameType = None,
+            index_constructor: tp.Optional[IndexConstructor] = None,
             ) -> FrameOrSeries:
         '''
         {doc} Returns a new container.
@@ -170,13 +208,20 @@ class IterNodeDelegate(tp.Generic[FrameOrSeries]):
             {mapping}
             {dtype}
         '''
+        if IterNodeApplyType.is_items(self._apply_type):
+            return self._apply_constructor(
+                    self.map_any_iter_items(mapping),
+                    dtype=dtype,
+                    index_constructor=index_constructor,
+                    name=name,
+                    )
         return self._apply_constructor(
-                self.map_any_iter_items(mapping),
+                self.map_any_iter(mapping),
                 dtype=dtype,
+                index_constructor=index_constructor,
                 name=name,
                 )
 
-
     #---------------------------------------------------------------------------
     @doc_inject(selector='map_fill')
     def map_fill_iter_items(self,
@@ -210,15 +255,20 @@ class IterNodeDelegate(tp.Generic[FrameOrSeries]):
             {mapping}
             {fill_value}
         '''
-        yield from (v for _, v in self.map_fill_iter_items(mapping, fill_value=fill_value))
+        get = getattr(mapping, 'get')
+        if self._yield_type is IterNodeType.VALUES:
+            yield from (get(v, fill_value) for v in self._func_values())
+        else:
+            yield from (get((k,  v), fill_value) for k, v in self._func_items())
 
     @doc_inject(selector='map_fill')
     def map_fill(self,
             mapping: Mapping,
             *,
             fill_value: tp.Any = np.nan,
-            dtype: DtypeSpecifier = None,  # can be DtypesSpecifier in some contexts
+            dtype: DtypeSpecifier = None,
             name: NameType = None,
+            index_constructor: tp.Optional[IndexConstructor] = None,
             ) -> FrameOrSeries:
         '''
         {doc} Returns a new container.
@@ -228,12 +278,21 @@ class IterNodeDelegate(tp.Generic[FrameOrSeries]):
             {fill_value}
             {dtype}
         '''
+        if IterNodeApplyType.is_items(self._apply_type):
+            return self._apply_constructor(
+                    self.map_fill_iter_items(mapping, fill_value=fill_value),
+                    dtype=dtype,
+                    name=name,
+                    index_constructor=index_constructor,
+                    )
         return self._apply_constructor(
-                self.map_fill_iter_items(mapping, fill_value=fill_value),
+                self.map_fill_iter(mapping, fill_value=fill_value),
                 dtype=dtype,
                 name=name,
+                index_constructor=index_constructor,
                 )
 
+
     #---------------------------------------------------------------------------
     @doc_inject(selector='map_all')
     def map_all_iter_items(self,
@@ -262,14 +321,19 @@ class IterNodeDelegate(tp.Generic[FrameOrSeries]):
         Args:
             {mapping}
         '''
-        yield from (v for _, v in self.map_all_iter_items(mapping=mapping))
+        func = getattr(mapping, '__getitem__')
+        if self._yield_type is IterNodeType.VALUES:
+            yield from (func(v) for v in self._func_values())
+        else:
+            yield from (func((k,  v)) for k, v in self._func_items())
 
     @doc_inject(selector='map_all')
     def map_all(self,
             mapping: Mapping,
             *,
-            dtype: DtypeSpecifier = None,  # can be DtypesSpecifier in some contexts
+            dtype: DtypeSpecifier = None,
             name: NameType = None,
+            index_constructor: tp.Optional[IndexConstructor] = None,
             ) -> FrameOrSeries:
         '''
         {doc} Returns a new container.
@@ -278,24 +342,33 @@ class IterNodeDelegate(tp.Generic[FrameOrSeries]):
             {mapping}
             {dtype}
         '''
+        if IterNodeApplyType.is_items(self._apply_type):
+            return self._apply_constructor(
+                    self.map_all_iter_items(mapping),
+                    dtype=dtype,
+                    name=name,
+                    index_constructor=index_constructor,
+                    )
         return self._apply_constructor(
-                self.map_all_iter_items(mapping),
+                self.map_all_iter(mapping),
                 dtype=dtype,
                 name=name,
+                index_constructor=index_constructor,
                 )
 
 
     #---------------------------------------------------------------------------
     @doc_inject(selector='apply')
     def apply_iter_items(self,
-            func: AnyCallable) -> tp.Iterator[tp.Tuple[tp.Any, tp.Any]]:
+            func: AnyCallable,
+            ) -> tp.Iterator[tp.Tuple[tp.Any, tp.Any]]:
         '''
         {doc} A generator of resulting key, value pairs.
 
         Args:
             {func}
         '''
-        # depend on yield type, we determine what the passed in function expects to take
+        # depend on yield type, we determine what the passed in function expects to
         if self._yield_type is IterNodeType.VALUES:
             yield from ((k, func(v)) for k, v in self._func_items())
         else:
@@ -311,14 +384,18 @@ class IterNodeDelegate(tp.Generic[FrameOrSeries]):
         Args:
             {func}
         '''
-        yield from (v for _, v in self.apply_iter_items(func=func))
+        if self._yield_type is IterNodeType.VALUES:
+            yield from (func(v) for v in self._func_values())
+        else:
+            yield from (func(k, v) for k, v in self._func_items())
 
     @doc_inject(selector='apply')
     def apply(self,
             func: AnyCallable,
             *,
-            dtype: DtypeSpecifier = None,  # can be DtypesSpecifier in some contexts
+            dtype: DtypeSpecifier = None,
             name: NameType = None,
+            index_constructor: tp.Optional[IndexConstructor]= None,
             ) -> FrameOrSeries:
         '''
         {doc} Returns a new container.
@@ -330,10 +407,16 @@ class IterNodeDelegate(tp.Generic[FrameOrSeries]):
         if not callable(func):
             raise RuntimeError('use map_fill(), map_any(), or map_all() for applying a mapping type')
 
+        if IterNodeApplyType.is_items(self._apply_type):
+            apply_func = self.apply_iter_items
+        else:
+            apply_func = self.apply_iter
+
         return self._apply_constructor(
-                self.apply_iter_items(func),
+                apply_func(func),
                 dtype=dtype,
                 name=name,
+                index_constructor=index_constructor,
                 )
 
     @doc_inject(selector='apply')
@@ -342,6 +425,7 @@ class IterNodeDelegate(tp.Generic[FrameOrSeries]):
             *,
             dtype: DtypeSpecifier = None,
             name: NameType = None,
+            index_constructor: tp.Optional[IndexConstructor]= None,
             max_workers: tp.Optional[int] = None,
             chunksize: int = 1,
             use_threads: bool = False
@@ -358,14 +442,20 @@ class IterNodeDelegate(tp.Generic[FrameOrSeries]):
             {chunksize}
             {use_threads}
         '''
+        # only use when we need pairs of values to dynamically create an Index
+        if IterNodeApplyType.is_items(self._apply_type):
+            apply_func = self._apply_iter_items_parallel
+        else:
+            apply_func = self._apply_iter_parallel
         return self._apply_constructor(
-                self._apply_iter_items_parallel(
-                        func=func,
+                apply_func(func,
                         max_workers=max_workers,
                         chunksize=chunksize,
-                        use_threads=use_threads),
+                        use_threads=use_threads,
+                        ),
                 dtype=dtype,
                 name=name,
+                index_constructor=index_constructor,
                 )
 
     def __iter__(self) -> tp.Union[
@@ -404,7 +494,7 @@ class IterNode(tp.Generic[FrameOrSeries]):
             function_values: tp.Callable[..., tp.Iterable[tp.Any]],
             function_items: tp.Callable[..., tp.Iterable[tp.Tuple[tp.Any, tp.Any]]],
             yield_type: IterNodeType,
-            apply_type: IterNodeApplyType = IterNodeApplyType.SERIES_ITEMS
+            apply_type: IterNodeApplyType,
             ) -> None:
         '''
         Args:
@@ -417,61 +507,191 @@ class IterNode(tp.Generic[FrameOrSeries]):
         self._yield_type = yield_type
         self._apply_type = apply_type
 
+    #---------------------------------------------------------------------------
+    # apply constructors
+
+    def to_series_values(self,
+            values: tp.Iterator[tp.Any],
+            *,
+            dtype: DtypeSpecifier,
+            name: NameType = None,
+            index_constructor: tp.Optional[IndexConstructor] = None,
+            axis: int = 0,
+            ) -> 'Series':
+        from static_frame.core.series import Series
+
+        # Creating a Series that will have the same index as source container
+        if self._container._NDIM == 2 and axis == 0:
+            index = self._container._columns #type: ignore
+            own_index = False
+        else:
+            index = self._container._index
+            own_index = True
+
+        if index_constructor is not None:
+            index = index_constructor(index)
+
+        # PERF: passing count here permits faster generator realization
+        values, _ = iterable_to_array_1d(
+                values,
+                count=index.shape[0],
+                dtype=dtype,
+                )
+        return Series(values,
+                name=name,
+                index=index,
+                own_index=own_index,
+                )
+
+    def to_series_items(self,
+            pairs: tp.Iterable[tp.Tuple[tp.Hashable, tp.Any]],
+            *,
+            dtype: DtypeSpecifier = None,
+            name: NameType = None,
+            index_constructor: tp.Optional[IndexConstructor]= None,
+            axis: int = 0,
+            ) -> 'Series':
+        from static_frame.core.series import Series
+
+        # apply_constructor should be implemented to take a pairs of label, value; only used for iter_window
+        # axis 0 iters windows labelled by the index, axis 1 iters windows labelled by the columns
+
+        if self._container._NDIM == 2 and axis == 1:
+            index_constructor = (index_constructor
+                    if index_constructor is not None
+                    else self._container._columns.from_labels) #type: ignore
+            name_index = self._container._columns._name #type: ignore
+        else:
+            index_constructor = (index_constructor
+                    if index_constructor is not None
+                    else self._container._index.from_labels)
+            name_index = self._container._index._name
+
+        index_constructor_final = partial(
+                index_constructor,
+                name=name_index,
+                )
+        # always return a Series
+        return Series.from_items(
+                pairs=pairs,
+                dtype=dtype,
+                name=name,
+                index_constructor=index_constructor_final,
+                )
+
+    def to_series_items_group(self,
+            pairs: tp.Iterable[tp.Tuple[tp.Hashable, tp.Any]],
+            *,
+            dtype: DtypeSpecifier = None,
+            name: NameType = None,
+            index_constructor: tp.Optional[IndexConstructor]= None,
+            name_index: NameType = None,
+            ) -> 'Series':
+        from static_frame.core.series import Series
+        from static_frame.core.index import Index
+
+        # NOTE: when used on labels, this key is given; when used on labels (indices) depth_level is given; only take the key if it is a hashable (a string or a tuple, not a slice, list, or array)
+
+        index_constructor = partial(
+                Index if index_constructor is None else index_constructor,
+                name=name_index,
+                )
+        return Series.from_items(
+                pairs=pairs,
+                dtype=dtype,
+                name=name,
+                index_constructor=index_constructor
+                )
+
+    def to_frame_elements(self,
+            items: tp.Iterable[tp.Tuple[
+                    tp.Tuple[tp.Hashable, tp.Hashable], tp.Any]],
+            *,
+            dtype: DtypeSpecifier = None,
+            name: NameType = None,
+            index_constructor: tp.Optional[IndexConstructor]= None,
+            axis: int = 0,
+            ) -> 'Frame':
+        from static_frame.core.frame import Frame
+
+        index_constructor = (self._container._index.from_labels
+                if index_constructor is None else index_constructor)
+
+        assert isinstance(self._container, Frame)
+        return self._container.__class__.from_element_items(
+                items,
+                index=self._container._index,
+                columns=self._container._columns,
+                axis=axis,
+                own_index=True,
+                index_constructor=index_constructor,
+                columns_constructor=self._container._columns.from_labels,
+                name=name,
+                )
+
+    def to_index_labels(self,
+            values: tp.Iterator[tp.Hashable], #pylint: disable=function-redefined
+            dtype: DtypeSpecifier = None,
+            name: NameType = None,
+            index_constructor: tp.Optional[IndexConstructor]= None,
+            ) -> np.ndarray:
+        # NOTE: name argument is for common interface
+        if index_constructor is not None:
+            raise RuntimeError('index_constructor not supported with this interface')
+        # PERF: passing count here permits faster generator realization
+        shape = self._container.shape
+        array, _ = iterable_to_array_1d(values, count=shape[0], dtype=dtype)
+        return array
+
+    #---------------------------------------------------------------------------
     def get_delegate(self,
-            *args: object,
             **kwargs: object
             ) -> IterNodeDelegate[FrameOrSeries]:
         '''
         In usage as an iteator, the args passed here are expected to be argument for the core iterators, i.e., axis arguments.
+
+        Args:
+            kwargs: kwarg args to be passed to both self._func_values and self._func_items
         '''
-        from static_frame.core.series import Series
         from static_frame.core.frame import Frame
 
-        assert not args # force all kwarg
-
+        # all kwargs, like ``drop```, are partialed into func_values, func_items
         func_values = partial(self._func_values, **kwargs)
         func_items = partial(self._func_items, **kwargs)
 
-        apply_constructor: tp.Callable[..., tp.Union[Frame, Series]]
+        axis = kwargs.get('axis', 0)
 
-        if self._apply_type is IterNodeApplyType.SERIES_ITEMS:
-            if isinstance(self._container, Frame) and kwargs['axis'] == 0:
-                index_constructor = self._container._columns.from_labels
-            else:
-                index_constructor = self._container._index.from_labels
-            # always return a Series
-            apply_constructor = partial(
-                    Series.from_items,
-                    index_constructor=index_constructor
+        apply_constructor: tp.Callable[..., tp.Union['Frame', 'Series']]
+
+        if self._apply_type is IterNodeApplyType.SERIES_VALUES:
+            apply_constructor = partial(self.to_series_values, axis=axis)
+
+        elif self._apply_type is IterNodeApplyType.SERIES_ITEMS:
+            apply_constructor = partial(self.to_series_items, axis=axis)
+
+        elif self._apply_type is IterNodeApplyType.SERIES_ITEMS_GROUP_VALUES:
+            try:
+                name_index = name_filter(kwargs.get('key', None))
+            except TypeError:
+                name_index = None
+            apply_constructor = partial(self.to_series_items_group,
+                    name_index=name_index,
+                    )
+
+        elif self._apply_type is IterNodeApplyType.SERIES_ITEMS_GROUP_LABELS:
+            # will always have `depth_level` in kwargs, and for Frame an axis; could attempt to get name from the index if it has a name
+            name_index = None
+            apply_constructor = partial(self.to_series_items_group,
+                    name_index=name_index,
                     )
-        elif self._apply_type is IterNodeApplyType.SERIES_ITEMS_FLAT:
-            # use default index constructor
-            apply_constructor = Series.from_items
 
         elif self._apply_type is IterNodeApplyType.FRAME_ELEMENTS:
             assert isinstance(self._container, Frame) # for typing
-            # for element-wise function application, axis will always be 0 or 1, as we always do full iteration; from_element_items accepts axis of None for incomplete specification, but that will never be used here.
-            apply_constructor = partial(
-                    self._container.__class__.from_element_items,
-                    index=self._container._index,
-                    columns=self._container._columns,
-                    axis=kwargs['axis'],
-                    own_index=True,
-                    index_constructor=self._container._index.from_labels,
-                    columns_constructor=self._container._columns.from_labels
-                    )
+            apply_constructor = partial(self.to_frame_elements, axis=axis)
+
         elif self._apply_type is IterNodeApplyType.INDEX_LABELS:
-            # apply_constructor = Series.from_items
-            def apply_constructor(items: tp.Iterable[tp.Tuple[tp.Hashable, tp.Any]], #pylint: disable=function-redefined
-                    dtype: DtypeSpecifier = None,
-                    name: NameType = None,
-                    ) -> np.ndarray:
-                # NOTE: cannot use name argument, here for compat
-                array, _ = iterable_to_array_1d(
-                        (v for _, v in items),
-                        dtype,
-                        )
-                return array
+            apply_constructor = self.to_index_labels
+
         else:
             raise NotImplementedError(self._apply_type) #pragma: no cover
 
@@ -479,7 +699,8 @@ class IterNode(tp.Generic[FrameOrSeries]):
                 func_values=func_values,
                 func_items=func_items,
                 yield_type=self._yield_type,
-                apply_constructor=tp.cast(tp.Callable[..., FrameOrSeries], apply_constructor)
+                apply_constructor=tp.cast(tp.Callable[..., FrameOrSeries], apply_constructor),
+                apply_type=self._apply_type,
                 )
 
 
@@ -543,9 +764,10 @@ class IterNodeGroupAxis(IterNode[FrameOrSeries]):
     def __call__(self,
             key: KEY_ITERABLE_TYPES, # type: ignore
             *,
-            axis: int = 0
+            axis: int = 0,
+            drop: bool = False,
             ) -> IterNodeDelegate[FrameOrSeries]:
-        return IterNode.get_delegate(self, key=key, axis=axis)
+        return IterNode.get_delegate(self, key=key, axis=axis, drop=drop)
 
 
 class IterNodeDepthLevel(IterNode[FrameOrSeries]):
