@@ -11,6 +11,7 @@ from static_frame.core.util import NULL_SLICE
 if tp.TYPE_CHECKING:
     from static_frame.core.batch import Batch  #pylint: disable = W0611 #pragma: no cover
     from static_frame.core.bus import Bus  #pylint: disable = W0611 #pragma: no cover
+    from static_frame.core.yarn import Yarn  #pylint: disable = W0611 #pragma: no cover
     from static_frame.core.frame import Frame  #pylint: disable = W0611 #pragma: no cover
     from static_frame.core.frame import FrameAsType  #pylint: disable = W0611 #pragma: no cover
     from static_frame.core.index import Index  #pylint: disable = W0611 #pragma: no cover
@@ -27,6 +28,7 @@ TContainer = tp.TypeVar('TContainer',
         'TypeBlocks',
         'Bus',
         'Batch',
+        'Yarn',
         # 'Quilt',
         'IndexHierarchy',
         )
@@ -45,11 +47,13 @@ class InterfaceGetItem(Interface[TContainer]):
     __slots__ = ('_func',)
     INTERFACE = ('__getitem__',)
 
+    _func: tp.Callable[[GetItemKeyType], TContainer]
+
     def __init__(self, func: tp.Callable[[GetItemKeyType], TContainer]) -> None:
-        self._func: tp.Callable[[GetItemKeyType], TContainer] = func
+        self._func = func #type: ignore
 
     def __getitem__(self, key: GetItemKeyType) -> TContainer:
-        return self._func(key)
+        return self._func(key) #type: ignore
 
 #-------------------------------------------------------------------------------
 
