@@ -1,7 +1,7 @@
 import typing as tp
+from itertools import repeat
 from itertools import zip_longest
 from functools import partial
-from copy import deepcopy
 
 import numpy as np
 
@@ -9,19 +9,21 @@ from static_frame.core.bus import Bus
 from static_frame.core.container import ContainerBase
 from static_frame.core.container_util import axis_window_items
 from static_frame.core.display import Display
+from static_frame.core.display import DisplayHeader
 from static_frame.core.display_config import DisplayConfig
 from static_frame.core.doc_str import doc_inject
 from static_frame.core.exception import AxisInvalid
 from static_frame.core.exception import ErrorInitQuilt
+from static_frame.core.exception import ErrorInitIndexNonUnique
 from static_frame.core.exception import NotImplementedAxis
 from static_frame.core.frame import Frame
 from static_frame.core.hloc import HLoc
 from static_frame.core.index_base import IndexBase
-from static_frame.core.index_hierarchy import IndexHierarchy
 from static_frame.core.node_iter import IterNodeAxis
 from static_frame.core.node_iter import IterNodeConstructorAxis
 from static_frame.core.node_iter import IterNodeType
 from static_frame.core.node_iter import IterNodeWindow
+from static_frame.core.node_iter import IterNodeApplyType
 from static_frame.core.node_selector import InterfaceGetItem
 from static_frame.core.series import Series
 from static_frame.core.store import Store
@@ -34,9 +36,8 @@ from static_frame.core.store_zip import StoreZipCSV
 from static_frame.core.store_zip import StoreZipParquet
 from static_frame.core.store_zip import StoreZipPickle
 from static_frame.core.store_zip import StoreZipTSV
+from static_frame.core.store_zip import StoreZipNPZ
 from static_frame.core.util import AnyCallable
-from static_frame.core.util import array_deepcopy
-from static_frame.core.util import duplicate_filter
 from static_frame.core.util import get_tuple_constructor
 from static_frame.core.util import GetItemKeyType
 from static_frame.core.util import GetItemKeyTypeCompound
@@ -45,86 +46,22 @@ from static_frame.core.util import NameType
 from static_frame.core.util import NULL_SLICE
 from static_frame.core.util import PathSpecifier
 from static_frame.core.util import concat_resolved
+from static_frame.core.style_config import StyleConfig
+from static_frame.core.index_hierarchy import IndexHierarchy
+from static_frame.core.yarn import Yarn
 
-
-def get_extractor(
-        deepcopy_from_bus: bool,
-        is_array: bool,
-        memo_active: bool,
-        ) -> AnyCallable:
-    '''
-    Args:
-        memo_active: enable usage of a common memoization dictionary accross all calls to extract from this extractor.
-    '''
-    if deepcopy_from_bus:
-        memo: tp.Optional[tp.Dict[int, tp.Any]] = None if not memo_active else {}
-        if is_array:
-            return partial(array_deepcopy, memo=memo)
-        return partial(deepcopy, memo=memo)
-    return lambda x: x
-
-class AxisMap:
-    '''
-    An AxisMap is a Series where index values point to string label in a Bus.
-    '''
-
-    @staticmethod
-    def get_axis_series(
-            tree: tp.Dict[tp.Hashable, IndexBase],
-            ) -> Series:
-
-        index = IndexHierarchy.from_tree(tree)
-        return Series(
-                index.values_at_depth(0), # store the labels as series values
-                index=index,
-                own_index=True,
-                )
-
-    @classmethod
-    def from_bus(cls,
-            bus: Bus,
-            axis: int,
-            deepcopy_from_bus: bool,
-            ) -> tp.Tuple[Series, IndexBase]:
-        '''
-        Given a :obj:`Bus` and an axis, derive a :obj:`Series` with an :obj:`IndexHierarchy`; also return and validate the :obj:`Index` of the opposite axis.
-        '''
-        # NOTE: need to extract just axis labels, not the full Frame; need new Store/Bus loaders just for label data
-
-        extractor = get_extractor(deepcopy_from_bus, is_array=False, memo_active=False)
-
-        tree = {}
-        opposite: tp.Optional[IndexBase] = None
-
-        for label, f in bus.items():
-            if axis == 0:
-                tree[label] = extractor(f.index)
-                if opposite is None:
-                    opposite = extractor(f.columns)
-                else:
-                    if not opposite.equals(f.columns):
-                        raise ErrorInitQuilt('opposite axis must have equivalent indices')
-            elif axis == 1:
-                tree[label] = extractor(f.columns)
-                if opposite is None:
-                    opposite = extractor(f.index)
-                else:
-                    if not opposite.equals(f.index):
-                        raise ErrorInitQuilt('opposite axis must have equivalent indices')
-            else:
-                raise AxisInvalid(f'invalid axis {axis}')
-        return cls.get_axis_series(tree), opposite # type: ignore
-
+from static_frame.core.axis_map import bus_to_hierarchy
+from static_frame.core.axis_map import get_extractor
 
 class Quilt(ContainerBase, StoreClientMixin):
     '''
-    A :obj:`Frame`-like view of the contents of a :obj:`Bus`. With the Quilt, :obj:`Frame` contained in a :obj:`Bus` can be conceived as stacking vertically (primary axis 0) or horizontally (primary axis 1). If the labels of the primary axis are unique accross all contained :obj:`Frame, ``retain_labels`` can be set to ``False`` and underlying labels are simply concatenated; otherwise, ``retain_labels`` must be set to ``True`` and an additional depth-level is added to the primary axis labels. A :obj:`Quilt` can only be created if labels of the opposite axis of all contained :obj:`Frame` are aligned.
+    A :obj:`Frame`-like view of the contents of a :obj:`Bus` or :obj:`Yarn`. With the Quilt, :obj:`Frame` contained in a :obj:`Bus` or :obj:`Yarn` can be conceived as stacking vertically (primary axis 0) or horizontally (primary axis 1). If the labels of the primary axis are unique accross all contained :obj:`Frame`, ``retain_labels`` can be set to ``False`` and underlying labels are simply concatenated; otherwise, ``retain_labels`` must be set to ``True`` and an additional depth-level is added to the primary axis labels. A :obj:`Quilt` can only be created if labels of the opposite axis of all contained :obj:`Frame` are aligned.
     '''
 
     __slots__ = (
             '_bus',
             '_axis',
-            '_axis_map',
+            '_axis_hierarchy',
             '_retain_labels',
             '_axis_opposite',
             '_assign_axis',
@@ -133,9 +70,9 @@ class Quilt(ContainerBase, StoreClientMixin):
             '_deepcopy_from_bus',
             )
 
-    _bus: Bus
+    _bus: tp.Union[Bus, Yarn]
     _axis: int
-    _axis_map: tp.Optional[Series]
+    _axis_hierarchy: tp.Optional[IndexHierarchy]
     _axis_opposite: tp.Optional[IndexBase]
     _columns: IndexBase
     _index: IndexBase
@@ -199,11 +136,11 @@ class Quilt(ContainerBase, StoreClientMixin):
         name = name if name else frame.name
         bus = Bus.from_frames(values(), config=config, name=name)
 
-        axis_map = AxisMap.get_axis_series(axis_map_components)
+        axis_hierarchy = IndexHierarchy.from_tree(axis_map_components)
 
         return cls(bus,
                 axis=axis,
-                axis_map=axis_map,
+                axis_hierarchy=axis_hierarchy,
                 axis_opposite=opposite,
                 retain_labels=retain_labels,
                 deepcopy_from_bus=deepcopy_from_bus,
@@ -308,6 +245,30 @@ class Quilt(ContainerBase, StoreClientMixin):
                 max_persist=max_persist,
                 )
 
+    @classmethod
+    @doc_inject(selector='quilt_constructor')
+    def from_zip_npz(cls,
+            fp: PathSpecifier,
+            *,
+            config: StoreConfigMapInitializer = None,
+            axis: int = 0,
+            retain_labels: bool,
+            deepcopy_from_bus: bool = False,
+            max_persist: tp.Optional[int] = None,
+            ) -> 'Quilt':
+        '''
+        Given a file path to zipped parquet :obj:`Quilt` store, return a :obj:`Quilt` instance.
+
+        {args}
+        '''
+        store = StoreZipNPZ(fp)
+        return cls._from_store(store,
+                config=config,
+                axis=axis,
+                retain_labels=retain_labels,
+                deepcopy_from_bus=deepcopy_from_bus,
+                max_persist=max_persist,
+                )
 
     @classmethod
     @doc_inject(selector='quilt_constructor')
@@ -334,7 +295,6 @@ class Quilt(ContainerBase, StoreClientMixin):
                 max_persist=max_persist,
                 )
 
-
     @classmethod
     @doc_inject(selector='quilt_constructor')
     def from_xlsx(cls,
@@ -455,11 +415,11 @@ class Quilt(ContainerBase, StoreClientMixin):
     #---------------------------------------------------------------------------
     @doc_inject(selector='quilt_init')
     def __init__(self,
-            bus: Bus,
+            bus: tp.Union[Bus, Yarn],
             *,
             axis: int = 0,
             retain_labels: bool,
-            axis_map: tp.Optional[Series] = None,
+            axis_hierarchy: tp.Optional[IndexHierarchy] = None,
             axis_opposite: tp.Optional[IndexBase] = None,
             deepcopy_from_bus: bool = False,
             ) -> None:
@@ -471,43 +431,56 @@ class Quilt(ContainerBase, StoreClientMixin):
         self._retain_labels = retain_labels
         self._deepcopy_from_bus = deepcopy_from_bus
 
-        if (axis_map is None) ^ (axis_opposite is None):
-            raise ErrorInitQuilt('if supplying axis_map, supply axis_opposite')
+        if (axis_hierarchy is None) ^ (axis_opposite is None):
+            raise ErrorInitQuilt('if supplying axis_hierarchy, supply axis_opposite')
 
         # can creation until needed
-        self._axis_map = axis_map
+        self._axis_hierarchy = axis_hierarchy
         self._axis_opposite = axis_opposite
-        # will be set with re-axis
-        # self._index = None
-        # self._columns = None
-        self._assign_axis = True
-
+        self._assign_axis = True # Boolean to control deferred axis index creation
 
     #---------------------------------------------------------------------------
     # deferred loading of axis info
+    @staticmethod
+    def _error_update_axis_labels(axis: int) -> ErrorInitQuilt:
+        axis_label = 'index' if axis == 0 else 'column'
+        axis_labels = 'indices' if axis == 0 else 'columns'
+        err_msg = f"Duplicate {axis_label} labels across frames. Either ensure all {axis_labels} are unique for all frames, or set retain_labels=True to obtain an IndexHierarchy"
+        return ErrorInitQuilt(err_msg)
 
     def _update_axis_labels(self) -> None:
-        if self._axis_map is None or self._axis_opposite is None:
-            self._axis_map, self._axis_opposite = AxisMap.from_bus(
+        if self._axis_hierarchy is None or self._axis_opposite is None:
+            self._axis_hierarchy, self._axis_opposite = bus_to_hierarchy(
                     self._bus,
                     axis=self._axis,
                     deepcopy_from_bus=self._deepcopy_from_bus,
+                    init_exception_cls=ErrorInitQuilt,
                     )
-
         if self._axis == 0:
             if not self._retain_labels:
-                self._index = self._axis_map.index.level_drop(1) #type: ignore
+                try:
+                    self._index = self._axis_hierarchy.level_drop(1)
+                except ErrorInitIndexNonUnique:
+                    raise self._error_update_axis_labels(self._axis) from None
             else: # get hierarchical
-                self._index = self._axis_map.index
+                self._index = self._axis_hierarchy
             self._columns = self._axis_opposite
         else:
             if not self._retain_labels:
-                self._columns = self._axis_map.index.level_drop(1) #type: ignore
+                try:
+                    self._columns = self._axis_hierarchy.level_drop(1)
+                except ErrorInitIndexNonUnique:
+                    raise self._error_update_axis_labels(self._axis) from None
             else:
-                self._columns = self._axis_map.index
+                self._columns = self._axis_hierarchy
             self._index = self._axis_opposite
         self._assign_axis = False
 
+    def unpersist(self) -> None:
+        '''For the :obj:`Bus` or :obj:`Yarn` contained in this object, replace all loaded :obj:`Frame` with :obj:`FrameDeferred`.
+        '''
+        self._bus.unpersist()
+
     #---------------------------------------------------------------------------
     # name interface
 
@@ -515,39 +488,85 @@ class Quilt(ContainerBase, StoreClientMixin):
     @doc_inject()
     def name(self) -> NameType:
         '''{}'''
-        return self._bus._series._name
+        return self._bus.name #type: ignore
 
     def rename(self, name: NameType) -> 'Quilt':
         '''
-        Return a new Quilt with an updated name attribute.
+        Return a new :obj:`Quilt` with an updated name attribute.
+
+        Args:
+            name
         '''
         return self.__class__(self._bus.rename(name),
                 axis=self._axis,
                 retain_labels=self._retain_labels,
                 deepcopy_from_bus=self._deepcopy_from_bus,
-                axis_map=self._axis_map,
+                axis_hierarchy=self._axis_hierarchy,
                 axis_opposite=self._axis_opposite,
                 )
 
     #---------------------------------------------------------------------------
 
-    def __repr__(self) -> str:
-        '''Provide a display of the :obj:`Quilt` that does not realize the entire :obj:`Frame`.
-        '''
-        if self.name:
-            header = f'{self.__class__.__name__}: {self.name}'
-        else:
-            header = self.__class__.__name__
-        return f'<{header} at {hex(id(self))}>'
-
+    @doc_inject()
     def display(self,
-            config: tp.Optional[DisplayConfig] = None
+            config: tp.Optional[DisplayConfig] = None,
+            *,
+            style_config: tp.Optional[StyleConfig] = None,
             ) -> Display:
-        '''Provide a :obj:`Frame`-style display of the :obj:`Quilt`.
+        '''{doc}
+
+        Args:
+            {config}
         '''
         if self._assign_axis:
             self._update_axis_labels()
-        return self.to_frame().display(config) #type: ignore
+
+        drop_column_dtype = False
+
+        if self._axis == 0:
+            if not self._retain_labels:
+                index = self.index.rename("Concatenated")
+            else:
+                index = self._bus.index.rename("Frames")
+            columns = self.columns.rename("Aligned")
+        else:
+            index = self.index.rename("Aligned")
+            if not self._retain_labels:
+                columns = self.columns.rename("Concatenated")
+            else:
+                columns = self._bus.index.rename("Frames")
+                drop_column_dtype = True
+
+        config = config or DisplayConfig()
+
+        def placeholder_gen() -> tp.Iterator[tp.Iterable[tp.Any]]:
+            assert config is not None
+            yield from repeat(tuple(repeat(config.cell_placeholder, times=len(index))), times=len(columns))
+
+        d = Display.from_params(
+                index=index,
+                columns=columns,
+                header=DisplayHeader(self.__class__, self.name),
+                column_forward_iter=placeholder_gen,
+                column_reverse_iter=placeholder_gen,
+                column_default_iter=placeholder_gen,
+                config=config,
+                style_config=style_config,
+                )
+
+        # Strip out the dtype information!
+        if config.type_show:
+            if drop_column_dtype:
+                # First Column Row -> last element is the dtype of the column
+                # Guaranteed to not be index hierarchy as buses cannot have index hierarchies
+                d._rows[1].pop()
+
+            # Since placeholder_gen is not a ndarray, there is no dtype to append in the final row
+            # However, in the case of a center ellipsis being added, an ellipsis will be
+            # awkwardly placed direclty adjacent to the index dtype information.
+            if d._rows[-1][-1] == Display.CELL_ELLIPSIS:
+                d._rows[-1].pop()
+        return d
 
     #---------------------------------------------------------------------------
     # accessors
@@ -836,7 +855,7 @@ class Quilt(ContainerBase, StoreClientMixin):
         '''
         Extract a consolidated array based on iloc selection.
         '''
-        assert self._axis_map is not None #mypy
+        assert self._axis_hierarchy is not None #mypy
 
         extractor = get_extractor(
                 self._deepcopy_from_bus,
@@ -871,25 +890,25 @@ class Quilt(ContainerBase, StoreClientMixin):
         sel_reduces = isinstance(sel_key, INT_TYPES)
         opposite_reduces = isinstance(opposite_key, INT_TYPES)
 
-        sel = np.full(len(self._axis_map), False)
+        sel = np.full(len(self._axis_hierarchy), False)
         sel[sel_key] = True
 
-        # get ordered unique Bus labels from AxisMap Series values; cannot use .unique as need order
-        axis_map_sub: tp.Union[Series, tp.Hashable] = self._axis_map.iloc[sel_key]
-        if not isinstance(axis_map_sub, Series): # we have an element integer
-            bus_keys = (axis_map_sub,)
+        # get ordered unique Bus labels
+        axis_map_sub = self._axis_hierarchy.iloc[sel_key]
+        if isinstance(axis_map_sub, tuple): # type: ignore
+            bus_keys = (axis_map_sub[0],) #type: ignore
         else:
-            bus_keys = duplicate_filter(axis_map_sub.values)
+            bus_keys = axis_map_sub._levels.index
 
         for key_count, key in enumerate(bus_keys):
-            sel_component = sel[self._axis_map.index._loc_to_iloc(HLoc[key])]
+            sel_component = sel[self._axis_hierarchy._loc_to_iloc(HLoc[key])]
 
             if self._axis == 0:
-                component = self._bus.loc[key]._extract_array(sel_component, opposite_key) #type: ignore [attr-defined]
+                component = self._bus.loc[key]._extract_array(sel_component, opposite_key) #type: ignore
                 if sel_reduces:
                     component = component[0]
             else:
-                component = self._bus.loc[key]._extract_array(opposite_key, sel_component) #type: ignore [attr-defined]
+                component = self._bus.loc[key]._extract_array(opposite_key, sel_component) #type: ignore
                 if sel_reduces:
                     if component.ndim == 1:
                         component = component[0]
@@ -914,7 +933,7 @@ class Quilt(ContainerBase, StoreClientMixin):
         '''
         Extract Container based on iloc selection.
         '''
-        assert self._axis_map is not None #mypy
+        assert self._axis_hierarchy is not None #mypy
 
         extractor = get_extractor(
                 self._deepcopy_from_bus,
@@ -944,7 +963,7 @@ class Quilt(ContainerBase, StoreClientMixin):
                     )
 
         parts: tp.List[tp.Any] = []
-        bus_keys: tp.Iterable[tp.Hashable]
+        frame_labels: tp.Iterable[tp.Hashable]
 
         if self._axis == 0:
             sel_key = row_key
@@ -955,18 +974,20 @@ class Quilt(ContainerBase, StoreClientMixin):
 
         sel_reduces = isinstance(sel_key, INT_TYPES)
 
-        sel = np.full(len(self._axis_map), False)
+        sel = np.full(len(self._axis_hierarchy), False)
         sel[sel_key] = True
 
-        # get ordered unique Bus labels from AxisMap Series values; cannot use .unique as need order
-        axis_map_sub: tp.Union[Series, int] = self._axis_map.iloc[sel_key]
-        if not isinstance(axis_map_sub, Series): # we have an element integer
-            bus_keys = (axis_map_sub,)
+        # get ordered unique Bus labels
+        axis_map_sub = self._axis_hierarchy.iloc[sel_key]
+        if isinstance(axis_map_sub, tuple): #type: ignore
+            frame_labels = (axis_map_sub[0],) #type: ignore
         else:
-            bus_keys = duplicate_filter(axis_map_sub.values)
+            # get the outer level, or just the unique frame labels needed
+            frame_labels = axis_map_sub._levels.index
 
-        for key_count, key in enumerate(bus_keys):
-            sel_component = sel[self._axis_map.index._loc_to_iloc(HLoc[key])]
+        for key_count, key in enumerate(frame_labels):
+            # get Boolean segment for this Frame
+            sel_component = sel[self._axis_hierarchy._loc_to_iloc(HLoc[key])]
 
             if self._axis == 0:
                 component = self._bus.loc[key].iloc[sel_component, opposite_key]
@@ -1002,6 +1023,37 @@ class Quilt(ContainerBase, StoreClientMixin):
             return Series.from_concat(parts)
         return Frame.from_concat(parts, axis=self._axis) #type: ignore
 
+    #---------------------------------------------------------------------------
+    @doc_inject(selector='sample')
+    def sample(self,
+            index: tp.Optional[int] = None,
+            columns: tp.Optional[int] = None,
+            *,
+            seed: tp.Optional[int] = None,
+            ) -> Frame:
+        '''
+        {doc}
+
+        Args:
+            {index}
+            {columns}
+            {seed}
+        '''
+        if self._assign_axis:
+            self._update_axis_labels()
+
+        if index is not None:
+            _, index_key = self._index._sample_and_key(count=index, seed=seed)
+        else:
+            index_key = None
+
+        if columns is not None:
+            _, columns_key = self._columns._sample_and_key(count=columns, seed=seed)
+        else:
+            columns_key = None
+
+        return self._extract(row_key=index_key, column_key=columns_key) #type: ignore
+
     #---------------------------------------------------------------------------
 
     def _extract_iloc(self, key: GetItemKeyTypeCompound) -> tp.Union[Series, Frame]:
@@ -1077,7 +1129,8 @@ class Quilt(ContainerBase, StoreClientMixin):
                 container=self,
                 function_values=self._axis_array,
                 function_items=self._axis_array_items,
-                yield_type=IterNodeType.VALUES
+                yield_type=IterNodeType.VALUES,
+                apply_type=IterNodeApplyType.SERIES_VALUES,
                 )
 
     @property
@@ -1091,7 +1144,8 @@ class Quilt(ContainerBase, StoreClientMixin):
                 container=self,
                 function_values=self._axis_array,
                 function_items=self._axis_array_items,
-                yield_type=IterNodeType.ITEMS
+                yield_type=IterNodeType.ITEMS,
+                apply_type=IterNodeApplyType.SERIES_VALUES,
                 )
 
     @property
@@ -1105,7 +1159,8 @@ class Quilt(ContainerBase, StoreClientMixin):
                 container=self,
                 function_values=self._axis_tuple,
                 function_items=self._axis_tuple_items,
-                yield_type=IterNodeType.VALUES
+                yield_type=IterNodeType.VALUES,
+                apply_type=IterNodeApplyType.SERIES_VALUES,
                 )
 
     @property
@@ -1119,7 +1174,8 @@ class Quilt(ContainerBase, StoreClientMixin):
                 container=self,
                 function_values=self._axis_tuple,
                 function_items=self._axis_tuple_items,
-                yield_type=IterNodeType.ITEMS
+                yield_type=IterNodeType.ITEMS,
+                apply_type=IterNodeApplyType.SERIES_VALUES,
                 )
 
     @property
@@ -1133,7 +1189,8 @@ class Quilt(ContainerBase, StoreClientMixin):
                 container=self,
                 function_values=self._axis_series,
                 function_items=self._axis_series_items,
-                yield_type=IterNodeType.VALUES
+                yield_type=IterNodeType.VALUES,
+                apply_type=IterNodeApplyType.SERIES_VALUES,
                 )
 
     @property
@@ -1147,7 +1204,8 @@ class Quilt(ContainerBase, StoreClientMixin):
                 container=self,
                 function_values=self._axis_series,
                 function_items=self._axis_series_items,
-                yield_type=IterNodeType.ITEMS
+                yield_type=IterNodeType.ITEMS,
+                apply_type=IterNodeApplyType.SERIES_VALUES,
                 )
 
 
@@ -1169,7 +1227,8 @@ class Quilt(ContainerBase, StoreClientMixin):
                 container=self,
                 function_values=function_values,
                 function_items=function_items,
-                yield_type=IterNodeType.VALUES
+                yield_type=IterNodeType.VALUES,
+                apply_type=IterNodeApplyType.SERIES_ITEMS,
                 )
 
     @property #type: ignore
@@ -1188,7 +1247,8 @@ class Quilt(ContainerBase, StoreClientMixin):
                 container=self,
                 function_values=function_values,
                 function_items=function_items,
-                yield_type=IterNodeType.ITEMS
+                yield_type=IterNodeType.ITEMS,
+                apply_type=IterNodeApplyType.SERIES_ITEMS,
                 )
 
     @property #type: ignore
@@ -1207,7 +1267,8 @@ class Quilt(ContainerBase, StoreClientMixin):
                 container=self,
                 function_values=function_values,
                 function_items=function_items,
-                yield_type=IterNodeType.VALUES
+                yield_type=IterNodeType.VALUES,
+                apply_type=IterNodeApplyType.SERIES_ITEMS,
                 )
 
     @property #type: ignore
@@ -1226,7 +1287,8 @@ class Quilt(ContainerBase, StoreClientMixin):
                 container=self,
                 function_values=function_values,
                 function_items=function_items,
-                yield_type=IterNodeType.ITEMS
+                yield_type=IterNodeType.ITEMS,
+                apply_type=IterNodeApplyType.SERIES_ITEMS,
                 )
 
     #---------------------------------------------------------------------------
@@ -1249,6 +1311,76 @@ class Quilt(ContainerBase, StoreClientMixin):
         '''
         return self.iloc[-count:]
 
+    #---------------------------------------------------------------------------
+    @doc_inject()
+    def equals(self,
+            other: tp.Any,
+            *,
+            compare_name: bool = False,
+            compare_dtype: bool = False,
+            compare_class: bool = False,
+            skipna: bool = True,
+            ) -> bool:
+        '''
+        {doc}
+
+        Note: this will attempt to load and compare all Frame managed by the Bus stored within this Quilt.
+
+        Args:
+            {compare_name}
+            {compare_dtype}
+            {compare_class}
+            {skipna}
+        '''
+        if id(other) == id(self):
+            return True
+
+        if compare_class and self.__class__ != other.__class__:
+            return False
+        elif not isinstance(other, Quilt):
+            return False
+
+        if self._axis != other._axis:
+            return False
+
+        if self._retain_labels != other._retain_labels:
+            return False
+
+        if compare_name and self.name != other.name:
+            return False
+
+        if self._assign_axis:
+            self._update_axis_labels()
+        if other._assign_axis:
+            other._update_axis_labels()
+
+        if not self._axis_hierarchy.equals( # type: ignore
+                other._axis_hierarchy,
+                compare_name=compare_name,
+                compare_dtype=compare_dtype,
+                compare_class=compare_class,
+                skipna=skipna,
+                ):
+            return False
+
+        if not self._axis_opposite.equals( # type: ignore
+                other._axis_opposite,
+                compare_name=compare_name,
+                compare_dtype=compare_dtype,
+                compare_class=compare_class,
+                skipna=skipna,
+                ):
+            return False
+
+        if not self._bus.equals(other._bus,
+                compare_name=compare_name,
+                compare_dtype=compare_dtype,
+                compare_class=compare_class,
+                skipna=skipna,
+                ):
+            return False
+
+        return True
 
     #---------------------------------------------------------------------------
     def to_frame(self) -> Frame:
