@@ -6,6 +6,7 @@ import os
 from itertools import chain
 from functools import partial
 from functools import wraps
+from weakref import WeakValueDictionary
 import numpy as np
 
 from static_frame.core.interface_meta import InterfaceMeta
@@ -21,7 +22,7 @@ from static_frame.core.util import DtypesSpecifier
 from static_frame.core.util import path_filter
 from static_frame.core.util import PathSpecifier
 from static_frame.core.util import DepthLevelSpecifier
-
+from static_frame.core.util import IndexConstructors
 
 #-------------------------------------------------------------------------------
 
@@ -32,8 +33,10 @@ class StoreConfigHE(metaclass=InterfaceMeta):
 
     index_depth: int
     index_name_depth_level: tp.Optional[DepthLevelSpecifier]
+    index_constructors: IndexConstructors
     columns_depth: int
     columns_name_depth_level: tp.Optional[DepthLevelSpecifier]
+    columns_constructors: IndexConstructors
     columns_select: tp.Optional[tp.Iterable[str]]
     dtypes: DtypesSpecifier
     consolidate_blocks: bool
@@ -54,8 +57,10 @@ class StoreConfigHE(metaclass=InterfaceMeta):
     __slots__ = (
             'index_depth',
             'index_name_depth_level',
+            'index_constructors',
             'columns_depth',
             'columns_name_depth_level',
+            'columns_constructors',
             'columns_select',
             'dtypes',
             'consolidate_blocks',
@@ -78,8 +83,10 @@ class StoreConfigHE(metaclass=InterfaceMeta):
             # constructors
             index_depth: int = 0, # this default does not permit round trip
             index_name_depth_level: tp.Optional[DepthLevelSpecifier] = None,
+            index_constructors: IndexConstructors = None,
             columns_depth: int = 1,
             columns_name_depth_level: tp.Optional[DepthLevelSpecifier] = None,
+            columns_constructors: IndexConstructors = None,
             columns_select: tp.Optional[tp.Iterable[str]] = None,
             dtypes: DtypesSpecifier = None,
             consolidate_blocks: bool = False,
@@ -108,8 +115,10 @@ class StoreConfigHE(metaclass=InterfaceMeta):
         # constructor
         self.index_depth = index_depth
         self.index_name_depth_level = index_name_depth_level
+        self.index_constructors = index_constructors
         self.columns_depth = columns_depth
         self.columns_name_depth_level = columns_name_depth_level
+        self.columns_constructors = columns_constructors
         self.columns_select = columns_select
         self.dtypes = dtypes
         self.consolidate_blocks = consolidate_blocks
@@ -159,7 +168,7 @@ class StoreConfigHE(metaclass=InterfaceMeta):
         if dtypes_specifier is None :
             return dtypes_specifier
         if isinstance(dtypes_specifier, dict):
-            return dtypes_specifier.items()
+            return tuple(dtypes_specifier.items())
         if isinstance(dtypes_specifier, list):
             return tuple(dtypes_specifier)
         return dtypes_specifier # type: ignore [return-value]
@@ -169,8 +178,10 @@ class StoreConfigHE(metaclass=InterfaceMeta):
             self._hash = hash((
                     self.index_depth, # int
                     self._hash_depth_specifier(self.index_name_depth_level),
+                    self.index_constructors, # class or callable
                     self.columns_depth, # int
                     self._hash_depth_specifier(self.columns_name_depth_level),
+                    self.columns_constructors, # class or callable
                     self.columns_select if self.columns_select is None else tuple(self.columns_select),
                     self._hash_dtypes_specifier(self.dtypes),
                     self.consolidate_blocks, # bool
@@ -222,8 +233,10 @@ class StoreConfig(StoreConfigHE):
     def __init__(self, *,
             index_depth: int = 0,
             index_name_depth_level: tp.Optional[DepthLevelSpecifier] = None,
+            index_constructors: IndexConstructors = None,
             columns_depth: int = 1,
             columns_name_depth_level: tp.Optional[DepthLevelSpecifier] = None,
+            columns_constructors: IndexConstructors = None,
             columns_select: tp.Optional[tp.Iterable[str]] = None,
             dtypes: DtypesSpecifier = None,
             consolidate_blocks: bool = False,
@@ -245,8 +258,10 @@ class StoreConfig(StoreConfigHE):
         StoreConfigHE.__init__(self,
                 index_depth=index_depth,
                 index_name_depth_level=index_name_depth_level,
+                index_constructors=index_constructors,
                 columns_depth=columns_depth,
                 columns_name_depth_level=columns_name_depth_level,
+                columns_constructors=columns_constructors,
                 columns_select=columns_select,
                 dtypes=dtypes,
                 consolidate_blocks=consolidate_blocks,
@@ -409,7 +424,7 @@ def store_coherent_non_write(f: AnyCallable) -> AnyCallable:
 
 
 def store_coherent_write(f: AnyCallable) -> AnyCallable:
-    '''Decorator for dervied Store classes implementation of write()
+    '''Decorator for derived Store classes implementation of write()
     '''
     @wraps(f)
     def wrapper(self: 'Store', *args: tp.Any, **kwargs: tp.Any) -> tp.Any:
@@ -426,7 +441,8 @@ class Store:
 
     __slots__ = (
             '_fp',
-            '_last_modified'
+            '_last_modified',
+            '_weak_cache',
             )
 
     def __init__(self, fp: PathSpecifier):
@@ -440,6 +456,7 @@ class Store:
         self._fp: str = fp
         self._last_modified = np.nan
         self._mtime_update()
+        self._weak_cache: tp.MutableMapping[tp.Hashable, Frame] = WeakValueDictionary()
 
     def _mtime_update(self) -> None:
         if os.path.exists(self._fp):
@@ -457,6 +474,12 @@ class Store:
             # file existed previously and we got a modification time, but now it does not exist
             raise StoreFileMutation(f'expected file {self._fp} no longer exists')
 
+    # def __copy__(self) -> 'Store':
+    #     '''
+    #     Return a new Store instance linked to the same file.
+    #     '''
+    #     return self.__class__(fp=self._fp)
+
     #---------------------------------------------------------------------------
     @staticmethod
     def get_field_names_and_dtypes(*,
@@ -517,12 +540,15 @@ class Store:
         if force_brackets:
             def gen() -> tp.Iterator[str]:
                 for name in field_names:
-                    name = str(name)
-                    if name.startswith('[') and name.endswith(']'):
-                        yield name
+                    name_str = str(name)
+                    if name_str.startswith('[') and name_str.endswith(']'):
+                        yield name_str
+                    elif isinstance(name, tuple):
+                        # make look like it came from the array
+                        yield f'[{" ".join((repr(n) for n in name))}]'
                     else:
-                        yield f'[{name}]'
-            field_names = tuple(gen())
+                        yield f'[{name_str}]'
+            field_names = list(gen())
 
         return field_names, dtypes
 
