@@ -1,9 +1,9 @@
-import unittest
 from datetime import date
 from datetime import datetime
 # from io import StringIO
 import typing as tp
 import numpy as np
+import frame_fixtures as ff
 
 from static_frame.core.frame import Frame
 from static_frame.core.bus import Bus
@@ -22,9 +22,13 @@ from static_frame.test.test_case import TestCase
 from static_frame.test.test_case import temp_file
 from static_frame.test.test_case import skip_win
 
+from static_frame.core.index_auto import IndexAutoFactory
 
 from static_frame.core.exception import ErrorInitBus
 from static_frame.core.exception import StoreFileMutation
+from static_frame.core.exception import ErrorInitIndexNonUnique
+
+from static_frame.core.index_datetime import IndexDate
 
 
 class TestUnit(TestCase):
@@ -48,6 +52,7 @@ class TestUnit(TestCase):
             b1.__dict__ #pylint: disable=W0104
 
     #---------------------------------------------------------------------------
+
     def test_bus_init_a(self) -> None:
 
         f1 = Frame.from_dict(
@@ -94,7 +99,6 @@ class TestUnit(TestCase):
         with self.assertRaises(ErrorInitBus):
             Bus(Series([3, 4], index=('a', 'b'), dtype=object))
 
-
     def test_bus_init_c(self) -> None:
 
         f1 = Frame.from_dict(
@@ -123,6 +127,34 @@ class TestUnit(TestCase):
             self.assertEqualFrames(f2, f2_loaded)
 
     #---------------------------------------------------------------------------
+
+    def test_bus_from_frames_a(self) -> None:
+
+        f1 = Frame.from_dict(
+                dict(a=(1,2), b=(3,4)),
+                index=('x', 'y'),
+                name='foo')
+        f2 = Frame.from_dict(
+                dict(a=(1,2,3), b=(4,5,6)),
+                index=('x', 'y', 'z'),
+                name='foo')
+
+        with self.assertRaises(ErrorInitIndexNonUnique):
+            _ = Bus.from_frames((f1, f2))
+
+    #---------------------------------------------------------------------------
+
+    def test_bus_from_series_a(self) -> None:
+        f1 = ff.parse('s(4,2)').rename('f1')
+        f2 = ff.parse('s(4,5)').rename('f2')
+        s1 = Series((f1, f2), index=('a', 'b'))
+        b1 = Bus.from_series(s1, own_data=True)
+        self.assertEqual(b1.status['shape'].to_pairs(),
+                (('a', (4, 2)), ('b', (4, 5))),
+                )
+
+    #---------------------------------------------------------------------------
+
     def test_bus_rename_a(self) -> None:
         f1 = Frame.from_dict(
                 dict(a=(1,2), b=(3,4)),
@@ -140,6 +172,7 @@ class TestUnit(TestCase):
         self.assertEqual(b2.name, 'bar')
 
     #---------------------------------------------------------------------------
+
     def test_bus_from_items_a(self) -> None:
         f1 = Frame.from_dict(
                 dict(a=(1,2), b=(3,4)),
@@ -153,8 +186,8 @@ class TestUnit(TestCase):
         b1 = Bus.from_items((('a', f1), ('b', f2)))
         self.assertEqual(b1.index.values.tolist(), ['a', 'b'])
 
-
     #---------------------------------------------------------------------------
+
     def test_bus_shapes_a(self) -> None:
         f1 = Frame.from_dict(
                 dict(a=(1,2), b=(3,4)),
@@ -221,7 +254,6 @@ class TestUnit(TestCase):
 
             self.assertEqual(b2.nbytes, 112)
 
-
     @skip_win # type: ignore
     def test_bus_dtypes_a(self) -> None:
         f1 = Frame.from_dict(
@@ -257,7 +289,6 @@ class TestUnit(TestCase):
                     (('b', (('f1', None), ('f2', np.dtype('int64')), ('f3', np.dtype('int64')))), ('c', (('f1', None), ('f2', np.dtype('int64')), ('f3', None))), ('d', (('f1', None), ('f2', None), ('f3', np.dtype('int64')))))
                     )
 
-
     @skip_win # type: ignore
     def test_bus_status_a(self) -> None:
         f1 = Frame.from_dict(
@@ -288,7 +319,6 @@ class TestUnit(TestCase):
                     b2.status.to_pairs(0),                                                           (('loaded', (('f1', True), ('f2', True), ('f3', True))), ('size', (('f1', 4.0), ('f2', 6.0), ('f3', 4.0))), ('nbytes', (('f1', 32.0), ('f2', 48.0), ('f3', 32.0))),('shape', (('f1', (2, 2)), ('f2', (3, 2)), ('f3', (2, 2)))))
             )
 
-
     def test_bus_keys_a(self) -> None:
         f1 = Frame.from_dict(
                 dict(a=(1,2), b=(3,4)),
@@ -323,7 +353,6 @@ class TestUnit(TestCase):
             self.assertEqual(b2.values[2].name, 'f3')
             self.assertTrue(b2._loaded_all)
 
-
     def test_bus_reversed_a(self) -> None:
         f1 = Frame.from_dict(
                 dict(a=(1,2), b=(3,4)),
@@ -341,7 +370,6 @@ class TestUnit(TestCase):
         b1 = Bus.from_frames((f1, f2, f3))
         self.assertEqual(list(reversed(b1)), ['f3', 'f2', 'f1'])
 
-
     def test_bus_display_a(self) -> None:
         f1 = Frame.from_dict(
                 dict(a=(1,2), b=(3,4)),
@@ -377,9 +405,8 @@ class TestUnit(TestCase):
                 include_index=False)).to_rows()
         self.assertEqual(rows2, ['Frame', 'Frame', 'Frame'])
 
-
-
     #---------------------------------------------------------------------------
+
     def test_bus_iloc_a(self) -> None:
         f1 = Frame.from_dict(
                 dict(a=(1,2), b=(3,4)),
@@ -444,7 +471,6 @@ class TestUnit(TestCase):
         self.assertEqual(len(b2), 2)
         self.assertEqual(b2.index.values.tolist(), ['f2', 'f3'])
 
-
     def test_bus_getitem_a(self) -> None:
         f1 = Frame.from_dict(
                 dict(a=(1,2), b=(3,4)),
@@ -470,6 +496,7 @@ class TestUnit(TestCase):
                     )
 
     #---------------------------------------------------------------------------
+
     def test_bus_to_xlsx_a(self) -> None:
         f1 = Frame.from_dict(
                 dict(a=(1,2), b=(3,4)),
@@ -502,7 +529,6 @@ class TestUnit(TestCase):
         for frame in (f1, f2, f3):
             self.assertEqualFrames(frame, b2[frame.name])
 
-
     def test_bus_to_xlsx_b(self) -> None:
 
         f1 = Frame.from_dict(
@@ -526,8 +552,6 @@ class TestUnit(TestCase):
         for frame in (f1, f2):
             self.assertEqualFrames(frame, b2[frame.name])
 
-
-
     def test_bus_to_xlsx_c(self) -> None:
         '''
         Test manipulating a file behind the Bus.
@@ -552,10 +576,9 @@ class TestUnit(TestCase):
 
             f2.to_xlsx(fp)
 
-            with self.assertRaises(StoreFileMutation):
+            with self.assertRaises((StoreFileMutation, KeyError)):
                 tuple(b2.items())
 
-
     def test_bus_to_xlsx_d(self) -> None:
         '''
         Test manipulating a file behind the Bus.
@@ -576,7 +599,6 @@ class TestUnit(TestCase):
         with self.assertRaises(StoreFileMutation):
             tuple(b2.items())
 
-
     def test_bus_to_xlsx_e(self) -> None:
 
         f1 = Frame.from_dict(
@@ -600,7 +622,6 @@ class TestUnit(TestCase):
         for frame in (f1, f2):
             self.assertTrue(frame.equals(b2[frame.name]))
 
-
     def test_bus_to_xlsx_f(self) -> None:
         f = Frame.from_records([
                 [np.datetime64('1983-02-20 05:34:18.763'), np.datetime64('2020-08-01')],
@@ -648,18 +669,23 @@ class TestUnit(TestCase):
                 label_encoder=str,
                 label_decoder=dt64,
                 )
-        b1 = Bus.from_frames((f1, f2), config=config)
+        b1 = Bus.from_frames((f1, f2),
+                config=config,
+                index_constructor=IndexDate)
 
         with temp_file('.xlsx') as fp:
             b1.to_xlsx(fp, config=config)
 
-            b2 = Bus.from_xlsx(fp, config=config)
+            b2 = Bus.from_xlsx(fp,
+                    config=config,
+                    index_constructor=IndexDate)
             tuple(b2.items()) # force loading all
 
         for frame in (f1, f2):
             self.assertEqualFrames(frame, b2[frame.name])
 
     #---------------------------------------------------------------------------
+
     def test_bus_to_sqlite_a(self) -> None:
         f1 = Frame.from_dict(
                 dict(a=(1,2), b=(3,4)),
@@ -686,7 +712,6 @@ class TestUnit(TestCase):
         for frame in frames:
             self.assertEqualFrames(frame, b2[frame.name])
 
-
     def test_bus_to_sqlite_b(self) -> None:
         '''
         Test manipulating a file behind the Bus.
@@ -707,7 +732,6 @@ class TestUnit(TestCase):
         with self.assertRaises(StoreFileMutation):
             tuple(b2.items())
 
-
     def test_bus_to_sqlite_c(self) -> None:
 
         dt64 = np.datetime64
@@ -726,12 +750,14 @@ class TestUnit(TestCase):
                 label_encoder=str,
                 label_decoder=dt64,
                 )
-        b1 = Bus.from_frames((f1, f2), config=config)
+        b1 = Bus.from_frames((f1, f2),
+                config=config,
+                index_constructor=IndexDate)
 
         with temp_file('.db') as fp:
             b1.to_sqlite(fp, config=config)
 
-            b2 = Bus.from_sqlite(fp, config=config)
+            b2 = Bus.from_sqlite(fp, config=config, index_constructor=IndexDate)
             tuple(b2.items()) # force loading all
             self.assertEqual(b2.index.dtype.kind, 'M')
 
@@ -739,9 +765,8 @@ class TestUnit(TestCase):
         for frame in (f1, f2):
             self.assertEqualFrames(frame, b2[frame.name])
 
-
-
     #---------------------------------------------------------------------------
+
     def test_bus_to_hdf5_a(self) -> None:
         f1 = Frame.from_dict(
                 dict(a=(1,2), b=(3,4)),
@@ -768,7 +793,6 @@ class TestUnit(TestCase):
         for frame in frames:
             self.assertEqualFrames(frame, b2[frame.name])
 
-
     def test_bus_to_hdf5_b(self) -> None:
         '''
         Test manipulating a file behind the Bus.
@@ -806,12 +830,12 @@ class TestUnit(TestCase):
                 label_encoder=str,
                 label_decoder=dt64,
                 )
-        b1 = Bus.from_frames((f1, f2), config=config)
+        b1 = Bus.from_frames((f1, f2), config=config, index_constructor=IndexDate)
 
         with temp_file('.h5') as fp:
             b1.to_hdf5(fp, config=config)
 
-            b2 = Bus.from_hdf5(fp, config=config)
+            b2 = Bus.from_hdf5(fp, config=config, index_constructor=IndexDate)
             tuple(b2.items()) # force loading all
             self.assertEqual(b2.index.dtype.kind, 'M')
 
@@ -821,7 +845,6 @@ class TestUnit(TestCase):
 
     #---------------------------------------------------------------------------
 
-
     def test_bus_equals_a(self) -> None:
 
         f1 = Frame.from_dict(
@@ -897,7 +920,6 @@ class TestUnit(TestCase):
         self.assertTrue(b1.equals(b2, compare_class=False))
         self.assertFalse(b1.equals('foo', compare_class=False))
 
-
     def test_bus_equals_c(self) -> None:
 
         f1 = Frame.from_dict(
@@ -918,10 +940,8 @@ class TestUnit(TestCase):
         self.assertTrue(b1.equals(b2))
         self.assertFalse(b1.equals(b2, compare_name=True))
 
-
     #---------------------------------------------------------------------------
 
-
     def test_bus_interface_b(self) -> None:
         f1 = Frame.from_dict(
                 dict(a=(1,2), b=(3,4)),
@@ -941,7 +961,6 @@ class TestUnit(TestCase):
         self.assertTrue(isinstance(post2, Frame))
         self.assertTrue(post2.shape, (41, 3)) #type: ignore
 
-
     #---------------------------------------------------------------------------
 
     def test_bus_mloc_a(self) -> None:
@@ -962,7 +981,6 @@ class TestUnit(TestCase):
             self.assertEqual(mloc.to_pairs(),
                     (('f1', None),))
 
-
     def test_bus_mloc_b(self) -> None:
 
         f1 = Frame.from_dict(
@@ -1017,6 +1035,7 @@ class TestUnit(TestCase):
             self.assertEqual(mloc1['f2'], b2.mloc.loc['f2'])
 
     #---------------------------------------------------------------------------
+
     def test_bus_update_series_cache_iloc(self) -> None:
 
         f1 = Frame.from_dict(
@@ -1035,6 +1054,7 @@ class TestUnit(TestCase):
             b1._update_series_cache_iloc(1)
 
     #---------------------------------------------------------------------------
+
     def test_bus_extract_loc_a(self) -> None:
         f1 = Frame.from_dict(
                 dict(a=(1,2), b=(3,4)),
@@ -1075,8 +1095,8 @@ class TestUnit(TestCase):
             self.assertEqual(b3.index.values.tolist(),
                     [('a', 1), ('b', 2), ('b', 1)])
 
-
     #---------------------------------------------------------------------------
+
     def test_bus_values_a(self) -> None:
 
         f1 = Frame.from_dict(
@@ -1094,6 +1114,7 @@ class TestUnit(TestCase):
         self.assertTrue(f2.equals(post[1]))
 
     #---------------------------------------------------------------------------
+
     def test_bus_to_parquet_a(self) -> None:
         f1 = Frame.from_dict(
                 dict(a=(1,2), b=(3,4)),
@@ -1126,7 +1147,6 @@ class TestUnit(TestCase):
             # parquet brings in characters as objects, thus forcing different dtypes
             self.assertEqualFrames(frame, b2[frame.name], compare_dtype=False)
 
-
     def test_bus_to_parquet_b(self) -> None:
 
         dt64 = np.datetime64
@@ -1153,19 +1173,40 @@ class TestUnit(TestCase):
                 label_decoder=dt64
                 )
 
-        b1 = Bus.from_frames((f1, f2, f3), config=config)
+        b1 = Bus.from_frames((f1, f2, f3), config=config, index_constructor=IndexDate)
         self.assertEqual(b1.index.dtype.kind, 'M')
 
         with temp_file('.zip') as fp:
             b1.to_zip_parquet(fp, config=config)
 
-            b2 = Bus.from_zip_parquet(fp, config=config)
+            b2 = Bus.from_zip_parquet(fp,
+                    config=config,
+                    index_constructor=IndexDate)
             self.assertEqual(b2.index.dtype.kind, 'M')
 
             key = dt64('2020-01-01')
             self.assertEqualFrames(b1[key], b2[key], compare_dtype=False)
 
+    def test_bus_to_parquet_c(self) -> None:
+        f1 = ff.parse('s(4,4)|i(ID,dtD)|v(int)').rename('a')
+        f2 = ff.parse('s(4,4)|i(ID,dtD)|v(int)').rename('b')
+
+        config = StoreConfig(
+                index_depth=1,
+                include_index=True,
+                index_constructors=IndexDate,
+                columns_depth=1,
+                include_columns=True,
+        )
+        b1 = Bus.from_frames((f1, f2))
+        with temp_file('.zip') as fp:
+            b1.to_zip_parquet(fp, config=config)
+            b2 = Bus.from_zip_parquet(fp, config=config)
+            self.assertIs(b2['a'].index.__class__, IndexDate)
+            self.assertIs(b2['b'].index.__class__, IndexDate)
+
     #---------------------------------------------------------------------------
+
     def test_bus_max_persist_a(self) -> None:
         def items() -> tp.Iterator[tp.Tuple[str, Frame]]:
             for i in range(20):
@@ -1193,7 +1234,6 @@ class TestUnit(TestCase):
             self.assertEqual(b2._loaded.tolist(),
                     [False, False, False, False, False, False, False, False, False, False, False, False, False, False, False, False, False, True, True, True])
 
-
     def test_bus_max_persist_b(self) -> None:
         def items() -> tp.Iterator[tp.Tuple[str, Frame]]:
             for i in range(20):
@@ -1274,8 +1314,6 @@ class TestUnit(TestCase):
             self.assertEqual(b3._loaded.tolist(),
                     [True, False, True, True])
 
-
-
     def test_bus_max_persist_d(self) -> None:
         def items() -> tp.Iterator[tp.Tuple[str, Frame]]:
             for i in range(5):
@@ -1328,7 +1366,6 @@ class TestUnit(TestCase):
             self.assertEqual(b2._loaded.tolist(),
                     [True, False, False, True, True])
 
-
     def test_bus_max_persist_e(self) -> None:
         def items() -> tp.Iterator[tp.Tuple[str, Frame]]:
             for i in range(4):
@@ -1364,8 +1401,6 @@ class TestUnit(TestCase):
             self.assertEqual(list(b2._last_accessed.keys()),
                     ['3', '0', '1', '2'])
 
-
-
     def test_bus_max_persist_f(self) -> None:
         def items() -> tp.Iterator[tp.Tuple[str, Frame]]:
             for i in range(5):
@@ -1393,7 +1428,6 @@ class TestUnit(TestCase):
             self.assertEqual(b2._loaded.tolist(),
                     [False, False, False, False, True])
 
-
     def test_bus_max_persist_g(self) -> None:
         def items() -> tp.Iterator[tp.Tuple[str, Frame]]:
             for i in range(5):
@@ -1420,9 +1454,69 @@ class TestUnit(TestCase):
             self.assertEqual(b2._loaded.tolist(),
                     [False, False, False, False, True])
 
+    def test_bus_max_persist_h(self) -> None:
+        f1 = ff.parse('s(4,2)').rename('f1')
+        f2 = ff.parse('s(4,5)').rename('f2')
+        f3 = ff.parse('s(2,2)').rename('f3')
+        s1 = Series((f1, f2, f3), index=('a', 'b', 'c'))
+        with self.assertRaises(ErrorInitBus):
+            # max_persist cannot be less than the number of already loaded Frames
+            Bus(s1, max_persist=2)
 
+    def test_bus_max_persist_i(self) -> None:
+        f1 = ff.parse('s(4,2)').rename('f1')
+        f2 = ff.parse('s(4,5)').rename('f2')
+        f3 = ff.parse('s(2,2)').rename('f3')
+        f4 = ff.parse('s(2,8)').rename('f4')
+        f5 = ff.parse('s(4,4)').rename('f5')
+        f6 = ff.parse('s(6,4)').rename('f6')
+
+        b1 = Bus.from_frames((f1, f2, f3, f4, f5, f6))
+
+        config = StoreConfig(
+                index_depth=1,
+                columns_depth=1,
+                include_columns=True,
+                include_index=True
+                )
+
+        with temp_file('.zip') as fp:
+            b1.to_zip_pickle(fp)
+            b2 = Bus.from_zip_pickle(fp, config=config, max_persist=2)
+            # NOTE: this type of selection forces _store_reader to use read_many at size of max_persist
+            b3 = b2[['f1', 'f2', 'f3', 'f4', 'f5']]
+            self.assertEqual(b3.status.loc[b3.status['loaded'], 'shape'].to_pairs(),
+                (('f4', (2, 8)), ('f5', (4, 4))),
+                )
+
+    def test_bus_max_persist_j(self) -> None:
+        f1 = ff.parse('s(4,2)').rename('f1')
+        f2 = ff.parse('s(4,5)').rename('f2')
+        f3 = ff.parse('s(2,2)').rename('f3')
+        f4 = ff.parse('s(2,8)').rename('f4')
+        f5 = ff.parse('s(4,4)').rename('f5')
+        f6 = ff.parse('s(6,4)').rename('f6')
+
+        b1 = Bus.from_frames((f1, f2, f3, f4, f5, f6))
+
+        config = StoreConfig(
+                index_depth=1,
+                columns_depth=1,
+                include_columns=True,
+                include_index=True
+                )
+
+        with temp_file('.zip') as fp:
+            b1.to_zip_pickle(fp)
+            b2 = Bus.from_zip_pickle(fp, config=config, max_persist=2)
+            # NOTE: this type of selection forces _store_reader to use read_many at size of max_persist
+            a1 = b2.values
+            self.assertNotEqual(id(a1), id(b2._values_mutable))
+            self.assertEqual(b2.status['loaded'].sum(), 2)
+            self.assertTrue(all([f.__class__ is Frame for f in a1]))
 
     #---------------------------------------------------------------------------
+
     def test_bus_sort_index_a(self) -> None:
         f1 = Frame.from_dict(
                 dict(a=(1,2), b=(3,4)),
@@ -1458,6 +1552,7 @@ class TestUnit(TestCase):
                     (('a', (('x', 1), ('y', 2))), ('b', (('x', 3), ('y', 4)))))
 
     #---------------------------------------------------------------------------
+
     def test_bus_sort_values_a(self) -> None:
         f1 = Frame.from_dict(
                 dict(a=(1,2), b=(30000,4)),
@@ -1476,13 +1571,39 @@ class TestUnit(TestCase):
         with temp_file('.zip') as fp:
             b1.to_zip_pickle(fp)
             b2 = Bus.from_zip_pickle(fp)
-            b3 = b2.sort_values(key=lambda b: b.iter_element().apply(lambda f: f.sum().sum()))
 
+
+            b3 = b2.sort_values(key=lambda b: b.iter_element().apply(lambda f: f.sum().sum()))
             self.assertEqual(b3.index.values.tolist(),
                 ['f3', 'f2', 'f1']
                 )
 
     #---------------------------------------------------------------------------
+
+    def test_bus_iter_element_apply_a(self) -> None:
+        f1 = Frame.from_dict(
+                dict(a=(1,2), b=(30000,4)),
+                index=('x', 'y'),
+                name='f1')
+        f2 = Frame.from_dict(
+                dict(c=(1,2,3), b=(4,5,6)),
+                index=('x', 'y', 'z'),
+                name='f2')
+        f3 = Frame.from_dict(
+                dict(d=(1,2), b=(5,6)),
+                index=('p', 'q'),
+                name='f3')
+
+        b1 = Bus.from_frames((f3, f2, f1))
+        b2 = b1.iter_element().apply(lambda f: f.sum().sum())
+
+        self.assertEqual(b2.to_pairs(),
+                (('f3', 14), ('f2', 21), ('f1', 30007)))
+
+        self.assertEqual(id(b1.index), id(b2.index))
+
+    #---------------------------------------------------------------------------
+
     def test_bus_drop_a(self) -> None:
         f1 = Frame.from_dict(
                 dict(a=(1,2), b=(3,4)),
@@ -1509,6 +1630,7 @@ class TestUnit(TestCase):
                     )
 
     #---------------------------------------------------------------------------
+
     def test_bus_from_dict_a(self) -> None:
         f1 = Frame.from_dict(
                 dict(a=(1,2), b=(3,4)),
@@ -1524,6 +1646,7 @@ class TestUnit(TestCase):
                 ['a', 'b'])
 
     #---------------------------------------------------------------------------
+
     def test_bus_iter_element_a(self) -> None:
         f1 = Frame.from_dict(
                 dict(a=(1,2), b=(3,4)),
@@ -1541,7 +1664,102 @@ class TestUnit(TestCase):
             (('c', (('x', 1), ('y', 2), ('z', 3))), ('b', (('x', 4), ('y', 5), ('z', 6))))
             )
 
+    def test_bus_iter_element_b(self) -> None:
+        f1 = ff.parse('s(4,2)').rename('f1')
+        f2 = ff.parse('s(4,5)').rename('f2')
+        f3 = ff.parse('s(2,2)').rename('f3')
+        f4 = ff.parse('s(2,8)').rename('f4')
+        f5 = ff.parse('s(4,4)').rename('f5')
+        f6 = ff.parse('s(6,4)').rename('f6')
+
+        b1 = Bus.from_frames((f1, f2, f3, f4, f5, f6))
+        config = StoreConfig(
+                index_depth=1,
+                columns_depth=1,
+                include_columns=True,
+                include_index=True
+                )
+
+        with temp_file('.zip') as fp:
+            b1.to_zip_pickle(fp)
+            b2 = Bus.from_zip_pickle(fp, config=config, max_persist=2)
+
+            frames = list(b2.iter_element())
+            self.assertTrue(all(x.__class__ is Frame for x in frames))
+            self.assertEqual(len(frames), 6)
+            self.assertEqual( b2.status['loaded'].sum(), 2)
+
+    def test_bus_iter_element_c(self) -> None:
+        f1 = ff.parse('s(4,2)').rename('f1')
+        f2 = ff.parse('s(4,5)').rename('f2')
+        f3 = ff.parse('s(2,2)').rename('f3')
+        f4 = ff.parse('s(2,8)').rename('f4')
+        f5 = ff.parse('s(4,4)').rename('f5')
+        f6 = ff.parse('s(6,4)').rename('f6')
+
+        b1 = Bus.from_frames((f1, f2, f3, f4, f5, f6))
+        config = StoreConfig(
+                index_depth=1,
+                columns_depth=1,
+                include_columns=True,
+                include_index=True
+                )
+
+        with temp_file('.zip') as fp:
+            b1.to_zip_pickle(fp)
+            b2 = Bus.from_zip_pickle(fp, config=config, max_persist=1)
+
+            frames = list(b2.iter_element())
+            self.assertTrue(all(x.__class__ is Frame for x in frames))
+            self.assertEqual(len(frames), 6)
+            self.assertEqual( b2.status['loaded'].sum(), 1)
+
+    def test_bus_iter_element_d(self) -> None:
+        f1 = ff.parse('s(4,2)').rename('f1')
+        f2 = ff.parse('s(4,5)').rename('f2')
+        f3 = ff.parse('s(2,2)').rename('f3')
+        f4 = ff.parse('s(2,8)').rename('f4')
+        f5 = ff.parse('s(4,4)').rename('f5')
+        f6 = ff.parse('s(6,4)').rename('f6')
+
+        b1 = Bus.from_frames((f1, f2, f3, f4, f5, f6))
+        config = StoreConfig(
+                index_depth=1,
+                columns_depth=1,
+                include_columns=True,
+                include_index=True
+                )
+
+        with temp_file('.zip') as fp:
+            b1.to_zip_pickle(fp)
+            b2 = Bus.from_zip_pickle(fp, config=config)
+
+            frames = list(b2.iter_element())
+            self.assertTrue(all(x.__class__ is Frame for x in frames))
+            self.assertEqual(len(frames), 6)
+            self.assertEqual( b2.status['loaded'].sum(), 6)
+
     #---------------------------------------------------------------------------
+
+    def test_bus_iter_element_items_a(self) -> None:
+        f1 = Frame.from_dict(
+                dict(a=(1,2), b=(3,4)),
+                index=('x', 'y'),
+                name='f1')
+        f2 = Frame.from_dict(
+                dict(c=(1,2,3), b=(4,5,6)),
+                index=('x', 'y', 'z'),
+                name='f2')
+
+        b1 = Bus.from_dict(dict(a=f1, b=f2))
+        post = tuple(b1.iter_element_items())
+        self.assertEqual(len(post), 2)
+        self.assertEqual([pair[0] for pair in post], ['a', 'b'])
+        self.assertTrue(post[0][1].equals(f1))
+        self.assertTrue(post[1][1].equals(f2))
+
+    #---------------------------------------------------------------------------
+
     def test_bus_reindex_a(self) -> None:
         f1 = Frame.from_dict(
                 dict(a=(1,2), b=(3,4)),
@@ -1561,7 +1779,37 @@ class TestUnit(TestCase):
         self.assertTrue(b2['f4'].equals(f3))
         self.assertTrue(b2.__class__ is Bus)
 
+    def test_bus_reindex_b(self) -> None:
+        def items() -> tp.Iterator[tp.Tuple[str, Frame]]:
+            for i in range(5):
+                yield 'abcde'[i], Frame(np.arange(i, i+10).reshape(2, 5))
+
+        s = Series.from_items(items(), dtype=object)
+        b1 = Bus(s)
+
+        config = StoreConfig(
+                index_depth=1,
+                columns_depth=1,
+                include_columns=True,
+                include_index=True
+                )
+
+        with temp_file('.zip') as fp:
+            b1.to_zip_pickle(fp)
+            # set max_persist to size to test when fully loaded with max_persist
+            b2 = Bus.from_zip_pickle(fp, config=config, max_persist=1)
+
+            b3 = b2.reindex(('c', 'd', 'q'), fill_value=Frame())
+            f1 = b3['c']
+            # this would fail if the Store was still associated with the Bus
+            f2 = b3['q']
+
+            self.assertEqual(b3._max_persist, None)
+            self.assertEqual(b3._store, None)
+            self.assertEqual(b2._max_persist, 1)
+
     #---------------------------------------------------------------------------
+
     def test_bus_relabel_a(self) -> None:
         f1 = Frame.from_dict(
                 dict(a=(1,2), b=(3,4)),
@@ -1583,6 +1831,7 @@ class TestUnit(TestCase):
         self.assertTrue(b2.__class__ is Bus)
 
     #---------------------------------------------------------------------------
+
     def test_bus_relabel_flat_a(self) -> None:
         f1 = Frame.from_dict(
                 dict(a=(1,2), b=(3,4)),
@@ -1606,6 +1855,7 @@ class TestUnit(TestCase):
         self.assertTrue(b2.__class__ is Bus)
 
     #---------------------------------------------------------------------------
+
     def test_bus_relabel_level_drop_a(self) -> None:
         f1 = Frame.from_dict(
                 dict(a=(1,2), b=(3,4)),
@@ -1627,6 +1877,7 @@ class TestUnit(TestCase):
         self.assertTrue(b3.index.equals(b1.index))
 
     #---------------------------------------------------------------------------
+
     def test_bus_relabel_rehierarch_a(self) -> None:
         f1 = Frame.from_dict(
                 dict(a=(1,2), b=(3,4)),
@@ -1648,6 +1899,7 @@ class TestUnit(TestCase):
                 [['f3', 'X'], ['f2', 'X'], ['f1', 'X']])
 
     #---------------------------------------------------------------------------
+
     def test_bus_roll_a(self) -> None:
         f1 = Frame.from_dict(
                 dict(a=(1,2), b=(3,4)),
@@ -1669,9 +1921,8 @@ class TestUnit(TestCase):
                 )
         self.assertTrue(b2['f2'].equals(b1['f2']))
 
-
-
     #---------------------------------------------------------------------------
+
     def test_bus_shift_a(self) -> None:
         f1 = Frame.from_dict(
                 dict(a=(1,2), b=(3,4)),
@@ -1692,11 +1943,270 @@ class TestUnit(TestCase):
         self.assertTrue(b2['f2'].equals(b1['f1']))
         self.assertTrue(b2['f1'].equals(b1['f1']))
 
+    #---------------------------------------------------------------------------
+
+    def test_bus_from_concat_a(self) -> None:
+        def items() -> tp.Iterator[tp.Tuple[str, Frame]]:
+            for i in range(5):
+                yield str(i), Frame(np.arange(i, i+10).reshape(2, 5))
+
+        s = Series.from_items(items(), dtype=object)
+        b1 = Bus(s)
+
+        config = StoreConfig(
+                index_depth=1,
+                columns_depth=1,
+                include_columns=True,
+                include_index=True
+                )
+
+        with temp_file('.zip') as fp:
+            b1.to_zip_pickle(fp)
+            b2 = Bus.from_zip_pickle(fp, config=config, max_persist=1)
+
+            b3 = b1.relabel(('a', 'b', 'c', 'd', 'e'))
+
+            # fully loaded in memory
+            b4 = Bus.from_concat((b2, b3))
+
+            self.assertEqual(b4.status['loaded'].to_pairs(),
+                    (('0', True), ('1', True), ('2', True), ('3', True), ('4', True), ('a', True), ('b', True), ('c', True), ('d', True), ('e', True))
+                    )
+            self.assertEqual(b2.status['loaded'].sum(), 1)
+
+            b5 = Bus.from_concat((b2, b3), name='foo', index=IndexAutoFactory)
+
+            self.assertEqual(b5.status['loaded'].to_pairs(),
+                    ((0, True), (1, True), (2, True), (3, True), (4, True), (5, True), (6, True), (7, True), (8, True), (9, True)))
+            self.assertEqual(b5.name, 'foo')
+
+    def test_bus_from_concat_b(self) -> None:
+        def items() -> tp.Iterator[tp.Tuple[str, Frame]]:
+            for i in range(5):
+                yield str(i), Frame(np.arange(i, i+10).reshape(2, 5))
+
+        s = Series.from_items(items(), dtype=object)
+        b1 = Bus(s)
+
+        config = StoreConfig(
+                index_depth=1,
+                columns_depth=1,
+                include_columns=True,
+                include_index=True
+                )
+
+        with temp_file('.zip') as fp:
+            b1.to_zip_pickle(fp)
+            # set max_persist to size to test when fully loaded with max_persist
+            b2 = Bus.from_zip_pickle(fp, config=config, max_persist=5)
+
+            b3 = b1.relabel(('a', 'b', 'c', 'd', 'e'))
+
+            # fully loaded in memory
+            b4 = Bus.from_concat((b2, b3))
+
+            self.assertEqual(b4.status['loaded'].to_pairs(),
+                    (('0', True), ('1', True), ('2', True), ('3', True), ('4', True), ('a', True), ('b', True), ('c', True), ('d', True), ('e', True))
+                    )
+            self.assertEqual(b2.status['loaded'].sum(), 5)
+
+    def test_bus_from_concat_c(self) -> None:
+        def items() -> tp.Iterator[tp.Tuple[str, Frame]]:
+            for i in range(5):
+                yield str(i), Frame(np.arange(i, i+10).reshape(2, 5))
+
+        s = Series.from_items(items(), dtype=object)
+        b1 = Bus(s)
+
+        config = StoreConfig(
+                index_depth=1,
+                columns_depth=1,
+                include_columns=True,
+                include_index=True
+                )
+
+        with temp_file('.zip') as fp:
+            b1.to_zip_pickle(fp)
+            # set max_persist to size to test when fully loaded with max_persist
+            b2 = Bus.from_zip_pickle(fp, config=config, max_persist=1)
+
+            s1 = b2.to_series()
+            self.assertEqual(b2.status['loaded'].sum(), 1)
+
+            self.assertEqual(
+                    [f.shape for f in s1.values],
+                    [(2, 5), (2, 5), (2, 5), (2, 5), (2, 5)]
+                    )
+
+    #---------------------------------------------------------------------------
+
+    def test_bus_contains_a(self) -> None:
+        f1 = ff.parse('s(4,2)').rename('f1')
+        f2 = ff.parse('s(4,5)').rename('f2')
+        b1 = Bus.from_frames((f1, f2))
+
+        self.assertTrue('f1' in b1)
+        self.assertFalse('f3' in b1)
+
+    #---------------------------------------------------------------------------
+
+    def test_bus_get_a(self) -> None:
+        f1 = ff.parse('s(4,2)').rename('f1')
+        f2 = ff.parse('s(4,5)').rename('f2')
+        b1 = Bus.from_frames((f1, f2))
+
+        self.assertTrue(b1.get('f1').equals(f1))
+        self.assertEqual(b1.get('f3'), None)
+
+    #---------------------------------------------------------------------------
+
+    def test_bus_head_a(self) -> None:
+        f1 = ff.parse('s(4,2)').rename('f1')
+        f2 = ff.parse('s(4,5)').rename('f2')
+        f3 = ff.parse('s(2,2)').rename('f3')
+        f4 = ff.parse('s(2,8)').rename('f4')
+        f5 = ff.parse('s(4,4)').rename('f5')
+        f6 = ff.parse('s(6,4)').rename('f6')
+
+        b1 = Bus.from_frames((f1, f2, f3, f4, f5, f6))
+
+        self.assertEqual(b1.head().index.values.tolist(), ['f1', 'f2', 'f3', 'f4', 'f5'])
+
+    #---------------------------------------------------------------------------
+
+    def test_bus_tail_a(self) -> None:
+        f1 = ff.parse('s(4,2)').rename('f1')
+        f2 = ff.parse('s(4,5)').rename('f2')
+        f3 = ff.parse('s(2,2)').rename('f3')
+        f4 = ff.parse('s(2,8)').rename('f4')
+        f5 = ff.parse('s(4,4)').rename('f5')
+        f6 = ff.parse('s(6,4)').rename('f6')
+
+        b1 = Bus.from_frames((f1, f2, f3, f4, f5, f6))
+
+        self.assertEqual(b1.tail().index.values.tolist(), ['f2', 'f3', 'f4', 'f5', 'f6'])
+
+    #---------------------------------------------------------------------------
+
+    def test_bus_unpersist_a(self) -> None:
+        f1 = ff.parse('s(4,2)').rename('f1')
+        f2 = ff.parse('s(4,5)').rename('f2')
+        f3 = ff.parse('s(2,2)').rename('f3')
+        f4 = ff.parse('s(2,8)').rename('f4')
+        f5 = ff.parse('s(4,4)').rename('f5')
+        f6 = ff.parse('s(6,4)').rename('f6')
+
+        b1 = Bus.from_frames((f1, f2, f3, f4, f5, f6))
+
+        config = StoreConfig(
+                index_depth=1,
+                columns_depth=1,
+                include_columns=True,
+                include_index=True
+                )
+
+        with temp_file('.zip') as fp:
+            b1.to_zip_pickle(fp)
+            # set max_persist to size to test when fully loaded with max_persist
+            b2 = Bus.from_zip_pickle(fp, config=config, max_persist=3)
+            b3 = b2['f2':]
+            self.assertEqual(b2.status['loaded'].sum(), 3)
+            b2.unpersist()
+            self.assertEqual(b2.status['loaded'].sum(), 0)
+            self.assertEqual(b2['f6'].shape, (6, 4))
+            self.assertEqual(b2.status['loaded'].sum(), 1)
+
+            self.assertEqual(b3.status['loaded'].sum(), 3)
+
+    def test_bus_unpersist_b(self) -> None:
+        f1 = ff.parse('s(4,2)').rename('f1')
+        f2 = ff.parse('s(4,5)').rename('f2')
+        f3 = ff.parse('s(2,2)').rename('f3')
+        f4 = ff.parse('s(2,8)').rename('f4')
+        f5 = ff.parse('s(4,4)').rename('f5')
+        f6 = ff.parse('s(6,4)').rename('f6')
+
+        b1 = Bus.from_frames((f1, f2, f3, f4, f5, f6))
+        b1.unpersist()
+
+        config = StoreConfig(
+                index_depth=1,
+                columns_depth=1,
+                include_columns=True,
+                include_index=True
+                )
+
+        with temp_file('.zip') as fp:
+            b1.to_zip_pickle(fp)
+            # set max_persist to size to test when fully loaded with max_persist
+            b2 = Bus.from_zip_pickle(fp, config=config, max_persist=2)
+            list(b2.items())
+            list(b2.items())
+            self.assertEqual(b2.status['loaded'].sum(), 2)
+
+            b2.unpersist()
+            self.assertEqual(b2.status['loaded'].sum(), 0)
 
 
+    def test_bus_unpersist_c(self) -> None:
+        f1 = ff.parse('s(4,2)').rename('f1')
+        f2 = ff.parse('s(4,5)').rename('f2')
+        f3 = ff.parse('s(2,2)').rename('f3')
+        f4 = ff.parse('s(2,8)').rename('f4')
+        f5 = ff.parse('s(4,4)').rename('f5')
+        f6 = ff.parse('s(6,4)').rename('f6')
+
+        b1 = Bus.from_frames((f1, f2, f3, f4, f5, f6))
+        b1.unpersist()
+
+        config = StoreConfig(
+                index_depth=1,
+                columns_depth=1,
+                include_columns=True,
+                include_index=True
+                )
+
+        with temp_file('.zip') as fp:
+            b1.to_zip_pickle(fp)
+            # set max_persist to size to test when fully loaded with max_persist
+            b2 = Bus.from_zip_pickle(fp, config=config)
+            list(b2.items())
+            self.assertEqual(b2.status['loaded'].sum(), 6)
+
+            b2.unpersist()
+            self.assertEqual(b2.status['loaded'].sum(), 0)
+
+            b3 = b2[['f2', 'f5', 'f6']]
+            self.assertEqual(b3.status['loaded'].sum(), 3)
+            self.assertEqual(b2.status['loaded'].sum(), 3)
+
+            b2.unpersist()
+            self.assertEqual(b2.status['loaded'].sum(), 0)
+
+    #---------------------------------------------------------------------------
+
+    def test_bus_npz_a(self) -> None:
+        f1 = ff.parse('s(4,2)').rename('f1')
+        f2 = ff.parse('s(4,5)').rename('f2')
+        f3 = ff.parse('s(2,2)').rename('f3')
+        f4 = ff.parse('s(2,8)').rename('f4')
+        f5 = ff.parse('s(4,4)').rename('f5')
+        f6 = ff.parse('s(6,4)').rename('f6')
+
+        b1 = Bus.from_frames((f1, f2, f3, f4, f5, f6))
+
+        config = StoreConfig()
+
+        with temp_file('.zip') as fp:
+            b1.to_zip_npz(fp)
+            # set max_persist to size to test when fully loaded with max_persist
+            b2 = Bus.from_zip_npz(fp, config=config, max_persist=3)
+            b3 = b2['f2':]
+            self.assertEqual(b3['f5'].to_pairs(),
+                ((0, ((0, 1930.4), (1, -1760.34), (2, 1857.34), (3, 1699.34))), (1, ((0, -610.8), (1, 3243.94), (2, -823.14), (3, 114.58))), (2, ((0, 694.3), (1, -72.96), (2, 1826.02), (3, 604.1))), (3, ((0, 1080.4), (1, 2580.34), (2, 700.42), (3, 3338.48))))
+                )
 
 
 if __name__ == '__main__':
-
+    import unittest
     unittest.main()
-
