@@ -1,9 +1,7 @@
-import unittest
 import datetime
 
 import numpy as np
 
-
 from static_frame.core.container_util import bloc_key_normalize
 from static_frame.core.container_util import get_col_dtype_factory
 from static_frame.core.container_util import index_from_optional_constructor
@@ -16,11 +14,16 @@ from static_frame.core.container_util import pandas_to_numpy
 from static_frame.core.container_util import pandas_version_under_1
 from static_frame.core.container_util import apex_to_name
 from static_frame.core.container_util import apply_binary_operator_blocks_columnar
+from static_frame.core.container_util import container_to_exporter_attr
+from static_frame.core.container_util import get_block_match
+
+
+from static_frame.core.frame import FrameHE
 
 from static_frame.test.test_case import TestCase
-
 from static_frame.core.exception import AxisInvalid
 
+
 from static_frame import Frame
 from static_frame import Index
 from static_frame import IndexDate
@@ -33,7 +36,6 @@ from static_frame import Series
 
 class TestUnit(TestCase):
 
-
     def test_is_static_a(self) -> None:
         self.assertTrue(is_static(Index))
         self.assertFalse(is_static(IndexGO))
@@ -57,7 +59,6 @@ class TestUnit(TestCase):
         self.assertFalse(is_static(IndexHierarchyGO.from_tree))
         self.assertFalse(is_static(IndexHierarchyGO.from_index_items))
 
-
     def test_index_from_optional_constructor_a(self) -> None:
         idx1 = index_from_optional_constructor([1, 3, 4],
                 default_constructor=Index)
@@ -79,7 +80,6 @@ class TestUnit(TestCase):
                 default_constructor=Index)
         self.assertEqual(idx4.__class__, IndexSecond)
 
-
     def test_index_from_optional_constructor_b(self) -> None:
         idx0 = IndexHierarchy.from_labels(
                 [('a', 0), ('a', 1), ('b', 0), ('b', 1)])
@@ -90,7 +90,6 @@ class TestUnit(TestCase):
         # Since the default constructo is static, we should be able to reuse the index
         self.assertEqual(id(idx0), id(idx1))
 
-
     def test_index_from_optional_constructor_c(self) -> None:
         idx0 = IndexHierarchyGO.from_labels(
                 [('a', 0), ('a', 1), ('b', 0), ('b', 1)])
@@ -102,7 +101,6 @@ class TestUnit(TestCase):
         self.assertNotEqual(id(idx0), id(idx1))
         self.assertTrue(idx1.STATIC)
 
-
     def test_index_from_optional_constructor_d(self) -> None:
         idx0 = IndexHierarchy.from_labels(
                 [('a', 0), ('a', 1), ('b', 0), ('b', 1)])
@@ -114,8 +112,6 @@ class TestUnit(TestCase):
         self.assertNotEqual(id(idx0), id(idx1))
         self.assertFalse(idx1.STATIC)
 
-
-
     def test_matmul_a(self) -> None:
         # lhs: frame, rhs: array
 
@@ -143,7 +139,6 @@ class TestUnit(TestCase):
                 (('x', 15), ('y', 22), ('z', 29))
                 )
 
-
     def test_matmul_b(self) -> None:
         # lhs: frame, rhs: array
 
@@ -172,7 +167,6 @@ class TestUnit(TestCase):
         with self.assertRaises(RuntimeError):
             matmul(f1, np.arange(20).reshape(5, 4))
 
-
     def test_matmul_d(self) -> None:
         # lhs: series, rhs: frame
 
@@ -200,7 +194,6 @@ class TestUnit(TestCase):
         self.assertEqual(matmul(s1, s2), 98)
         self.assertEqual(matmul(s1, s2.values), 98)
 
-
     def test_matmul_f(self) -> None:
         # lhs: array 1D, rhs: array 2D, Frame
 
@@ -214,7 +207,6 @@ class TestUnit(TestCase):
                 (('a', 26), ('b', 50))
                 )
 
-
     def test_matmul_g(self) -> None:
         # lhs: array 1D, rhs: array 1D, Series
 
@@ -225,7 +217,6 @@ class TestUnit(TestCase):
         with self.assertRaises(RuntimeError):
             self.assertEqual(matmul(s1, [10, 11]), 98)
 
-
     def test_matmul_h(self) -> None:
         # lhs: array 2D, rhs: array 2D, Frame
 
@@ -244,7 +235,6 @@ class TestUnit(TestCase):
         with self.assertRaises(RuntimeError):
             matmul(f1, np.arange(25).reshape(5, 5))
 
-
     def test_matmul_i(self) -> None:
         import itertools as it
 
@@ -261,7 +251,6 @@ class TestUnit(TestCase):
                 elif isinstance(post, np.ndarray):
                     self.assertTrue(post.tolist(), (pair[0].values @ pair[1].values).tolist())
 
-
     def test_matmul_j(self) -> None:
 
         f1 = Frame.from_dict(dict(a=(1, 2, 3), b=(5, 6, 7)),
@@ -272,8 +261,8 @@ class TestUnit(TestCase):
         with self.assertRaises(RuntimeError):
             _ = matmul(a1, f1)
 
-
     #---------------------------------------------------------------------------
+
     def test_key_to_ascending_key_a(self) -> None:
         self.assertEqual(key_to_ascending_key([9, 5, 1], 3), [1, 5, 9])
         self.assertEqual(key_to_ascending_key(np.array([9, 5, 1]), 3).tolist(), [1, 5, 9]) # type: ignore
@@ -291,7 +280,6 @@ class TestUnit(TestCase):
         f2 = key_to_ascending_key(f1, f1.shape[1])
         self.assertEqual(f2.columns.values.tolist(), ['a', 'b']) # type: ignore
 
-
     def test_key_to_ascending_key_b(self) -> None:
 
         with self.assertRaises(RuntimeError):
@@ -321,8 +309,6 @@ class TestUnit(TestCase):
             a4 = pandas_to_numpy(s2[:2], own_data=False)
             self.assertEqual(a4.dtype, np.dtype('bool'))
 
-
-
     def test_bloc_key_normalize_a(self) -> None:
         f1 = Frame.from_dict(dict(b=(1, 2), a=(5, 6)), index=tuple('yz'))
 
@@ -335,17 +321,21 @@ class TestUnit(TestCase):
         post2 = bloc_key_normalize(f1 < 5, f1)
         self.assertEqual(post2.tolist(), [[True, False], [True, False]])
 
-
-    def test_index_many_concat_a(self) -> None:
+    def test_index_many_concat_a1(self) -> None:
 
         idx0 = Index(('1997-01-01', '1997-01-02'), name='foo')
         idx1 = IndexDate(('2020-01-01', '2020-01-02'), name='foo')
         idx2 = IndexDate(('2020-02-01', '2020-02-02'))
 
-
         post1 = index_many_concat((idx0,  idx1), Index)
         assert isinstance(post1, Index)
 
+        post1b = index_many_concat((idx0,  idx1), Index, IndexDate)
+        assert isinstance(post1b, IndexDate)
+
+        post1c = index_many_concat((idx0,  idx1, idx2), Index, IndexDate)
+        assert isinstance(post1c, IndexDate)
+
         self.assertEqual(post1.values.tolist(),
                 ['1997-01-01',
                 '1997-01-02',
@@ -354,9 +344,14 @@ class TestUnit(TestCase):
         self.assertEqual(post1.name, 'foo')
         self.assertEqual(post1.__class__, Index)
 
+    def test_index_many_concat_a2(self) -> None:
+
+        # idx0 = Index(('1997-01-01', '1997-01-02'), name='foo')
+        idx1 = IndexDate(('2020-01-01', '2020-01-02'), name='foo')
+        idx2 = IndexDate(('2020-02-01', '2020-02-02'))
+
         post2 = index_many_concat((idx1,  idx2), Index)
         assert isinstance(post2, Index)
-
         self.assertEqual(post2.__class__, IndexDate)
         self.assertEqual(post2.values.tolist(),
                 [datetime.date(2020, 1, 1),
@@ -417,7 +412,6 @@ class TestUnit(TestCase):
                 [[1, date(2019, 1, 1)], [2, date(2019, 1, 2)], [2, date(2019, 1, 3)], [3, date(2019, 1, 4)]]
                 )
 
-
     def test_index_many_concat_e(self) -> None:
 
         idx1 = IndexDateGO(('2020-01-01', '2020-01-02'))
@@ -431,6 +425,7 @@ class TestUnit(TestCase):
                 )
 
     #---------------------------------------------------------------------------
+
     def test_index_many_set_a(self) -> None:
 
         idx0 = Index(('1997-01-01', '1997-01-02'), name='foo')
@@ -443,12 +438,19 @@ class TestUnit(TestCase):
 
         self.assertEqual(post1.name, 'foo')
         self.assertEqual(post1.__class__, Index)
-        self.assertEqual(set(post1.values),
-                {'1997-01-02',
-                '1997-01-01',
-                np.datetime64('2020-01-01'),
-                np.datetime64('2020-01-02')})
 
+        # self.assertEqual(set(post1.values),
+        #         {'1997-01-02',
+        #         '1997-01-01',
+        #         np.datetime64('2020-01-01'),
+        #         np.datetime64('2020-01-02')})
+
+        # the result of this operation is an unstable ordering
+        values = set(post1.values)
+        self.assertTrue('1997-01-01' in values)
+        self.assertTrue('1997-01-02' in values)
+        self.assertTrue(datetime.date(2020, 1, 1) in values)
+        self.assertTrue(datetime.date(2020, 1, 2) in values)
 
         post2 = index_many_set((idx1,  idx2), Index, union=True)
         assert isinstance(post2, Index)
@@ -480,7 +482,6 @@ class TestUnit(TestCase):
         post2 = index_many_set((idx1,  idx2), IndexGO, union=False)
         self.assertEqual(post2.__class__, IndexDateGO)
 
-
     def test_index_many_set_c(self) -> None:
         idx1 = IndexDate(('2020-02-01', '2020-02-02'))
 
@@ -492,7 +493,40 @@ class TestUnit(TestCase):
         post2 = index_many_set((), Index, union=True)
         self.assertEqual(len(post2), 0) #type: ignore
 
+    def test_index_many_set_d(self) -> None:
+        idx1 = Index(range(3), loc_is_iloc=True)
+        idx2 = Index(range(3), loc_is_iloc=True)
+        idx3 = index_many_set((idx1, idx2), Index, union=True)
+        self.assertTrue(idx3._map is None) #type: ignore
+        self.assertEqual(idx3.values.tolist(), [0, 1, 2]) #type: ignore
+
+    def test_index_many_set_e(self) -> None:
+        idx1 = Index(range(2), loc_is_iloc=True)
+        idx2 = Index(range(4), loc_is_iloc=True)
+        idx3 = index_many_set((idx1, idx2), Index, union=True)
+        self.assertTrue(idx3._map is None) #type: ignore
+        self.assertEqual(idx3.values.tolist(), [0, 1, 2, 3]) #type: ignore
+
+    def test_index_many_set_f(self) -> None:
+        idx1 = Index(range(2), loc_is_iloc=True)
+        idx2 = Index(range(4), loc_is_iloc=True)
+        idx3 = index_many_set((idx1, idx2), Index, union=False)
+        self.assertTrue(idx3._map is None) #type: ignore
+        self.assertEqual(idx3.values.tolist(), [0, 1]) #type: ignore
+
+    def test_index_many_set_g(self) -> None:
+        idx1 = Index(range(2), loc_is_iloc=True)
+        idx2 = Index([3, 2, 1, 0])
+        idx3 = index_many_set((idx1, idx2), Index, union=False)
+        self.assertTrue(idx3._map is not None) #type: ignore
+        self.assertEqual(idx3.values.tolist(), [0, 1]) #type: ignore
+
+    def test_index_many_set_h(self) -> None:
+        post1 = index_many_set((), Index, union=True, explicit_constructor=IndexDate)
+        self.assertIs(post1.__class__, IndexDate)
+
     #---------------------------------------------------------------------------
+
     def test_get_col_dtype_factory_a(self) -> None:
 
         func1 = get_col_dtype_factory((np.dtype(float), np.dtype(object)), None)
@@ -513,8 +547,8 @@ class TestUnit(TestCase):
         with self.assertRaises(RuntimeError):
             _ = get_col_dtype_factory(dict(bar=np.dtype(bool)), None)
 
-
     #---------------------------------------------------------------------------
+
     def test_apex_to_name_a(self) -> None:
         self.assertEqual(
                 apex_to_name([['foo']],
@@ -611,8 +645,8 @@ class TestUnit(TestCase):
         with self.assertRaises(AxisInvalid):
             _ = apex_to_name([['foo']], depth_level=-1, axis=3, axis_depth=1)
 
-
     #---------------------------------------------------------------------------
+
     def test_apply_binary_operator_blocks_by_column_a(self) -> None:
         blocks = (np.arange(3), np.arange(6).reshape(3, 2), np.arange(3) * 10)
         other = np.array([1, 0, 1])
@@ -624,6 +658,46 @@ class TestUnit(TestCase):
         self.assertEqual(np.stack(post, axis=1).tolist(),
                 [[0, 0, 1, 0], [0, 0, 0, 0], [2, 4, 5, 20]])
 
+    #---------------------------------------------------------------------------
+
+    def test_container_to_exporter_attr(self) -> None:
+        self.assertEqual(container_to_exporter_attr(Frame), 'to_frame')
+        self.assertEqual(container_to_exporter_attr(FrameHE), 'to_frame_he')
+
+        with self.assertRaises(NotImplementedError):
+            container_to_exporter_attr(Series)
+
+    #---------------------------------------------------------------------------
+
+    def test_block_match_a(self) -> None:
+
+        # opperate on the back of the list
+        stack = [np.arange(4).reshape(2, 2), np.arange(0, 2), np.arange(6).reshape(2, 3)]
+        post1 = list(get_block_match(1, stack))
+
+        # takes front of each block in reverse order
+        self.assertEqual(post1[0].tolist(), [0, 3])
+
+        self.assertEqual([a.shape for a in post1],
+                [(2,)])
+        self.assertEqual([a.shape for a in stack],
+                [(2, 2), (2,), (2, 2)])
+
+    def test_block_match_b(self) -> None:
+
+        # opperate on the back of the list
+        stack = [np.arange(4).reshape(2, 2), np.arange(0, 2), np.arange(6).reshape(2, 3)]
+        post1 = list(get_block_match(5, stack))
+
+        # takes front of each block in reverse order
+        self.assertEqual(post1[0].tolist(), [[0, 1, 2], [3, 4, 5]])
+
+        self.assertEqual([a.shape for a in post1],
+                [(2, 3), (2,), (2, 1)])
+        self.assertEqual([a.shape for a in stack],
+                [(2, 1)])
+
 
 if __name__ == '__main__':
+    import unittest
     unittest.main()
