@@ -6,6 +6,7 @@ import hashlib
 import json
 
 import numpy as np
+import frame_fixtures as ff
 
 from static_frame.test.test_case import TestCase
 from static_frame.test.test_case import skip_win
@@ -22,13 +23,16 @@ from static_frame import DisplayConfig
 from static_frame import DisplayConfigs
 from static_frame import DisplayFormats
 from static_frame import DisplayActive
+from static_frame.core.display_color import HexColor
 
 from static_frame.core.display_config import DisplayFormatLaTeX
 from static_frame.core.display import DisplayTypeCategoryFactory
 from static_frame.core.display import terminal_ansi
+from static_frame.core.display import DisplayCell
+from static_frame.core.display import FORMAT_EMPTY
 from static_frame.core.display import DisplayTypeInt
-from static_frame.test.test_case import temp_file
 
+from static_frame.test.test_case import temp_file
 
 nan = np.nan
 
@@ -42,7 +46,10 @@ class TestUnit(TestCase):
 
     def test_display_config_a(self) -> None:
         config = DisplayConfig.from_default(type_color=False)
-        d = Display.from_values(np.array([[1, 2], [3, 4]], dtype=np.int64), 'header', config=config)
+        d = Display.from_values(
+                np.array([[1, 2], [3, 4]], dtype=np.int64),
+                header='header',
+                config=config)
         self.assertEqual(d.to_rows(),
                 ['header', '1 2', '3 4', '<int64>'])
 
@@ -51,15 +58,12 @@ class TestUnit(TestCase):
 
         self.assertFalse(post.cell_align_left)
 
-
-
     def test_display_config_c(self) -> None:
         config_right = sf.DisplayConfig.from_default(cell_align_left=False)
         config_left = sf.DisplayConfig.from_default(cell_align_left=True)
 
         msg = config_right.to_json()
 
-
     def test_display_config_d(self) -> None:
 
         with temp_file('.json') as fp:
@@ -75,7 +79,6 @@ class TestUnit(TestCase):
         dc1 = DisplayConfig()
         self.assertTrue(str(dc1).startswith('<DisplayConfig'))
 
-
     #---------------------------------------------------------------------------
 
     @skip_win  #type: ignore
@@ -87,7 +90,6 @@ class TestUnit(TestCase):
             DisplayActive.write(fp2) #type: ignore
             DisplayActive.read(fp2) #type: ignore
 
-
     #---------------------------------------------------------------------------
 
     def test_display_cell_align_left_a(self) -> None:
@@ -103,9 +105,6 @@ class TestUnit(TestCase):
                 index.display(config=config_right).to_rows(),
                 ['<Index>', '      a', '      b', '      c', '  <<U1>'])
 
-
-
-
     @skip_win  #type: ignore
     def test_display_cell_align_left_b(self) -> None:
         config_right = sf.DisplayConfig.from_default(cell_align_left=False, type_color=False)
@@ -148,7 +147,6 @@ class TestUnit(TestCase):
                 '      y       2      95     c  False  False',
                 '  <<U1> <int64> <int64> <<U1> <bool> <bool>'])
 
-
     @skip_win  # type: ignore
     def test_display_type_show_a(self) -> None:
         config_type = sf.DisplayConfig.from_default(
@@ -198,7 +196,6 @@ class TestUnit(TestCase):
                 'y       2       95      c     False  False',
                 '<<U1>   <int64> <int64> <<U1> <bool> <bool>'])
 
-
     @skip_win  # type: ignore
     def test_display_type_show_b(self) -> None:
         config_type = sf.DisplayConfig.from_default(
@@ -251,7 +248,6 @@ class TestUnit(TestCase):
 
         s = Series(chunks(20, 3), index=('a', 'b', 'c'))
 
-
         self.assertEqual(s.display(config=config_width_12).to_rows(),
                 [
                 '<Series>',
@@ -301,7 +297,6 @@ class TestUnit(TestCase):
                 '<<U1>  <<U10> <<U10> <<U10> <<U10>']
                 )
 
-
     def test_display_display_rows_a(self) -> None:
 
         config_rows_12 = sf.DisplayConfig.from_default(display_rows=12, type_color=False)
@@ -341,7 +336,6 @@ class TestUnit(TestCase):
                 'yz       324',
                 '<<U2>    <int64>'])
 
-
     def test_display_rows_b(self) -> None:
         # this isseu was found only with Frame, not with Series
         dc = DisplayConfig(display_rows=8, type_color=False)
@@ -388,7 +382,6 @@ class TestUnit(TestCase):
                 '<int64> <int64>']
                 )
 
-
     @skip_win  # type: ignore
     def test_display_display_columns_a(self) -> None:
 
@@ -423,7 +416,6 @@ class TestUnit(TestCase):
                 '3         0       1       ... 323     324',
                 '<int64>   <int64> <int64> ... <int64> <int64>'])
 
-
     @skip_win  # type: ignore
     def test_display_display_columns_b(self) -> None:
 
@@ -462,7 +454,6 @@ class TestUnit(TestCase):
                 '<<U1>   <int64> <int64> <<U1> <bool> <bool>']
                 )
 
-
     def test_display_truncate_a(self) -> None:
 
         config_rows_12_cols_8 = sf.DisplayConfig.from_default(display_rows=12, display_columns=8)
@@ -489,7 +480,6 @@ class TestUnit(TestCase):
         self.assertEqual(
                 len(tuple(f.display(config_rows_7_cols_5).to_rows())), 11)
 
-
     def test_display_type_attributes_a(self) -> None:
 
         x, z = Display.type_attributes(np.dtype('int8'), DisplayConfigs.DEFAULT)
@@ -500,7 +490,6 @@ class TestUnit(TestCase):
         with self.assertRaises(NotImplementedError):
             x, z = Display.type_attributes(None, DisplayConfigs.DEFAULT)
 
-
     def test_display_type_category_a(self) -> None:
 
         x = DisplayTypeCategoryFactory.to_category(np.dtype(int))
@@ -528,11 +517,17 @@ class TestUnit(TestCase):
     def test_display_flatten_a(self) -> None:
         config = DisplayConfig.from_default(type_color=False)
 
-        d1 = Display.from_values(np.array([1, 2, 3, 4], dtype=np.int64), 'header', config=config)
+        d1 = Display.from_values(
+                np.array([1, 2, 3, 4], dtype=np.int64),
+                header='header',
+                config=config)
         self.assertEqual(d1.flatten().to_rows(), ['header 1 2 3 4 <int64>'])
 
 
-        d2 = Display.from_values(np.array([5, 6, 7, 8], dtype=np.int64), 'header', config=config)
+        d2 = Display.from_values(
+                np.array([5, 6, 7, 8], dtype=np.int64),
+                header='header',
+                config=config)
 
         # mutates in place
         d1.extend_display(d2)
@@ -546,7 +541,6 @@ class TestUnit(TestCase):
         self.assertEqual(d1.transform().to_rows(),
                 ['header 1 2 3 4 <int64>', 'header 5 6 7 8 <int64>'])
 
-
     @skip_win  # type: ignore
     def test_display_html_pre_a(self) -> None:
         f = Frame.from_dict(dict(a=(1, 2),
@@ -566,7 +560,6 @@ class TestUnit(TestCase):
 
         self.assertEqualLines(html, str(expected))
 
-
     @skip_win  # type: ignore
     def test_display_html_table_a(self) -> None:
         f = sf.Frame.from_dict(
@@ -578,21 +571,21 @@ class TestUnit(TestCase):
         expected1 = f.display(sf.DisplayConfig(
                 display_format='html_table', type_color=False))
 
-        html1 = '''<table border="1"><thead><tr><th>&lt;Frame&gt;</th><th></th><th></th><th></th><th></th><th></th></tr><tr><th>&lt;IndexHierarchy&gt;</th><th></th><th>J</th><th>J</th><th>J</th><th>&lt;&lt;U1&gt;</th></tr><tr><th></th><th></th><th>I</th><th>I</th><th>I</th><th>&lt;&lt;U1&gt;</th></tr><tr><th></th><th></th><th>a</th><th>b</th><th>c</th><th>&lt;&lt;U1&gt;</th></tr><tr><th>&lt;IndexHierarchy: (&#x27;a&#x27;, &#x27;b&#x27;)&gt;</th><th></th><th></th><th></th><th></th><th></th></tr></thead><tbody><tr><th>1</th><th>True</th><td>1</td><td>True</td><td>q</td><td></td></tr><tr><th>2</th><th>False</th><td>2</td><td>False</td><td>r</td><td></td></tr><tr><th>3</th><th>True</th><td>3</td><td>True</td><td>s</td><td></td></tr><tr><th>4</th><th>False</th><td>4</td><td>False</td><td>t</td><td></td></tr><tr><th>&lt;int64&gt;</th><th>&lt;bool&gt;</th><td>&lt;int64&gt;</td><td>&lt;bool&gt;</td><td>&lt;&lt;U1&gt;</td><td></td></tr></tbody></table>
+        html1 = '''<table><thead><tr><th>&lt;Frame&gt;</th><th></th><th></th><th></th><th></th><th></th></tr><tr><th>&lt;IndexHierarchy&gt;</th><th></th><th>J</th><th>J</th><th>J</th><th>&lt;&lt;U1&gt;</th></tr><tr><th></th><th></th><th>I</th><th>I</th><th>I</th><th>&lt;&lt;U1&gt;</th></tr><tr><th></th><th></th><th>a</th><th>b</th><th>c</th><th>&lt;&lt;U1&gt;</th></tr><tr><th>&lt;IndexHierarchy: (&#x27;a&#x27;, &#x27;b&#x27;)&gt;</th><th></th><th></th><th></th><th></th><th></th></tr></thead><tbody><tr><th>1</th><th>True</th><td>1</td><td>True</td><td>q</td><td></td></tr><tr><th>2</th><th>False</th><td>2</td><td>False</td><td>r</td><td></td></tr><tr><th>3</th><th>True</th><td>3</td><td>True</td><td>s</td><td></td></tr><tr><th>4</th><th>False</th><td>4</td><td>False</td><td>t</td><td></td></tr><tr><th>&lt;int64&gt;</th><th>&lt;bool&gt;</th><td>&lt;int64&gt;</td><td>&lt;bool&gt;</td><td>&lt;&lt;U1&gt;</td><td></td></tr></tbody></table>
         '''
         self.assertEqual(html1.strip(), str(expected1).strip())
 
         expected2 = f.display(sf.DisplayConfig(
                 display_format='html_table', type_color=False, type_show=False))
 
-        html2 = '''<table border="1"><thead><tr><th></th><th></th><th>J</th><th>J</th><th>J</th></tr><tr><th></th><th></th><th>I</th><th>I</th><th>I</th></tr><tr><th></th><th></th><th>a</th><th>b</th><th>c</th></tr></thead><tbody><tr><th>1</th><th>True</th><td>1</td><td>True</td><td>q</td></tr><tr><th>2</th><th>False</th><td>2</td><td>False</td><td>r</td></tr><tr><th>3</th><th>True</th><td>3</td><td>True</td><td>s</td></tr><tr><th>4</th><th>False</th><td>4</td><td>False</td><td>t</td></tr></tbody></table>
+        html2 = '''<table><thead><tr><th></th><th></th><th>J</th><th>J</th><th>J</th></tr><tr><th></th><th></th><th>I</th><th>I</th><th>I</th></tr><tr><th></th><th></th><th>a</th><th>b</th><th>c</th></tr></thead><tbody><tr><th>1</th><th>True</th><td>1</td><td>True</td><td>q</td></tr><tr><th>2</th><th>False</th><td>2</td><td>False</td><td>r</td></tr><tr><th>3</th><th>True</th><td>3</td><td>True</td><td>s</td></tr><tr><th>4</th><th>False</th><td>4</td><td>False</td><td>t</td></tr></tbody></table>
         '''
         self.assertEqual(html2.strip(), str(expected2).strip())
 
         expected3 = f.display(sf.DisplayConfig(
                 display_format='html_table', type_color=False, type_show=False, include_index=False))
 
-        html3 = '''<table border="1"><thead><tr><th>J</th><th>J</th><th>J</th></tr><tr><th>I</th><th>I</th><th>I</th></tr><tr><th>a</th><th>b</th><th>c</th></tr></thead><tbody><tr><td>1</td><td>True</td><td>q</td></tr><tr><td>2</td><td>False</td><td>r</td></tr><tr><td>3</td><td>True</td><td>s</td></tr><tr><td>4</td><td>False</td><td>t</td></tr></tbody></table>
+        html3 = '''<table><thead><tr><th>J</th><th>J</th><th>J</th></tr><tr><th>I</th><th>I</th><th>I</th></tr><tr><th>a</th><th>b</th><th>c</th></tr></thead><tbody><tr><td>1</td><td>True</td><td>q</td></tr><tr><td>2</td><td>False</td><td>r</td></tr><tr><td>3</td><td>True</td><td>s</td></tr><tr><td>4</td><td>False</td><td>t</td></tr></tbody></table>
         '''
         self.assertEqual(html3.strip(), str(expected3).strip())
 
@@ -600,7 +593,7 @@ class TestUnit(TestCase):
         expected4 = f.display(sf.DisplayConfig(
                 display_format='html_table', type_color=False, type_show=False, include_columns=False))
 
-        html4 = '''<table border="1"><tbody><tr><th>1</th><th>True</th><td>1</td><td>True</td><td>q</td></tr><tr><th>2</th><th>False</th><td>2</td><td>False</td><td>r</td></tr><tr><th>3</th><th>True</th><td>3</td><td>True</td><td>s</td></tr><tr><th>4</th><th>False</th><td>4</td><td>False</td><td>t</td></tr></tbody></table>
+        html4 = '''<table><tbody><tr><th>1</th><th>True</th><td>1</td><td>True</td><td>q</td></tr><tr><th>2</th><th>False</th><td>2</td><td>False</td><td>r</td></tr><tr><th>3</th><th>True</th><td>3</td><td>True</td><td>s</td></tr><tr><th>4</th><th>False</th><td>4</td><td>False</td><td>t</td></tr></tbody></table>
         '''
         self.assertEqual(html4.strip(), str(expected4).strip())
 
@@ -612,13 +605,10 @@ class TestUnit(TestCase):
                 include_index=False,
                 include_columns=False))
 
-        html5 = '''<table border="1"><tbody><tr><td>1</td><td>True</td><td>q</td></tr><tr><td>2</td><td>False</td><td>r</td></tr><tr><td>3</td><td>True</td><td>s</td></tr><tr><td>4</td><td>False</td><td>t</td></tr></tbody></table>
+        html5 = '''<table><tbody><tr><td>1</td><td>True</td><td>q</td></tr><tr><td>2</td><td>False</td><td>r</td></tr><tr><td>3</td><td>True</td><td>s</td></tr><tr><td>4</td><td>False</td><td>t</td></tr></tbody></table>
         '''
         self.assertEqual(html5.strip(), str(expected5).strip())
 
-
-
-
     def test_display_html_table_b(self) -> None:
         records = (
                 (2, 'a', False),
@@ -628,8 +618,7 @@ class TestUnit(TestCase):
                 columns=('p', 'q', 'r'),
                 index=('w', 'x'))
         self.assertEqual(f._repr_html_(),
-            '<table border="1"><thead><tr><th></th><th>p</th><th>q</th><th>r</th></tr></thead><tbody><tr><th>w</th><td>2</td><td>a</td><td>False</td></tr><tr><th>x</th><td>30</td><td>b</td><td>False</td></tr></tbody></table>')
-
+            '<table><thead><tr><th></th><th>p</th><th>q</th><th>r</th></tr></thead><tbody><tr><th>w</th><td>2</td><td>a</td><td>False</td></tr><tr><th>x</th><td>30</td><td>b</td><td>False</td></tr></tbody></table>')
 
     def test_display_html_series_a(self) -> None:
         records = (
@@ -641,7 +630,7 @@ class TestUnit(TestCase):
                 index=('w', 'x'))
 
         self.assertEqual(f['q']._repr_html_(),
-            '<table border="1"><tbody><tr><th>w</th><td>a</td></tr><tr><th>x</th><td>b</td></tr></tbody></table>')
+            '<table><tbody><tr><th>w</th><td>a</td></tr><tr><th>x</th><td>b</td></tr></tbody></table>')
 
     def test_display_html_index_a(self) -> None:
         records = (
@@ -653,11 +642,11 @@ class TestUnit(TestCase):
                 index=('w', 'x'))
 
         self.assertEqual(f.index._repr_html_(),
-                '<table border="1"><tbody><tr><td>w</td></tr><tr><td>x</td></tr></tbody></table>')
+                '<table><tbody><tr><td>w</td></tr><tr><td>x</td></tr></tbody></table>')
 
         f1 = f.set_index_hierarchy(('p', 'q'))
         self.assertEqual(f1.index._repr_html_(),
-                '<table border="1"><tbody><tr><td>2</td><td>a</td></tr><tr><td>30</td><td>b</td></tr></tbody></table>'
+                '<table><tbody><tr><td>2</td><td>a</td></tr><tr><td>30</td><td>b</td></tr></tbody></table>'
                 )
 
     def test_display_html_index_b(self) -> None:
@@ -685,8 +674,6 @@ class TestUnit(TestCase):
         self.assertEqual(post[2],
                 '<span style="color: #777777">&lt;IndexHierarchy: (&#x27;p&#x27;, &#x27;q&#x27;)&gt;</span>')
 
-
-
     def test_display_max_width_a(self) -> None:
         records = (
                 (2, 'a', False),
@@ -734,8 +721,6 @@ class TestUnit(TestCase):
                 '<int64>  <float64>']
                 )
 
-
-
     def test_display_float_scientific_b(self) -> None:
 
         s1 = sf.Series([3.1j, 5.2j]) ** 40
@@ -767,13 +752,13 @@ class TestUnit(TestCase):
                 '<int64>  <complex128>']
                 )
 
-
     def test_display_tall(self) -> None:
         f = Frame.from_element(None, index=range(40), columns=range(20))
         self.assertEqual(len(f.display_tall().to_rows()), 44)
         self.assertEqual(len(f.display_wide().to_rows()), 39)
 
     #---------------------------------------------------------------------------
+
     def test_display_format_latex_a(self) -> None:
 
         post = DisplayFormatLaTeX.markup_outermost('x', identifier='foo')
@@ -781,7 +766,6 @@ class TestUnit(TestCase):
             '''\\begin{table}[ht]\n\\centering\nx\n\\label{table:foo}\n\\end{table}'''
             )
 
-
     @skip_win #type: ignore
     def test_display_type_color_markup_a(self) -> None:
 
@@ -797,8 +781,6 @@ class TestUnit(TestCase):
         post2 = Display.type_color_markup(DisplayTypeInt, config2)
         self.assertEqual(post2, '<span style="color: #505050">{}</span>')
 
-
-
     def test_display_include_index_a(self) -> None:
 
         f1 = Frame.from_element('b', index=range(3), columns=range(2))
@@ -821,8 +803,39 @@ class TestUnit(TestCase):
                 ['b b', 'b b', 'b b']
                 )
 
+    #---------------------------------------------------------------------------
 
+    def test_display_value_color_a(self) -> None:
 
+        f1 = ff.parse('s(10,3)|i(I,str)')
+        s2 = f1.assign[1].apply(lambda s: s.iter_element().apply(lambda e: HexColor.format_terminal('green' if e > 0 else 'blue', str(e))))
+        post = s2.display()
+
+    #---------------------------------------------------------------------------
+
+    def test_display_get_max_width_pad_width_a(self) -> None:
+
+        row1 = [DisplayCell(FORMAT_EMPTY, 'foo'), DisplayCell(FORMAT_EMPTY, 'bar')]
+        row2 = [DisplayCell(FORMAT_EMPTY, 'foo')]
+
+        post = Display._get_max_width_pad_width(rows=[row1, row2],
+                col_idx_src=1,
+                col_last_src=1,
+                row_indices=range(2),
+                )
+        self.assertEqual(post, (3, 3))
+
+    #---------------------------------------------------------------------------
+
+    def test_display_repr_a(self) -> None:
+
+        row1 = [DisplayCell(FORMAT_EMPTY, 'foo'), DisplayCell(FORMAT_EMPTY, 'bar')]
+        row2 = [DisplayCell(FORMAT_EMPTY, 'foo'), DisplayCell(FORMAT_EMPTY, 'bar')]
+
+        post = Display(rows=[row1, row2],
+                outermost=False,
+                )
+        self.assertEqual(post.__repr__(), 'foo bar\nfoo bar')
 
     #---------------------------------------------------------------------------
 
@@ -914,6 +927,5 @@ class TestUnit(TestCase):
         f.assign.loc[8, :](s1, fill_value='')
 
 
-
 if __name__ == '__main__':
     unittest.main()
