@@ -781,7 +781,7 @@ dtype('int64')
 <Index: signature>
 sort_index(*, ascending, kind, key)  Series   Method Return a new Seri...
 sort_values(*, ascending, kind, key) Series   Method Return a new Seri...
-<<U94>                               <<U6>    <<U17> <<U83>
+<<U94>                               <<U6>    <<U27> <<U83>
 
 #end_Series-interface
 
@@ -1292,7 +1292,7 @@ d'Arrest 6.51           1.346
 sort_columns(*, ascending, kind, ... Frame    Method Return a new Fram...
 sort_index(*, ascending, kind, key)  Frame    Method Return a new Fram...
 sort_values(label, *, ascending, ... Frame    Method Return a new Fram...
-<<U94>                               <<U5>    <<U18> <<U83>
+<<U94>                               <<U5>    <<U27> <<U83>
 
 
 #end_Frame-interface
@@ -2211,13 +2211,13 @@ quark                              strange 0.1       -0.333
 
 >>> f.relabel_flat(index=True)
 <Frame>
-<Index>              mass      charge    <<U6>
-<Index>
-('lepton', 'muon')   0.106     -1.0
-('lepton', 'tau')    1.777     -1.0
-('quark', 'charm')   1.3       0.666
-('quark', 'strange') 0.1       -0.333
-<object>             <float64> <float64>
+<Index>                   mass      charge    <<U6>
+<Index: ('type', 'name')>
+('lepton', 'muon')        0.106     -1.0
+('lepton', 'tau')         1.777     -1.0
+('quark', 'charm')        1.3       0.666
+('quark', 'strange')      0.1       -0.333
+<object>                  <float64> <float64>
 
 #end_Frame-relabel_flat()
 
@@ -2262,13 +2262,13 @@ quark                              strange 0.1       -0.333
 
 >>> f.relabel_level_drop(index=1)
 <Frame>
-<Index> mass      charge    <<U6>
-<Index>
-muon    0.106     -1.0
-tau     1.777     -1.0
-charm   1.3       0.666
-strange 0.1       -0.333
-<<U7>   <float64> <float64>
+<Index>       mass      charge    <<U6>
+<Index: name>
+muon          0.106     -1.0
+tau           1.777     -1.0
+charm         1.3       0.666
+strange       0.1       -0.333
+<<U7>         <float64> <float64>
 
 #end_Frame-relabel_level_drop()
 
@@ -2402,11 +2402,12 @@ Encke             3.3       0.847        11.8
 <Index>                              cls_name group    doc                  <<U18>
 <Index: signature>
 drop_duplicated(*, axis, exclude_... FrameGO  Method   Return a Frame wi...
+dropfalsy(axis, condition)           FrameGO  Method   Return a new Fram...
 dropna(axis, condition)              FrameGO  Method   Return a new Fram...
 drop[key]                            FrameGO  Selector Label-based selec...
 drop.iloc[key]                       FrameGO  Selector
 drop.loc[key]                        FrameGO  Selector
-<<U94>                               <<U7>    <<U18>   <<U83>
+<<U94>                               <<U7>    <<U27>   <<U83>
 
 #end_FrameGO-interface
 
@@ -2436,9 +2437,12 @@ Saturn    120536   568.0     60268.0
 <Index>                       cls_name group    doc                  <<U18>
 <Index: signature>
 to_hdf5(fp, *, config)        Bus      Exporter Write the complet...
+to_series()                   Bus      Exporter Return a Series w...
 to_sqlite(fp, *, config)      Bus      Exporter Write the complet...
+to_visidata()                 Bus      Exporter Open an interacti...
 to_xlsx(fp, *, config)        Bus      Exporter Write the complet...
 to_zip_csv(fp, *, config)     Bus      Exporter Write the complet...
+to_zip_npz(fp, *, config)     Bus      Exporter Write the complet...
 to_zip_parquet(fp, *, config) Bus      Exporter Write the complet...
 to_zip_pickle(fp, *, config)  Bus      Exporter Write the complet...
 to_zip_tsv(fp, *, config)     Bus      Exporter Write the complet...
@@ -2459,7 +2463,7 @@ to_zip_tsv(fp, *, config)     Bus      Exporter Write the complet...
 <Index: signature>
 relabel(mapper)    Index    Method Return a new Inde...
 rename(name)       Index    Method Return a new Fram...
-<<U68>             <<U5>    <<U17> <<U83>
+<<U68>             <<U5>    <<U27> <<U83>
 
 #end_Index-interface
 
@@ -2515,11 +2519,12 @@ NE
 <Frame: IndexGO>
 <Index>                              cls_name group    doc                  <<U18>
 <Index: signature>
-to_html(config)                      IndexGO  Exporter Return an HTML ta...
+to_html(config, style_config)        IndexGO  Exporter Return an HTML ta...
 to_html_datatables(fp, *, show, c... IndexGO  Exporter Return a complete...
 to_pandas()                          IndexGO  Exporter Return a Pandas I...
 to_series()                          IndexGO  Exporter Return a Series w...
-<<U68>                               <<U7>    <<U17>   <<U83>
+to_visidata()                        IndexGO  Exporter Open an interacti...
+<<U68>                               <<U7>    <<U27>   <<U83>
 
 #end_IndexGO-interface
 
@@ -2549,9 +2554,9 @@ from_labels(labels, *, name, reor... IndexHierarchy Constructor Construct an Ind
 from_labels_delimited(labels, *, ... IndexHierarchy Constructor Construct an Inde...
 from_names(names)                    IndexHierarchy Constructor Construct a zero-...
 from_pandas(value)                   IndexHierarchy Constructor Given a Pandas in...
-from_product(*, name, *levels)       IndexHierarchy Constructor Given groups of i...
-from_tree(tree, *, name)             IndexHierarchy Constructor Convert into a In...
-<<U68>                               <<U14>         <<U18>      <<U83>
+from_product(*, name, index_const... IndexHierarchy Constructor Given groups of i...
+from_tree(tree, *, name, index_co... IndexHierarchy Constructor Convert into a In...
+<<U68>                               <<U14>         <<U27>      <<U83>
 
 #end_IndexHierarchy-interface
 
@@ -2574,12 +2579,13 @@ quark            down
 #start_IndexHierarchyGO-interface
 >>> sf.IndexHierarchyGO.interface.loc[sf.IndexHierarchyGO.interface.index.via_str.startswith('re')]
 <Frame: IndexHierarchyGO>
-<Index>                   cls_name         group  doc                  <<U18>
+<Index>                              cls_name         group  doc                  <<U18>
 <Index: signature>
-rehierarch(depth_map)     IndexHierarchyGO Method Return a new Inde...
-relabel(mapper)           IndexHierarchyGO Method Return a new Inde...
-rename(name)              IndexHierarchyGO Method Return a new Fram...
-<<U68>                    <<U16>           <<U18> <<U83>
+rehierarch(depth_map)                IndexHierarchyGO Method Return a new Inde...
+relabel(mapper)                      IndexHierarchyGO Method Return a new Inde...
+relabel_at_depth(mapper, depth_le... IndexHierarchyGO Method Return a new Inde...
+rename(name)                         IndexHierarchyGO Method Return a new Fram...
+<<U68>                               <<U16>           <<U27> <<U83>
 
 #end_IndexHierarchyGO-interface
 
@@ -2634,7 +2640,7 @@ from_labels(labels, *, name)         IndexYear Constructor Construct an Inde...
 from_pandas(value)                   IndexYear Constructor Given a Pandas in...
 from_year_month_range(start, stop... IndexYear Constructor Get an IndexYearM...
 from_year_range(start, stop, step... IndexYear Constructor Get an IndexDate ...
-<<U68>                               <<U9>     <<U17>      <<U83>
+<<U68>                               <<U9>     <<U27>      <<U83>
 
 #end_IndexYear-interface
 
@@ -2651,7 +2657,7 @@ from_labels(labels, *, name)         IndexYearGO Constructor Construct an Inde..
 from_pandas(value)                   IndexYearGO Constructor Given a Pandas in...
 from_year_month_range(start, stop... IndexYearGO Constructor Get an IndexYearM...
 from_year_range(start, stop, step... IndexYearGO Constructor Get an IndexDate ...
-<<U68>                               <<U11>      <<U17>      <<U83>
+<<U68>                               <<U11>      <<U27>      <<U83>
 
 #end_IndexYearGO-interface
 
@@ -2669,7 +2675,7 @@ from_labels(labels, *, name)         IndexYearMonthGO Constructor Construct an I
 from_pandas(value)                   IndexYearMonthGO Constructor Given a Pandas in...
 from_year_month_range(start, stop... IndexYearMonthGO Constructor Get an IndexYearM...
 from_year_range(start, stop, step... IndexYearMonthGO Constructor Get an IndexYearM...
-<<U68>                               <<U16>           <<U17>      <<U83>
+<<U68>                               <<U16>           <<U27>      <<U83>
 
 #end_IndexYearMonth-interface
 
@@ -2687,7 +2693,7 @@ from_labels(labels, *, name)         IndexYearMonthGO Constructor Construct an I
 from_pandas(value)                   IndexYearMonthGO Constructor Given a Pandas in...
 from_year_month_range(start, stop... IndexYearMonthGO Constructor Get an IndexYearM...
 from_year_range(start, stop, step... IndexYearMonthGO Constructor Get an IndexYearM...
-<<U68>                               <<U16>           <<U17>      <<U83>
+<<U68>                               <<U16>           <<U27>      <<U83>
 
 #end_IndexYearMonthGO-interface
 
@@ -2705,7 +2711,7 @@ from_labels(labels, *, name)         IndexDate Constructor Construct an Inde...
 from_pandas(value)                   IndexDate Constructor Given a Pandas in...
 from_year_month_range(start, stop... IndexDate Constructor Get an IndexDate ...
 from_year_range(start, stop, step... IndexDate Constructor Get an IndexDate ...
-<<U68>                               <<U9>     <<U17>      <<U83>
+<<U68>                               <<U9>     <<U27>      <<U83>
 
 #end_IndexDate-interface
 
@@ -2723,7 +2729,7 @@ from_labels(labels, *, name)         IndexDateGO Constructor Construct an Inde..
 from_pandas(value)                   IndexDateGO Constructor Given a Pandas in...
 from_year_month_range(start, stop... IndexDateGO Constructor Get an IndexDate ...
 from_year_range(start, stop, step... IndexDateGO Constructor Get an IndexDate ...
-<<U68>                               <<U11>      <<U17>      <<U83>
+<<U68>                               <<U11>      <<U27>      <<U83>
 
 #end_IndexDateGO-interface
 
