@@ -1,44 +1,24 @@
-import unittest
 import typing as tp
 
 import numpy as np
 import frame_fixtures as ff
 
 import static_frame as sf
-# from static_frame import Index
 from static_frame import IndexHierarchy
-# from static_frame import IndexHierarchyGO
-# from static_frame import IndexYearMonth
-# from static_frame import IndexYearGO
-# from static_frame import IndexYear
 from static_frame import IndexDate
-# from static_frame import IndexDateGO
 from static_frame import Series
 from static_frame import Frame
 from static_frame import FrameGO
 from static_frame import TypeBlocks
-# from static_frame import mloc
-# from static_frame import ILoc
 from static_frame import HLoc
-# from static_frame import DisplayConfig
-# from static_frame import IndexAutoFactory
-
 from static_frame.test.test_case import TestCase
-# from static_frame.test.test_case import skip_win
-# from static_frame.test.test_case import skip_linux_no_display
-# from static_frame.test.test_case import skip_pylt37
-# from static_frame.test.test_case import temp_file
-# from static_frame.core.exception import ErrorInitFrame
-# from static_frame.core.exception import ErrorInitIndex
 from static_frame.core.exception import AxisInvalid
 
 nan = np.nan
 
 
-
 class TestUnit(TestCase):
 
-
     #---------------------------------------------------------------------------
 
     def test_frame_iter_a(self) -> None:
@@ -55,7 +35,6 @@ class TestUnit(TestCase):
         self.assertEqual([x for x in f1.columns], ['p', 'q', 'r', 's', 't'])
         self.assertEqual([x for x in f1], ['p', 'q', 'r', 's', 't'])
 
-
     def test_frame_iter_array_a(self) -> None:
 
         records = (
@@ -74,7 +53,6 @@ class TestUnit(TestCase):
                 next(iter(f1.iter_array(axis=1))).tolist(),
                 [1, 2, 'a', False, True])
 
-
     def test_frame_iter_array_b(self) -> None:
 
         arrays = list(np.random.rand(1000) for _ in range(100))
@@ -124,7 +102,6 @@ class TestUnit(TestCase):
                 ((0, ((0, 0), (1, 0), (2, 0), (3, 0), (4, 0), (5, 0), (6, 0), (7, 0))), (1, ((0, 1), (1, 1), (2, 1), (3, 1), (4, 1), (5, 1), (6, 1), (7, 1))), (2, ((0, 2), (1, 2), (2, 2), (3, 2), (4, 2), (5, 2), (6, 2), (7, 2))), (3, ((0, 3), (1, 3), (2, 3), (3, 3), (4, 3), (5, 3), (6, 3), (7, 3))), (4, ((0, 4), (1, 4), (2, 4), (3, 4), (4, 4), (5, 4), (6, 4), (7, 4))), (5, ((0, 5), (1, 5), (2, 5), (3, 5), (4, 5), (5, 5), (6, 5), (7, 5))), (6, ((0, 6), (1, 6), (2, 6), (3, 6), (4, 6), (5, 6), (6, 6), (7, 6))), (7, ((0, 7), (1, 7), (2, 7), (3, 7), (4, 7), (5, 7), (6, 7), (7, 7))))
                 )
 
-
     def test_frame_iter_array_e(self) -> None:
 
         f = sf.Frame.from_dict(
@@ -136,7 +113,6 @@ class TestUnit(TestCase):
         post = f.iter_array(axis=0).apply(np.sum)
         self.assertTrue(post.dtype == float)
 
-
     def test_frame_iter_array_f(self) -> None:
 
         f = sf.Frame(np.arange(12).reshape(3,4),
@@ -164,13 +140,12 @@ class TestUnit(TestCase):
         post = list(f.iter_array(axis=1))
         self.assertEqual([x.tolist() for x in post], [[], [], []])
 
-
     #---------------------------------------------------------------------------
+
     def test_frame_iter_tuple_a(self) -> None:
         post = tuple(sf.Frame.from_elements(range(5)).iter_tuple(axis=0, constructor=tuple))
         self.assertEqual(post, ((0, 1, 2, 3, 4),))
 
-
     def test_frame_iter_tuple_b(self) -> None:
         post = tuple(sf.Frame.from_elements(range(3), index=tuple('abc')).iter_tuple(axis=0))
         self.assertEqual(post, ((0, 1, 2),))
@@ -212,6 +187,7 @@ class TestUnit(TestCase):
                 [(1, 30), (2, 50), ('a', 'b'), (False, True), (True, False)])
 
     #---------------------------------------------------------------------------
+
     def test_frame_iter_series_a(self) -> None:
         f1 = ff.parse('f(Fg)|s(2,8)|i(I,str)|c(Ig,str)|v(int)')
         post1 = tuple(f1.iter_series(axis=0))
@@ -224,8 +200,32 @@ class TestUnit(TestCase):
         self.assertEqual(post2[0].to_pairs(),
                 (('zZbu', -88017), ('ztsv', 162197), ('zUvW', -3648), ('zkuW', 129017), ('zmVj', 58768), ('z2Oo', 84967), ('z5l6', 146284), ('zCE3', 137759)))
 
+    def test_frame_iter_series_b(self) -> None:
+        f1 = ff.parse('s(10,4)|i(I,str)|c(I,str)|v(float)')
+
+        post1 = f1.iter_series(axis=1).apply(lambda s: s.sum(), dtype=float)
+        self.assertEqual(post1.dtype, float)
+        self.assertEqual(post1.shape, (10,))
+
+        post2 = f1[sf.ILoc[0]].iter_element().apply(lambda e: e == 647.9, dtype=bool)
+        self.assertEqual(post2.dtype, bool)
+        self.assertEqual(post2.shape, (10,))
+        self.assertEqual(post2.sum(), 1)
+
+        post3 = f1.iter_series(axis=1).apply(lambda s: str(s.sum()), dtype=str)
+        self.assertEqual(post3.dtype, np.dtype('<U18'))
+        self.assertEqual(post3.shape, (10,))
+
+        post4 = f1.iter_series(axis=1).apply(lambda s: int(s.sum()), dtype=int)
+        self.assertEqual(post4.dtype, np.dtype(int))
+        self.assertEqual(post4.shape, (10,))
+
+        post5 = f1.iter_series(axis=1).apply(lambda s: int(s.sum()), dtype=object)
+        self.assertEqual(post5.dtype, object)
+        self.assertEqual(post5.shape, (10,))
 
     #---------------------------------------------------------------------------
+
     def test_frame_iter_tuple_items_a(self) -> None:
         records = (
                 (1, 2, 'a', False, True),
@@ -238,9 +238,8 @@ class TestUnit(TestCase):
         post1 = list(f1.iter_tuple_items(constructor=list))
         self.assertEqual(post1, [('p', [1, 30]), ('q', [2, 50]), ('r', ['a', 'b']), ('s', [False, True]), ('t', [True, False])])
 
-
-
     #---------------------------------------------------------------------------
+
     def test_frame_iter_element_a(self) -> None:
         # reindex both axis
         records = (
@@ -276,7 +275,10 @@ class TestUnit(TestCase):
                 (('p', (('w', '_2_'), ('x', '_30_'), ('y', '_2_'), ('z', '_30_'))), ('q', (('w', '_2_'), ('x', '_34_'), ('y', '_95_'), ('z', '_73_'))), ('r', (('w', '_a_'), ('x', '_b_'), ('y', '_c_'), ('z', '_d_'))), ('s', (('w', '_False_'), ('x', '_True_'), ('y', '_False_'), ('z', '_True_'))), ('t', (('w', '_False_'), ('x', '_False_'), ('y', '_False_'), ('z', '_True_')))))
 
 
+        post3 = f1.iter_element(axis=1).apply(lambda x: '_' + str(x) + '_', dtype=str)
 
+        self.assertEqual(post3.to_pairs(0),
+                (('p', (('w', '_2_'), ('x', '_30_'), ('y', '_2_'), ('z', '_30_'))), ('q', (('w', '_2_'), ('x', '_34_'), ('y', '_95_'), ('z', '_73_'))), ('r', (('w', '_a_'), ('x', '_b_'), ('y', '_c_'), ('z', '_d_'))), ('s', (('w', '_False_'), ('x', '_True_'), ('y', '_False_'), ('z', '_True_'))), ('t', (('w', '_False_'), ('x', '_False_'), ('y', '_False_'), ('z', '_True_')))))
 
     def test_frame_iter_element_b(self) -> None:
         # reindex both axis
@@ -327,18 +329,18 @@ class TestUnit(TestCase):
                 ((('i', 'a'), (('a', 'a'), ('b', 'b'), ('c', 'c'))), (('i', 'b'), (('a', 'tru'), ('b', 'fals'), ('c', 'tru'))), (('ii', 'a'), (('a', 'non'), ('b', 'non'), ('c', 'non'))), (('ii', 'b'), (('a', 'non'), ('b', '1'), ('c', '5'))))
                 )
 
-
     def test_frame_iter_element_d(self) -> None:
         f1 = sf.Frame.from_elements(['I', 'II', 'III'], columns=('A',))
         f2 = sf.Frame.from_elements([67, 28, 99], columns=('B',), index=('I', 'II', 'IV'))
 
         post = f1['A'].iter_element().map_any(f2['B'])
 
+        # same index is given to returned Series
+        self.assertEqual(id(f1.index), id(post.index))
         # if we do not match the mapping, we keep the value.
         self.assertEqual(post.to_pairs(),
                 ((0, 67), (1, 28), (2, 'III')))
 
-
     def test_frame_iter_element_e(self) -> None:
         f1 = Frame.from_records(np.arange(9).reshape(3, 3))
 
@@ -350,7 +352,6 @@ class TestUnit(TestCase):
         self.assertEqual([d.kind for d in f2.dtypes.values],
                 ['i', 'i', 'i'])
 
-
     #---------------------------------------------------------------------------
 
     def test_frame_iter_group_a(self) -> None:
@@ -372,7 +373,7 @@ class TestUnit(TestCase):
         self.assertEqual(post.to_pairs(),
                 ((False, (2, 3)), (True, (2, 3)))
                 )
-
+        self.assertEqual(post.index.name, 's')
 
     def test_frame_iter_group_b(self) -> None:
         columns = tuple('pqrst')
@@ -389,7 +390,6 @@ class TestUnit(TestCase):
                 ((('A', 1), 1), (('A', 2), 1), (('B', 1), 1), (('B', 2), 1))
                 )
 
-
     def test_frame_iter_group_c(self) -> None:
         columns = tuple('pqrst')
         index = tuple('zxwy')
@@ -432,8 +432,22 @@ class TestUnit(TestCase):
         post2 = tuple(f.iter_group('a'))
         self.assertEqual(post2, ())
 
+    def test_frame_iter_group_g(self) -> None:
+        f = sf.Frame.from_records(
+                [['1998-10-12', 1], ['1998-10-13', 2]],
+                columns=['A', 'B'],
+                dtypes=['datetime64[D]', int]
+                )
+        post = f.iter_group('A').apply(lambda v: v['B'].sum(),
+                index_constructor=IndexDate)
+
+        self.assertEqual(post.index.__class__, IndexDate)
+        self.assertEqual(post.to_pairs(),
+            ((np.datetime64('1998-10-12'), 1),
+            (np.datetime64('1998-10-13'), 2)))
 
     #---------------------------------------------------------------------------
+
     def test_frame_iter_group_items_a(self) -> None:
 
         # testing a hierarchical index and columns, selecting column with a tuple
@@ -458,7 +472,6 @@ class TestUnit(TestCase):
         self.assertEqual(groups[1][1].to_pairs(0),
                 ((('i', 'a'), ((('b', 999999), 'b'), (('b', 201810), 'b'))), (('i', 'b'), ((('b', 999999), 999999), (('b', 201810), 201810))), (('i', 'c'), ((('b', 999999), 0.4), (('b', 201810), 0.4)))))
 
-
     def test_frame_iter_group_items_b(self) -> None:
         columns = tuple('pqrst')
         index = tuple('zxwy')
@@ -476,8 +489,7 @@ class TestUnit(TestCase):
                 ((False, 'False: 2'), (True, 'True: 2'))
                 )
 
-
-    def test_frame_iter_group_items_c(self) -> None:
+    def test_frame_iter_group_items_c1(self) -> None:
         # Test optimized sorting approach. Data must have a non-object dtype and key must be single
         data = np.array([[0, 1, 1, 3],
                          [3, 3, 2, 3],
@@ -502,19 +514,26 @@ class TestUnit(TestCase):
         self.assertEqual(expected_pairs, [group[1].to_pairs(axis=0) for group in groups])
 
 
-        # Index
+    def test_frame_iter_group_items_c2(self) -> None:
+        # Test optimized sorting approach. Data must have a non-object dtype and key must be single
+        data = np.array([[0, 1, 1, 3],
+                         [3, 3, 2, 3],
+                         [5, 5, 1, 3],
+                         [7, 2, 2, 4]])
+
+        frame = sf.Frame(data, columns=tuple('abcd'), index=tuple('wxyz'))
         groups = list(frame.iter_group_items('w', axis=1))
         expected_pairs = [
-                (('a', (('w', 0), ('x', 3), ('y', 5), ('z', 7))),), #type: ignore
-                (('b', (('w', 1), ('x', 3), ('y', 5), ('z', 2))), #type: ignore
+                (('a', (('w', 0), ('x', 3), ('y', 5), ('z', 7))),),
+                (('b', (('w', 1), ('x', 3), ('y', 5), ('z', 2))),
                  ('c', (('w', 1), ('x', 2), ('y', 1), ('z', 2)))),
-                (('d', (('w', 3), ('x', 3), ('y', 3), ('z', 4))),)] #type: ignore
+                (('d', (('w', 3), ('x', 3), ('y', 3), ('z', 4))),)]
 
         self.assertEqual([0, 1, 3], [group[0] for group in groups])
         self.assertEqual(expected_pairs, [group[1].to_pairs(axis=0) for group in groups])
 
 
-    def test_frame_iter_group_items_d(self) -> None:
+    def test_frame_iter_group_items_d1(self) -> None:
         # Test iterating with multiple key selection
         data = np.array([[0, 1, 1, 3],
                          [3, 3, 2, 3],
@@ -525,6 +544,7 @@ class TestUnit(TestCase):
 
         # Column
         groups = list(frame.iter_group_items(['c', 'd'], axis=0))
+        self.assertEqual([(1, 3), (2, 3), (2, 4)], [group[0] for group in groups])
         expected_pairs = [
                 (('a', (('w', 0), ('y', 5))),
                  ('b', (('w', 1), ('y', 5))),
@@ -539,23 +559,30 @@ class TestUnit(TestCase):
                  ('c', (('z', 2),)),
                  ('d', (('z', 4),)))]
 
-        self.assertEqual([(1, 3), (2, 3), (2, 4)], [group[0] for group in groups])
         self.assertEqual(expected_pairs, [group[1].to_pairs(axis=0) for group in groups])
 
 
+    def test_frame_iter_group_items_d2(self) -> None:
+        # Test iterating with multiple key selection
+        data = np.array([[0, 1, 1, 3],
+                         [3, 3, 2, 3],
+                         [5, 5, 1, 3],
+                         [7, 2, 2, 4]])
+
+        frame = sf.Frame(data, columns=tuple('abcd'), index=tuple('wxyz'))
         # Index
         groups = list(frame.iter_group_items(['x', 'y'], axis=1))
+        self.assertEqual([(2, 1), (3, 3), (3, 5)], [group[0] for group in groups])
+
         expected_pairs = [
-                (('c', (('w', 1), ('x', 2), ('y', 1), ('z', 2))),), #type: ignore
-                (('d', (('w', 3), ('x', 3), ('y', 3), ('z', 4))),), #type: ignore
-                (('a', (('w', 0), ('x', 3), ('y', 5), ('z', 7))), #type: ignore
+                (('c', (('w', 1), ('x', 2), ('y', 1), ('z', 2))),),
+                (('d', (('w', 3), ('x', 3), ('y', 3), ('z', 4))),),
+                (('a', (('w', 0), ('x', 3), ('y', 5), ('z', 7))),
                  ('b', (('w', 1), ('x', 3), ('y', 5), ('z', 2)))),
         ]
 
-        self.assertEqual([(2, 1), (3, 3), (3, 5)], [group[0] for group in groups])
         self.assertEqual(expected_pairs, [group[1].to_pairs(axis=0) for group in groups])
 
-
     def test_frame_iter_group_items_e(self) -> None:
         columns = tuple('pqrst')
         index = tuple('zxwy')
@@ -587,7 +614,6 @@ class TestUnit(TestCase):
                 ((False, False), (True, False), (True, True))
                 )
 
-
     def test_frame_iter_group_items_f(self) -> None:
 
         objs = [object() for _ in range(2)]
@@ -618,8 +644,8 @@ class TestUnit(TestCase):
         self.assertEqual(post2[obj_b].to_pairs(0),
                 (('a', ((2, 5),)), ('b', ((2, 6),)), ('c', ((2, obj_b),))))
 
-
     #---------------------------------------------------------------------------
+
     def test_frame_iter_group_index_a(self) -> None:
 
         records = (
@@ -679,7 +705,6 @@ class TestUnit(TestCase):
                 (('a', 34), ('b', 131))
                 )
 
-
     def test_frame_iter_group_index_c(self) -> None:
         columns = tuple('pqrst')
         index = tuple('zxwy')
@@ -716,7 +741,6 @@ class TestUnit(TestCase):
 
         self.assertTrue(tuple(reversed(f)) == tuple(reversed(columns)))
 
-
     #---------------------------------------------------------------------------
 
     def test_frame_axis_window_items_a(self) -> None:
@@ -749,8 +773,6 @@ class TestUnit(TestCase):
         self.assertEqual(post1[-1][1].__class__, Frame)
         self.assertEqual(post1[-1][1].shape, (20, 2))
 
-
-
     def test_frame_axis_window_items_b(self) -> None:
 
         base = np.array([1, 2, 3, 4])
@@ -781,8 +803,6 @@ class TestUnit(TestCase):
         self.assertEqual(post1[-1][1].__class__, np.ndarray)
         self.assertEqual(post1[-1][1].shape, (20, 2))
 
-
-
     def test_frame_iter_window_a(self) -> None:
 
         base = np.array([1, 2, 3, 4])
@@ -801,9 +821,8 @@ class TestUnit(TestCase):
         self.assertEqual(len(post), 18)
         self.assertTrue(all(f.shape == (3, 4) for f in post))
 
-
-
     #---------------------------------------------------------------------------
+
     def test_frame_axis_interface_a(self) -> None:
         # reindex both axis
         records = (
@@ -845,8 +864,8 @@ class TestUnit(TestCase):
         self.assertEqual(list(s3.items()),
                 [('w', None), ('x', 64), ('y', None), ('z', 138)])
 
-
     #---------------------------------------------------------------------------
+
     def test_frame_group_a(self) -> None:
         # reindex both axis
         records = (
@@ -860,10 +879,6 @@ class TestUnit(TestCase):
                 columns=('p', 'q', 'r', 's', 't'),
                 index=('w', 'x', 'y', 'z'))
 
-        with self.assertRaises(AxisInvalid):
-            post = tuple(f1._axis_group_iloc_items(4, axis=-1))
-
-
         post = tuple(f1._axis_group_iloc_items(4, axis=0)) # row iter, group by column 4
 
         group1, group_frame_1 = post[0]
@@ -878,7 +893,6 @@ class TestUnit(TestCase):
         self.assertEqual(group_frame_2.to_pairs(0),
                 (('p', (('z', 30),)), ('q', (('z', 73),)), ('r', (('z', 'd'),)), ('s', (('z', True),)), ('t', (('z', True),))))
 
-
     def test_frame_group_b(self) -> None:
         # reindex both axis
         records = (
@@ -908,7 +922,40 @@ class TestUnit(TestCase):
         self.assertEqual(post[2][1].to_pairs(0),
                 (('r', (('w', 'a'), ('x', 'b'), ('y', 'c'), ('z', 'd'))),))
 
+    def test_frame_group_c(self) -> None:
+        f = ff.parse('s(10,3)|v(int,str,bool)').assign[0].apply(lambda s: s % 4)
+        post1 = tuple(f.iter_group(0, axis=0, drop=True))
+        self.assertEqual(len(post1), 2)
+        self.assertEqual(post1[0].to_pairs(),
+                ((1, ((3, 'zuVU'), (5, 'zJXD'), (6, 'zPAQ'), (7, 'zyps'))), (2, ((3, True), (5, False), (6, True), (7, True))))
+                )
 
+        self.assertEqual(post1[1].to_pairs(),
+                ((1, ((0, 'zaji'), (1, 'zJnC'), (2, 'zDdR'), (4, 'zKka'), (8, 'zyG5'), (9, 'zvMu'))), (2, ((0, True), (1, False), (2, False), (4, False), (8, True), (9, False))))
+                )
+
+        post2 = tuple(f.iter_group_items(0, axis=0, drop=True))
+        self.assertEqual(len(post2), 2)
+        self.assertEqual(post2[0][0], 0)
+        self.assertEqual(post2[0][1].shape, (4, 2))
+        self.assertEqual(post2[1][0], 3)
+        self.assertEqual(post2[1][1].shape, (6, 2))
+
+    def test_frame_group_d(self) -> None:
+        f = ff.parse('s(3,6)|v(bool)')
+        post1 = tuple(f.iter_group(0, axis=1, drop=True))
+        self.assertEqual(len(post1), 2)
+        self.assertEqual(post1[0].shape, (2, 4))
+        self.assertEqual(post1[1].shape, (2, 2))
+
+        post2 = tuple(f.iter_group_items(0, axis=1, drop=True))
+        self.assertEqual(len(post1), 2)
+        self.assertEqual(post2[0][0], False)
+        self.assertEqual(post2[0][1].shape, (2, 4))
+        self.assertEqual(post2[1][0], True)
+        self.assertEqual(post2[1][1].shape, (2, 2))
+
+    #---------------------------------------------------------------------------
 
     def test_frame_axis_interface_b(self) -> None:
         # reindex both axis
@@ -935,8 +982,7 @@ class TestUnit(TestCase):
         s1 = f1.iter_group('p', axis=0).apply(lambda f: f['q'].values.sum())
         self.assertEqual(list(s1.items()), [(2, 97), (30, 107)])
 
+
 if __name__ == '__main__':
+    import unittest
     unittest.main()
-
-
-
