@@ -6,6 +6,7 @@ import typing as tp
 from io import StringIO
 
 import numpy as np
+from arraykit import mloc
 
 from static_frame import Index
 from static_frame import IndexGO
@@ -16,12 +17,16 @@ from static_frame import Series
 from static_frame import IndexYear
 from static_frame import Frame
 from static_frame import ILoc
+from static_frame import IndexAutoFactory
 from static_frame.test.test_case import TestCase
 from static_frame.core.index import _index_initializer_needs_init
 from static_frame.core.exception import ErrorInitIndex
 from static_frame.core.exception import LocInvalid
+from static_frame.core.exception import ErrorInitIndexNonUnique
+
 from static_frame.core.util import PositionsAllocator
-from static_frame.core.util import mloc
+from static_frame.core.util import arrays_equal
+from static_frame.core.util import NULL_SLICE
 
 
 class TestUnit(TestCase):
@@ -44,8 +49,8 @@ class TestUnit(TestCase):
         with self.assertRaises(AttributeError):
             idx1.__dict__ #pylint: disable=W0104
 
-
     #---------------------------------------------------------------------------
+
     def test_index_init_a(self) -> None:
         idx1 = Index(('a', 'b', 'c', 'd'), name='foo')
         idx2 = Index(idx1)
@@ -53,7 +58,6 @@ class TestUnit(TestCase):
         self.assertEqual(idx1.name, 'foo')
         self.assertEqual(idx2.name, 'foo')
 
-
     def test_index_init_b(self) -> None:
 
         idx1 = IndexHierarchy.from_product(['A', 'B'], [1, 2])
@@ -63,7 +67,6 @@ class TestUnit(TestCase):
         self.assertEqual(idx2.values.tolist(),
             [('A', 1), ('A', 2), ('B', 1), ('B', 2)])
 
-
     def test_index_init_c(self) -> None:
 
         s1 = Series(('a', 'b', 'c'))
@@ -103,17 +106,31 @@ class TestUnit(TestCase):
                 [(0, 1, 2), (3, 4, 5)]
                 )
 
-
     def test_index_init_h(self) -> None:
         index = Index(range(10, 20, 2))
         self.assertEqual(index.values.tolist(), list(range(10, 20, 2)))
 
-
     def test_index_init_i(self) -> None:
         i1 = Index([10, 20, 30], name='foo')
         i2 = Index(i1)
         self.assertEqual(i2.name, 'foo')
 
+    def test_index_init_j(self) -> None:
+        from itertools import chain
+
+        with self.assertRaises(ErrorInitIndexNonUnique):
+            idx1 = Index(list(chain(range(100), range(50, 200))))
+
+        with self.assertRaises(ErrorInitIndexNonUnique):
+            idx1 = Index(list(chain(range(100), range(50, 200))))
+
+    def test_index_init_k(self) -> None:
+        with self.assertRaises(ErrorInitIndexNonUnique):
+            _ = Index((x for x in (3, 5, 3)))
+
+    def test_index_init_l(self) -> None:
+        with self.assertRaises(ErrorInitIndex):
+            idx = Index('bar')
 
     #---------------------------------------------------------------------------
 
@@ -129,7 +146,6 @@ class TestUnit(TestCase):
         self.assertEqual(idx._loc_to_iloc(slice('b','d')), slice(1, 4, None))
         self.assertEqual(idx._loc_to_iloc('d'), 3)
 
-
     def test_index_loc_to_iloc_b(self) -> None:
         idx = Index(('a', 'b', 'c', 'd'))
         post = idx._loc_to_iloc(Series(['b', 'c']))
@@ -143,7 +159,6 @@ class TestUnit(TestCase):
         post = idx._loc_to_iloc(['c', 'd', 'e'], partial_selection=True)
         self.assertEqual(post, [2, 3])
 
-
     def test_index_loc_to_iloc_d(self) -> None:
         # testing the public interface
         idx1 = Index(('a', 'b', 'c', 'd'))
@@ -187,9 +202,67 @@ class TestUnit(TestCase):
         with self.assertRaises(IndexError):
             _ = idx2.loc_to_iloc(np.array([False, True, False]))
 
+    def test_index_loc_to_iloc_f(self) -> None:
+        dt = datetime.date
+        dt64 = np.datetime64
 
+        idx1 = Index((
+                dt(2020,12,31),
+                dt(2021,1,15),
+                dt(2021,1,31),
+                ))
+
+        self.assertEqual(
+                idx1.loc_to_iloc(dt64('2021-01-15')),
+                1
+                )
+        # NOTE: this fails as we only see a list of dt64s and cannot match them in the AutoMap dictionary unless we were to directly examine and conert each element
+        with self.assertRaises(KeyError):
+            _ = idx1.loc_to_iloc([dt64(d) for d in reversed(idx1)])
+
+        post = idx1.loc_to_iloc(np.array([dt64(d) for d in reversed(idx1)]))
+        self.assertEqual(post, [2, 1, 0])
+
+    def test_index_loc_to_iloc_g(self) -> None:
+        dt = datetime.date
+        dt64 = np.datetime64
+
+        idx1 = IndexYear(('2021', '2018', '2001'))
+
+        with self.assertRaises(KeyError):
+            idx1.loc_to_iloc(dt64('2001-01-01'))
+
+        with self.assertRaises(KeyError):
+            idx1.loc_to_iloc(np.array((dt64('2001-01-01'), dt64('2018-01-01'))))
+
+    def test_index_loc_to_iloc_h(self) -> None:
+        dt = datetime.date
+        dt64 = np.datetime64
+
+        idx1 = IndexDate(('2021-01-01', '2021-01-02', '1543-08-31', '1988-05-01', '1988-05-02'))
+
+        self.assertEqual(idx1.loc_to_iloc(dt64('2021-01')).tolist(), [0, 1]) #type: ignore
+
+        self.assertEqual(
+                idx1.loc_to_iloc(np.array((dt64('2021'), dt64('1988')))).tolist(), #type: ignore
+                [0, 1, 3, 4]
+                )
+
+    def test_index_loc_to_iloc_i(self) -> None:
+        idx1 = Index(range(4), loc_is_iloc=True)
+        self.assertTrue(idx1._map is None)
+
+        idx2 = idx1[:]
+        self.assertTrue(idx2._map is None) #type: ignore
+
+    def test_index_loc_to_iloc_j(self) -> None:
+        idx1 = IndexAutoFactory.from_optional_constructor(10,
+                default_constructor=Index)
+        post = idx1.loc_to_iloc(NULL_SLICE)
+        self.assertEqual(post, NULL_SLICE)
 
     #---------------------------------------------------------------------------
+
     def test_index_mloc_a(self) -> None:
         idx = Index(('a', 'b', 'c', 'd'))
         self.assertTrue(idx.mloc == idx[:2].mloc) #type: ignore
@@ -199,28 +272,24 @@ class TestUnit(TestCase):
         idx.append('e')
         self.assertTrue(idx.mloc == idx[:2].mloc) #type: ignore
 
-
     def test_index_dtype_a(self) -> None:
         idx = IndexGO(('a', 'b', 'c', 'd'))
         self.assertEqual(str(idx.dtype), '<U1')
         idx.append('eee')
         self.assertEqual(str(idx.dtype), '<U3')
 
-
     def test_index_shape_a(self) -> None:
         idx = IndexGO(('a', 'b', 'c', 'd'))
         self.assertEqual(idx.shape, (4,))
         idx.append('e')
         self.assertEqual(idx.shape, (5,))
 
-
     def test_index_ndim_a(self) -> None:
         idx = IndexGO(('a', 'b', 'c', 'd'))
         self.assertEqual(idx.ndim, 1)
         idx.append('e')
         self.assertEqual(idx.ndim, 1)
 
-
     def test_index_size_a(self) -> None:
         idx = IndexGO(('a', 'b', 'c', 'd'))
         self.assertEqual(idx.size, 4)
@@ -256,8 +325,7 @@ class TestUnit(TestCase):
         idx1.append('e')
         self.assertEqual(idx1.positions.tolist(), list(range(5)))
 
-
-    def test_index_unique(self) -> None:
+    def test_index_unique_a(self) -> None:
 
         with self.assertRaises(ErrorInitIndex):
             idx = Index(('a', 'b', 'c', 'a'))
@@ -277,12 +345,13 @@ class TestUnit(TestCase):
         # acceptable but not advisiable
         idx = Index([0, '0'])
 
+    def test_index_unique_b(self) -> None:
+        idx = Index(('a', 'b', 'c', 'd'))
+        self.assertEqual(idx.unique().tolist(), idx.values.tolist())
 
     def test_index_creation_a(self) -> None:
         idx = Index(('a', 'b', 'c', 'd'))
 
-        #idx2 = idx['b':'d']
-
         self.assertEqual(idx.values.tolist(), ['a', 'b', 'c', 'd'])
 
         self.assertEqual(idx[2:].values.tolist(), ['c', 'd']) #type: ignore
@@ -304,7 +373,6 @@ class TestUnit(TestCase):
         idxgo.extend(('f', 'g'))
         self.assertEqual(idxgo.values.tolist(), ['a', 'b', 'c', 'd', 'e', 'f', 'g'])
 
-
     def test_index_creation_b(self) -> None:
         idx = Index((x for x in ('a', 'b', 'c', 'd') if x in {'b', 'd'}))
         self.assertEqual(idx._loc_to_iloc('b'), 0)
@@ -324,14 +392,12 @@ class TestUnit(TestCase):
         self.assertEqual(not_idx.tolist(),
                 [-21, -31, -41, -51],)
 
-
     def test_index_unary_operators_b(self) -> None:
         idx = IndexGO((20, 30, 40))
         idx.append(50)
         a1 = -idx
         self.assertEqual(a1.tolist(), [-20, -30, -40, -50])
 
-
     def test_index_binary_operators_a(self) -> None:
         idx = Index((20, 30, 40, 50))
 
@@ -349,7 +415,6 @@ class TestUnit(TestCase):
                 (2 - idx).tolist(),
                 [-18, -28, -38, -48])
 
-
     def test_index_binary_operators_b(self) -> None:
         '''Both operands are Index instances
         '''
@@ -382,8 +447,11 @@ class TestUnit(TestCase):
         self.assertEqual(('_' + idx1).tolist(), ['_a', '_b', '_c'])
         self.assertEqual((idx1 * 3).tolist(), ['aaa', 'bbb', 'ccc'])
 
-
-
+    def test_index_binary_operators_g(self) -> None:
+        idx1 = Index((1, 2, 3))
+        s1 = Series(('a', 'b', 'c'))
+        with self.assertRaises(ValueError):
+            _ = idx1 * s1
 
     #---------------------------------------------------------------------------
 
@@ -401,7 +469,6 @@ class TestUnit(TestCase):
         idx.append(10)
         self.assertEqual(idx.sum(), 100)
 
-
     def test_index_isin_a(self) -> None:
 
         idx = Index((30, 40, 50))
@@ -430,6 +497,7 @@ class TestUnit(TestCase):
                 )
 
     #---------------------------------------------------------------------------
+
     def test_index_copy_a(self) -> None:
         idx1 = IndexGO(('a', 'b', 'c'))
         idx1.append('d')
@@ -438,7 +506,6 @@ class TestUnit(TestCase):
         self.assertEqual(idx2.values.tolist(), ['a', 'b', 'c', 'd', 'e'])
         self.assertEqual(idx1.values.tolist(), ['a', 'b', 'c', 'd'])
 
-
     #---------------------------------------------------------------------------
 
     def test_index_contains_a(self) -> None:
@@ -472,7 +539,6 @@ class TestUnit(TestCase):
         self.assertEqual(index3.values.tolist(), ['c', 'd', 'e', 'f', 'h', 'i']) #type: ignore
         self.assertEqual(index.values.tolist(), ['a', 'b', 'c', 'd', 'e', 'f', 'h'])
 
-
     def test_index_go_b(self) -> None:
 
         index = IndexGO(('a', 'b', 'c'))
@@ -488,7 +554,6 @@ class TestUnit(TestCase):
         with self.assertRaises(KeyError):
             index.append((2, 5))
 
-
     def test_index_go_d(self) -> None:
 
         index = IndexGO((), loc_is_iloc=True)
@@ -505,8 +570,6 @@ class TestUnit(TestCase):
         self.assertTrue('a' in index)
         self.assertTrue(1 in index)
 
-
-
     def test_index_go_e(self) -> None:
 
         index = IndexGO((), loc_is_iloc=True)
@@ -523,12 +586,16 @@ class TestUnit(TestCase):
         self.assertTrue(-1 in index)
         self.assertTrue(1 in index)
 
+    def test_index_go_f(self) -> None:
 
-
+        idx1 = IndexAutoFactory.from_optional_constructor(3,
+                default_constructor=IndexGO)
+        idx1.append(3) # type: ignore
+        post = idx1._loc_to_iloc(np.array([True, False, True, False]), 2) #type: ignore
+        self.assertEqual(post.tolist(), [2, 4]) #type: ignore
 
     #---------------------------------------------------------------------------
 
-
     def test_index_sort_a(self) -> None:
 
         index = Index(('a', 'c', 'd', 'e', 'b'))
@@ -539,8 +606,6 @@ class TestUnit(TestCase):
                 [index.sort(ascending=False)._loc_to_iloc(x) for x in sorted(index.values)],
                 [4, 3, 2, 1, 0])
 
-
-
     def test_index_sort_b(self) -> None:
 
         index = Index(('ax', 'cb', 'dg', 'eb', 'bq'))
@@ -551,6 +616,11 @@ class TestUnit(TestCase):
                 ['cb', 'eb', 'dg', 'bq', 'ax']
                 )
 
+    def test_index_sort_c(self) -> None:
+        index = Index(('a', 'c', 'd', 'e', 'b'))
+        with self.assertRaises(RuntimeError):
+            index.sort(ascending=(True, False))
+
     #---------------------------------------------------------------------------
 
     def test_index_relable(self) -> None:
@@ -577,9 +647,6 @@ class TestUnit(TestCase):
         self.assertEqual(index.relabel({'e': 'E'}).values.tolist(),
                 ['a', 'c', 'd', 'E', 'b'])
 
-
-
-
     def test_index_tuples_a(self) -> None:
 
         index = Index([('a','b'), ('b','c'), ('c','d')])
@@ -599,7 +666,6 @@ class TestUnit(TestCase):
         self.assertEqual(s2[[(1, 7), (1, 5), (1, 0)]].values.tolist(),
                 [7, 5, 0])
 
-
     def test_index_pickle_a(self) -> None:
         a = Index([('a','b'), ('b','c'), ('c','d')])
         b = Index([1, 2, 3, 4])
@@ -627,8 +693,8 @@ class TestUnit(TestCase):
         self.assertEqual(index._drop_loc(slice('b', None)).values.tolist(),
                 ['a'])
 
-
     #---------------------------------------------------------------------------
+
     def test_index_iloc_loc_to_iloc_a(self) -> None:
 
         idx = Index(('a', 'b', 'c', 'd'))
@@ -644,7 +710,6 @@ class TestUnit(TestCase):
         self.assertEqual(post.values.tolist(), #type: ignore
                 ['a', 'b', 'c', 'd', 'e'])
 
-
     def test_index_drop_iloc_a(self) -> None:
 
         idx = IndexGO(('a', 'b', 'c', 'd'))
@@ -653,7 +718,6 @@ class TestUnit(TestCase):
         self.assertEqual(post.values.tolist(),
                 ['a', 'd', 'e'])
 
-
     def test_index_drop_iloc_b(self) -> None:
 
         idx = IndexGO(('a', 'b', 'c', 'd'))
@@ -662,7 +726,6 @@ class TestUnit(TestCase):
         self.assertEqual(post.values.tolist(),
                 ['a', 'b', 'c', 'd', 'e'])
 
-
     #---------------------------------------------------------------------------
 
     def test_index_loc_to_iloc_boolen_a(self) -> None:
@@ -681,7 +744,6 @@ class TestUnit(TestCase):
         assert isinstance(post, np.ndarray)
         self.assertEqual(post.tolist(), [1, 3])
 
-
     def test_index_loc_to_iloc_boolen_b(self) -> None:
 
         idx = Index(('a', 'b', 'c', 'd'))
@@ -700,7 +762,6 @@ class TestUnit(TestCase):
         assert isinstance(post, np.ndarray)
         self.assertEqual(post.tolist(), [1,3])
 
-
     def test_index_drop_b(self) -> None:
 
         idx = Index(('a', 'b', 'c', 'd'))
@@ -747,7 +808,6 @@ class TestUnit(TestCase):
         self.assertEqual(idx4.values.tolist(),
                 [10, 20, 30.1])
 
-
     def test_index_fillna_b(self) -> None:
 
         idx1 = Index(('a', 'b', 'c'))
@@ -758,6 +818,13 @@ class TestUnit(TestCase):
         idx4 = idx3.fillna('d')
         self.assertNotEqual(id(idx3), id(idx4))
 
+    def test_index_fillna_c(self) -> None:
+
+        idx1 = Index((3.0, 2.0, np.nan))
+        idx2 = idx1.fillna('foo')
+        self.assertEqual(idx2.values.tolist(),
+                [3.0, 2.0, 'foo'],
+                )
 
     #---------------------------------------------------------------------------
 
@@ -769,7 +836,6 @@ class TestUnit(TestCase):
         self.assertEqual(idx.ndim, 1)
         self.assertEqual(idx.nbytes, 16)
 
-
     def test_index_name_a(self) -> None:
 
         idx1 = Index(('a', 'b', 'c', 'd'), name='foo')
@@ -788,7 +854,6 @@ class TestUnit(TestCase):
         with self.assertRaises(TypeError):
             Index(('a', 'b', 'c', 'd'), name={'x', 'y'})
 
-
     def test_index_name_c(self) -> None:
 
         idx1 = IndexGO(('a', 'b', 'c', 'd'), name='foo')
@@ -807,6 +872,7 @@ class TestUnit(TestCase):
                 ['a', 'b', 'c', 'd', 'x'])
 
     #---------------------------------------------------------------------------
+
     def test_index_to_series_a(self) -> None:
 
         idx1 = IndexGO(('a', 'b', 'c', 'd'), name='foo')
@@ -816,7 +882,6 @@ class TestUnit(TestCase):
                 ((0, 'a'), (1, 'b'), (2, 'c'), (3, 'd'))
                 )
 
-
     #---------------------------------------------------------------------------
 
     def test_index_to_pandas_a(self) -> None:
@@ -826,7 +891,6 @@ class TestUnit(TestCase):
         self.assertEqual(pdidx.name, idx1.name)
         self.assertTrue((pdidx.values == idx1.values).all())
 
-
     def test_index_to_pandas_b(self) -> None:
         import pandas
         idx1 = IndexDate(('2018-01-01', '2018-06-01'), name='foo')
@@ -837,7 +901,6 @@ class TestUnit(TestCase):
 
     #---------------------------------------------------------------------------
 
-
     def test_index_from_pandas_a(self) -> None:
         import pandas
 
@@ -845,7 +908,6 @@ class TestUnit(TestCase):
         idx = Index.from_pandas(pdidx)
         self.assertEqual(idx.values.tolist(), ['a', 'b', 'c', 'd'])
 
-
     def test_index_from_pandas_b(self) -> None:
         import pandas
 
@@ -864,7 +926,6 @@ class TestUnit(TestCase):
                 [datetime.date(2018, 1, 1), datetime.date(2018, 6, 1)]
                 )
 
-
     def test_index_from_pandas_d(self) -> None:
         import pandas
         pdidx = pandas.DatetimeIndex(('2018-01-01', '2018-06-01'), name='foo')
@@ -892,8 +953,8 @@ class TestUnit(TestCase):
         with self.assertRaises(ErrorInitIndex):
             idx = Index.from_pandas(Index(('a', 'b')))
 
-
     #---------------------------------------------------------------------------
+
     def test_index_iter_a(self) -> None:
 
         idx1 = IndexGO(('a', 'b', 'c'), name='foo')
@@ -927,8 +988,27 @@ class TestUnit(TestCase):
         post = idx1.iter_label(0).apply(lambda x: x.upper())
         self.assertEqual(post.tolist(), ['A', 'B', 'C', 'D'])
 
+    def test_index_iter_label_b(self) -> None:
+
+        idx1 = IndexGO(('a', 'b', 'c', 'd'), name='foo')
+        post = tuple(idx1.iter_label().apply_iter_items(
+            lambda x: x.upper()
+            ))
+        self.assertEqual(post,
+            ((0, 'A'), (1, 'B'), (2, 'C'), (3, 'D')),
+            )
+
+    def test_index_iter_label_c(self) -> None:
+
+        idx1 = IndexGO(('a', 'b', 'c', 'd'), name='foo')
+        with self.assertRaises(RuntimeError):
+            _ = tuple(idx1.iter_label().apply(
+                    lambda x: x.upper(),
+                    index_constructor=IndexDate,
+                    ))
 
     #---------------------------------------------------------------------------
+
     def test_index_intersection_a(self) -> None:
 
         idx1 = Index(('a', 'b', 'c', 'd', 'e'))
@@ -940,7 +1020,6 @@ class TestUnit(TestCase):
         self.assertEqual(idx2.values.tolist(),
                 ['a', 'b', 'c'])
 
-
     def test_index_intersection_b(self) -> None:
 
         idx1 = Index(('c', 'b', 'a'))
@@ -983,7 +1062,6 @@ class TestUnit(TestCase):
         self.assertEqual(len(idx4), 0)
         self.assertEqual(idx4.dtype, np.dtype(object))
 
-
     def test_index_intersection_f(self) -> None:
 
         idx1 = IndexDate.from_date_range('2020-01-02', '2020-01-08')
@@ -1016,7 +1094,6 @@ class TestUnit(TestCase):
         idx4 = idx1.intersection(idx2, idx3)
         self.assertEqual(idx4.values.tolist(), ['b'])
 
-
     #---------------------------------------------------------------------------
 
     def test_index_union_a(self) -> None:
@@ -1030,7 +1107,6 @@ class TestUnit(TestCase):
         self.assertEqual(idx2.values.tolist(),
                 ['a', 'b', 'c', 'd', 'dd', 'e', 'f'])
 
-
     def test_index_union_b(self) -> None:
 
         idx1 = Index(('c', 'b', 'a'))
@@ -1041,7 +1117,6 @@ class TestUnit(TestCase):
                 ['c', 'b', 'a']
                 )
 
-
     def test_index_union_c(self) -> None:
 
         idx1 = Index(('c', 'b', 'a'))
@@ -1104,13 +1179,15 @@ class TestUnit(TestCase):
                 set([np.nan, None, 4.4, obj])
                 ) # Note: order is lost...
 
-
-
     def test_index_to_html_a(self) -> None:
 
         idx1 = IndexGO(('a', 'b', 'c'))
+
         self.assertEqual(idx1.to_html(),
-                '<table border="1"><tbody><tr><td>a</td></tr><tr><td>b</td></tr><tr><td>c</td></tr></tbody></table>')
+                '<table style="border-collapse:collapse;border-width:1px;border-color:#898b8e;border-style:solid"><tbody><tr><td style="background-color:#ffffff;font-weight:normal;padding:2px;font-size:14px;border-width:1px;border-color:#898b8e;border-style:solid;color:#2b2a2a">a</td></tr><tr><td style="background-color:#f2f2f2;font-weight:normal;padding:2px;font-size:14px;border-width:1px;border-color:#898b8e;border-style:solid;color:#2b2a2a">b</td></tr><tr><td style="background-color:#ffffff;font-weight:normal;padding:2px;font-size:14px;border-width:1px;border-color:#898b8e;border-style:solid;color:#2b2a2a">c</td></tr></tbody></table>'
+                )
+        self.assertEqual(idx1.to_html(style_config=None),
+                '<table><tbody><tr><td>a</td></tr><tr><td>b</td></tr><tr><td>c</td></tr></tbody></table>')
 
     def test_index_to_html_datatables_a(self) -> None:
 
@@ -1123,7 +1200,6 @@ class TestUnit(TestCase):
         self.assertEqual(post, None)
         self.assertTrue(len(sio.read()) > 1200)
 
-
     def test_index_empty_a(self) -> None:
         idx1 = Index(())
         idx2 = Index(iter(()))
@@ -1149,14 +1225,11 @@ class TestUnit(TestCase):
             (('a', 1), ('b', 1), ('c', 1), ('d', 1), ('e', 1))
             )
 
-
     def test_index_label_widths_at_depth_b(self) -> None:
         idx1 = IndexGO(('a', 'b', 'c', 'd', 'e'))
         with self.assertRaises(RuntimeError):
             next(idx1.label_widths_at_depth(1))
 
-
-
     #---------------------------------------------------------------------------
 
     def test_index_bool_a(self) -> None:
@@ -1198,7 +1271,6 @@ class TestUnit(TestCase):
         with self.assertRaises(RuntimeError):
             idx1.values_at_depth(1)
 
-
     def test_index_values_at_depth_b(self) -> None:
 
         idx1 = IndexGO(('a', 'b', 'c', 'd', 'e'))
@@ -1211,8 +1283,6 @@ class TestUnit(TestCase):
         self.assertEqual(post.tolist(),
                 ['a', 'b', 'c', 'd', 'e'])
 
-
-
     #---------------------------------------------------------------------------
 
     def test_index_head_a(self) -> None:
@@ -1225,8 +1295,8 @@ class TestUnit(TestCase):
         idx1 = IndexGO(('a', 'b', 'c', 'd', 'e'))
         self.assertEqual(idx1.tail(2).values.tolist(), ['d' ,'e'])
 
-
     #---------------------------------------------------------------------------
+
     def test_index_via_str_a(self) -> None:
 
         idx1 = IndexGO(('a', 'b', 'c', 'd', 'e'))
@@ -1236,7 +1306,6 @@ class TestUnit(TestCase):
                 ['A', 'B', 'C', 'D', 'E']
                 )
 
-
     def test_index_via_str_b(self) -> None:
 
         idx1 = IndexGO(('a', 'b', 'c', 'd', 'e'))
@@ -1249,7 +1318,6 @@ class TestUnit(TestCase):
                 ['A', 'B', 'C', 'D', 'E', 'F', 'G']
                 )
 
-
     def test_index_via_dt_a(self) -> None:
 
         idx1 = IndexDate(('2020-01-01', '2021-02-05', '2019-03-17'))
@@ -1272,7 +1340,6 @@ class TestUnit(TestCase):
                 [2, 4, 6, 2, 4]
                 )
 
-
     #---------------------------------------------------------------------------
 
     def test_index_equals_a(self) -> None:
@@ -1297,7 +1364,6 @@ class TestUnit(TestCase):
         self.assertEqual(idx1.equals(idx1), True)
         self.assertEqual(idx1.equals(idx4), True)
 
-
     def test_index_equals_b(self) -> None:
 
         idx1 = Index((5, 3, 20), dtype=np.int64)
@@ -1306,11 +1372,10 @@ class TestUnit(TestCase):
         self.assertFalse(idx1.equals(idx2, compare_dtype=True))
         self.assertTrue(idx1.equals(idx2, compare_dtype=False))
 
-
     def test_index_equals_c(self) -> None:
 
         idx1 = IndexDate.from_year_range('2010', '2011')
-        idx2 = Index(idx1.values)
+        idx2 = Index(idx1.values.astype(object))
 
         self.assertFalse(idx1.equals(idx2, compare_class=True))
         self.assertTrue(idx1.equals(idx2, compare_class=False),)
@@ -1327,7 +1392,6 @@ class TestUnit(TestCase):
         # nan and None are not treated equivalent, even with skipna true
         self.assertFalse(idx1.equals(idx3, compare_dtype=False, skipna=True))
 
-
     def test_index_equals_e(self) -> None:
         a = Index([1, 2, 3])
         b = Index([1, 2, 3])
@@ -1341,14 +1405,28 @@ class TestUnit(TestCase):
         self.assertTrue(not a.equals(d))
         self.assertTrue(not a.equals(e))
 
-
     def test_index_equals_f(self) -> None:
         a = IndexGO([1, 2, 3])
         b = IndexGO([1, 2, 3])
         b.append(4)
         self.assertFalse(a.equals(b))
 
+    def test_index_equals_g(self) -> None:
+        dt64 = np.datetime64
+
+        a = IndexDate([dt64('2021-01-01'), dt64('1954-01-01')])
+        b = IndexYear([dt64('2021'), dt64('1954')])
+
+        self.assertFalse(arrays_equal(a, b, skipna=True))
+
+    def test_index_equals_h(self) -> None:
+        a = IndexGO([1, 2, 3])
+        a.append(4)
+        b = IndexGO([1, 2, 3])
+        self.assertFalse(a.equals(b))
+
     #---------------------------------------------------------------------------
+
     def test_index_sample_a(self) -> None:
         a = IndexGO([1, 2, 3])
         a.append(None)
@@ -1357,6 +1435,7 @@ class TestUnit(TestCase):
         self.assertEqual(b.values.tolist(), [2, None])
 
     #---------------------------------------------------------------------------
+
     def test_index_deepcopy_a(self) -> None:
 
         idx1 = IndexGO(('a', 'b', 'c', 'd'))
@@ -1377,7 +1456,15 @@ class TestUnit(TestCase):
         self.assertTrue(id(idx1._labels) != id(idx2._labels))
         self.assertTrue(idx2._map is None)
 
+    def test_index_deepcopy_c(self) -> None:
+        idx1 = IndexGO(range(3))
+        idx1.append(100)
+        idx2 = copy.deepcopy(idx1)
+        idx1.append(200)
+        self.assertEqual(idx2.values.tolist(), [0, 1, 2, 100])
+
     #---------------------------------------------------------------------------
+
     def test_index_iloc_searchsorted_a(self) -> None:
 
         idx1 = IndexGO(('a', 'b', 'c', 'd'))
@@ -1396,7 +1483,41 @@ class TestUnit(TestCase):
                 ('a', 'e'), side_left=False, fill_value=None).tolist(),
                 ['b', None])
 
+    def test_index_iloc_searchsorted_c(self) -> None:
+        idx1 = Index(())
+        self.assertEqual(idx1.loc_searchsorted(0, fill_value=None), None)
+
+    #---------------------------------------------------------------------------
+
+    def test_index_via_re_a(self) -> None:
+
+        idx1 = IndexGO(('aabbcc', 'bbcccc', 'cc', 'ddddbb'))
+        idx1.append('q')
+        a1 = idx1.via_re('bb').search()
+
+        self.assertEqual(a1.tolist(),
+                [True, True, False, True, False])
+
+    #---------------------------------------------------------------------------
+
+    def test_index_index_types_a(self) -> None:
+
+        idx1 = IndexGO(('a', 'b', 'c', 'd'), name='foo')
+        self.assertEqual(idx1.index_types.to_pairs(), (('foo', IndexGO),))
+
+    #---------------------------------------------------------------------------
+
+    def test_index_level_add_a(self) -> None:
+        idx1 = Index(('a', 'b', 'c'), name='foo')
+        idx2 = idx1.level_add('2012', index_constructor=IndexYear)
+        self.assertEqual(idx2.name, 'foo')
+        self.assertEqual(idx1.index_types.values.tolist(), [Index])
+        self.assertEqual(idx2.index_types.values.tolist(), [IndexYear, Index])
+
+        self.assertTrue(
+            (idx2.values_at_depth(0) == np.array(['2012', '2012', '2012'], dtype='datetime64[Y]')).all()
+            )
+
 
 if __name__ == '__main__':
     unittest.main()
-
