@@ -1,7 +1,8 @@
-
+import re
 import unittest
 from collections import OrderedDict
 import copy
+import datetime
 
 import numpy as np
 
@@ -10,6 +11,9 @@ from static_frame import HLoc
 from static_frame import ILoc
 from static_frame import Index
 from static_frame import IndexDate
+from static_frame import IndexDateGO
+from static_frame import IndexYearMonthGO
+from static_frame import IndexSecond
 from static_frame import IndexGO
 from static_frame import IndexHierarchy
 from static_frame import IndexLevel
@@ -19,9 +23,9 @@ from static_frame.core.exception import ErrorInitIndexLevel
 from static_frame.test.test_case import skip_win
 from static_frame.test.test_case import TestCase
 
-
 from static_frame.core.util import EMPTY_ARRAY
 
+
 class TestUnit(TestCase):
 
     def test_index_level_a(self) -> None:
@@ -55,12 +59,10 @@ class TestUnit(TestCase):
 
         self.assertEqual(level2.depth, next(level2.depths()))
 
-
     def test_index_level_b(self) -> None:
         with self.assertRaises(ErrorInitIndexLevel):
             _ = IndexLevel(('A', 'B')) #type: ignore
 
-
     def test_index_level_dtypes_all_a(self) -> None:
         level0 = IndexLevel(index=Index(('a', 'b')), targets=None)
         post = tuple(level0.dtypes_iter())
@@ -68,7 +70,6 @@ class TestUnit(TestCase):
         self.assertEqual(len(post), 1)
         self.assertEqual(level0.depth, next(level0.depths()))
 
-
     #---------------------------------------------------------------------------
 
     def test_index_level_dtypes_a(self) -> None:
@@ -80,7 +81,6 @@ class TestUnit(TestCase):
         level0 = IndexLevelGO(index=groups, targets=targets)
         self.assertEqual([d.kind for d in level0.dtype_per_depth()], ['U', 'U'])
 
-
     def test_index_level_dtypes_b(self) -> None:
         idx1 = Index(('A', 'B'))
         idx2 = IndexDate.from_date_range('2019-01-05', '2019-01-08')
@@ -92,7 +92,6 @@ class TestUnit(TestCase):
                 ['U', 'M', 'i'],
                 )
 
-
     def test_index_level_dtypes_c(self) -> None:
         level0 = IndexLevel(index=Index(('a', 'b')), targets=None)
         post = tuple(level0.dtype_per_depth())
@@ -116,7 +115,6 @@ class TestUnit(TestCase):
                 (np.dtype('<U1'), np.dtype('<U1'), np.dtype('<U3'))
                 )
 
-
     def test_index_level_values_at_depth_a(self) -> None:
         hidx = IndexHierarchy.from_labels((('a', 1, 'x'), ('a', 2, 'y'), ('b', 1, 'foo'), ('b', 1, 'bar')))
         lvl = hidx._levels
@@ -129,7 +127,6 @@ class TestUnit(TestCase):
         lvl = hidx._levels
         self.assertEqual(lvl.values_at_depth(2).tolist(), ['x', 'y', 'foo', None])
 
-
     #---------------------------------------------------------------------------
 
     def test_index_level_index_types_a(self) -> None:
@@ -143,33 +140,62 @@ class TestUnit(TestCase):
                 ['Index', 'IndexDate', 'Index'])
         self.assertEqual(lvl.depth, next(lvl.depths()))
 
-
     def test_index_level_index_types_b(self) -> None:
         level0 = IndexLevel(index=Index(('a', 'b')), targets=None)
         post = tuple(level0.index_types())
         self.assertEqual(post[0], Index)
         self.assertEqual(len(post), 1)
 
-
     #---------------------------------------------------------------------------
     def test_index_level_contains_a(self) -> None:
         level0 = IndexLevel(index=Index(('a', 'b')), targets=None)
         self.assertFalse(('c',) in level0)
         self.assertTrue(('a',) in level0)
 
-
-
     #---------------------------------------------------------------------------
 
     def test_index_level_extend_a(self) -> None:
         level0 = IndexLevelGO(index=IndexGO(('a', 'b')), targets=None)
+        # RuntimeError: found IndexLevel with None as targets
         with self.assertRaises(RuntimeError):
             level0.extend(level0)
 
+    def test_index_level_extend_b(self) -> None:
+        level0 = IndexLevelGO(index=IndexGO(('a', 'b')), targets=None)
+        level1 = IndexLevelGO(index=IndexGO(('c', 'd')), targets=None)
+        # RuntimeError: found IndexLevel with None as targets
+        with self.assertRaises(RuntimeError):
+            level0.extend(level1)
+
+    def test_index_level_extend_c(self) -> None:
+        observations0 = IndexGO(('x', 'y'))
+        targets0 = ArrayGO(
+                (IndexLevelGO(index=observations0),
+                IndexLevelGO(observations0, offset=2)))
+        level0 = IndexLevelGO(index=IndexDateGO(('2021', '2022')), targets=targets0)
+
+        observations1 = IndexGO(('x', 'y'))
+        targets1 = ArrayGO(
+                (IndexLevelGO(index=observations1),
+                IndexLevelGO(observations1, offset=2)))
+        level1 = IndexLevelGO(index=IndexYearMonthGO(('1853-03', '1873-12')), targets=targets1)
+        # RuntimeError: level for extension does not have corresponding types: <class 'static_frame.core.index_datetime.IndexDateGO'>, <class 'static_frame.core.index_datetime.IndexYearMonthGO'>
+        with self.assertRaises(RuntimeError):
+            level0.extend(level1)
+
+    def test_index_level_extend_d(self) -> None:
+        tree = {
+            "I": {"A": (1, 2), "B": (1, 2, 3), "C": (2, 3)},
+            "II": {"D": (1, 2, 3), "E": (1,)},
+        }
+        lvl0 = IndexLevelGO.from_tree(tree)
+        lvl1 = IndexLevelGO(index=IndexGO(()), targets=None, depth_reference=3)
+        # RuntimeError: found IndexLevel with None as targets
+        with self.assertRaises(RuntimeError):
+            lvl1.extend(lvl0)
 
     #---------------------------------------------------------------------------
 
-
     def test_index_level_get_labels_a(self) -> None:
         groups = IndexGO(('A', 'B'))
         observations = IndexGO(('x', 'y'))
@@ -219,14 +245,12 @@ class TestUnit(TestCase):
         self.assertEqual(lvl0.leaf_loc_to_iloc(('B', '2018-01-04', 'y'),), 15)
         self.assertEqual(lvl0.leaf_loc_to_iloc(('A', '2018-01-01', 'y')), 1)
 
-
     def test_index_level_leaf_loc_to_iloc_b(self) -> None:
 
         level0 = IndexLevel(index=Index(('a', 'b')), targets=None)
         self.assertEqual(level0.leaf_loc_to_iloc(('b',)), 1)
         self.assertEqual(level0.leaf_loc_to_iloc(ILoc[1]), 1)
 
-
     def test_index_level_leaf_loc_to_iloc_c(self) -> None:
 
         groups = Index(('A', 'B', 'C'))
@@ -281,13 +305,11 @@ class TestUnit(TestCase):
         with self.assertRaises(KeyError):
             level0.loc_to_iloc('a')
 
-
     def test_index_level_loc_to_iloc_b(self) -> None:
         level0 = IndexLevel(index=Index(('a', 'b')), targets=None)
         with self.assertRaises(KeyError):
             level0.loc_to_iloc(HLoc['c',])
 
-
     #---------------------------------------------------------------------------
 
     def test_index_level_append_a(self) -> None:
@@ -305,7 +327,6 @@ class TestUnit(TestCase):
                 targets=lvl2_targets,
                 offset=0)
 
-
         # must create new index levels for each lower node
         lvl2c = IndexLevelGO(index=observations)
         lvl2d = IndexLevelGO(index=observations, offset=2)
@@ -363,7 +384,6 @@ class TestUnit(TestCase):
         self.assertEqual(lvl1a.values.tolist(),
                 [['A', 'x'], ['A', 'y'], ['B', 'x'], ['B', 'y'], [1, 2]])
 
-
     def test_index_level_append_c(self) -> None:
 
         levels1 = IndexLevelGO(IndexGO(()), depth_reference=3)
@@ -382,7 +402,6 @@ class TestUnit(TestCase):
         self.assertEqual(levels1.values.tolist(),
                 [['III', 'A', 1], ['III', 'A', 2]])
 
-
     #---------------------------------------------------------------------------
 
     def test_index_level_iter_a(self) -> None:
@@ -409,7 +428,6 @@ class TestUnit(TestCase):
         post = list(levels.index_at_depth(2))
         self.assertEqual(post, [1, 2, 1, 2, 3, 2, 3, 1, 2, 3, 1])
 
-
     def test_index_level_iter_b(self) -> None:
         OD = OrderedDict
         tree = OD([
@@ -430,7 +448,6 @@ class TestUnit(TestCase):
                 (('I', 'A', 1), ('I', 'A', 2), ('I', 'B', 1), ('I', 'B', 2), ('I', 'B', 3), ('I', 'C', 2), ('I', 'C', 3), ('II', 'A', 1), ('II', 'A', 2), ('II', 'A', 3), ('II', 'B', 1))
                 )
 
-
     def test_index_level_label_widths_at_depth_a(self) -> None:
         OD = OrderedDict
         tree = OD([
@@ -464,7 +481,6 @@ class TestUnit(TestCase):
             (((1, 1), (2, 1), (1, 1), (2, 1), (3, 1), (2, 1), (3, 1), (1, 1), (1, 1), (1, 1), (2, 1), (3, 1), (1, 1)))
         )
 
-
     def test_index_level_label_widths_at_depth_b(self) -> None:
         f = Frame.from_dict(
             dict(a=(1,2,3,4), b=(True, False, True, False), c=list('qrst')))
@@ -475,7 +491,6 @@ class TestUnit(TestCase):
         post2 = tuple(f.index._levels.label_widths_at_depth(1))
         self.assertEqual(post2, ((True, 1), (False, 1), (True, 1), (False, 1)))
 
-
     def test_index_level_label_widths_at_depth_c(self) -> None:
         labels = (
                 ('I', 'A', 1),
@@ -517,7 +532,25 @@ class TestUnit(TestCase):
                 ((1, 1), (2, 1), (3, 1), (3, 1), (4, 1))
         )
 
+    def test_index_level_label_widths_at_depth_e(self) -> None:
+        tree = {
+            "I": {"A": (1, 2), "B": (1, 2, 3), "C": (2, 3)},
+            "II": {"D": (1, 2, 3), "E": (1,)},
+        }
+        lvl = IndexLevel.from_tree(tree)
+        widths = tuple(lvl.label_widths_at_depth(1))
+        self.assertEqual(widths, (('A', 2), ('B', 3), ('C', 2), ('D', 3), ('E', 1)))
 
+    def test_index_level_label_widths_at_depth_f(self) -> None:
+        tree = {
+            "I": {"A": (1, 2), "B": (1, 2, 3),},
+            "II": {"D": (1, 2, 3), "E": (1,)},
+        }
+        lvl = IndexLevel.from_tree(tree)
+        widths = tuple(lvl.label_widths_at_depth(0))
+        self.assertEqual(widths, (('I', 5), ('II', 4)))
+
+    #---------------------------------------------------------------------------
     def test_index_levels_with_tuple_a(self) -> None:
         OD = OrderedDict
         tree = OD([
@@ -535,8 +568,6 @@ class TestUnit(TestCase):
         self.assertEqual(levels.depth, 3)
         self.assertEqual(levels.loc_to_iloc((('II', 'II'), 'B', 1)), 8)
 
-
-
     #---------------------------------------------------------------------------
     def test_index_levels_equals_a(self) -> None:
         OD = OrderedDict
@@ -579,8 +610,6 @@ class TestUnit(TestCase):
 
         levels3 = IndexLevel.from_tree(tree3)
 
-
-
         self.assertTrue(levels1.equals(levels1))
         self.assertTrue(levels1.equals(levels2))
         self.assertTrue(levels2.equals(levels1))
@@ -644,7 +673,6 @@ class TestUnit(TestCase):
         # differeing depth
         self.assertFalse(levels2.equals(levels3))
 
-
     def test_index_levels_equals_d(self) -> None:
 
         levels1 = IndexLevel(Index(('a', 'b', 'c')), targets=None)
@@ -652,15 +680,39 @@ class TestUnit(TestCase):
 
         self.assertTrue(levels1.equals(levels2))
 
+    def test_index_levels_equals_e(self) -> None:
+        tree1 = {
+            "I": {"A": (1, 2), "B": (1, 2, 3)},
+            "II": {"D": (1, 2, 3), "E": (1,)},
+        }
+        lvl1 = IndexLevel.from_tree(tree1)
+
+        tree2 = {
+            "I": {"A": (1, 2), "B": (1, 2, 3)},
+            "II": (1, 2, 3),
+        }
+        lvl2 = IndexLevel.from_tree(tree2)
+
+        self.assertFalse(lvl1.equals(lvl2))
+
     #---------------------------------------------------------------------------
 
     def test_index_levels_to_type_blocks_a(self) -> None:
-
         levels1 = IndexLevel(Index(()), targets=None, depth_reference=3)
         tb = levels1.to_type_blocks()
         # NOTE: this will be updated to (0, 0) with IndexLevel support for zero size
         self.assertEqual(tb.shape, (0, 3))
 
+    def test_index_levels_to_type_blocks_b(self) -> None:
+        levels1 = IndexLevel(Index(()), targets=None, depth_reference=0)
+        tb = levels1.to_type_blocks()
+        self.assertEqual(tb.shape, (0, 0))
+
+    def test_index_levels_to_type_blocks_c(self) -> None:
+        levels1 = IndexLevel(Index(()), targets=None, depth_reference=3)
+        tb = levels1.to_type_blocks()
+        self.assertEqual(tb.shape, (0, 3))
+
     #---------------------------------------------------------------------------
 
     def test_index_level_depth_reference_a(self) -> None:
@@ -679,6 +731,18 @@ class TestUnit(TestCase):
         tb = lvl1.to_type_blocks()
         self.assertEqual(tb.shape, (0, 3))
 
+    #---------------------------------------------------------------------------
+
+    def test_index_level_depth_a(self) -> None:
+        levels1 = IndexLevel(Index(()), targets=None, depth_reference=3)
+        self.assertEqual(levels1.depth, 3)
+
+        levels1 = IndexLevel(Index(()), targets=None, depth_reference=1)
+        self.assertEqual(levels1.depth, 1)
+
+        levels1 = IndexLevel(Index((3, 4)), targets=None)
+        self.assertEqual(levels1.depth, 1)
+
     #---------------------------------------------------------------------------
     def test_index_level_from_depth_a(self) -> None:
 
@@ -686,6 +750,27 @@ class TestUnit(TestCase):
         self.assertEqual(lvl1.depth, 4)
         self.assertEqual(len(lvl1), 0)
 
+    #---------------------------------------------------------------------------
+    def test_index_level_from_tree_a(self) -> None:
+
+        OD = OrderedDict
+        tree = OD([
+                (0, OD([
+                        (8, (1, 2)), (9, (1, 2, 3)), (10, (2, 3))
+                        ])
+                ),
+                (1, OD([
+                        (3, (1,)), (5, (1, 2))
+                        ])
+                ),
+                ])
+        levels = IndexLevel.from_tree(tree, index_constructors=IndexSecond)
+        dt = datetime.datetime
+
+        self.assertEqual(levels.to_type_blocks().values.tolist(),
+                [[dt(1970, 1, 1, 0, 0), dt(1970, 1, 1, 0, 0, 8), dt(1970, 1, 1, 0, 0, 1)], [dt(1970, 1, 1, 0, 0), dt(1970, 1, 1, 0, 0, 8), dt(1970, 1, 1, 0, 0, 2)], [dt(1970, 1, 1, 0, 0), dt(1970, 1, 1, 0, 0, 9), dt(1970, 1, 1, 0, 0, 1)], [dt(1970, 1, 1, 0, 0), dt(1970, 1, 1, 0, 0, 9), dt(1970, 1, 1, 0, 0, 2)], [dt(1970, 1, 1, 0, 0), dt(1970, 1, 1, 0, 0, 9), dt(1970, 1, 1, 0, 0, 3)], [dt(1970, 1, 1, 0, 0), dt(1970, 1, 1, 0, 0, 10), dt(1970, 1, 1, 0, 0, 2)], [dt(1970, 1, 1, 0, 0), dt(1970, 1, 1, 0, 0, 10), dt(1970, 1, 1, 0, 0, 3)], [dt(1970, 1, 1, 0, 0, 1), dt(1970, 1, 1, 0, 0, 3), dt(1970, 1, 1, 0, 0, 1)], [dt(1970, 1, 1, 0, 0, 1), dt(1970, 1, 1, 0, 0, 5), dt(1970, 1, 1, 0, 0, 1)], [dt(1970, 1, 1, 0, 0, 1), dt(1970, 1, 1, 0, 0, 5), dt(1970, 1, 1, 0, 0, 2)]]
+        )
+
     #---------------------------------------------------------------------------
     def test_index_level_labels_at_depth_a(self) -> None:
 
@@ -713,7 +798,6 @@ class TestUnit(TestCase):
         self.assertEqual(tuple(levels.labels_at_depth(2)),
                 (1, 2, 1, 2, 3, 2, 3, 1, 1, 1, 2, 3, 1))
 
-
     #---------------------------------------------------------------------------
 
     def test_index_level_deepcopy_a(self) -> None:
@@ -736,7 +820,79 @@ class TestUnit(TestCase):
                 id(post.targets[0].index._labels), #type: ignore
                 id(post.targets[1].index._labels)) #type: ignore
 
+    #---------------------------------------------------------------------------
+
+    def test_index_level_traverse(self) -> None:
+        tree = {
+            "I": {"A": (1, 2), "B": (1, 2, 3), "C": (2, 3)},
+            "II": {"D": (1, 2, 3), "E": (1,)},
+        }
+        lvl = IndexLevel.from_tree(tree)
+
+        tree = lvl.traverse() # type: ignore
+
+        self.assertEqual(tuple(tree), ("I", "II"))
+        self.assertEqual(tuple(tree["I"]), ("A", "B", "C"))
+        self.assertEqual(tuple(tree["II"]), ("D", "E"))
+
+        self.assertTrue(tree["I"]["A"].equals(lvl.targets[0].targets[0].index)) # type: ignore
+        self.assertTrue(tree["I"]["B"].equals(lvl.targets[0].targets[1].index)) # type: ignore
+        self.assertTrue(tree["I"]["C"].equals(lvl.targets[0].targets[2].index)) # type: ignore
+
+        self.assertTrue(tree["II"]["D"].equals(lvl.targets[1].targets[0].index)) # type: ignore
+        self.assertTrue(tree["II"]["E"].equals(lvl.targets[1].targets[1].index)) # type: ignore
+
+    #---------------------------------------------------------------------------
+
+    def test_index_level_repr_a(self) -> None:
+        """Small tree"""
+        tree = {
+                "I": {"A": (1, 2), "B": (1, 2, 3), "C": (2, 3)},
+                "II": {"D": (1, 2, 3), "E": (1,)},
+        }
+        lvl = IndexLevel.from_tree(tree)
+        msg = repr(lvl)
+        self.assertEqual(len(msg.split()), 52)
+
+        num_ellipsis = re.findall(r"\.\.\.", msg)
+        num_labels = re.findall(r"IndexLevel", msg)
+        self.assertEqual(len(num_ellipsis), 0)
+        self.assertEqual(len(num_labels), 8)
+
+    def test_index_level_repr_b(self) -> None:
+        """Very deep tree"""
+
+        index = IndexHierarchy.from_product(*(range(i, i+2) for i in range(0, 20, 2)))
+
+        msg = repr(index._levels)
+        self.assertEqual(len(msg.split()), 67)
+
+        num_ellipsis = re.findall(r"\.\.\.", msg)
+        num_labels = re.findall(r"IndexLevel", msg)
+        self.assertEqual(len(num_ellipsis), 8)
+        self.assertEqual(len(num_labels), 15)
+
+    def test_index_level_repr_c(self) -> None:
+        """Very wide tree"""
+
+        index = IndexHierarchy.from_product(range(10), range(10, 20))
+
+        msg = repr(index._levels)
+        self.assertEqual(len(msg.split()), 70)
+
+        num_ellipsis = re.findall(r"\.\.\.", msg)
+        num_labels = re.findall(r"IndexLevel", msg)
+        self.assertEqual(len(num_ellipsis), 1)
+        self.assertEqual(len(num_labels), 5)
+
+    #---------------------------------------------------------------------------
+
+    def test_index_level_dtypes_at_depth_a(self) -> None:
+
+        levels1 = IndexLevel(Index(()), targets=None, depth_reference=3)
+        with self.assertRaises(RuntimeError):
+            _ = tuple(levels1.dtypes_at_depth(10))
+
 
 if __name__ == '__main__':
     unittest.main()
-
