@@ -1,5 +1,4 @@
-
-import unittest
+import typing as tp
 
 import frame_fixtures as ff
 import numpy as np
@@ -7,47 +6,26 @@ import numpy as np
 from static_frame.test.test_case import TestCase
 
 from static_frame.core.quilt import Quilt
-from static_frame.core.quilt import AxisMap
-from static_frame.core.index import Index
 from static_frame.core.hloc import HLoc
 from static_frame.core.display_config import DisplayConfig
 from static_frame.core.index import ILoc
+from static_frame.core.index import Index
 from static_frame.core.frame import Frame
 from static_frame.core.bus import Bus
+from static_frame.core.yarn import Yarn
 from static_frame.core.batch import Batch
 from static_frame.core.store import StoreConfig
 
 from static_frame.test.test_case import temp_file
 from static_frame.core.exception import ErrorInitQuilt
-from static_frame.core.exception import ErrorInitIndexNonUnique
 from static_frame.core.exception import AxisInvalid
+from static_frame.core.axis_map import bus_to_hierarchy
+
 
 class TestUnit(TestCase):
 
-    def test_axis_map_a(self) -> None:
-
-        components = dict(
-                x=Index(('a', 'b', 'c')),
-                y=Index(('a', 'b', 'c')),
-                )
-
-        am = AxisMap.get_axis_series(components) #type: ignore
-        self.assertEqual(am.to_pairs(),
-                ((('x', 'a'), 'x'), (('x', 'b'), 'x'), (('x', 'c'), 'x'), (('y', 'a'), 'y'), (('y', 'b'), 'y'), (('y', 'c'), 'y')))
-
-
-    def test_axis_map_b(self) -> None:
-
-        f1 = ff.parse('s(4,4)|v(int,float)').rename('f1')
-        f2 = ff.parse('s(4,4)|v(str)').rename('f2')
-        f3 = ff.parse('s(4,4)|v(bool)').rename('f3')
-
-        b1 = Bus.from_frames((f1, f2, f3))
-
-        with self.assertRaises(AxisInvalid):
-            AxisMap.from_bus(b1, deepcopy_from_bus=False, axis=3)
-
     #---------------------------------------------------------------------------
+
     def test_quilt_init_a(self) -> None:
 
         f1 = Frame.from_dict(
@@ -100,10 +78,9 @@ class TestUnit(TestCase):
 
         # must retain labels for non-unique axis
         q3 = Quilt(b1, retain_labels=False, axis=1)
-        with self.assertRaises(ErrorInitIndexNonUnique):
+        with self.assertRaises(ErrorInitQuilt):
             self.assertEqual(q3.shape, (7, 2))
 
-
     def test_quilt_init_c(self) -> None:
 
         f1 = ff.parse('s(4,4)|v(int,float)').rename('f1')
@@ -111,13 +88,101 @@ class TestUnit(TestCase):
         f3 = ff.parse('s(4,4)|v(bool)').rename('f3')
 
         b1 = Bus.from_frames((f1, f2, f3))
-        axis_map = AxisMap.from_bus(b1, axis=0, deepcopy_from_bus=True)
+        from static_frame.core.index_hierarchy import IndexHierarchy
+        y1 = Yarn((b1,), index=IndexHierarchy.from_labels(
+                ((1, 'a'), (1, 'b'), (2, 'a')),
+                ))
+        with self.assertRaises(ErrorInitQuilt):
+            Quilt(y1, retain_labels=False, axis=0).shape
 
         with self.assertRaises(ErrorInitQuilt):
-            _ = Quilt(b1, retain_labels=True, axis=0, axis_map=axis_map, axis_opposite=None)
+            Quilt(y1, retain_labels=False, axis=1).shape
 
+    def test_quilt_init_d(self) -> None:
+        f1 = ff.parse('s(4,2)').rename('f1')
+        f2 = ff.parse('s(4,2)').rename('f2')
+        f3 = ff.parse('s(2,2)').rename('f3')
+        f4 = ff.parse('s(2,2)').rename('f4')
+        f5 = ff.parse('s(4,2)').rename('f5')
+        f6 = ff.parse('s(6,2)').rename('f6')
+
+        b1 = Bus.from_frames((f1, f2, f3), name='b1')
+        b2 = Bus.from_frames((f4,), name='b2')
+        b3 = Bus.from_frames((f5, f6), name='b3')
+
+        y1 = Yarn.from_buses((b1, b2, b3), retain_labels=False)
+
+        q1 = Quilt(y1, retain_labels=True)
+        self.assertEqual(q1.shape, (22, 2))
+        self.assertEqual(q1.loc[[('f6', 3)]].to_pairs(),
+                ((0, ((('f6', 3), 1699.34),)), (1, ((('f6', 3), 114.58),)))
+                )
+
+    def test_quilt_init_e(self) -> None:
+        f1 = ff.parse('s(4,2)').rename('f1')
+        f2 = ff.parse('s(4,2)').rename('f2')
+        f3 = ff.parse('s(2,2)').rename('f3')
+        f4 = ff.parse('s(2,2)').rename('f4')
+        f5 = ff.parse('s(4,2)').rename('f5')
+        f6 = ff.parse('s(6,2)').rename('f6')
+
+        bm1 = Bus.from_frames((f1, f2, f3), name='b1')
+        bm2 = Bus.from_frames((f4,), name='b2')
+        bm3 = Bus.from_frames((f5, f6), name='b3')
+
+        with temp_file('.zip') as fp1, temp_file('.zip') as fp2, temp_file('.zip') as fp3:
+            bm1.to_zip_pickle(fp1)
+            bm2.to_zip_pickle(fp2)
+            bm3.to_zip_pickle(fp3)
+
+            b1 = Bus.from_zip_pickle(fp1, max_persist=1).rename('a')
+            b2 = Bus.from_zip_pickle(fp2, max_persist=1).rename('b')
+            b3 = Bus.from_zip_pickle(fp3, max_persist=1).rename('c')
+
+            y1 = Yarn.from_buses((b1, b2, b3), retain_labels=False)
+
+            q1 = Quilt(y1, retain_labels=True)
+            self.assertEqual(q1.shape, (22, 2))
+            self.assertEqual(q1.loc[[('f6', 3)]].to_pairs(),
+                    ((0, ((('f6', 3), 1699.34),)), (1, ((('f6', 3), 114.58),)))
+                    )
+
+    def test_quilt_init_f(self) -> None:
+        f1 = ff.parse('s(4,2)').rename('f1')
+        f2 = ff.parse('s(4,2)').rename('f2')
+        f3 = ff.parse('s(2,2)').rename('f3')
+        f4 = ff.parse('s(2,2)').rename('f4')
+        f5 = ff.parse('s(4,2)').rename('f5')
+        f6 = ff.parse('s(6,2)').rename('f6')
+
+        b1 = Bus.from_frames((f1, f2, f3), name='b1')
+        b2 = Bus.from_frames((f4,), name='b2')
+        b3 = Bus.from_frames((f5, f6), name='b3')
+
+        y1 = Yarn((b1, b2, b3), index=tuple('abcdef'))
+
+        q1 = Quilt(y1, retain_labels=True)
+        self.assertEqual(q1.index.values.tolist(),
+                [['a', 0], ['a', 1], ['a', 2], ['a', 3], ['b', 0], ['b', 1], ['b', 2], ['b', 3], ['c', 0], ['c', 1], ['d', 0], ['d', 1], ['e', 0], ['e', 1], ['e', 2], ['e', 3], ['f', 0], ['f', 1], ['f', 2], ['f', 3], ['f', 4], ['f', 5]]
+                )
+        self.assertEqual(q1.shape, (22, 2))
+        self.assertEqual(q1.loc[[('f', 3)]].to_pairs(),
+                ((0, ((('f', 3), 1699.34),)), (1, ((('f', 3), 114.58),)))
+                )
+
+    def test_quilt_init_g(self) -> None:
+
+        f1 = ff.parse('s(4,100)|v(int)|i(I,str)|c(I,str)')
+
+        q1 = Quilt.from_frame(f1, chunksize=10, axis=1, retain_labels=False)
+
+        axis_hierarchy, opp = bus_to_hierarchy(q1._bus, q1._axis, deepcopy_from_bus=False, init_exception_cls=ErrorInitQuilt)
+
+        with self.assertRaises(ErrorInitQuilt):
+            Quilt(q1._bus, axis=1, retain_labels=False, axis_hierarchy=axis_hierarchy)
 
     #---------------------------------------------------------------------------
+
     def test_quilt_from_items_a(self) -> None:
 
         f1 = Frame.from_dict(
@@ -140,8 +205,8 @@ class TestUnit(TestCase):
         self.assertEqual(q1.to_frame().to_pairs(),
                 ((('f1', 'a'), (('x', 1), ('y', 2))), (('f1', 'c'), (('x', 3), ('y', 4))), (('f2', 'a'), (('x', 2), ('y', 3))), (('f2', 'b'), (('x', 4), ('y', 6))), (('f3', 'c'), (('x', 10), ('y', 20))), (('f3', 'b'), (('x', 50), ('y', 60)))))
 
-
     #---------------------------------------------------------------------------
+
     def test_quilt_from_frames_a(self) -> None:
 
         f1 = Frame.from_dict(
@@ -164,18 +229,50 @@ class TestUnit(TestCase):
         self.assertEqual(q1.to_frame().to_pairs(),
                 ((('f1', 'a'), (('x', 1), ('y', 2))), (('f1', 'c'), (('x', 3), ('y', 4))), (('f2', 'a'), (('x', 2), ('y', 3))), (('f2', 'b'), (('x', 4), ('y', 6))), (('f3', 'c'), (('x', 10), ('y', 20))), (('f3', 'b'), (('x', 50), ('y', 60)))))
 
+    def test_quilt_from_frames_b(self) -> None:
+        index_a = Index(list(map(str, range(100))), name="a")
+        index_b = Index(list(map(str, range(100, 200))), name="b")
+        columns_a = Index(tuple("ABCDEFGHIJKLMNOPQRSTUVWXYZ"), name="a")
+        columns_b = Index(tuple("abcdefghijklmnopqrstuvwxyz"), name="b")
 
+        valid_construction_args = [
+            (index_a, columns_a, index_a, columns_a, True, 0),
+            (index_a, columns_a, index_a, columns_a, True, 1),
+            (index_a, columns_a, index_a, columns_b, True, 1),
+            (index_a, columns_a, index_a, columns_b, False, 1),
+            (index_a, columns_a, index_b, columns_a, True, 0),
+            (index_a, columns_a, index_b, columns_a, False, 0),
+            (index_a, columns_b, index_a, columns_a, True, 1),
+            (index_a, columns_b, index_a, columns_a, False, 1),
+            (index_a, columns_b, index_a, columns_b, True, 0),
+            (index_a, columns_b, index_a, columns_b, True, 1),
+            (index_a, columns_b, index_b, columns_b, True, 0),
+            (index_a, columns_b, index_b, columns_b, False, 0),
+            (index_b, columns_a, index_a, columns_a, True, 0),
+            (index_b, columns_a, index_a, columns_a, False, 0),
+            (index_b, columns_a, index_b, columns_a, True, 0),
+            (index_b, columns_a, index_b, columns_a, True, 1),
+            (index_b, columns_a, index_b, columns_b, True, 1),
+            (index_b, columns_a, index_b, columns_b, False, 1),
+            (index_b, columns_b, index_a, columns_b, True, 0),
+            (index_b, columns_b, index_a, columns_b, False, 0),
+            (index_b, columns_b, index_b, columns_a, True, 1),
+            (index_b, columns_b, index_b, columns_a, False, 1),
+            (index_b, columns_b, index_b, columns_b, True, 0),
+            (index_b, columns_b, index_b, columns_b, True, 1),
+        ]
+
+        def build_quilt(args: tp.Any) -> Quilt:
+            f1_index, f1_columns, f2_index, f2_columns, retain_labels, axis = args
+            f1 = Frame.from_element("f1", index=f1_index, columns=f1_columns, name="f1")
+            f2 = Frame.from_element("f2", index=f2_index, columns=f2_columns, name="f2")
+            return Quilt.from_frames((f1, f2), retain_labels=retain_labels, axis=axis)
+
+        # This checks that the init was successful, and that the display call also is successful
+        for quilt in map(build_quilt, valid_construction_args):
+            quilt.display()
 
     #---------------------------------------------------------------------------
-    def test_quilt_display_a(self) -> None:
-
-        dc = DisplayConfig(type_show=False)
-
-        f1 = ff.parse('s(10,4)|v(int)|i(I,str)|c(I,str)').rename('foo')
-        q1 = Quilt.from_frame(f1, chunksize=2, retain_labels=False)
-        self.assertEqual(
-                q1.display(dc).to_rows(),
-                f1.display(dc).to_rows())
 
     def test_quilt_values_a(self) -> None:
         f1 = ff.parse('s(6,4)|v(int)|i(I,str)|c(I,str)')
@@ -183,7 +280,6 @@ class TestUnit(TestCase):
         self.assertEqual(q1.values.tolist(),
                 [[-88017, 162197, -3648, 129017], [92867, -41157, 91301, 35021], [84967, 5729, 30205, 166924], [13448, -168387, 54020, 122246], [175579, 140627, 129017, 197228], [58768, 66269, 35021, 105269]])
 
-
     def test_quilt_nbytes_a(self) -> None:
 
         dc = DisplayConfig(type_show=False)
@@ -193,6 +289,7 @@ class TestUnit(TestCase):
         self.assertEqual(q1.nbytes, f1.nbytes)
 
     #---------------------------------------------------------------------------
+
     def test_quilt_from_frame_a(self) -> None:
 
         f1 = ff.parse('s(100,4)|v(int)|i(I,str)|c(I,str)').rename('foo')
@@ -201,9 +298,9 @@ class TestUnit(TestCase):
 
         self.assertEqual(q1.name, 'foo')
         self.assertEqual(q1.rename('bar').name, 'bar')
-        self.assertTrue(repr(q1).startswith('<Quilt: foo'))
+        self.assertTrue(repr(q1.display(DisplayConfig(type_color=False))).startswith('<Quilt: foo'))
 
-        post, opp = AxisMap.from_bus(q1._bus, q1._axis, deepcopy_from_bus=True)
+        post, opp = bus_to_hierarchy(q1._bus, q1._axis, deepcopy_from_bus=True, init_exception_cls=ErrorInitQuilt)
         self.assertEqual(len(post), 100)
         self.assertEqual(len(opp), 4)
 
@@ -222,14 +319,13 @@ class TestUnit(TestCase):
         f3 = q1.loc['zQuq':, 'zUvW':] #type: ignore
         self.assertEqual(f3.shape, (6, 2))
 
-
     def test_quilt_from_frame_b(self) -> None:
 
         f1 = ff.parse('s(4,100)|v(int)|i(I,str)|c(I,str)')
 
         q1 = Quilt.from_frame(f1, chunksize=10, axis=1, retain_labels=False)
 
-        post, opp = AxisMap.from_bus(q1._bus, q1._axis, deepcopy_from_bus=False)
+        post, opp = bus_to_hierarchy(q1._bus, q1._axis, deepcopy_from_bus=False, init_exception_cls=ErrorInitQuilt)
         self.assertEqual(len(post), 100)
         self.assertEqual(len(opp), 4)
 
@@ -246,7 +342,6 @@ class TestUnit(TestCase):
         with self.assertRaises(AxisInvalid):
             q1 = Quilt.from_frame(f1, chunksize=10, axis=2, retain_labels=False)
 
-
     #---------------------------------------------------------------------------
 
     def test_quilt_extract_a1(self) -> None:
@@ -258,7 +353,6 @@ class TestUnit(TestCase):
         self.assertEqual(q1['zkuW':'zTSt'].shape, (4, 95)) #type: ignore
         self.assertEqual(q1.loc[ILoc[-2:], 'zaji': 'zsa5'].shape, (2, 17)) #type: ignore
 
-
     def test_quilt_extract_a2(self) -> None:
 
         f1 = ff.parse('s(4,100)|v(int)|i(I,str)|c(I,str)')
@@ -268,8 +362,6 @@ class TestUnit(TestCase):
         self.assertEqual(q1['zkuW':'zTSt'].shape, (4, 95)) #type: ignore
         self.assertEqual(q1.loc[ILoc[-2:], 'zaji': 'zsa5'].shape, (2, 17)) #type: ignore
 
-
-
     def test_quilt_extract_b1(self) -> None:
 
         f1 = ff.parse('s(4,10)|v(int)|i(I,str)|c(I,str)')
@@ -320,8 +412,6 @@ class TestUnit(TestCase):
                 (('zZbu', 146284), ('ztsv', 170440), ('zUvW', 32395), ('zkuW', 137759))
                 )
 
-
-
     def test_quilt_extract_c1(self) -> None:
 
         f1 = ff.parse('s(20,4)|v(int)|i(I,str)|c(I,str)')
@@ -338,7 +428,6 @@ class TestUnit(TestCase):
         # return an element
         self.assertEqual(q1.loc[('zOyq', 'zIA5'), 'zkuW'], 92867)
 
-
     def test_quilt_extract_c2(self) -> None:
 
         f1 = ff.parse('s(20,4)|v(int)|i(I,str)|c(I,str)')
@@ -355,7 +444,6 @@ class TestUnit(TestCase):
         # return an element
         self.assertEqual(q1.loc[('zOyq', 'zIA5'), 'zkuW'], 92867)
 
-
     def test_quilt_extract_d1(self) -> None:
 
         f1 = ff.parse('s(4,20)|v(int)|i(I,str)|c(I,str)')
@@ -406,7 +494,6 @@ class TestUnit(TestCase):
         self.assertEqual(s1.name, 'zkuW')
         self.assertEqual(s1.shape, (4,))
 
-
     def test_quilt_extract_f1(self) -> None:
 
         f1 = ff.parse('s(4,20)|v(int)|i(I,str)|c(I,str)')
@@ -417,7 +504,6 @@ class TestUnit(TestCase):
 
         self.assertEqual(q1.loc['zkuW', 'zwIp'], -112188)
 
-
     def test_quilt_extract_f2(self) -> None:
 
         f1 = ff.parse('s(4,20)|v(int)|i(I,str)|c(I,str)')
@@ -460,9 +546,8 @@ class TestUnit(TestCase):
             self.assertEqual(post.shape, (10, 1))
             self.assertEqual(set(post.index.values_at_depth(0)), {'c'})
 
-
-
     #---------------------------------------------------------------------------
+
     def test_quilt_extract_array_a1(self) -> None:
 
         f1 = ff.parse('s(20,4)|v(int)|i(I,str)|c(I,str)')
@@ -537,7 +622,6 @@ class TestUnit(TestCase):
         self.assertEqual(q1._extract_array(2, 2), 30205)
         self.assertEqual(q1._extract_array(-1, -1), -112188)
 
-
     def test_quilt_extract_array_b2(self) -> None:
 
         f1 = ff.parse('s(4,20)|v(int)|i(I,str)|c(I,str)')
@@ -565,16 +649,15 @@ class TestUnit(TestCase):
         f1 = ff.parse('s(4,4)|v(int)|i(I,str)|c(I,str)')
 
         q1 = Quilt.from_frame(f1, chunksize=4, axis=1, retain_labels=False, deepcopy_from_bus=False)
-        a1_id_in_bus = id(q1._bus._series.values[0].values)
+        a1_id_in_bus = id(q1._bus._values_mutable[0].values) #type: ignore
         a1_id_via_quilt = id(q1._extract_array())
         self.assertEqual(a1_id_in_bus, a1_id_via_quilt)
 
         q2 = Quilt.from_frame(f1, chunksize=4, axis=1, retain_labels=False, deepcopy_from_bus=True)
-        a2_id_in_bus = id(q2._bus._series.values[0].values)
+        a2_id_in_bus = id(q2._bus._values_mutable[0].values) #type: ignore
         a2_id_via_quilt = id(q2._extract_array())
         self.assertNotEqual(a2_id_in_bus, a2_id_via_quilt)
 
-
     def test_quilt_extract_array_c(self) -> None:
 
         f1 = ff.parse('s(4,4)|v(int,float)').rename('f1')
@@ -586,8 +669,8 @@ class TestUnit(TestCase):
         self.assertTrue(a1.tolist(),
                 [[-88017.0, -610.8, -3648.0, 1080.4, False, False, True, False], [92867.0, 3243.94, 91301.0, 2580.34, False, False, False, False], [84967.0, -823.14, 30205.0, 700.42, False, False, False, True], [13448.0, 114.58, 54020.0, 3338.48, True, False, True, True]])
 
-
     #---------------------------------------------------------------------------
+
     def test_quilt_retain_labels_a(self) -> None:
 
         dc = DisplayConfig(type_show=False)
@@ -610,8 +693,8 @@ class TestUnit(TestCase):
                 (('zZbu', ((('zUvW', 'zUvW'), 84967), (('zUvW', 'zkuW'), 13448), (('zmVj', 'zmVj'), 175579), (('zmVj', 'z2Oo'), 58768), (('z5l6', 'z5l6'), 146284), (('z5l6', 'zCE3'), 170440))), ('ztsv', ((('zUvW', 'zUvW'), 5729), (('zUvW', 'zkuW'), -168387), (('zmVj', 'zmVj'), 140627), (('zmVj', 'z2Oo'), 66269), (('z5l6', 'z5l6'), -171231), (('z5l6', 'zCE3'), -38997))), ('zUvW', ((('zUvW', 'zUvW'), 30205), (('zUvW', 'zkuW'), 54020), (('zmVj', 'zmVj'), 129017), (('zmVj', 'z2Oo'), 35021), (('z5l6', 'z5l6'), 166924), (('z5l6', 'zCE3'), 122246))), ('zkuW', ((('zUvW', 'zUvW'), 166924), (('zUvW', 'zkuW'), 122246), (('zmVj', 'zmVj'), 197228), (('zmVj', 'z2Oo'), 105269), (('z5l6', 'z5l6'), 119909), (('z5l6', 'zCE3'), 194224))))
                 )
 
-
     #---------------------------------------------------------------------------
+
     def test_quilt_items_store_a(self) -> None:
 
         f1 = ff.parse('s(10,4)|v(int)|i(I,str)|c(I,str)')
@@ -620,6 +703,7 @@ class TestUnit(TestCase):
         self.assertEqual(len(tuple(q1._items_store())), 5)
 
     #---------------------------------------------------------------------------
+
     def test_quilt_keys_a(self) -> None:
 
         f1 = ff.parse('s(4,4)|v(int)|i(I,str)|c(I,str)')
@@ -645,8 +729,8 @@ class TestUnit(TestCase):
         self.assertEqual(q1.get('zZbu').shape, (4,)) #type: ignore
         self.assertEqual(q1.get(''), None)
 
-
     #---------------------------------------------------------------------------
+
     def test_quilt_from_zip_pickle_a(self) -> None:
 
         # indexes are heterogenous but columns are not
@@ -690,8 +774,8 @@ class TestUnit(TestCase):
             q2 = Quilt.from_zip_pickle(fp, config=sc, retain_labels=True)
             self.assertTrue(q2.to_frame().equals(q1.to_frame()))
 
-
     #---------------------------------------------------------------------------
+
     def test_quilt_from_zip_tsv_a(self) -> None:
 
         f1 = ff.parse('s(4,4)|v(int,float)').rename('f1')
@@ -709,6 +793,7 @@ class TestUnit(TestCase):
             self.assertTrue(q2.to_frame().equals(q1.to_frame()))
 
     #---------------------------------------------------------------------------
+
     def test_quilt_from_zip_csv_a(self) -> None:
 
         f1 = ff.parse('s(4,4)|v(int,float)').rename('f1')
@@ -726,6 +811,7 @@ class TestUnit(TestCase):
             self.assertTrue(q2.to_frame().equals(q1.to_frame()))
 
     #---------------------------------------------------------------------------
+
     def test_quilt_from_zip_parquet_a(self) -> None:
 
         f1 = ff.parse('s(4,4)|v(int,float)').rename('f1')
@@ -744,6 +830,7 @@ class TestUnit(TestCase):
             self.assertTrue((q2.to_frame().values == q1.to_frame().values).all())
 
     #---------------------------------------------------------------------------
+
     def test_quilt_from_xlsx_a(self) -> None:
 
         f1 = ff.parse('s(4,4)|v(int,float)').rename('f1')
@@ -761,6 +848,7 @@ class TestUnit(TestCase):
             self.assertTrue(q2.to_frame().equals(q1.to_frame()))
 
     #---------------------------------------------------------------------------
+
     def test_quilt_from_sqlite_a(self) -> None:
 
         f1 = ff.parse('s(4,4)|v(int,float)').rename('f1')
@@ -777,6 +865,7 @@ class TestUnit(TestCase):
             self.assertTrue((q2.to_frame().values == q1.to_frame().values).all())
 
     #---------------------------------------------------------------------------
+
     def test_quilt_from_hdf5_a(self) -> None:
 
         f1 = ff.parse('s(4,4)|v(int,float)|c(I,str)').rename('f1')
@@ -792,8 +881,6 @@ class TestUnit(TestCase):
             q2 = Quilt.from_hdf5(fp, config=sc, retain_labels=True)
             self.assertTrue((q2.to_frame().values == q1.to_frame().values).all())
 
-
-
     #---------------------------------------------------------------------------
 
     def test_quilt_iter_array_a1(self) -> None:
@@ -834,7 +921,6 @@ class TestUnit(TestCase):
         with self.assertRaises(NotImplementedError):
             _ = tuple(q1.iter_array_items(axis=0))
 
-
     def test_quilt_iter_array_a2(self) -> None:
 
         # indexes are hetergenous but columns are not
@@ -873,8 +959,6 @@ class TestUnit(TestCase):
         with self.assertRaises(NotImplementedError):
             _ = tuple(q1.iter_array_items(axis=0))
 
-
-
     def test_quilt_iter_array_b1(self) -> None:
 
         f1 = ff.parse('s(2,6)|v(int)|i(I,str)|c(I,str)')
@@ -895,8 +979,6 @@ class TestUnit(TestCase):
         with self.assertRaises(NotImplementedError):
             _ = tuple(q1.iter_array_items(axis=1))
 
-
-
     def test_quilt_iter_array_b2(self) -> None:
 
         f1 = ff.parse('s(2,6)|v(int)|i(I,str)|c(I,str)')
@@ -917,18 +999,17 @@ class TestUnit(TestCase):
         with self.assertRaises(NotImplementedError):
             _ = tuple(q1.iter_array_items(axis=1))
 
-
     def test_quilt_iter_array_b3(self) -> None:
 
         f1 = ff.parse('s(2,6)|v(int,str)|i(I,str)|c(I,str)')
 
         q2 = Quilt.from_frame(f1, chunksize=3, axis=1, retain_labels=False, deepcopy_from_bus=False)
-        a2_src_id = id(q2._bus._series.values[0]._extract_array(None, 0))
+        a2_src_id = id(q2._bus._values_mutable[0]._extract_array(None, 0)) #type: ignore
         a2_dst_id = id(next(iter(q2.iter_array(axis=0))))
         self.assertTrue(a2_src_id == a2_dst_id)
 
         q1 = Quilt.from_frame(f1, chunksize=3, axis=1, retain_labels=False, deepcopy_from_bus=True)
-        a1_src_id = id(q1._bus._series.values[0]._extract_array(None, 0))
+        a1_src_id = id(q1._bus._values_mutable[0]._extract_array(None, 0)) #type: ignore
         a1_dst_id = id(next(iter(q1.iter_array(axis=0))))
         self.assertTrue(a1_src_id != a1_dst_id)
 
@@ -1004,8 +1085,8 @@ class TestUnit(TestCase):
         with self.assertRaises(NotImplementedError):
             _ = tuple(q1.items())
 
-
     #---------------------------------------------------------------------------
+
     def test_quilt_iter_tuple_a1(self) -> None:
 
         f1 = ff.parse('s(2,6)|v(int)|i(I,str)|c(I,str)')
@@ -1028,7 +1109,6 @@ class TestUnit(TestCase):
         with self.assertRaises(NotImplementedError):
             _ = tuple(q1.iter_tuple(axis=1))
 
-
     def test_quilt_iter_tuple_b1(self) -> None:
 
         f1 = ff.parse('s(2,6)|v(int)|i(I,str)|c(I,str)')
@@ -1066,6 +1146,7 @@ class TestUnit(TestCase):
         self.assertEqual(post[0].__class__, NT)
 
     #---------------------------------------------------------------------------
+
     def test_quilt_to_frame_a1(self) -> None:
 
         f1 = ff.parse('s(4,4)|v(int)|i(I,str)|c(I,str)')
@@ -1087,6 +1168,7 @@ class TestUnit(TestCase):
         self.assertTrue(q2.to_frame().equals(f1))
 
     #---------------------------------------------------------------------------
+
     def test_quilt_iter_window_a1(self) -> None:
 
         f1 = ff.parse('s(20,2)|v(int)|i(I,str)|c(I,str)')
@@ -1153,8 +1235,6 @@ class TestUnit(TestCase):
             self.assertEqual(f1.shape, (75, 3))
             self.assertEqual(q1.status['loaded'].sum(), 1)
 
-
-
     def test_quilt_iter_window_b2(self) -> None:
         from string import ascii_lowercase
         # indexes are heterogenous but columns are not
@@ -1209,6 +1289,7 @@ class TestUnit(TestCase):
 )
 
     #---------------------------------------------------------------------------
+
     def test_quilt_iter_window_array_b1(self) -> None:
 
         f1 = ff.parse('s(20,2)|v(int)|i(I,str)|c(I,str)')
@@ -1232,6 +1313,7 @@ class TestUnit(TestCase):
                 (('zmVj', 377853), ('zr4u', 597832), ('zGDJ', 46996), ('zO5l', 391169)))
 
     #---------------------------------------------------------------------------
+
     def test_quilt_iter_window_array_items_a(self) -> None:
 
         f1 = ff.parse('s(4,4)|v(int,float)').rename('f1')
@@ -1239,12 +1321,20 @@ class TestUnit(TestCase):
         b1 = Bus.from_frames((f1, f2))
         q1 = Quilt(b1, retain_labels=True, axis=1)
 
+        # axis 0 will produce windows labelled by the Index
         s1 = q1.iter_window_array_items(size=2, step=1).apply(lambda _, a: a.sum())
         self.assertEqual(round(s1, 2).to_pairs(),
                 ((1, 98797.88), (2, 305042.56), (3, 185974.34)))
 
+        s2 = q1.iter_window_array_items(size=2, step=1, axis=1).apply(lambda _, a: a.sum())
+        self.assertEqual(round(s2, 2).to_pairs(),
+                ((('f1', 1), 105189.58), (('f1', 2), 173802.58), (('f1', 3), 179577.64), (('f2', 0), 7700.64), (('f2', 1), 1.0), (('f2', 2), 2.0), (('f2', 3), 4.0))
+                )
+
     #---------------------------------------------------------------------------
+
     def test_quilt_repr_a(self) -> None:
+        config = DisplayConfig.from_default(cell_align_left=True, type_color=False)
 
         f1 = ff.parse('s(4,4)|v(int,float)').rename('f1')
         f2 = ff.parse('s(4,4)|v(bool)').rename('f2')
@@ -1255,10 +1345,19 @@ class TestUnit(TestCase):
         q1 = Quilt(b1, retain_labels=True)
         q2 = Quilt(b2, retain_labels=True)
 
-        self.assertTrue(repr(q1).startswith('<Quilt: foo'))
-        self.assertTrue(repr(q2).startswith('<Quilt at'))
+        common_rows = [
+            '<Index: Aligned> 0 1 2 3 <int64>',
+            '<Index: Frames>',
+            'f1               . . . .',
+            'f2               . . . .',
+            '<<U2>']
+
+
+        self.assertEqual(q1.display(config=config).to_rows(), ['<Quilt: foo>', * common_rows])
+        self.assertEqual(q2.display(config=config).to_rows(), ['<Quilt>', * common_rows])
 
     #---------------------------------------------------------------------------
+
     def test_quilt_columns_a(self) -> None:
 
         f1 = ff.parse('s(4,4)|v(int,float)').rename('f1')
@@ -1269,6 +1368,7 @@ class TestUnit(TestCase):
                 [['f1', 0], ['f1', 1], ['f1', 2], ['f1', 3], ['f2', 0], ['f2', 1], ['f2', 2], ['f2', 3]])
 
     #---------------------------------------------------------------------------
+
     def test_quilt_size_a(self) -> None:
 
         f1 = ff.parse('s(4,4)|v(int,float)').rename('f1')
@@ -1278,6 +1378,7 @@ class TestUnit(TestCase):
         self.assertEqual(q1.size, 32)
 
     #---------------------------------------------------------------------------
+
     def test_quilt_items_a(self) -> None:
 
         f1 = ff.parse('s(4,4)|v(int,float)').rename('f1')
@@ -1289,8 +1390,8 @@ class TestUnit(TestCase):
         self.assertEqual(tuple(items.keys()),
                 (('f1', 0), ('f1', 1), ('f1', 2), ('f1', 3), ('f2', 0), ('f2', 1), ('f2', 2), ('f2', 3)))
 
-
     #---------------------------------------------------------------------------
+
     def test_quilt_axis_array_a(self) -> None:
 
         f1 = ff.parse('s(4,4)|v(int,float)').rename('f1')
@@ -1302,6 +1403,7 @@ class TestUnit(TestCase):
             _ = tuple(q1._axis_array(3))
 
     #---------------------------------------------------------------------------
+
     def test_quilt_axis_tuple_a(self) -> None:
 
         f1 = ff.parse('s(4,4)|v(int,float)').rename('f1')
@@ -1331,6 +1433,7 @@ class TestUnit(TestCase):
                 (False, False, True, True)))
 
     #---------------------------------------------------------------------------
+
     def test_quilt_iter_series_items_a(self) -> None:
 
         f1 = ff.parse('s(4,4)|v(int,float)').rename('f1')
@@ -1344,6 +1447,7 @@ class TestUnit(TestCase):
                 )
 
     #---------------------------------------------------------------------------
+
     def test_quilt_head_a(self) -> None:
 
         f1 = ff.parse('s(4,4)|v(int,float)').rename('f1')
@@ -1353,8 +1457,8 @@ class TestUnit(TestCase):
         self.assertEqual(q1.head(2).to_pairs(),
                 ((('f1', 0), ((0, -88017), (1, 92867))), (('f1', 1), ((0, -610.8), (1, 3243.94))), (('f1', 2), ((0, -3648), (1, 91301))), (('f1', 3), ((0, 1080.4), (1, 2580.34))), (('f2', 0), ((0, False), (1, False))), (('f2', 1), ((0, False), (1, False))), (('f2', 2), ((0, True), (1, False))), (('f2', 3), ((0, False), (1, False)))))
 
-
     #---------------------------------------------------------------------------
+
     def test_quilt_tail_a(self) -> None:
 
         f1 = ff.parse('s(4,4)|v(int,float)').rename('f1')
@@ -1365,9 +1469,152 @@ class TestUnit(TestCase):
                 ((('f1', 0), ((2, 84967), (3, 13448))), (('f1', 1), ((2, -823.14), (3, 114.58))), (('f1', 2), ((2, 30205), (3, 54020))), (('f1', 3), ((2, 700.42), (3, 3338.48))), (('f2', 0), ((2, False), (3, True))), (('f2', 1), ((2, False), (3, False))), (('f2', 2), ((2, False), (3, True))), (('f2', 3), ((2, True), (3, True))))
                 )
 
+    #---------------------------------------------------------------------------
+
+    def test_quilt_unpersist_a(self) -> None:
+
+        f1 = ff.parse('s(4,4)|v(int,float)').rename('f1')
+        f2 = ff.parse('s(4,4)|v(bool)').rename('f2')
+        b1 = Bus.from_frames((f1, f2))
+        q1 = Quilt(b1, retain_labels=True, axis=1)
+        items1 = dict(q1.items())
+        self.assertTrue(len(items1) == 8)
+
+        self.assertEqual(b1.status['loaded'].sum(), 2)
+        q1.unpersist()
+
+        # there is no Store with this Bus so unpersits makes not change
+        self.assertEqual(b1.status['loaded'].sum(), 2)
+
+    def test_quilt_unpersist_b(self) -> None:
+
+        f1 = ff.parse('s(4,4)|v(int,float)|c(I,str)').rename('f1')
+        f2 = ff.parse('s(4,4)|v(str)|c(I,str)').rename('f2')
+        f3 = ff.parse('s(4,4)|v(bool)|c(I,str)').rename('f3')
+
+        q1 = Quilt.from_frames((f1, f2, f3), retain_labels=True, axis=1)
+
+        with temp_file('.zip') as fp:
+            q1.to_zip_pickle(fp)
+            q2 = Quilt.from_zip_pickle(fp, retain_labels=True, axis=1)
+            self.assertEqual(len(dict(q2.items())), 12)
+
+            self.assertEqual(q2._bus.status['loaded'].sum(), 3)
+            q2.unpersist()
+
+            self.assertEqual(q2._bus.status['loaded'].sum(), 0)
+            self.assertEqual(len(dict(q2.items())), 12)
+            self.assertEqual(q2._bus.status['loaded'].sum(), 3)
+
+    #---------------------------------------------------------------------------
+
+    def test_quilt_sample_a(self) -> None:
+
+        f1 = ff.parse('s(20,20)|v(int)|c(I,str)|i(I,str)').rename('f1')
+        f2 = ff.parse('s(20,20)|v(bool)|c(I,str)|i(I,str)').rename('f2')
+        b1 = Bus.from_frames((f1, f2))
+        q1 = Quilt(b1, retain_labels=True, axis=1)
+
+        self.assertEqual(q1.sample(4, 4, seed=0).to_pairs(),
+                ((('f1', 'zmVj'), (('ztsv', 146284), ('zr4u', 126025), ('zB7E', 164351), ('zwIp', 195850))), (('f2', 'zZbu'), (('ztsv', False), ('zr4u', False), ('zB7E', False), ('zwIp', False))), (('f2', 'zUvW'), (('ztsv', False), ('zr4u', True), ('zB7E', False), ('zwIp', True))), (('f2', 'z2Oo'), (('ztsv', True), ('zr4u', True), ('zB7E', True), ('zwIp', False))))
+                )
+
+        self.assertEqual(q1.sample(index=4).shape, (4, 40))
+        self.assertEqual(q1.sample(columns=4).shape, (20, 4))
+
+    #---------------------------------------------------------------------------
+
+    def test_quilt_to_zip_npz_a(self) -> None:
+
+        f1 = ff.parse('s(4,4)|v(int,float)|c(I,str)').rename('f1')
+        f2 = ff.parse('s(4,4)|v(str)|c(I,str)').rename('f2')
+        f3 = ff.parse('s(4,4)|v(bool)|c(I,str)').rename('f3')
+        q1 = Quilt.from_frames((f1, f2, f3), retain_labels=True, axis=1)
+
+        with temp_file('.zip') as fp:
+            q1.to_zip_npz(fp)
+            q2 = Quilt.from_zip_npz(fp, retain_labels=True, axis=1)
+
+            self.assertTrue(q1.equals(q2, compare_class=True, compare_dtype=True, compare_name=True))
+
+    #---------------------------------------------------------------------------
+
+    def test_quilt_equals_a(self) -> None:
+
+        f1 = ff.parse('s(4,4)|v(int,float)|c(I,str)').rename('f1')
+        f2 = ff.parse('s(4,4)|v(str)|c(I,str)').rename('f2')
+        q1 = Quilt.from_frames((f1, f2), retain_labels=True, axis=1)
+
+        self.assertTrue(q1.equals(q1, compare_class=True, compare_dtype=True, compare_name=True))
+
+        self.assertFalse(q1.equals(None, compare_class=True))
+        self.assertFalse(q1.equals(None))
+
+    def test_quilt_equals_b(self) -> None:
+
+        f1 = ff.parse('s(4,4)|v(int,float)|c(I,str)').rename('f1')
+        f2 = ff.parse('s(4,4)|v(str)|c(I,str)').rename('f2')
+        q1 = Quilt.from_frames((f1, f2), retain_labels=True, axis=1, name='foo')
+        q2 = Quilt.from_frames((f1, f2), retain_labels=True, axis=1, name='bar')
+
+        self.assertFalse(q1.equals(q2, compare_class=True, compare_dtype=True, compare_name=True))
+
+    def test_quilt_equals_c(self) -> None:
+
+        f1 = ff.parse('s(4,4)|v(int,float)|c(I,str)').rename('f1')
+        f2 = ff.parse('s(4,4)|v(str)|c(I,str)').rename('f2')
+        q1 = Quilt.from_frames((f1, f2), retain_labels=True, axis=1)
+        q2 = Quilt.from_frames((f1, f2), retain_labels=True, axis=0)
+
+        self.assertFalse(q1.equals(q2, compare_class=True, compare_dtype=True, compare_name=True))
+
+    def test_quilt_equals_d(self) -> None:
+
+        f1 = ff.parse('s(3,4)|v(int,float)|c(I,str)').rename('f1').reindex((1, 2, 3))
+        f2 = ff.parse('s(3,4)|v(str)|c(I,str)').rename('f2').reindex((4, 5, 6))
+        q1 = Quilt.from_frames((f1, f2), retain_labels=True)
+        q2 = Quilt.from_frames((f1, f2), retain_labels=False)
+
+        self.assertFalse(q1.equals(q2, compare_class=True, compare_dtype=True, compare_name=True))
+
+    def test_quilt_equals_e(self) -> None:
+
+        f1 = ff.parse('s(3,4)|v(int,float)|c(I,str)').rename('f1').reindex((1, 2, 3))
+        f2 = ff.parse('s(3,4)|v(str)|c(I,str)').rename('f2').reindex((4, 5, 6))
+        q1 = Quilt.from_frames((f1, f2), retain_labels=True)
+
+        f3 = ff.parse('s(3,4)|v(int,float)|c(I,str)').rename('f1').reindex((1, 2, 5))
+        f4 = ff.parse('s(3,4)|v(str)|c(I,str)').rename('f2').reindex((4, 8, 6))
+        q2 = Quilt.from_frames((f3, f4), retain_labels=True)
+
+        self.assertFalse(q1.equals(q2, compare_class=True, compare_dtype=True, compare_name=True))
+
+    def test_quilt_equals_f(self) -> None:
+
+        f1 = ff.parse('s(3,4)|v(int,float)|c(I,str)').rename('f1')
+        f2 = ff.parse('s(3,4)|v(str)|c(I,str)').rename('f2')
+        q1 = Quilt.from_frames((f1, f2), retain_labels=True)
+
+        f3 = ff.parse('s(3,4)|v(int,float)|c(I,str)').rename('f1').relabel(
+                columns=lambda x: x.upper())
+        f4 = ff.parse('s(3,4)|v(str)|c(I,str)').rename('f2').relabel(
+                columns=lambda x: x.upper())
+        q2 = Quilt.from_frames((f3, f4), retain_labels=True)
+
+        self.assertFalse(q1.equals(q2, compare_class=True, compare_dtype=True, compare_name=True))
+
+    def test_quilt_equals_g(self) -> None:
+
+        f1 = ff.parse('s(3,4)|v(int,float)|c(I,str)').rename('f1')
+        f2 = ff.parse('s(3,4)|v(str)|c(I,str)').rename('f2')
+        q1 = Quilt.from_frames((f1, f2), retain_labels=True)
+
+        f3 = ff.parse('s(3,4)|v(str,bool)|c(I,str)').rename('f2')
+        q2 = Quilt.from_frames((f1, f3), retain_labels=True)
+
+        self.assertFalse(q1.equals(q2, compare_class=True, compare_dtype=True, compare_name=True))
 
 
 if __name__ == '__main__':
+    import unittest
     unittest.main()
-
-
