@@ -1,6 +1,4 @@
-import unittest
 from itertools import product
-# from io import StringIO
 import numpy as np
 
 from static_frame.core.store import Store
@@ -10,14 +8,12 @@ from static_frame.core.store import StoreConfigMap
 from static_frame.core.frame import Frame
 
 from static_frame.test.test_case import TestCase
-# from static_frame.test.test_case import temp_file
 from static_frame.core.exception import ErrorInitStoreConfig
-
+from static_frame.core.exception import StoreParameterConflict
 
 
 class TestUnit(TestCase):
 
-
     #---------------------------------------------------------------------------
 
     def test_store_init_a(self) -> None:
@@ -79,7 +75,6 @@ class TestUnit(TestCase):
         sc1m = StoreConfigMap(maps2, default=default)
         self.assertEqual(sc1m.default.read_chunksize, 2)
 
-
     def test_store_config_map_init_e(self) -> None:
         maps1 = {'a': StoreConfig(write_max_workers=2),
                 'b': StoreConfig(write_max_workers=3)}
@@ -161,30 +156,32 @@ class TestUnit(TestCase):
             )
             self.assertNotEqual(config_he, config)
             self.assertEqual(config_he, config.to_store_config_he())
+            self.assertTrue(isinstance(hash(config_he), int))
 
     def test_store_config_he_b(self) -> None:
-        def compare_configs(config1: StoreConfigHE, config2: StoreConfigHE) -> None:
-            self.assertNotEqual(config1, config2)
 
         config1 = StoreConfigHE(index_depth=1)
         config2 = StoreConfigHE(index_depth=2)
-        compare_configs(config1, config2)
+        self.assertNotEqual(config1, config2)
+
+        self.assertTrue(config1 is not None)
+        self.assertFalse(config1 == None) #pylint: disable=C0121
 
         config1 = StoreConfigHE(columns_select=['a'])
         config2 = StoreConfigHE(columns_select=('b',))
-        compare_configs(config1, config2)
+        self.assertNotEqual(config1, config2)
 
         config1 = StoreConfigHE(dtypes=str)
         config2 = StoreConfigHE(dtypes=int)
-        compare_configs(config1, config2)
+        self.assertNotEqual(config1, config2)
 
         config1 = StoreConfigHE(dtypes=str)
         config2 = StoreConfigHE(dtypes=dict(a=int))
-        compare_configs(config1, config2)
+        self.assertNotEqual(config1, config2)
 
         config1 = StoreConfigHE(index_name_depth_level=None)
         config2 = StoreConfigHE(index_name_depth_level=(1, 2))
-        compare_configs(config1, config2)
+        self.assertNotEqual(config1, config2)
 
     def test_store_config_not_hashable(self) -> None:
         with self.assertRaises(NotImplementedError):
@@ -203,7 +200,6 @@ class TestUnit(TestCase):
         self.assertEqual(sc2m['a'].include_index, False)
         self.assertEqual(sc2m['b'].include_index, False)
 
-
     def test_store_config_map_b(self) -> None:
 
         maps = {'a': StoreConfig(index_depth=2),
@@ -230,7 +226,6 @@ class TestUnit(TestCase):
         self.assertEqual(sc4m['a'].index_depth, 2)
         self.assertEqual(sc4m['b'].index_depth, 3)
 
-
     def test_store_config_map_d(self) -> None:
         with self.assertRaises(ErrorInitStoreConfig):
             _ = StoreConfigMap({'a': object()}) #type: ignore
@@ -238,7 +233,6 @@ class TestUnit(TestCase):
         with self.assertRaises(ErrorInitStoreConfig):
             _ = StoreConfigMap(default=object()) #type: ignore
 
-
     def test_store_get_field_names_and_dtypes_a(self) -> None:
 
         f1 = Frame.from_records((('a', True, None),), index=(('a',)), columns=(('x', 'y', 'z')))
@@ -268,6 +262,67 @@ class TestUnit(TestCase):
         self.assertEqual(dtypes,
                 [np.dtype('<U1'), np.dtype('bool'), np.dtype('O')])
 
+    def test_store_get_field_names_and_dtypes_c(self) -> None:
+
+        f1 = Frame.from_records((('a', True, None),), index=(('a',)), columns=(('x', 'y', 'z'))).rename(columns='foo')
+
+        with self.assertRaises(StoreParameterConflict):
+            field_names, dtypes = Store.get_field_names_and_dtypes(frame=f1,
+                    include_index=False,
+                    include_index_name=True,
+                    include_columns=True,
+                    include_columns_name=True,
+                    )
+
+        field_names, dtypes = Store.get_field_names_and_dtypes(frame=f1,
+                include_index=True,
+                include_index_name=False,
+                include_columns=True,
+                include_columns_name=True,
+                )
+        self.assertEqual(field_names, ['foo', 'x', 'y', 'z'])
+        self.assertTrue(len(field_names) == len(dtypes))
+
+    def test_store_get_field_names_and_dtypes_d(self) -> None:
+
+        from static_frame.core.index_hierarchy import IndexHierarchy
+        columns = IndexHierarchy.from_labels(((1, 'a'), (1, 'b'), (2, 'c')), name=('foo', 'bar'))
+        f1 = Frame.from_records((('a', True, None),), index=(('a',)), columns=columns)
+
+        field_names, dtypes = Store.get_field_names_and_dtypes(frame=f1,
+                include_index=True,
+                include_index_name=False,
+                include_columns=True,
+                include_columns_name=True,
+                )
+        self.assertEqual(field_names, [('foo', 'bar'), "[1 'a']", "[1 'b']", "[2 'c']"])
+        self.assertTrue(len(field_names) == len(dtypes))
+
+
+        field_names, dtypes = Store.get_field_names_and_dtypes(frame=f1,
+                include_index=True,
+                include_index_name=False,
+                include_columns=True,
+                include_columns_name=True,
+                force_brackets=True,
+                )
+        self.assertEqual(field_names, ["['foo' 'bar']", "[1 'a']", "[1 'b']", "[2 'c']"])
+
+        with self.assertRaises(StoreParameterConflict):
+            field_names, dtypes = Store.get_field_names_and_dtypes(frame=f1,
+                    include_index=True,
+                    include_index_name=False,
+                    include_columns=True,
+                    include_columns_name=False,
+                    )
+
+        with self.assertRaises(StoreParameterConflict):
+            field_names, dtypes = Store.get_field_names_and_dtypes(frame=f1,
+                    include_index=False,
+                    include_index_name=False,
+                    include_columns=True,
+                    include_columns_name=True,
+                    )
 
     #---------------------------------------------------------------------------
 
@@ -278,8 +333,7 @@ class TestUnit(TestCase):
         sc1m = StoreConfigMap.from_initializer(maps)
         self.assertTrue(sc1m.default == StoreConfigMap._DEFAULT)
 
-    #---------------------------------------------------------------------------
-
 
 if __name__ == '__main__':
+    import unittest
     unittest.main()
